#!/usr/bin/env python
# coding: utf-8
# $Id: dm_fir_reff_fast.py 374 2016-06-14 05:38:21Z emo $
from __future__ import print_function
import io
import os
import math
import numpy
import re
import time
import egdbcommon
import zipfile
import io
import subprocess
from MyViewGlobal import logger
from functools import reduce
from util import ParseShotNo
from fence import Fence
#import dataSource
import pickle
import pandas

def ApplyFilter(form, xdata, xedata, ydata, yedata, zdata, zedata):
    result = ydata if len(zdata)==0 else zdata
    sz = len(xdata)
    if len(xedata) == 0:
        xedata = [0.0] * sz
    if len(yedata) == 0:
        yedata = [0.0] * sz
    if len(zedata) == 0:
        zedata = [0.0] * sz
    if len(zdata) == 0:
        zdata = [0.0] * sz

    if form is None:
        return result

    try:
        result_eval = [eval(form, {'x':x, 'xe': xe, 'y':y, 'ye':ye, 'z':z, 'ze':ze, 'm':math}) 
                for x, xe, y, ye, z, ze in zip(xdata, xedata, ydata, yedata, zdata, zedata)]
    except:
        result_eval = result
    return result_eval

def ApplyFilterD(form, xdata, ydata, zdata):
    x, y, z = 1.0, 1.0, 1.0
    eval(form, locals={'x':1.0, 'y':1.0, 'z':1.0})
    zdata_new = [eval(form) for x, y, z in zip(xdata, ydata, zdata)]
    return zdata_new

def FindIndicesBeside(v, lst):
    if v <= lst[0]:
        return 0, 0
    if v >= lst[-1]:
        return len(lst)-1, len(lst)-1
    for i, vi in enumerate(lst):
        if vi > v:
            return i-1, i
    return None, None

def NormAngleDeg(deg):
    # convert angle between 0 to 360 degree
    if deg < 0.0 or deg >=360.0:
        return deg % 360
    return deg

def FindNear(theta_phi, theta, phi):
    t0, p0 = theta_phi[0]
    d2min = (theta-t0)**2+(phi-p0)**2
    imin = 0
    for i, (t, p) in enumerate(theta_phi):
        d2 = (theta-t)**2+(phi-p)**2
        if d2 < d2min:
            imin = i
            d2min = d2
    return imin

#------------------------------------------------------
def deg2rad(deg):
    return numpy.pi * deg/180.0

def rad2deg(rad):
    return 180.0 * rad/numpy.pi

def AngleToXcYc(crange, ctheta, cphi):
    rs = crange*(1-cphi/(math.pi/2))
    x = rs * numpy.cos(ctheta)
    y = -rs * numpy.sin(ctheta)
    return x, y

def AstroAngleToCanvasAngle(theta, phi):
    ctheta = -1 * (theta + 90)
    cphi = phi
    return ctheta, cphi

def CanvasAngleToAstroAngle(ctheta, cphi):
    theta = -1*ctheta -90
    phi = cphi
    return theta, phi

def PhiToRadiusOnImage(radius0_on_image, phi):
    rs = radius0_on_image*(1-phi/(math.pi/2))
    #rs = radius0_on_image*math.sin((math.pi/2-phi)/2)
    #print(radius0_on_image, phi, rs)
    return rs

def RadiusOnImageToPhi(radius0_on_image, rs):
    phi = (1 - rs/radius0_on_image) * (math.pi/2) 
    #print(radius0_on_image, phi, rs)
    return phi

def AngleToXYOnImage(cx, cy, radius0_on_image, oft, theta, phi, scl):
    """
    cx, cy: center of image (phi=90 degree)
    image_radius: phi=0degree circle raius (x at theta=0, phi=0 degree)
    theta : azimuthal angle
    phi : gyoukaku 
    """
    #print("AngleToXY input:", cx, cy, radius0_on_image, oft, theta, phi, scl)
    rs = PhiToRadiusOnImage(radius0_on_image, phi)
    dx = rs * math.cos(theta)
    dy = rs * math.sin(theta)
    #print('Angle=>XY', dx, dy, rs, math.atan2(dy, dx))
    #exit(-1)

    x = dx * math.cos(oft) - dy * math.sin(oft)
    y = dx * math.sin(oft) + dy * math.cos(oft)
    #print('X, Y, oft', x, y, oft)
    #exit(-1)

    xs = cx + x
    ys = cy + y/scl
    return xs, ys

def XYOnImageToAngle(cx, cy, radius0_on_image, oft, scl, xs, ys):
    # for canvas angle (NOT astro angle)
    #print("XYToAngle input:", cx, cy, radius0_on_image, oft, scl, xs, ys)
    x = xs - cx
    y = (ys - cy) * scl

    a11, a12 = math.cos(oft), -math.sin(oft)
    a21, a22 = math.sin(oft), math.cos(oft)
    den = (a11*a22-a12*a21)

    dx = (x * a22 - a12 * y)/den
    dy = (a11 * y - x * a21)/den
    rs = math.sqrt(dx**2 + dy**2)

    theta = math.atan2(dy, dx)
    phi = RadiusOnImageToPhi(radius0_on_image, rs) 
    return theta, phi

def RadiusOnCanvasToPhi(crange, r):
    phi = (math.pi/2) * (crange - r)/crange
    return phi

def XYOnCanvasToAngle(crange,  x, y):
    r = math.sqrt(x**2+y**2)
    phi = RadiusOnCanvasToPhi(crange, r)
    theta = math.atan2(y, x)
    return theta, phi


def SkyAngleToCanvasAngle(theta, phi):
    ctheta = -1 * (theta + 90)
    cphi = phi
    return ctheta, cphi


#----------------------------------------------------------

def GetValues(eg, vname, tidx):
    xs = eg.dims(1)
    ys = eg.dims(2)
    width , height = len(xs), len(ys)
    values = numpy.zeros((width, height))
    #print('vname: ', vname)
    #print('Valnames: ', eg.valnames)
    for yi in range(height):
        values[:, yi] = eg.trace_of(vname, 1, tidx, yi)
    #import matplotlib.pyplot as plt
    #plt.plot(values[:, 30])
    #plt.show()
    #exit(-1)
    return values

def FindIndexNear(v, lst):
    dv_i = [(abs(v-vi), i) for i, vi in enumerate(lst)]
    dv_i.sort()
    return dv_i[0][1]

def LimitValue(v, vmin, vmax):
    if vmin > v:
        return vmin
    if vmax <= v:
        return vmax
    return v

def AverageGroupOfIndices(sz, avgn):
    lst = [i for i in range(sz)]
    paires = [lst[i: i+avgn] for i in range(0, sz, avgn)] 
    return paires

def AveragePartOfList(lst, indices):
    return sum([lst[i] for i in indices]) / len(indices)

def AverageLists(lsts):
    """
    [[v1, v2, v3], [v4, v5, v6]]
    =>
    [avg([v1, v4]), avg([v2, v5]), avg([v3, v6])]
    """
    n = len(lsts)
    return [sum(values)/float(n) for values in zip(*lsts)]

def Flatten(lstsOflst):
    """
    [[a, b], [c, d], [e, f]] => [a, b, c, d, e, f]
    """
    return reduce((lambda x, y: x + y), lstsOflst, [])

def StandardDeviation(lstsOflst):  
    values = Flatten(lstsOflst)
    n = len(values)
    avg = sum(values)/n
    return math.sqrt(sum([(v-avg)**2 for v in values])/n)

def StandardDeviationError(lstsOflst):  
    values = Flatten(lstsOflst)
    n = len(values)
    avg = sum(values)/n
    return math.sqrt(sum([(v-avg)**2 for v in values])/n/n)

def StandardDeviationLists(lsts):
    """
    [[l1, l2, l3], [l4, l5, l6]]
    =>
    [sd([l1, l4]), sd([l2, l5]), sd([l3, l6])]
    """
    #n = len(lsts)
    return [StandardDeviation(lstsOflst) for lstsOflst in zip(*lsts)]

def StandardDeviationErrorLists(lsts):
    """
    [[l1, l2, l3], [l4, l5, l6]]
    =>
    [sd([l1, l4]), sd([l2, l5]), sd([l3, l6])]
    """
    #n = len(lsts)
    return [StandardDeviationError(lstsOflst) for lstsOflst in zip(*lsts)]

def PartOfList(lsts, idxs):
    return [lsts[i] for i in idxs]

def AverageErrorLists(lsts, indices):
    """
     indices = [[0, 1, 2], [3, 4, 5], ...]
     lsts = [[v1, v2, v3, v4, ...],
              [v1, v2, v3, v4, ...],
              ...]
     values = [[v1, v1, ..], [v2, v2, ..], ...]
     groups = [[[v1, v1, ...], [v2, v2, ...], ..], [[], [], ..], ...]
    """

    values = [list(item) for item in zip(*lsts)]
    groups = [PartOfList(values, idxs) for idxs in indices] 
    packs = [JoinList(grp) for grp in groups]
    sz = float(len(packs[0]))
    return [math.sqrt(sum([x*x for x in pack]))/sz/math.sqrt(sz) for pack in packs]

def JoinList(lst):
    return reduce(lambda x, y: x+y, lst, [])


def StringToInteger(vstr):
    try:
        return int(vstr)
    except ValueError:
        return 0

def ParseDataID(dataid):
    if -1 == dataid.find(">"):
        # not found
        return None, None, None, None

    clms = dataid.split(">")
    name_shot = clms[1]
    shot, sub = ParseShotNo(shot)
    return clms[0], name, shot, sub

def helptext():
    text = """
    <p>Here is an expanation of this module <p><br>
    Module name : 2dspec <br>
    Synopsis    : plot 2dspec_wimage data . <br>
    Script name : dm_2dspec_wimage.py <br>
    Data source : local file storage <br>
    Author name : M. Yoshinuma <br>
    Contact to  : yoshinuma@nifs.ac.jp <br>
    Plot type   : xy, xy2, xyz
    <br>
    <p>Options<p><br>
    use n1avg or n2avg option to average the data on the original dimension.
    nxavg=#; : average # points on dimension 1 (1..340)<br>
    nyavg=#; : average # points on dimension 2 (1..256)<br>
    step=#; : step of mesh on canvas axis.
    slicedat=#; : slice position for x-y or x-y2 plot. used if (theta, phi) = (0, 0)
                  choice time for xyz plot.
    theta=#;phi=# : make slicedat by using theta and phi. default (theta, phi) = (0, 0)
    force; : load from original file (not use pickle)
    <br>
    <p>Load data from EG data server<br>
    1)Select Add for shot number. <br>
    2)Set string like '>diagname%shot%' in the TextControl of the Add. 
    <br>
    """
    return text
    
class InterFace(object):
    def __init__(self, dataid, **kwargs):
        """
        kwargs:
            option: option string for trace
            cache: myview cache object
            cachebutton: state of button cache on myView.
                         "CACHE": 
                         "ONLINE":
                         "OFFLINE":
            temppath: temp path set with myView dialog.
            cmdpath: command path set with myView dialog
        if there is some troubole to make instance, raise "Error"
        """
        self.dataid = dataid
        self.option = kwargs.get('option')
        self.cache = kwargs.get('cache')
        self.cachebutton = kwargs.get('cachebutton')
        self.temppath = kwargs.get('temppath')
        self.cmdpath = kwargs.get('cmdpath')
        self.flgloaded = False
        self.with_errbar = False
        self.with_sd = False
        self.with_sde = False
        self.parse_option(self.option)
        self.image_cache = {}

        base, name = os.path.split(self.dataid)
        self.diagname = '%s' % name
        self.cachepath = None

    def parse_option(self, option):
        with_errbar = False
        force = False
        nxavg = 1
        nyavg = 1
        crange = 90
        cstep = 2.0
        sliced_at = 0
        theta_at = 0
        phi_at = 0
        ff = None
        for p in [o.strip() for o in option.split(';')]:
            keyvalue = [item.strip() for item in p.split('=')]
            if len(keyvalue) == 1:
                key, = keyvalue
                if key.lower() == 'werr':
                    with_errbar = True
                if key.lower() == 'force':
                    force = True
            if len(keyvalue) == 2:
                key, value = keyvalue
                if key == 'nxavg':
                    try:
                        nxavg = int(value)
                    except ValueError:
                        nxavg = 1
                    nxavg = LimitValue(nxavg, 1, 340)
                
                if key == 'nyavg':
                    try:
                        nyavg = int(value)
                    except ValueError:
                        nyavg = 1
                    nyavg = LimitValue(nyavg, 1, 256)
                
                if key == 'step':
                    try:
                        cstep = float(value)
                    except ValueError:
                        cstep = 2.0
                    cstep = LimitValue(cstep, 0.0001, 45)
                if key == 'slicedat':
                    try:
                        sliced_at = float(value)
                    except ValueError:
                        sliced_at = 0.0
                if key == 'theta':
                    try:
                        theta_at = float(value)
                    except ValueError:
                        theta_at = 0.0
                    theta_at = NormAngleDeg(theta_at)
                if key == 'phi':
                    try:
                        phi_at = float(value)
                    except ValueError:
                        phi_at = 0.0
                if key == 'ff':
                    ff = value

        self.force = force
        self.nxavg = nxavg
        self.nyavg = nyavg
        self.crange = crange
        self.cstep = cstep
        self.sliced_at = sliced_at
        self.theta_at = theta_at
        self.phi_at = phi_at
        self.ff = ff
        #print "#Plot3D : setting n1avg=%d n2avg=%d" % (self.n1avg, self.n2avg)

    def load(self, **kwargs):
        print("# ************* DATA LOAD ************")
        self.cachebutton = kwargs.get('cachebutton')
        shot, subshot = ParseShotNo(self.dataid)
        if shot is None:
            self.eg, pkl_path = EG_2D_pickle(self.dataid, force=self.force)
        else:
            self.load_target(shot, subshot)
            
        if self.eg == None or self.eg.dimno != 3:
            self.eg = None
            self.flgloaded = False
        else:
            self.flgloaded = True
            self.init_dimensions()
        print("# **** LOAD FINISHED [%s] ***" % self.dataid)

    def load_target(self, shot, subshot):
        if self.flgloaded:
            return

        diagname = 'mimage_time'
        shot, subshot = ParseShotNo(self.dataid)
        key = '%s_%d_%d' % (diagname, shot, subshot)
        pkl_key = 'pkl_%s_%d_%d' % (diagname, shot, subshot)
        self.cachepath = None
        zippath = None
        #self.cache.opencache()

        # try to get datafile from cache
        filetime = self.cache.findTime(key) 

        fence = Fence(key)
        try:
            fence.lock()

            if self.cachebutton == 'ONLINE' and egdbcommon.compareTime(diagname, shot, subshot, filetime):
                zippath=None
                logger.debug("*** DISCARD CACHE FILE FOR %s@%s ***" % (diagname, self.dataid))
            else:
                if self.flgloaded:
                    return
                logger.debug("*** USE CACHE FILE FOR %s@%s ***" % (diagname, self.dataid))
                self.cachepath = self.cache.find(key) 

            if not self.cachepath and self.cachebutton == 'ONLINE':
                # try to get datafile from server
                logger.debug("load from network")
                fname = "%s.zip" % key
                fname = self.cache.makeUniqueFilename(fname, self.temppath)
                zippath = os.path.join(self.temppath, fname) 
                cmdstr = "igetfile -s %d -m %d -d %s -Z -o %s" % (
                    shot, subshot, diagname, zippath)
                #cmdstr = os.path.join(self.cmdpath, cmdstr)
                #print cmdstr
                if egdbcommon.icheckfile(diagname, shot, subshot):
                    subprocess.check_call(cmdstr,shell=True)
                    if os.path.exists(zippath):
                        logger.debug("Register the Datafile %s" % zippath)
                        zippath = self.cache.registFile(key, zippath)
                    else:
                        zippath = None
                else:
                    zippath = None

            if self.cachepath:
#            self.eg = EG_3D(self.cachepath)
                self.eg,  pkl_path = EG_3D_pickle(self.cachepath)
                self.cache.releasefile(self.cachepath)
            else:
                if zippath:
                    self.eg, pkl_path = EG_3D_pickle(zippath, True)
                    self.cachepath = self.cache.registFile(pkl_key, pkl_path)
                    self.cache.releasefile(self.cachepath)
                else:
                    self.eg = None
        finally:
            fence.release()

    def init_dimensions(self):
        nxavg, nyavg = self.nxavg, self.nyavg
        self.times = self.eg.dims(0)
        xs = self.eg.dims(1)
        ys = self.eg.dims(2)
        self.tsize = len(self.times)
        self.xsize = len(xs)
        self.ysize = len(ys)
        cx = float(self.eg.keyvalue['cx'])
        cy = float(self.eg.keyvalue['cy'])
        ri = float(self.eg.keyvalue['ri'])
        oft = float(self.eg.keyvalue['oft'])
        scl = float(self.eg.keyvalue['scl'])
        self.cx, self.cy, self.ri, self.oft, self.scl = cx, cy, ri, oft, scl
        self.xsize_avg = self.xsize // nxavg
        self.ysize_avg = self.ysize // nyavg
        self.xs_avg = [sum(xs[i*nxavg: (i+1)*nxavg])/nxavg for i in range(self.xsize_avg)]
        self.ys_avg = [sum(ys[j*nyavg: (j+1)*nyavg])/nyavg for j in range(self.ysize_avg)]
        print(self.xsize_avg, self.ysize_avg)

    def datainfo(self, **kwargs):
        ptype = kwargs.get('type', None)
        if not self.flgloaded:
            self.load(**kwargs)

        if not self.eg:
            return "Data Loading Failed."
        return "\n".join(self.eg.headlines)

    def varsx(self, **kwargs):
        ptype = kwargs.get('type', None)
        if not self.flgloaded:
            self.load(**kwargs)

        if self.eg is None:
            return []
        
        if ptype == 'xyz':
            return ['EW']
        return ['time', 'EW', 'SN']

    def varsy(self, **kwargs):
        ptype = kwargs.get('type', None)
        varx = kwargs.get('varx', None)
        
        if not self.flgloaded:
            self.load(**kwargs)
        if not self.eg:
            return []

        if ptype == 'xyz':
            return ['SN']

        return self.eg.valnames

    def varsz(self, **kwargs):
        ptype = kwargs.get('type', None)
        varx = kwargs.get('varx', None)
        vary = kwargs.get('vary', None)
        if ptype == 'xyz':
            return self.eg.valnames
        return []

    def number_not_in_list(self, lst):
        notin = [i for i in range(0, 3) if i not in lst]
        return notin[0]
       
    def indexs(self, **kwargs):
        ptype = kwargs.get('type', None)
        varx = kwargs.get('varx', None)
        vary = kwargs.get('vary', None)
        #if ptype == 'xyz':
        #    return []
        #if ptype == 'xyz':
        
        names = ['1'] + self.eg.valnames[2:]
        return ["%d_%s" % (i, n) for i, n in enumerate(names)]

        #idxs = [ v for v in range(-self.crange, self.crange+1)]  
        #return ["%d_%d" % (i, idx) for i, idx in enumerate(idxs)]
    
    def refresh_index(self, kwargs):
        try:
            index = kwargs.get('index', None)
            trace = kwargs.get('trace', None)
            indexs = self.indexs(**kwargs)
            if index and trace:
                index = egdbcommon.get_index(indexs, index)
                kwargs['index'] = index
                trace = kwargs['trace']
                trace.index = index
        except:
            pass

    def dataset(self, **kwargs):
        """
        return list of dataset for varx, vary, varz, index, option.
        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog
            index: 

        dataset := [title, x, xe, y, ye, z, ze]
        x = [x1, x2, x3, ... , xn]
        y = [y1, y2, x3, ... , ym] 
        z = [[z(x1, y1), z(x2, y1), z(x3, y1),...,z(xn, y1)],
             [z(x1, y2), z(x2, y2), z(x3, y2),...,z(xn, y2)],
             [z(x1, y3), z(x2, y3), z(x3, y3),...,z(xn, y3)], 
             ......
             [z(x1, ym), z(x2, ym), z(x3, ym),...,z(xn, ym)]] 
        len(z) = len(y)
        len(z[i]) = len(x)
        """
        ptype = kwargs.get('type', None)
        varx = kwargs.get('varx', None)
        vary = kwargs.get('vary', None)

        if not self.flgloaded:
            self.load(**kwargs)

        dataset = ['%s No file' % self.dataid, [],[],[],[],[],[]]
        if not self.eg:
            return dataset

        dataset = ['%s No data' % self.dataid, [],[],[],[],[],[]]
        
        self.refresh_index(kwargs)
        
        if ptype == 'xyz':
            dataset = self.mapxyz(**kwargs)
            return dataset

        dataset = self.trace(**kwargs)
        return dataset

    def take_averaged_image(self, vname, tidx):
        print("Taking averaged image [%s]." % (vname))
        key = vname+"%d" % tidx
        image = self.image_cache.get(key, None)
        if image is not None:
            return image

        nxavg, nyavg = self.nxavg, self.nyavg
        #img = numpy.zeros((self.xsize, self.ysize))
        #integrate(average) x, y direction
        #print("GetVaues()")
        img = GetValues(self.eg, vname, tidx)
        #print("Average %d %d" % (self.xsize_avg, self.ysize_avg))
        img_avg = [img[i*nxavg:(i+1)*nxavg, j*nyavg:(j+1)*nyavg].sum()/nxavg/nyavg
                for i in range(self.xsize_avg) for j in range(self.ysize_avg)]
        #print("Reshape")
        img_avg = numpy.array(img_avg).reshape((self.xsize_avg, self.ysize_avg))
        self.image_cache[key] = img_avg
        #print("Finished")
        return img_avg

    def getvalue(self, img, x, y, xlst, ylst):
        def dist(x0, y0, x1, y1):
            return math.sqrt((x0-x1)**2 + (y0-y1)**2)

        xmin, xmax = min(xlst), max(xlst)
        ymin, ymax = min(ylst), max(ylst)

        if (xmin > x) or (xmax < x) or (ymin > y) or (ymax < y):
            return 0.0
        i0, i1 = FindIndicesBeside(x, xlst) 
        j0, j1 = FindIndicesBeside(y, ylst) 
        x0, x1 = xlst[i0], xlst[i1]
        y0, y1 = ylst[j0], ylst[j1]
        v00 = img[i0, j0]
        v01 = img[i0, j1]
        v11 = img[i1, j1]
        v10 = img[i1, j0]
        d00 = dist(x, y, x0, y0)
        d01 = dist(x, y, x0, y1)
        d11 = dist(x, y, x1, y1)
        d10 = dist(x, y, x1, y0)
        dw00 = 1/d00 if (d00 > 0) and (v00 != 0.0) else 0
        dw01 = 1/d01 if (d01 > 0) and (v01 != 0.0) else 0
        dw11 = 1/d11 if (d11 > 0) and (v11 != 0.0) else 0
        dw10 = 1/d10 if (d10 > 0) and (v10 != 0.0) else 0
        dwm = dw00 + dw01 + dw11 + dw10
        if dwm == 0:
            return v00
        v = (v00*dw00 + v01*dw01 + v11*dw11 + v10*dw10)/dwm
        #v = (v00 + v01 + v11 + v10)/4
        return v

    def render_data(self, img, xlst, ylst):
        crange, cstep = self.crange, self.cstep
        cx, cy, ri, oft_rad, scl = self.cx, self.cy, self.ri, deg2rad(self.oft), self.scl
        xdata = []
        ydata = []
        zdata = []
        caxis = numpy.linspace(-crange, crange, int(2*crange/cstep)+1)
        #print("caxis:", caxis)
        for y in caxis: #range(-crange, crange+1):
            for x in caxis: #range(-crange, crange+1):
                #print(x, y)
                theta, phi = XYOnCanvasToAngle(crange, x, -y)
                xs, ys = AngleToXYOnImage(cx, cy, ri, oft_rad, theta, phi, scl)
                v = self.getvalue(img, xs, ys, xlst, ylst)
                xdata.append(x)
                ydata.append(y)
                zdata.append(v)
        return xdata, ydata, zdata

    def render_at_y(self, img, xlst, ylst, sliced_at):
        crange, cstep = self.crange, self.cstep
        cx, cy, ri, oft_rad, scl = self.cx, self.cy, self.ri, deg2rad(self.oft), self.scl
        xdata = []
        ydata = []
        zdata = []
        #idxv = [v for v in range(-crange, crange+1)]

        caxis = numpy.linspace(-crange, crange, int(2*crange/cstep)+1)
        y = sliced_at #idxv[idxy]
        xss, yss = [], []
        for i, x in enumerate(caxis): #enumerate(range(-crange, crange+1)):
            theta, phi = XYOnCanvasToAngle(crange, x, -y)
            xs, ys = AngleToXYOnImage(cx, cy, ri, oft_rad, theta, phi, scl)
            v = self.getvalue(img, xs, ys, xlst, ylst)
            xdata.append(x)
            ydata.append(v)
            xss.append(xs)
            yss.append(ys)
        return xdata, ydata, xss, yss

    def render_at_x(self, img, xlst, ylst, sliced_at):
        crange, cstep = self.crange, self.cstep
        cx, cy, ri, oft_rad, scl = self.cx, self.cy, self.ri, deg2rad(self.oft), self.scl
        xdata = []
        ydata = []
        zdata = []
        #idxv = [v for v in range(-crange, crange+1)]
        caxis = numpy.linspace(-crange, crange, int(2*crange/cstep)+1)
        x = sliced_at #idxv[idxx]
        xss, yss = [], []
        for j, y in enumerate(caxis): #enumerate(range(-crange, crange+1)):
            theta, phi = XYOnCanvasToAngle(crange, x, -y)
            xs, ys = AngleToXYOnImage(cx, cy, ri, oft_rad, theta, phi, scl)
            v = self.getvalue(img, xs, ys, xlst, ylst)
            xdata.append(y)
            ydata.append(v)
            xss.append(xs)
            yss.append(ys)
        return xdata, ydata, xss, yss

    def mapxyz(self, **kwargs):
        varx = kwargs.get('varx', None)
        vary = kwargs.get('vary', None)
        varz = kwargs.get('varz', None)
        index = kwargs.get('index', None)
        idx = int(index.split('_')[0])
        #den_name = index.split('_')[1] 
        den_name = "_".join(index.split('_')[1:]) 
        xdata = []
        xedata = []
        ydata = []
        yedata = []
        zdata = []
        zedata = []

        #crange = self.crange
        nxavg, nyavg = self.nxavg, self.nyavg

        tidx = FindIndexNear(self.sliced_at, self.times)
        tm = self.times[tidx]

        vidx = self.eg.vidx(varz)
        #print("vidx:", vidx)
        unit = self.eg.valunits[vidx]
        den_vidx = self.eg.vidx(den_name)
        if den_vidx < 0:
            title = '%s_%s(%s)@%gs' % (self.diagname, varz, unit, tm)
            img_avg = self.take_averaged_image(varz, tidx)
        else:
            title = '%s_%s/%s@%gs' % (self.diagname, varz, den_name, tm)
            img_avg_nom = self.take_averaged_image(varz, tidx)
            img_avg_den = self.take_averaged_image(den_name, tidx)
            img_avg = img_avg_nom/img_avg_den
        
        #import matplotlib.pyplot as plt
        #plt.imshow(img_avg)
        #plt.show()

        #print("Render data")
        xdata, ydata, zdata = self.render_data(img_avg, self.xs_avg, self.ys_avg)
        #print("Finished")

        if varz == 'theta':
            zdata = [NormAngleDeg(z) for z in zdata]
       
        """
        xc_yc = [AngleToXcYc(crange, t, p) for t, p in self.theta_phi]
        print(len(xc_yc))
        xdata = [xc for xc, yc in xc_yc]
        ydata = [yc for xc, yc in xc_yc]
        zdata = img_avg
        """

        zdata = ApplyFilter(self.ff, xdata, xedata, ydata, yedata, zdata, zedata)

        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

    def trace_time(self, **kwargs):
        varx = kwargs.get('varx', None)
        vary = kwargs.get('vary', None)
        index = kwargs.get('index', None)
        idx = int(index.split('_')[0])
        den_name = "_".join(index.split('_')[1:]) 
        xdata = []
        xedata = []
        ydata = []
        yedata = []
        zdata = []
        zedata = []
        nxavg, nyavg = self.nxavg, self.nyavg
        
        tidx = FindIndexNear(self.sliced_at, self.times)
        tm = self.times[tidx]

        #Find (Xs, Ys) for (Theta,  Phi)
        ctheta_deg, cphi_deg = SkyAngleToCanvasAngle(self.theta_at, self.phi_at)
        ctheta, cphi = deg2rad(ctheta_deg), deg2rad(cphi_deg)
        sliced_at_x, sliced_at_y = AngleToXcYc(self.crange, ctheta, cphi)
        #print("(theta, phi)=>(xc, yc) : (%g, %g)=>(%g, %g)" % 
        #        (self.theta_at, self.phi_at, sliced_at_x, sliced_at_y)) 
        theta, phi = XYOnCanvasToAngle(self.crange, sliced_at_x, -sliced_at_y)
        #print("(theta_c, phi_c) = (%g, %g)" % (rad2deg(theta), rad2deg(phi)))
        cx, cy, ri, oft_rad, scl = self.cx, self.cy, self.ri, deg2rad(self.oft), self.scl
        xs, ys = AngleToXYOnImage(cx, cy, ri, oft_rad, theta, phi, scl)

        vidx = self.eg.vidx(vary)
        unit = self.eg.valunits[vidx]
        den_vidx = self.eg.vidx(den_name)
       
        #Fetch Data
        ydata = []
        if den_vidx < 0:
            title = '%s_%s(%s)_at%.1f_%.1f' % (self.diagname, vary, unit, 
                    self.phi_at, self.theta_at)
        else:
            title = '%s_%s/%s_at%.1f_%.1f' % (self.diagname, vary, den_name, 
                    self.phi_at, self.theta_at)

        for tidx, tm in enumerate(self.times):
            if den_vidx < 0:
                img_avg = self.take_averaged_image(vary, tidx)
            else:
                img_avg_nom = self.take_averaged_image(vary, tidx)
                img_avg_den = self.take_averaged_image(den_name, tidx)
                img_avg = img_avg_nom/img_avg_den

            v = self.getvalue(img_avg, xs, ys, self.xs_avg, self.ys_avg)
            #print("Value : %g @ (theta %g, phi %g) , (xs %g, ys %g)" % 
            #        (v, self.theta_at, self.phi_at, xs, ys))
            ydata.append(v)
        xdata = self.times
        ydata = ApplyFilter(self.ff, xdata, xedata, ydata, yedata, zdata, zedata)
        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

    def trace(self, **kwargs):
        varx = kwargs.get('varx', None)
        vary = kwargs.get('vary', None)
        index = kwargs.get('index', None)
        idx = int(index.split('_')[0])
        #den_name = index.split('_')[1] 
        den_name = "_".join(index.split('_')[1:]) 
        xdata = []
        xedata = []
        ydata = []
        yedata = []
        if varx == 'time':
            return self.trace_time(**kwargs)

        nxavg, nyavg = self.nxavg, self.nyavg
        #idx = int(index.split('_')[0])
        #idxv = int(index.split('_')[1])

        if self.theta_at == 0.0 and self.phi_at == 0.0:
            sliced_at = self.sliced_at
        else:
            ctheta_deg, cphi_deg = SkyAngleToCanvasAngle(self.theta_at, self.phi_at)
            ctheta, cphi = deg2rad(ctheta_deg), deg2rad(cphi_deg)
            sliced_at_x, sliced_at_y = AngleToXcYc(self.crange, ctheta, cphi)
            if varx == 'EW':
                sliced_at = sliced_at_y
            else:
                sliced_at = sliced_at_x
            print("(theta, phi)=>(xc, yc) : (%g, %g)=>(%g, %g)" % 
                    (self.theta_at, self.phi_at, sliced_at_x, sliced_at_y)) 
            theta, phi = XYOnCanvasToAngle(self.crange, sliced_at_x, -sliced_at_y)
            print("(theta_c, phi_c) = (%g, %g)" % (rad2deg(theta), rad2deg(phi)))
            cx, cy, ri, oft_rad, scl = self.cx, self.cy, self.ri, deg2rad(self.oft), self.scl
            xs, ys = AngleToXYOnImage(cx, cy, ri, oft_rad, theta, phi, scl)

        vidx = self.eg.vidx(vary)
        unit = self.eg.valunits[vidx]
        den_vidx = self.eg.vidx(den_name)
        if den_vidx < 0:
            title = '%s_%s(%s)_at%.1f' % (self.diagname, vary, unit, sliced_at)
            img_avg = self.take_averaged_image(vary)
        else:
            title = '%s_%s/%s_at%.1f' % (self.diagname, vary, den_name, sliced_at)
            img_avg_nom = self.take_averaged_image(vary)
            img_avg_den = self.take_averaged_image(den_name)
            img_avg = img_avg_nom/img_avg_den


        if self.theta_at == 0.0 and self.phi_at == 0.0:
            pass
        else:
            v = self.getvalue(img_avg, xs, ys, self.xs_avg, self.ys_avg)
            print("Value : %g @ (theta %g, phi %g) , (xs %g, ys %g)" % 
                    (v, self.theta_at, self.phi_at, xs, ys))

        if varx == 'EW':
            xdata, ydata, xss, yss = self.render_at_y(img_avg, self.xs_avg, self.ys_avg, sliced_at)
        else:
            xdata, ydata, xss, yss = self.render_at_x(img_avg, self.xs_avg, self.ys_avg, sliced_at)
        """
        for xs, ys in zip(xss, yss):
            print("%g\t%g" % (xs, ys))
        print("")
        """
        #print("Len:", len(xdata))

        zdata = []
        zedata = []
        ydata = ApplyFilter(self.ff, xdata, xedata, ydata, yedata, zdata, zedata)
        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

def ReadFromZipfile(filepath):
    zf = zipfile.ZipFile(filepath, 'r')
    target = zf.namelist()[0]
    cbuff = zf.read(target)
    zf.close()
    return cbuff

def OpenFile(filepath):
    if filepath[-3:].upper() == 'ZIP':
        zf = zipfile.ZipFile(filepath, 'r')
        target = zf.namelist()[0]
        return zf, io.TextIOWrapper(io.BufferedReader(zf.open(target, 'r')),encoding='ascii')
    else:
        return None, open(filepath, "r")

def CloseFile(zf, fp):
    if zf:
        fp.close()
        zf.close()
    else:
        fp.close()

def CalcWeights(lst):
    if len(lst) == 0:
        return [1]

    subweights = CalcWeights(lst[1:])
    return [lst[0] * subweights[0]] + subweights

def EG_3D_pickle(filepath, force=False):
    print("TRY TO OPEN PICKLE DATA")
    base, ext = os.path.splitext(filepath)
    picklefilepath = base + '.pkl'
    if (not force) and os.path.exists(picklefilepath):
        print("FOUND PICKE DATA")
        with open(picklefilepath, "rb") as f:
            eg = pickle.load(f)
        print("PICKE DATA LOADED")
    else:
        print("NOT FOUND PICKE DATA")
        eg = EG_3D(filepath)
        with open(picklefilepath, "wb") as f:
            pickle.dump(eg, f)
    print("FINISHED LOAD DATA")
    return eg, picklefilepath

class EG_3D(object):
    def __init__(self, filepath):
        """
        self.data[idx]
        iline => self.line_index() 
        size = self.dimno + self.valno
        idx = self.line_size * iline + vidx
        """
        self.filepath = filepath
        self.keyvalue = {}
        self.headlines = []
        self.load(filepath)

    def load(self, filepath):
        zf, fp = OpenFile(filepath)
        l = fp.readline()
        hlines = []
        while l:
            l = l.strip()
            hlines.append(l)
            if len(l) == 0:
                l = fp.readline()
                continue
            if l[0] == '#':
                clm = l[1:].split('=')
                if len(clm) == 2:
                    self.keyvalue[clm[0].strip()] = clm[1].strip()
                l = fp.readline()
                continue
            break
        self.headlines = hlines[:-1]

        cdimnames = self.keyvalue.get('DimName')
        cdimunits = self.keyvalue.get('DimUnit')
        cdimno = self.keyvalue.get('DimNo')
        cdimsize = self.keyvalue.get('DimSize')
        cvalno = self.keyvalue.get('ValNo')
        cvalnames = self.keyvalue.get('ValName')
        cvalunits = self.keyvalue.get('ValUnit')
        
        dimno = int(cdimno)
        valno = int(cvalno)
        line_size = dimno + valno
            
        dimsizes = [int(c.strip()) for c in cdimsize.split(',')]
        line_size = dimno + valno
        data_size = line_size * reduce((lambda x, y: x * y), dimsizes, 1)
        
        # allocate memory
        self.data = numpy.zeros(data_size)

        # make Headder info
        self.dimnames = [c.strip("' ") for c in cdimnames.split(',')] 
        self.dimunits = [c.strip("' ") for c in cdimunits.split(',')] 
        self.valnames = [c.strip("' ") for c in cvalnames.split(',')] 
        self.valunits = [c.strip("' ") for c in cvalunits.split(',')] 
        self.dimno = dimno
        self.valno = valno
        self.line_size = line_size
        self.dimsizes = dimsizes
        self.weights = CalcWeights(self.dimsizes[1:])

        CloseFile(zf, fp)
        zf, fp = OpenFile(filepath)
        data = pandas.read_csv(fp, comment='#', header=None)

        self.data = data.values.reshape(-1)

#        idx = 0
#        while l:
#            l = l.strip()
#            if len(l) == 0:
#                l = fp.readline()
#                continue
#            #for c in l.split(','):
#            for c in re.split(r',\s*|\s+', l):
#                if c != "":
#                    try:
#                        self.data[idx] = float(c)
#                    except ValueError:
#                        self.data[idx] = numpy.nan
#                    idx = idx+1
#            l = fp.readline()

        CloseFile(zf, fp)

    def line_index(self, *dimidxs):
        weighted_indexs = [i * w for i, w in zip(dimidxs, self.weights)]
        return reduce((lambda x, y: x+y), weighted_indexs, 0)

    def get_line(self, *dimidxs):
        idx = self.line_size * self.line_index(*dimidxs)
        return self.data[idx: idx+self.line_size]
    
    def dims(self, dimidx):
        if dimidx == 0:
            pos = [[i, 0, 0] for i in range(self.dimsizes[dimidx])]
        elif dimidx == 1:
            pos = [[0, i, 0] for i in range(self.dimsizes[dimidx])]
        else:
            pos = [[0, 0, i] for i in range(self.dimsizes[dimidx])]
        return [self.get_line(*p)[dimidx] for p in pos]

    def vdata(self, idxv, *dimidxs):
        return self.get_line(*dimidxs)[self.dimno+idxv]

    def vidx(self, name):
        if name in self.valnames:
            return self.valnames.index(name)
        else:
            return -1

    def trace_of_idx(self, vidx, dimidx, idx1, idx2):
        """
        dimidx: idx of variable dims for trace
        ex)
          0: time trace
          1: radial profile
        """
        dimno = self.dimno
        dimsize = self.dimsizes[dimidx]
        if dimidx == 0:
            pos = [[i, idx1, idx2] for i in range(dimsize)]
        elif dimidx == 1:
            pos = [[idx1, i, idx2] for i in range(dimsize)]
        else:
            pos = [[idx1, idx2, i] for i in range(dimsize)]
        return [self.get_line(*p)[self.dimno + vidx] for p in pos]

    def trace_of(self, name, dimidx, idx1, idx2):
        """
        dimidx: idx of variable dims for trace
        ex)
          0: time trace
          1: radial profile
        """
        vidx = self.vidx(name)
        if vidx == -1:
            return []
        dimno = self.dimno
        dimsize = self.dimsizes[dimidx]
        if dimidx == 0:
            pos = [[i, idx1, idx2] for i in range(dimsize)]
        elif dimidx == 1:
            pos = [[idx1, i, idx2] for i in range(dimsize)]
        else:
            pos = [[idx1, idx2, i] for i in range(dimsize)]
        return [self.get_line(*p)[self.dimno + vidx] for p in pos]



def TestPlot(x1, y1, x2, y2):
    import matplotlib.pyplot as plt
    plt.plot(x1, y1, '-')
    plt.plot(x2, y2, '-')
    plt.xlim(-90, 90)
    plt.ylim(0, 20)
    plt.show()

def TestShow(xdata, ydata, zdata):
    import matplotlib.pyplot as plt
    print(len(xdata), len(ydata), len(zdata))
    #sct = plt.scatter(xdata, ydata, c=zdata, s=5, marker='o', 
    #        vmin=-180, vmax=180, cmap='jet')
    sct = plt.scatter(xdata, ydata, c=zdata, s=5, marker='o', 
            vmin=-4, vmax=4, cmap='jet')
    plt.colorbar(sct)
    plt.xlim(-90, 90)
    plt.ylim(-90, 90)
    plt.show()

def TakeValueAt(x, xdata, ydata):
    i0, i1 = FindIndicesBeside(x, xdata)
    x0, x1 = xdata[i0], xdata[i1]
    y0, y1 = ydata[i0], ydata[i1]
    return y0 + (y1-y0)*(x-x0)/(x1-x0)

if __name__ == '__main__':
    from MyViewCache import *
    #option = 'addaxv=reff;wsde;n1avg=1;n2avg=1'
    #option = 'n1avg=2;n2avg=1'
    #option = 'n1avg=2'
    #option = 'addaxv=reff;werr;n1avg=1;n2avg=1'
    option = 'nxavg=1;nyavg=1;theta=10.6;phi=77.6;step=2'
    cache = MyViewCache('cache/cache.txt')
    #ifc = InterFace('samples\\CES2@16514.txt', temppath='temp', 
    #ifc = InterFace('samples\\thomson_109693.zip', temppath='temp', 
    #ifc = InterFace('samples\\tsmap_113287.zip', temppath='temp', 
    #idstr ='samples\\CES2@19389.txt'
    #idstr = 'samples\\DRMP_AC2_ece_100kHz_cave_dT_3D@1060829.dat'
    #idstr = 'samples\\mimage@20231020_213121.txt'
    idstr = 'samples\\mimage_time@20231020_180121.zip'
    #idstr = 'samples\\spec2d_wimage@20231020_213121.txt'

    ifc = InterFace(idstr, temppath='temp', 
            cachebutton='ONLINE', cache=cache,
            cmdpath = 'bin', option=option) 
    ifc.load(cachebutton='ONLINE')
    
    #print('#',ifc.varsx())
    #print('#',ifc.varsy(varx='EW', type='xy'))
    #print('#',ifc.indexs(varx='SN', vary='intensity', type='xy'))
    title, x, xe, y, ye, z, ze =ifc.dataset(
            varx='time', vary='OI(1S)(557.70nm)', varz='', index='0_1', type='xy')
    #varx='time', vary='OI(1S)(557.70nm)', varz='', index='5_OI(1S)(557.70nm)', type='xy')
    print('#', title, x, y)
    for xi, yi in zip(x, y):
        print("%g\t%g" % (xi, yi))
        

    """
    title, x1, xe, y1, ye, z, ze =ifc.dataset(
            varx='EW', vary='OI(1S)(557.70nm)', varz='', 
            index='5_OI(1S)(557.70nm)', type='xy')
    title, x2, xe, y2, ye, z, ze =ifc.dataset(
            varx='SN', vary='OI(1S)(557.70nm)', varz='', 
            index='5_OI(1S)(557.70nm)', type='xy')
    title, x1, xe, y1, ye, z, ze =ifc.dataset(
            varx='EW', vary='OI(1S)(557.70nm)', varz='', 
            index='0_1', type='xy')
    print(title, TakeValueAt(-2.3, x1, y1))
    title, x2, xe, y2, ye, z, ze =ifc.dataset(
            varx='SN', vary='OI(1S)(557.70nm)', varz='', 
            index='0_1', type='xy')
    print(title, TakeValueAt(12.2, x2, y2))
    #TestPlot(x1, y1, x2, y2)
   
    """
    """
    print('#',ifc.varsx(type='xyz'))
    print('#',ifc.varsy(varx='EW', type='xyz'))
    print('#',ifc.varsz(varx='EW', vary='SN', type='xyz'))
    print('#',ifc.indexs(varx='EW', vary='SN', type='xyz'))
    
    title, x, xe, y, ye, z, ze =ifc.dataset(
            varx='EW', vary='SN', varz='OI(1S)(557.70nm)', index='5_OI(1S)(557.70nm)', type='xyz')

    #title, x, xe, y, ye, z, ze =ifc.dataset(
    #        varx='EW', vary='SN', varz='phi', index='0_1', type='xyz')
    print('#', title)
    print(len(x), len(y), len(z))
    print(min(z), max(z))
    #import matplotlib.pyplot as plt
    #plt.plot(z)
    #plt.show()

    #TestShow(x, y, z)

    """
