#!/usr/bin/env python
# coding: utf-8
# $Id: dm_fir_reff_fast.py 374 2016-06-14 05:38:21Z emo $
import io
import os
import math
import numpy
import re
import time
import egdbcommon
import io
import zipfile
import subprocess
import kaisekiInterFace
from MyViewGlobal import logger
from functools import reduce

#import dataSource

#def LimitValue(v, vmin, vmax):
#    if vmin > v:
#        return vmin
#    if vmax <= v:
#        return vmax
#    return v

#def AverageGroupOfIndices(sz, avgn):
#    lst = [i for i in range(sz)]
#    paires = [lst[i: i+avgn] for i in range(0, sz, avgn)] 
#    return paires

#def AveragePartOfList(lst, indices):
#    return sum([lst[i] for i in indices]) / len(indices)
#
#def AverageListWithN(lst, avgn):
#    indices = AverageGroupOfIndices(len(lst), avgn)
#    return [AveragePartOfList(lst, idx) for idx in indices]

#def AverageErrListWithN(lst, avgn):
#    indices = AverageGroupOfIndices(len(lst), avgn)
#    m = math.sqrt(avgn)
#    return [AveragePartOfList(lst, idx)/m for idx in indices]

#def StringToInteger(vstr):
#    try:
#        return int(vstr)
#    except ValueError:
#        return 0
#
#def ParseDataID(dataid):
#    if -1 == dataid.find(">"):
#        # not found
#        return None, None, None
#
#    clms = dataid.split(">")
#    name_shot = clms[1]
#    name, shot = name_shot.split("@")
#    return clms[0], name, StringToInteger(shot)  

def helptext():
    text = """
    <p>Here is an expanation of this module <p><br>
    Module name : plot_eg_1D <br>
    Synopsis    : Plot 1 DimNo EG formatted data on local file storage. <br>
    Script name : dm_plot_eg_1D.py <br>
    Data source : local file storage <br>
    Author name : M. Yoshinuma <br>
    Contact to  : yoshinuma@nifs.ac.jp <br>
    Plot type   : 2D 
    <br>
    <p>Options<p><br>
    use navg option to average the data on the dimension.
    navg=#; : average # points on the dimension (1..20)<br>
    <br>
    use werr option to draw err bar. <br>
    The data of err is the value at the next column of vary. <br>
    werr;<br>
    <br >
    <p>Load data from EG data server<br>
    1)Select Add for shot number. <br>
    2)Set string like '>diagname%shot%' in the TextControl of the Add. 
    <br>
    """
    return text
    
#class InterFace( kaisekiInterFace.kaisekiInterFace ):
class InterFace(object):
    def __init__(self, dataid, **kwargs):
        """
        kwargs:
            option: option string for trace
            cache: myview cache object
            cachebutton: state of button cache on myView.
                         "CACHE": 
                         "ONLINE":
                         "OFFLINE":
            temppath: temp path set with myView dialog.
            cmdpath: command path set with myView dialog
        if there is some troubole to make instance, raise "Error"
        """
        self.dataid = dataid
        self.option = kwargs.get('option')
        self.cache = kwargs.get('cache')
        self.cachebutton = kwargs.get('cachebutton')
        self.temppath = kwargs.get('temppath')
        self.cmdpath = kwargs.get('cmdpath')
        self.flgloaded = False
        self.parse_option(self.option)

        base, name = os.path.split(self.dataid)
        self.diagname = '%s' % name
        self.cachepath = None

    def parse_option(self, option):
        with_errbar = False
        navg = 1
        for p in [o.strip() for o in option.split(';')]:
            keyvalue = [item.strip() for item in p.split('=')]
            if len(keyvalue) == 1:
                key, = keyvalue
                if key.lower() == 'werr':
                    with_errbar = True
            if len(keyvalue) == 2:
                key, value = keyvalue
                if key.lower() == 'navg':
                    try:
                        navg = int(value)
                    except ValueError:
                        navg = 1
                    navg = kaisekiInterFace.LimitValue(navg, 1, 100)

        self.with_errbar = with_errbar
        self.navg = navg

    def load_target(self, diagname, shot, sub):
        # try to get datafile from cache
        str_shot = "%d_%d" % (shot, sub)
        key = '%s_%s' % (diagname, str_shot)
        cachepath = None
        filetime = self.cache.findTime(key) 
        if (self.cachebutton == 'ONLINE' and 
                egdbcommon.compareTime(diagname, shot, sub, filetime)):
            cachepath=None
            logger.debug("*** DISCARD CACHE FILE FOR %s@%s ***" % (diagname, str_shot))
        else:
            cachepath = self.cache.find(key) 
            if cachepath:
                logger.debug("*** USE CACHE FILE FOR %s@%s ***" % (diagname, str_shot))

        if not cachepath and self.cachebutton == 'ONLINE':
            # try to get datafile from server
            logger.debug("load from network")
            fname = "%s.zip" % key
            fname = self.cache.makeUniqueFilename(fname, self.temppath)
            cachepath = os.path.join(self.temppath, fname) 
            cmdstr = "igetfile -s %d -m %d -d %s -Z -o %s" % (
                        shot, sub, diagname, cachepath) 
            #cmdstr = os.path.join(self.cmdpath, cmdstr)
            #print cmdstr
            if egdbcommon.icheckfile(diagname, shot, sub):
                subprocess.check_call(cmdstr,shell=True)
                if os.path.exists(cachepath):
                    logger.debug("Register the Datafile %s" % cachepath)
                    cachepath = self.cache.registFile(key, cachepath)
                else:
                    cachepath = None
            else:
                cachepath = None
        return cachepath

    def load(self, **kwargs):
        """
        try:
            self.eg = EG_1D(self.dataid)
        except:
            self.eg = None
        """
        print("&&self.dataid=",self.dataid)
        directive, diagname, shot, sub = kaisekiInterFace.ParseDataID(self.dataid)
        print("&&shot=%d,sub=%d" % (shot, sub))

        if diagname is None:
            self.eg = EG_1D(self.dataid)
        else:
            cachepath = self.load_target(diagname, shot, sub)
            print("cachepath=",cachepath)
            if cachepath is not None:
                self.eg = EG_1D(cachepath)
                self.cache.releasefile(cachepath)

        if self.eg.dimno != 1:
            self.eg = None
            self.flgloaded = False
        else:
            self.flgloaded = True
       
    def datainfo(self, **kwargs):
        ptype = kwargs.get('type', None)
        if not self.flgloaded:
            self.load(**kwargs)

        if not self.eg:
            return "Data Loading Failed."
        return "\n".join(self.eg.headlines)

    def varsx(self, **kwargs):
        ptype = kwargs.get('type', None)
        if not self.flgloaded:
            self.load(**kwargs)

        if not self.eg:
            return []
        return self.eg.dimnames + self.eg.valnames

    def varsy(self, **kwargs):
        ptype = kwargs.get('type', None)
        
        if not self.flgloaded:
            self.load(**kwargs)
        if not self.eg:
            return []
        
        if ptype == 'xyz':
            return []

        return self.eg.dimnames + self.eg.valnames

    def varsz(self, **kwargs):
        return []

    def indexs(self, **kwargs):
        return []
    
    def refresh_index(self, kwargs):
        try:
            index = kwargs.get('index', None)
            trace = kwargs.get('trace', None)
            indexs = self.indexs(**kwargs)
            if index and trace:
                index = egdbcommon.get_index(indexs, index)
                kwargs['index'] = index
                trace = kwargs['trace']
                trace.index = index
        except:
            pass

    def dataset(self, **kwargs):
        """
        return list of dataset for varx, vary, varz, index, option.
        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog
            index: 

        dataset := [title, x, xe, y, ye, z, ze]
        x = [x1, x2, x3, ... , xn]
        y = [y1, y2, x3, ... , ym] 
        z = [[z(x1, y1), z(x2, y1), z(x3, y1),...,z(xn, y1)],
             [z(x1, y2), z(x2, y2), z(x3, y2),...,z(xn, y2)],
             [z(x1, y3), z(x2, y3), z(x3, y3),...,z(xn, y3)], 
             ......
             [z(x1, ym), z(x2, ym), z(x3, ym),...,z(xn, ym)]] 
        len(z) = len(y)
        len(z[i]) = len(x)
        """
        ptype = kwargs.get('type', None)
        varx = kwargs.get('varx', None)
        vary = kwargs.get('vary', None)

        if not self.flgloaded:
            self.load(**kwargs)

        if not self.eg:
            dataset = ['%s No file' % self.dataid, [],[],[],[],[],[]]
            return dataset

        if self.eg.vidx(vary) < 0:
            dataset = ['%s No data for %s' % (self.dataid, vary), [],[],[],[],[],[]]
            return dataset

        #dimidx = self.eg.dimnames.index(varx)
        dimdata = self.eg.dims(0)
        if 0 == len(dimdata):
            dataset = ['%s No data' % self.dataid, [],[],[],[],[],[]]
            return dataset

        self.refresh_index(kwargs)
        
        if ptype == 'xyz':
            dataset = ['%s Not supported (xy-z)' % self.dataid, [],[],[],[],[],[]]
            return dataset

        dataset = self.trace(**kwargs)
        return dataset

    def trace(self, **kwargs):
        varx = kwargs.get('varx', None)
        vary = kwargs.get('vary', None)
        index = kwargs.get('index', None)
        xdata = []
        xedata = []
        ydata = []
        yedata = []

        if varx in self.eg.dimnames:
            xdata = self.eg.dims(0)
            xdata = kaisekiInterFace.AverageListWithN(xdata, self.navg)
        else:
            xdata = self.eg.trace_of(0, varx)
            xdata = kaisekiInterFace.AverageListWithN(xdata, self.navg)

        if vary in self.eg.dimnames:
            ydata = self.eg.dims(0)
            ydata = kaisekiInterFace.AverageListWithN(ydata, self.navg)
        else:
            ydata = self.eg.trace_of(0, vary)
            ydata = kaisekiInterFace.AverageListWithN(ydata, self.navg)

        if vary in self.eg.dimnames:
            unit = self.eg.dimunits[0]
        else:
            vidx = self.eg.vidx(vary)
            unit = self.eg.valunits[vidx]

        title = '%s_%s(%s)' % (self.diagname, vary, unit)

        yedata = []
        if self.with_errbar and vary in self.eg.dimnames:
            pass
        else:
            if self.with_errbar and vidx+1 < self.eg.valno:
                #yedata = self.eg.trace_of_idx(dimidx, vidx+1)
                yedata = self.eg.trace_of_idx(0, vidx+1)
                yedata = kaisekiInterFace.AverageErrListWithN(yedata, self.navg)

        zdata = []
        zedata = []
        return [title, xdata, xedata, ydata, yedata, zdata, zedata]


#def ReadFromZipfile(filepath):
#    zf = zipfile.ZipFile(filepath, 'r')
#    target = zf.namelist()[0]
#    cbuff = zf.read(target)
#    zf.close()
#    return cbuff

#def OpenFile(filepath):
#    if filepath[-3:].upper() == 'ZIP':
#        zf = zipfile.ZipFile(filepath, 'r')
#        target = zf.namelist()[0]
#        return zf, io.TextIOWrapper(io.BufferedReader(zf.open(target, 'r')),encoding='ascii')
#    else:
#        return None, open(filepath, "r")

#def CloseFile(zf, fp):
#    if zf:
#        fp.close()
#        zf.close()
#    else:
#        fp.close()

class EG_1D(object):
    def __init__(self, filepath):
        """
        self.data[idx]
        iline => self.line_index() 
        size = self.dimno + self.valno
        idx = self.line_size * iline + vidx
        """
        self.filepath = filepath
        self.keyvalue = {}
        self.headlines = []
        self.load(filepath)

    def load(self, filepath):
        zf, fp = kaisekiInterFace.OpenFile(filepath)
        l = fp.readline()
        hlines = []
        while l:
            l = l.strip()
            hlines.append(l)
            if len(l) == 0:
                l = fp.readline()
                continue
            if l[0] == '#':
                clm = l[1:].split('=')
                if len(clm) == 2:
                    self.keyvalue[clm[0].strip()] = clm[1].strip()
                l = fp.readline()
                continue
            break
        self.headlines = hlines[:-1]

        cdimnames = self.keyvalue.get('DimName')
        cdimno = self.keyvalue.get('DimNo')
        cdimunits = self.keyvalue.get('DimUnit')
        cdimsize = self.keyvalue.get('DimSize')
        cvalno = self.keyvalue.get('ValNo')
        cvalnames = self.keyvalue.get('ValName')
        cvalunits = self.keyvalue.get('ValUnit')
        
        dimno = int(cdimno)
        valno = int(cvalno)
        line_size = dimno + valno

        dimsizes = [int(c.strip()) for c in cdimsize.split(',')]
        line_size = dimno + valno
        data_size = line_size * (reduce((lambda x, y: x * y), dimsizes, 1) * 2)
        
        # allocate memory
        self.data = numpy.zeros(data_size)

        # make Headder info
        self.dimnames = [c.strip("' ") for c in cdimnames.split(',')] 
        self.dimunits = [c.strip("' ") for c in cdimunits.split(',')] 
        self.valnames = [c.strip("' ") for c in cvalnames.split(',')] 
        self.valunits = [c.strip("' ") for c in cvalunits.split(',')] 
        self.dimno = dimno
        self.valno = valno
        self.line_size = line_size
        self.dimsizes = dimsizes
        self.t_size, = dimsizes

        idx = 0
        while l:
            l = l.strip()
            if len(l) == 0:
                l = fp.readline()
                continue
            #for c in l.split(','):
            for c in re.split(r',\s*|\s+', l):
                try:
                    self.data[idx] = float(c)
                except ValueError:
                    self.data[idx] = numpy.nan
                idx = idx+1
            l = fp.readline()

        kaisekiInterFace.CloseFile(zf, fp)
        self.timelist = self.times()

    def get_line(self, idx_time):
        idx = self.line_size * idx_time
        return self.data[idx: idx+self.line_size]

    
    def dims(self, dimidx):
        return [self.get_line(i)[dimidx] for i in range(self.dimsizes[dimidx])]

    def times(self):
        return [self.get_line(i)[0] for i in range(self.t_size)]

    def vdata(self, idx_time, idx_v):
        return self.get_line(idx_time)[self.dimno+idx_v]

    def vidx(self, name):
        if name in self.valnames:
            return self.valnames.index(name)
        else:
            return -1

    def trace_of_idx(self, dimidx, vidx):
        dimno = self.dimno
        dimsize = self.dimsizes[dimidx]
        pos = [ ([0] * dimidx) + [i] + ([0] * (dimno - dimidx -1)) 
                for i in range(dimsize)]
        return [self.get_line(*p)[self.dimno + vidx] for p in pos]

    def trace_of(self, dimidx, name):
        vidx = self.vidx(name)
        if vidx == -1:
            return []
        dimno = self.dimno
        dimsize = self.dimsizes[dimidx]
        pos = [ ([0] * dimidx) + [i] + ([0] * (dimno - dimidx -1)) 
                for i in range(dimsize)]
        return [self.get_line(*p)[self.dimno + vidx] for p in pos]


if __name__ == '__main__':
    from MyViewCache import *
    #option = 'navg=1'
    option = 'werr'
    cache = MyViewCache('cache/cache.txt')
    #ifc = InterFace('samples\\nb5a_fraction@128056.dat', temppath='temp', 
    #ifc = InterFace('samples\\PCRMPTJULI.txt', temppath='temp', 
    #ifc = InterFace('>ip@125561', temppath='temp', 
    ifc = InterFace('>ti_crystal@178473', temppath='temp', 
            cachebutton='ONLINE', cache=cache,
            cmdpath = 'bin', option=option) 
    ifc.load(cachebutton='ONLINE')
    #ifc.parse_option('werr')

    print(('#',ifc.varsx()))
    print(('#',ifc.varsy(varx='time')))
    #print('#',ifc.indexs(varx='time', vary='avg_fraction_1st_nb5a'))
    
    title, x, xe, y, ye, z, ze =ifc.dataset(varx='Time_corr', vary='Ti0')
    print( len(x), len(y), len(ye))
    for i, xv in enumerate(x):
        print("%g\t%g\t%g" % (xv, y[i], ye[i]))
        #print(("%g\t%g" % (xv, y[i])))
        pass

        


