#!/usr/bin/env python
# coding: utf-8
# $Id: dm_fir_reff_fast.py 374 2016-06-14 05:38:21Z emo $
import io
import os
import math
import numpy
import re
import time
import egdbcommon
import zipfile
import io
import subprocess
import kaisekiInterFace
from MyViewGlobal import logger
from functools import reduce
#import dataSource

#def LimitValue(v, vmin, vmax):
#    if vmin > v:
#        return vmin
#    if vmax <= v:
#        return vmax
#    return v

#def AverageGroupOfIndices(sz, avgn):
#    lst = [i for i in range(sz)]
#    paires = [lst[i: i+avgn] for i in range(0, sz, avgn)] 
#    return paires

#def AveragePartOfList(lst, indices):
#    return sum([lst[i] for i in indices]) / len(indices)

#def AverageLists(lsts):
#    """
#    [[v1, v2, v3], [v4, v5, v6]]
#    =>
#    [avg([v1, v4]), avg([v2, v5]), avg([v3, v6])]
#    """
#    n = len(lsts)
#    return [sum(values)/float(n) for values in zip(*lsts)]

def Flatten(lstsOflst):
    """
    [[a, b], [c, d], [e, f]] => [a, b, c, d, e, f]
    """
    return reduce((lambda x, y: x + y), lstsOflst, [])

def StandardDeviation(lstsOflst):  
    values = Flatten(lstsOflst)
    n = len(values)
    avg = sum(values)/n
    return math.sqrt(sum([(v-avg)**2 for v in values])/n)

def StandardDeviationError(lstsOflst):  
    values = Flatten(lstsOflst)
    n = len(values)
    avg = sum(values)/n
    return math.sqrt(sum([(v-avg)**2 for v in values])/n/n)

def StandardDeviationLists(lsts):
    """
    [[l1, l2, l3], [l4, l5, l6]]
    =>
    [sd([l1, l4]), sd([l2, l5]), sd([l3, l6])]
    """
    #n = len(lsts)
    return [StandardDeviation(lstsOflst) for lstsOflst in zip(*lsts)]

def StandardDeviationErrorLists(lsts):
    """
    [[l1, l2, l3], [l4, l5, l6]]
    =>
    [sd([l1, l4]), sd([l2, l5]), sd([l3, l6])]
    """
    #n = len(lsts)
    return [StandardDeviationError(lstsOflst) for lstsOflst in zip(*lsts)]

def PartOfList(lsts, idxs):
    return [lsts[i] for i in idxs]

def AverageErrorLists(lsts, indices):
    """
     indices = [[0, 1, 2], [3, 4, 5], ...]
     lsts = [[v1, v2, v3, v4, ...],
              [v1, v2, v3, v4, ...],
              ...]
     values = [[v1, v1, ..], [v2, v2, ..], ...]
     groups = [[[v1, v1, ...], [v2, v2, ...], ..], [[], [], ..], ...]
    """

    values = [list(item) for item in zip(*lsts)]
    groups = [PartOfList(values, idxs) for idxs in indices] 
    packs = [JoinList(grp) for grp in groups]
    sz = float(len(packs[0]))
    return [math.sqrt(sum([x*x for x in pack]))/sz/math.sqrt(sz) for pack in packs]

def JoinList(lst):
    return reduce(lambda x, y: x+y, lst, [])


#def StringToInteger(vstr):
#    try:
#        return int(vstr)
#    except ValueError:
#        return 0

#def ParseDataID(dataid):
#    if -1 == dataid.find(">"):
#        # not found
#        return None, None, None
#
#    clms = dataid.split(">")
#    name_shot = clms[1]
#    name, shot = name_shot.split("@")
#    return clms[0], name, StringToInteger(shot)

def helptext():
    text = """
    <p>Here is an expanation of this module <p><br>
    Module name : plot_eg_2D <br>
    Synopsis    : Plot 2 DimNo EG formatted data on local file storage. <br>
    Script name : dm_plot_eg_1D.py <br>
    Data source : local file storage <br>
    Author name : M. Yoshinuma <br>
    Contact to  : yoshinuma@nifs.ac.jp <br>
    Plot type   : xy, xy2, xyz
    <br>
    <p>Options<p><br>
    addaxv='valname_i', 'valname_j';<br>
    add valnames to the x or y axis list.<br>
    ex) addaxv=reff; : you can add value 'reff' to the x or y axis list.<br>

    use index option to set the dimensioin index for additional valunames.<br>
    index=1 or 2 : select dimension for index (default 1)<br>
    <br> 
    use n1avg or n2avg option to average the data on the original dimension.
    n1avg=#; : average # points on dimension 1 (1..20)<br>
    n2avg=#; : average # points on dimension 2 (1..20)<br>
    <br>
    use werr option to draw err bar. <br>
    The data of err is the value at the next column of vary. <br>
    werr;<br>
    <br>
    use wsd option to draw standard deviation of an averaged data as err bar. <br>
    No error bar if the n1avg=1 and n2avg=1. <br>
    wsde;<br>
    <br>
    use wsde option to draw standard deviation of an averaged data divided with sqrt(n) as err bar. <br>
    No error bar if the n1avg=1 and n2avg=1. <br>
    wsde;<br>
    <br >
    <p>Load data from EG data server<br>
    1)Select Add for shot number. <br>
    2)Set string like '>diagname%shot%' in the TextControl of the Add. 
    <br>
    """
    return text
    
class InterFace(object):
    def __init__(self, dataid, **kwargs):
        """
        kwargs:
            option: option string for trace
            cache: myview cache object
            cachebutton: state of button cache on myView.
                         "CACHE": 
                         "ONLINE":
                         "OFFLINE":
            temppath: temp path set with myView dialog.
            cmdpath: command path set with myView dialog
        if there is some troubole to make instance, raise "Error"
        """
        self.dataid = dataid
        self.option = kwargs.get('option')
        self.cache = kwargs.get('cache')
        self.cachebutton = kwargs.get('cachebutton')
        self.temppath = kwargs.get('temppath')
        self.cmdpath = kwargs.get('cmdpath')
        self.flgloaded = False
        self.n1avg = 1
        self.n2avg = 1
        self.with_errbar = False
        self.with_sd = False
        self.with_sde = False
        self.parse_option(self.option)

        base, name = os.path.split(self.dataid)
        self.diagname = '%s' % name
        self.cachepath = None

    def parse_option(self, option):
        with_errbar = False
        with_sd = False
        with_sde = False
        axvs = []
        idxs = []
        n1avg = 1
        n2avg = 1
        for p in [o.strip() for o in option.split(';')]:
            keyvalue = [item.strip() for item in p.split('=')]
            if len(keyvalue) == 1:
                key, = keyvalue
                if key.lower() == 'werr':
                    with_errbar = True
                if key.lower() == 'wsd':
                    with_sd = True
                if key.lower() == 'wsde':
                    with_sde = True
            if len(keyvalue) == 2:
                key, value = keyvalue
                if key == 'addaxv':
                    axvs = [v.strip() for v in value.split(',')] 
                if key == 'index':
                    for v in value.split(','):
                        try:
                            idx = int(v.strip()) -1
                        except ValueError:
                            idx = 0
                        if 0 > idx > 1:
                            idx = 0
                        idxs.append(idx)
                if key == 'n1avg':
                    try:
                        n1avg = int(value)
                    except ValueError:
                        n1avg = 1
                    n1avg = kaisekiInterFace.LimitValue(n1avg, 1, 100)

                if key == 'n2avg':
                    try:
                        n2avg = int(value)
                    except ValueError:
                        n2avg = 1
                    n2avg = kaisekiInterFace.LimitValue(n2avg, 1, 100)

        idxs = idxs + [0] * (len(axvs) - len(idxs))
        self.additional_axis_values = axvs
        self.indices_for_additional_axis_values = idxs
        self.with_errbar = with_errbar
        self.with_sd = with_sd
        self.with_sde = with_sde
        self.n1avg = n1avg
        self.n2avg = n2avg
        print(("#Plot2D : setting n1avg=%d n2avg=%d" % (self.n1avg, self.n2avg)))

    def load_target(self, diagname, shot, sub):
        # try to get datafile from cache
        str_shot = "%d_%d" % (shot, sub)
        key = '%s_%s' % (diagname, str_shot)
        cachepath = None
        filetime = self.cache.findTime(key) 
        if (self.cachebutton == 'ONLINE' and 
                egdbcommon.compareTime(diagname, shot, sub, filetime)):
            cachepath=None
            logger.debug("*** DISCARD CACHE FILE FOR %s@%s ***" % (diagname, str_shot))
        else:
            cachepath = self.cache.find(key) 
            if cachepath:
                logger.debug("*** USE CACHE FILE FOR %s@%s ***" % (diagname, str_shot))

        if not cachepath and self.cachebutton == 'ONLINE':
            # try to get datafile from server
            logger.debug("load from network")
            fname = "%s.zip" % key
            fname = self.cache.makeUniqueFilename(fname, self.temppath)
            cachepath = os.path.join(self.temppath, fname) 
            cmdstr = "igetfile -s %d -m %d -d %s -Z -o %s" % (
                        shot, sub, diagname, cachepath) 
            #cmdstr = os.path.join(self.cmdpath, cmdstr)
            #print cmdstr
            if egdbcommon.icheckfile(diagname, shot, sub):
                subprocess.check_call(cmdstr,shell=True)
                if os.path.exists(cachepath):
                    logger.debug("Register the Datafile %s" % cachepath)
                    cachepath = self.cache.registFile(key, cachepath)
                else:
                    cachepath = None
            else:
                cachepath = None
        return cachepath

    def load(self, **kwargs):

        """
        try:
            self.eg = EG_2D(self.dataid)
        except:
            self.eg = None
        """
        directive, diagname, shot, sub = kaisekiInterFace.ParseDataID(self.dataid)
        print(directive, diagname, shot, sub)
        

        if diagname is None:
            self.eg = EG_2D(self.dataid)
        else:
            cachepath = self.load_target(diagname, shot, sub)
            if cachepath:
                self.eg = EG_2D(cachepath)
                self.cache.releasefile(cachepath)

        if self.eg.dimno != 2:
            self.eg = None
            self.flgloaded = False
        else:
            self.flgloaded = True
            self.init_dimensions()
        print(("**** LOAD FINISHED [%s] ***" % self.dataid))
            
    def init_dimensions(self):
        dim1data = self.eg.dims(1)
        dim0data = self.eg.dims(0)
        dim0_indices = kaisekiInterFace.AverageGroupOfIndices(len(dim0data), self.n1avg)
        dim1_indices = kaisekiInterFace.AverageGroupOfIndices(len(dim1data), self.n2avg)
        dim0 = [kaisekiInterFace.AveragePartOfList(dim0data, idxs) for idxs in dim0_indices]
        dim1 = [kaisekiInterFace.AveragePartOfList(dim1data, idxs) for idxs in dim1_indices]
        self.dim_indices = [dim0_indices, dim1_indices]
        self.dim_list = [dim0, dim1]

    def datainfo(self, **kwargs):
        ptype = kwargs.get('type', None)
        if not self.flgloaded:
            self.load(**kwargs)

        if not self.eg:
            return "Data Loading Failed."
        return "\n".join(self.eg.headlines)

    def varsx(self, **kwargs):
        ptype = kwargs.get('type', None)
        if not self.flgloaded:
            self.load(**kwargs)

        if not self.eg:
            return []
        return self.eg.dimnames + self.additional_axis_values

    def varsy(self, **kwargs):
        ptype = kwargs.get('type', None)
        varx = kwargs.get('varx', None)
        
        if not self.flgloaded:
            self.load(**kwargs)
        if not self.eg:
            return []
       
        if varx in self.additional_axis_values:
            varx_idx = -1
        else:
            varx_idx = self.eg.dimnames.index(varx) 

        if ptype == 'xyz':
            if varx_idx == 0:
                return [self.eg.dimnames[1]] + self.additional_axis_values
            elif varx_idx == 1:
                return [self.eg.dimnames[0]] + self.additional_axis_values
            else:
                return self.eg.dimnames # varx is in the additional axis values

        return self.eg.valnames

    def varsz(self, **kwargs):
        ptype = kwargs.get('type', None)
        if ptype == 'xyz':
            return self.eg.valnames
        return []

    def indexs(self, **kwargs):
        if not self.flgloaded:
            self.load(**kwargs)
        ptype = kwargs.get('type', None)
        varx = kwargs.get('varx', None)
        if ptype == 'xyz':
            return []
       
        if varx in self.additional_axis_values:
            idx = self.additional_axis_values.index(varx)
            idxidx = self.indices_for_additional_axis_values[idx]
            #dimdata = self.eg.dims(idxidx)
            dimdata = self.dim_list[idxidx]
            dimunit = self.eg.dimunits[idxidx]
        else:
            varx_idx = self.eg.dimnames.index(varx) 
            if varx_idx == 0:
                #dimdata = self.eg.dims(1)
                dimunit = self.eg.dimunits[1]
                dimdata = self.dim_list[1]
            else:
                #dimdata = self.eg.dims(0)
                dimunit = self.eg.dimunits[0]
                dimdata = self.dim_list[0]

        return ["%d_%g%s" % (i, v, dimunit) for i, v in enumerate(dimdata)]
    
    def refresh_index(self, kwargs):
        try:
            index = kwargs.get('index', None)
            trace = kwargs.get('trace', None)
            indexs = self.indexs(**kwargs)
            if index and trace:
                index = egdbcommon.get_index(indexs, index)
                kwargs['index'] = index
                trace = kwargs['trace']
                trace.index = index
        except:
            pass

    def dataset(self, **kwargs):
        """
        return list of dataset for varx, vary, varz, index, option.
        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog
            index: 

        dataset := [title, x, xe, y, ye, z, ze]
        x = [x1, x2, x3, ... , xn]
        y = [y1, y2, x3, ... , ym] 
        z = [[z(x1, y1), z(x2, y1), z(x3, y1),...,z(xn, y1)],
             [z(x1, y2), z(x2, y2), z(x3, y2),...,z(xn, y2)],
             [z(x1, y3), z(x2, y3), z(x3, y3),...,z(xn, y3)], 
             ......
             [z(x1, ym), z(x2, ym), z(x3, ym),...,z(xn, ym)]] 
        len(z) = len(y)
        len(z[i]) = len(x)
        """
        ptype = kwargs.get('type', None)
        varx = kwargs.get('varx', None)
        vary = kwargs.get('vary', None)

        if not self.flgloaded:
            self.load(**kwargs)

        dataset = ['%s No file' % self.dataid, [],[],[],[],[],[]]
        if not self.eg:
            return dataset

        dataset = ['%s No data' % self.dataid, [],[],[],[],[],[]]
        
        if varx in self.additional_axis_values:
            dimidx = 0
        else:
            dimidx = self.eg.dimnames.index(varx)
        dimdata = self.eg.dims(dimidx)
        if 0 == len(dimdata):
            return dataset

        self.refresh_index(kwargs)
        
        if ptype == 'xyz':
            dataset = self.mapxyz(**kwargs)
            return dataset

        dataset = self.trace(**kwargs)
        return dataset

    def mapxyz_with_additional_x_axis(self, **kwargs):
        varx = kwargs.get('varx', None)
        vary = kwargs.get('vary', None)
        varz = kwargs.get('varz', None)
        index = kwargs.get('index', None)
        xdata = []
        xedata = []
        ydata = []
        yedata = []
        zdata = []
        zedata = []

        ydimidx = self.eg.dimnames.index(vary)

        if ydimidx == 0:
            #xdimdata = self.eg.dims(1)
            #ydimdata = self.eg.dims(0)
            xdimdata = self.dim_list[1]
            ydimdata = self.dim_list[0]
            xindices = self.dim_indices[1]
            yindices = self.dim_indices[0]
        else:
            xdimdata = self.dim_list[0]
            ydimdata = self.dim_list[1]
            xindices = self.dim_indices[0]
            yindices = self.dim_indices[1]

        xidx = self.eg.vidx(varx)
        vidx = self.eg.vidx(varz)
        unit = self.eg.valunits[vidx]
        title = '%s_%s(%s)' % (self.diagname, varz, unit)


        for i, ii in enumerate(xindices):
            dupe_check_list = []
            xo = xdimdata[i]
            for j, jj in enumerate(yindices):
                y = ydimdata[j]

                xpack = []
                vpack = []

                for xi in ii:
                    for yj in jj:
                        if ydimidx == 0:
                            xpack.append(self.eg.vdata(xidx, yj, xi))
                            vpack.append(self.eg.vdata(vidx, yj, xi))
                        else:
                            xpack.append(self.eg.vdata(xidx, xi, yj))
                            vpack.append(self.eg.vdata(vidx, xi, yj))
                x = sum(xpack)/float(len(xpack))
                v = sum(vpack)/float(len(vpack))

                if (x, y) in dupe_check_list:
                    continue

                dupe_check_list.append((x, y))
                xdata.append(x)
                ydata.append(y)
                zdata.append(v)

        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

    def mapxyz_with_additional_y_axis(self, **kwargs):
        varx = kwargs.get('varx', None)
        vary = kwargs.get('vary', None)
        varz = kwargs.get('varz', None)
        index = kwargs.get('index', None)
        xdata = []
        xedata = []
        ydata = []
        yedata = []
        zdata = []
        zedata = []

        xdimidx = self.eg.dimnames.index(varx)

        if xdimidx == 0:
            xdimdata = self.dim_list[0]
            ydimdata = self.dim_list[1]
            xindices = self.dim_indices[0]
            yindices = self.dim_indices[1]
        else:
            xdimdata = self.dim_list[1]
            ydimdata = self.dim_list[0]
            xindices = self.dim_indices[1]
            yindices = self.dim_indices[0]

        yidx = self.eg.vidx(vary) #
        vidx = self.eg.vidx(varz)
        unit = self.eg.valunits[vidx]
        title = '%s_%s(%s)' % (self.diagname, varz, unit)

        for i, ii in enumerate(xindices):
            dupe_check_list = []
            x = xdimdata[i]
            for j, jj in enumerate(yindices):
                yo = ydimdata[j]

                ypack = []
                vpack = []

                for xi in ii:
                    for yj in jj:
                        if xdimidx == 0:
                            ypack.append(self.eg.vdata(yidx, xi, yj))
                            vpack.append(self.eg.vdata(vidx, xi, yj))
                        else:
                            ypack.append(self.eg.vdata(yidx, yj, xi))
                            vpack.append(self.eg.vdata(vidx, yj, xi))
                y = sum(ypack)/float(len(ypack))
                v = sum(vpack)/float(len(vpack))
                
                if (x, y) in dupe_check_list:
                    continue
                dupe_check_list.append((x, y))
                xdata.append(x)
                ydata.append(y)
                zdata.append(v)

        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

    def mapxyz(self, **kwargs):
        varx = kwargs.get('varx', None)
        vary = kwargs.get('vary', None)
        varz = kwargs.get('varz', None)
        index = kwargs.get('index', None)
        xdata = []
        xedata = []
        ydata = []
        yedata = []
        zdata = []
        zedata = []

        if varx in self.additional_axis_values:
            return self.mapxyz_with_additional_x_axis(**kwargs)

        if vary in self.additional_axis_values:
            return self.mapxyz_with_additional_y_axis(**kwargs)

        dimidx = self.eg.dimnames.index(varx)
        #xdimdata = self.eg.dims(dimidx)
        xdimdata = self.dim_list[dimidx]
        xdimindices = self.dim_indices[dimidx]

        if dimidx == 0:
            #ydimdata = self.eg.dims(1)
            ydimdata = self.dim_list[1]
            ydimindices = self.dim_indices[1]
        else:
            #ydimdata = self.eg.dims(0)
            ydimdata = self.dim_list[0]
            ydimindices = self.dim_indices[0]

        vidx = self.eg.vidx(varz)
        unit = self.eg.valunits[vidx]
        title = '%s_%s(%s)' % (self.diagname, varz, unit)

        for i, x in enumerate(xdimdata):
            xindices = xdimindices[i]
            for j, y in enumerate(ydimdata):
                xdata.append(x)
                ydata.append(y)
                yindices = ydimindices[j]
                vpack = []
                for ii in xindices:
                    for jj in yindices:
                        if dimidx == 0:
                            vpack.append(self.eg.vdata(vidx, ii, jj))
                        else:
                            vpack.append(self.eg.vdata(vidx, jj, ii))
                v = sum(vpack)/len(vpack)
                zdata.append(v)

        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

    def trace(self, **kwargs):
        varx = kwargs.get('varx', None)
        vary = kwargs.get('vary', None)
        index = kwargs.get('index', None)
        xdata = []
        xedata = []
        ydata = []
        yedata = []

        #print index
        idx = int(index.split('_')[0])
        
        if varx in self.additional_axis_values:
            avidx = self.additional_axis_values.index(varx)
            idxdim = self.indices_for_additional_axis_values[avidx]
            varx_dimidx = 1 if idxdim == 0 else 0 
        else:
            varx_dimidx = self.eg.dimnames.index(varx)
            idxdim = 1 if varx_dimidx == 0 else 0

        idxindices = self.dim_indices[idxdim]
        varxindices = self.dim_indices[varx_dimidx]
       
        #print 'idxdim=', idxdim
        #print 'idx=', idx
        #dimdata_of_index = self.eg.dims(idxdim)
        dimdata_of_index = self.dim_list[idxdim]
        #print dimdata_of_index
        
        idx_dimunit = self.eg.dimunits[idxdim]
        at = dimdata_of_index[idx]

        vidx = self.eg.vidx(vary)
        unit = self.eg.valunits[vidx]
        title = '%s_%s(%s)@%g%s' % (self.diagname, vary, unit, at, idx_dimunit)

       
        if varx in self.additional_axis_values:
            xd_pack = []
            for i in idxindices[idx]:
                xd_org = self.eg.trace_of(varx_dimidx, i, varx)
                xd = [kaisekiInterFace.AveragePartOfList(xd_org, ii) for ii in varxindices]
                xd_pack.append(xd)
            xdata = kaisekiInterFace.AverageLists(xd_pack)
        else:
            xdata = self.dim_list[varx_dimidx]

        yd_pack = []
        ye_sd_pack = []
        for i in idxindices[idx]:
            yd_org = self.eg.trace_of(varx_dimidx, i, vary)
            yd = [kaisekiInterFace.AveragePartOfList(yd_org, ii) for ii in varxindices]
            ye_sd = [[yd_org[j] for j in ii] for ii in varxindices]
            yd_pack.append(yd)
            ye_sd_pack.append(ye_sd)
        ydata = kaisekiInterFace.AverageLists(yd_pack)

        if self.with_errbar and vidx+1 < self.eg.valno:
            ye_pack = []
            for i in idxindices[idx]:
                ye_org = self.eg.trace_of_idx(varx_dimidx, i, vidx+1)
                ye_pack.append(ye_org)
            yedata = AverageErrorLists(ye_pack, varxindices) 
        elif self.with_sd and ((self.n1avg > 1  or self.n2avg > 1)):
            yedata = StandardDeviationLists(ye_sd_pack)
        elif self.with_sde and ((self.n1avg > 1  or self.n2avg > 1)):
            yedata = StandardDeviationErrorLists(ye_sd_pack)
        else:
            yedata = []

        zdata = []
        zedata = []
        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

#def ReadFromZipfile(filepath):
#    zf = zipfile.ZipFile(filepath, 'r')
#    target = zf.namelist()[0]
#    cbuff = zf.read(target)
#    zf.close()
#    return cbuff

#def OpenFile(filepath):
#    if filepath[-3:].upper() == 'ZIP':
#        zf = zipfile.ZipFile(filepath, 'r')
#        target = zf.namelist()[0]
#        return zf, io.TextIOWrapper(io.BufferedReader(zf.open(target, 'r')),encoding='ascii')
#    else:
#        return None, open(filepath, "r")

#def CloseFile(zf, fp):
#    if zf:
#        fp.close()
#        zf.close()
#    else:
#        fp.close()

#def CalcWeights(lst):
#    if len(lst) == 0:
#        return [1]
#
#    subweights = CalcWeights(lst[1:])
#    return [lst[0] * subweights[0]] + subweights

class EG_2D(object):
    def __init__(self, filepath):
        """
        self.data[idx]
        iline => self.line_index() 
        size = self.dimno + self.valno
        idx = self.line_size * iline + vidx
        """
        self.filepath = filepath
        self.keyvalue = {}
        self.headlines = []
        self.load(filepath)

    def load(self, filepath):
        zf, fp = kaisekiInterFace.OpenFile(filepath)
        l = fp.readline()
        hlines = []
        while l:
            l = l.strip()
            hlines.append(l)
            if len(l) == 0:
                l = fp.readline()
                continue
            if l[0] == '#':
                clm = l[1:].split('=')
                if len(clm) == 2:
                    self.keyvalue[clm[0].strip()] = clm[1].strip()
                l = fp.readline()
                continue
            break
        self.headlines = hlines[:-1]

        cdimnames = self.keyvalue.get('DimName')
        cdimunits = self.keyvalue.get('DimUnit')
        cdimno = self.keyvalue.get('DimNo')
        cdimsize = self.keyvalue.get('DimSize')
        cvalno = self.keyvalue.get('ValNo')
        cvalnames = self.keyvalue.get('ValName')
        cvalnames = self.keyvalue.get('Valname', cvalnames)
        cvalunits = self.keyvalue.get('ValUnit')
        
        dimno = int(cdimno)
        valno = int(cvalno)
        line_size = dimno + valno
            
        dimsizes = [int(c.strip()) for c in cdimsize.split(',')]
        line_size = dimno + valno
        data_size = line_size * reduce((lambda x, y: x * y), dimsizes, 1)
        
        # allocate memory
        self.data = numpy.zeros(data_size)

        # make Headder info
        self.dimnames = [c.strip("' ") for c in cdimnames.split(',')] 
        self.dimunits = [c.strip("' ") for c in cdimunits.split(',')] 
        self.valnames = [c.strip("' ") for c in cvalnames.split(',')] 
        self.valunits = [c.strip("' ") for c in cvalunits.split(',')] 
        self.dimno = dimno
        self.valno = valno
        self.line_size = line_size
        self.dimsizes = dimsizes
        self.weights = kaisekiInterFace.CalcWeights(self.dimsizes[1:])

        idx = 0
        while l:
            l = l.strip()
            if len(l) == 0:
                l = fp.readline()
                continue
            #for c in l.split(','):
            for c in re.split(r',\s*|\s+', l):
                if c != "":
                    try:
                        self.data[idx] = float(c)
                    except ValueError:
                        self.data[idx] = numpy.nan
                    idx = idx+1
            l = fp.readline()

        kaisekiInterFace.CloseFile(zf, fp)

    def line_index(self, *dimidxs):
        weighted_indexs = [i * w for i, w in zip(dimidxs, self.weights)]
        return reduce((lambda x, y: x+y), weighted_indexs, 0)

    def get_line(self, *dimidxs):
        idx = self.line_size * self.line_index(*dimidxs)
        return self.data[idx: idx+self.line_size]
    
    def dims(self, dimidx):
        if dimidx == 0:
            pos = [[i, 0] for i in range(self.dimsizes[dimidx])]
        else:
            pos = [[0, i] for i in range(self.dimsizes[dimidx])]
        return [self.get_line(*p)[dimidx] for p in pos]

    def vdata(self, idxv, *dimidxs):
        return self.get_line(*dimidxs)[self.dimno+idxv]

    def vidx(self, name):
        if name in self.valnames:
            return self.valnames.index(name)
        else:
            return -1

    def trace_of_idx(self, dimidx, idx, vidx):
        """
        dimidx: idx of variable dims for trace
        ex)
          0: time trace
          1: radial profile
        """
        dimno = self.dimno
        dimsize = self.dimsizes[dimidx]
        if dimidx == 0:
            pos = [[i, idx] for i in range(dimsize)]
        else:
            pos = [[idx, i] for i in range(dimsize)]
        return [self.get_line(*p)[self.dimno + vidx] for p in pos]

    def trace_of(self, dimidx, idx, name):
        """
        dimidx: idx of variable dims for trace
        ex)
          0: time trace
          1: radial profile
        """
        vidx = self.vidx(name)
        if vidx == -1:
            return []
        dimno = self.dimno
        dimsize = self.dimsizes[dimidx]
        if dimidx == 0:
            pos = [[i, idx] for i in range(dimsize)]
        else:
            pos = [[idx, i] for i in range(dimsize)]
        return [self.get_line(*p)[self.dimno + vidx] for p in pos]


if __name__ == '__main__':
    from MyViewCache import *
    #option = 'addaxv=reff;wsde;n1avg=1;n2avg=1'
    #option = 'n1avg=2;n2avg=1'
    option = 'n1avg=2'
    #option = 'addaxv=reff;werr;n1avg=1;n2avg=1'
    cache = MyViewCache('cache/cache.txt')
    #ifc = InterFace('samples\\CES2@16514.txt', temppath='temp', 
    #ifc = InterFace('samples\\thomson_109693.zip', temppath='temp', 
    #ifc = InterFace('samples\\tsmap_113287.zip', temppath='temp', 
    ifc = InterFace('samples\\CES2@19389.txt', temppath='temp', 
            cachebutton='ONLINE', cache=cache,
            cmdpath = 'bin', option=option) 
    ifc.load(cachebutton='ONLINE')

    #print '#',ifc.varsx()
    #print '#',ifc.varsy(varx='Time', type='xy')
    #print '#',ifc.varsy(varx='Time', type='xyz')
    #print '#',ifc.varsy(varx='R', type='xyz')
    #print '#',ifc.varsy(varx='reff', type='xyz')
    #print '#',ifc.varsz(varx='Time', vary='reff', type='xyz')

    """
    print '#',ifc.varsy(varx='Time', vary='R', type='xyz')
    print '#',ifc.varsy(varx='R', vary='Time', type='xyz')
    print '#',ifc.varsz(varx='Time')
    print '#',ifc.varsz(varx='R')
    print '#',ifc.varsz(varx='Time', vary='R', type='xyz')
    print '#',ifc.varsz(varx='R', vary='Time', type='xyz')
    """
    #print '#',ifc.indexs(varx='Time', vary='Ti')
    #print '#',ifc.indexs(varx='R', vary='Ti')
    #print '#',ifc.indexs(varx='reff', vary='Te', type='xy')
   
    #title, x, xe, y, ye, z, ze =ifc.dataset(varx='reff', vary='Te', index='54_5.44s')
    #title, x, xe, y, ye, z, ze =ifc.dataset(varx='R', vary='Te', index='54_5.44s')
    #title, x, xe, y, ye, z, ze =ifc.dataset(varx='R', vary='Te', index='27_5.47s')
    #title, x, xe, y, ye, z, ze =ifc.dataset(varx='reff', vary='Te', index='27_5.47s')
    #print len(x), len(y)
    #title, x, xe, y, ye, z, ze =ifc.dataset(varx='R', vary='Te', index='15_4006ms')
    """
    title, x, xe, y, ye, z, ze =ifc.dataset(varx='Time', vary='Te', index='15_4006ms')
    print '#', title
    for i, xv in enumerate(x):
        print("%g\t%g\t%g" % (xv, y[i], ye[i]))
    
    """
    
    title, x, xe, y, ye, z, ze =ifc.dataset(varx='Time', vary='R', varz='Ti', type='xyz')
    print(('#', title))
    """
    for i, xv in enumerate(x):
        print("%g\t%g\t%g" % (xv, y[i], z[i]))
        if (i+1) % 32 == 0:
            print
    """


        


