#!/usr/bin/env python
# coding: utf-8
# $Id: dm_fir_reff_fast.py 374 2016-06-14 05:38:21Z emo $
from __future__ import print_function
import io
import os
import sys
sys.path.append(os.getcwd())
import math
import numpy
import re
import time
import egdbcommon
import zipfile
import io
import subprocess
import kaisekiInterFace
from MyViewGlobal import logger
from functools import reduce
#import dataSource

#def FindIndexNear(v, lst):
#    dv_i = [(abs(v-vi), i) for i, vi in enumerate(lst)]
#    dv_i.sort()
#    return dv_i[0][1]

#def LimitValue(v, vmin, vmax):
#    if vmin > v:
#        return vmin
#    if vmax <= v:
#        return vmax
#    return v

#def AverageGroupOfIndices(sz, avgn):
#    lst = [i for i in range(sz)]
#    paires = [lst[i: i+avgn] for i in range(0, sz, avgn)] 
#    return paires

#def AveragePartOfList(lst, indices):
#    return sum([lst[i] for i in indices]) / len(indices)

#def AverageLists(lsts):
#    """
#    [[v1, v2, v3], [v4, v5, v6]]
#    =>
#    [avg([v1, v4]), avg([v2, v5]), avg([v3, v6])]
#    """
#    n = len(lsts)
#    return [sum(values)/float(n) for values in zip(*lsts)]

def Flatten(lstsOflst):
    """
    [[a, b], [c, d], [e, f]] => [a, b, c, d, e, f]
    """
    return reduce((lambda x, y: x + y), lstsOflst, [])

def StandardDeviation(lstsOflst):  
    values = Flatten(lstsOflst)
    n = len(values)
    avg = sum(values)/n
    return math.sqrt(sum([(v-avg)**2 for v in values])/n)

def StandardDeviationError(lstsOflst):  
    values = Flatten(lstsOflst)
    n = len(values)
    avg = sum(values)/n
    return math.sqrt(sum([(v-avg)**2 for v in values])/n/n)

def StandardDeviationLists(lsts):
    """
    [[l1, l2, l3], [l4, l5, l6]]
    =>
    [sd([l1, l4]), sd([l2, l5]), sd([l3, l6])]
    """
    #n = len(lsts)
    return [StandardDeviation(lstsOflst) for lstsOflst in zip(*lsts)]

def StandardDeviationErrorLists(lsts):
    """
    [[l1, l2, l3], [l4, l5, l6]]
    =>
    [sd([l1, l4]), sd([l2, l5]), sd([l3, l6])]
    """
    #n = len(lsts)
    return [StandardDeviationError(lstsOflst) for lstsOflst in zip(*lsts)]

def PartOfList(lsts, idxs):
    return [lsts[i] for i in idxs]

def AverageErrorLists(lsts, indices):
    """
     indices = [[0, 1, 2], [3, 4, 5], ...]
     lsts = [[v1, v2, v3, v4, ...],
              [v1, v2, v3, v4, ...],
              ...]
     values = [[v1, v1, ..], [v2, v2, ..], ...]
     groups = [[[v1, v1, ...], [v2, v2, ...], ..], [[], [], ..], ...]
    """

    values = [list(item) for item in zip(*lsts)]
    groups = [PartOfList(values, idxs) for idxs in indices] 
    packs = [JoinList(grp) for grp in groups]
    sz = float(len(packs[0]))
    return [math.sqrt(sum([x*x for x in pack]))/sz/math.sqrt(sz) for pack in packs]

def JoinList(lst):
    return reduce(lambda x, y: x+y, lst, [])


#def StringToInteger(vstr):
#    try:
#        return int(vstr)
#    except ValueError:
#        return 0

#def ParseDataID(dataid):
#    if -1 == dataid.find(">"):
#        # not found
#        return None, None, None
#
#    clms = dataid.split(">")
#    name_shot = clms[1]
#    name, shot = name_shot.split("@")
#    return clms[0], name, StringToInteger(shot)

def helptext():
    text = """
    <p>Here is an expanation of this module <p><br>
    Module name : plot_eg_3D <br>
    Synopsis    : Plot 3 DimNo EG formatted data on local file storage. <br>
    Script name : dm_plot_eg_3D.py <br>
    Data source : local file storage <br>
    Author name : M. Yoshinuma <br>
    Contact to  : yoshinuma@nifs.ac.jp <br>
    Plot type   : xy, xy2, xyz
    <br>
    <p>Options<p><br>
    addaxv='valname_i', 'valname_j';<br>
    add valnames to the x or y axis list.<br>
    ex) addaxv=reff; : you can add value 'reff' to the x or y axis list.<br>

    use index option to set the dimensioin index for additional valunames.<br>
    index=1 or 2 : select dimension for index (default 1)<br>
    <br> 
    slicedat=value; <br>
    ex) slicedat=3.5;
    value at position where you want to slice on 3rd dimension for 2D trace plot. 
    (index is used for selecting slice position on 2nd dimmension)
    <br> 
    <br> 
    use n1avg or n2avg option to average the data on the original dimension.
    n1avg=#; : average # points on dimension 1 (1..20)<br>
    n2avg=#; : average # points on dimension 2 (1..20)<br>
    n3avg=#; : average # points on dimension 3 (1..20)<br>
    <br>
    use werr option to draw err bar for 2D plot. <br>
    The data of err is the value at the next column of vary. <br>
    werr;<br>
    <br>
    use wsd option to draw standard deviation of an averaged data as err bar. <br>
    No error bar if the n1avg=1, n2avg=1 and n3avg=1. <br>
    wsde;<br>
    <br>
    use wsde option to draw standard deviation of an averaged data divided with sqrt(n) as err bar. <br>
    No error bar if the n1avg=1, n2avg=1 and n3avg=1. <br>
    wsde;<br>
    <br >
    <p>Load data from EG data server<br>
    1)Select Add for shot number. <br>
    2)Set string like '>diagname%shot%' in the TextControl of the Add. 
    <br>
    """
    return text
    
class InterFace( object ):
    def __init__(self, dataid, **kwargs):
        """
        kwargs:
            option: option string for trace
            cache: myview cache object
            cachebutton: state of button cache on myView.
                         "CACHE": 
                         "ONLINE":
                         "OFFLINE":
            temppath: temp path set with myView dialog.
            cmdpath: command path set with myView dialog
        if there is some troubole to make instance, raise "Error"
        """
        self.dataid = dataid
        self.option = kwargs.get('option')
        self.cache = kwargs.get('cache')
        self.cachebutton = kwargs.get('cachebutton')
        self.temppath = kwargs.get('temppath')
        self.cmdpath = kwargs.get('cmdpath')
        self.flgloaded = False
        self.n1avg = 1
        self.n2avg = 1
        self.n3avg = 1
        self.with_errbar = False
        self.with_sd = False
        self.with_sde = False
        self.parse_option(self.option)

        base, name = os.path.split(self.dataid)
        self.diagname = '%s' % name
        self.cachepath = None

    def parse_option(self, option):
        with_errbar = False
        with_sd = False
        with_sde = False
        axvs = []
        idxs = [] # dim no used for index; setup repeatedly
        optdims = [] # dim no set slice with option; setup repeatedly
        """
        like: addaxv=reff,reff/a99;index=0,0;optdims=2,2;
        """
        slicedat = 0.0
        n1avg = 1
        n2avg = 1
        n3avg = 1
        sliced_near1 = 0.0 #used for 2D plot
        sliced_near2 = 0.0 #used for 2D plot 
        sliced_near3 = 0.0 #used for 2D plot 
        for p in [o.strip() for o in option.split(';')]:
            keyvalue = [item.strip() for item in p.split('=')]
            if len(keyvalue) == 1:
                key, = keyvalue
                if key.lower() == 'werr':
                    with_errbar = True
                if key.lower() == 'wsd':
                    with_sd = True
                if key.lower() == 'wsde':
                    with_sde = True
            if len(keyvalue) == 2:
                key, value = keyvalue
                if key == 'addaxv':
                    axvs = [v.strip() for v in value.split(',')] 
                if key == 'index':
                    for v in value.split(','):
                        try:
                            idx = int(v.strip()) -1
                        except ValueError:
                            idx = 0
                        if 0 > idx > 2:
                            idx = 0
                        idxs.append(idx)
                if key == 'optdim':
                    for v in value.split(','):
                        try:
                            idx = int(v.strip()) -1
                        except ValueError:
                            idx = 0
                        if 0 > idx > 2:
                            idx = 0
                        optdims.append(idx)
                if key == 'slicedat':
                    try:
                        slicedat= float(value)
                    except ValueError:
                        slicedat= 0.0
                if key == 'n1avg':
                    try:
                        n1avg = int(value)
                    except ValueError:
                        n1avg = 1
                    n1avg = kaisekiInterFace.LimitValue(n1avg, 1, 20)

                if key == 'n2avg':
                    try:
                        n2avg = int(value)
                    except ValueError:
                        n2avg = 1
                    n2avg = kaisekiInterFace.LimitValue(n2avg, 1, 20)
                
                if key == 'n3avg':
                    try:
                        n3avg = int(value)
                    except ValueError:
                        n3avg = 1
                    n3avg = kaisekiInterFace.LimitValue(n3avg, 1, 20)

        idxs = idxs + [0] * (len(axvs) - len(idxs))
        self.additional_axis_values = axvs
        self.indices_for_additional_axis_values = idxs
        self.optdims_for_additional_axis_values = optdims
        self.with_errbar = with_errbar
        self.with_sd = with_sd
        self.with_sde = with_sde
        self.slicedat = slicedat
        self.n1avg = n1avg
        self.n2avg = n2avg
        self.n3avg = n3avg
        #print "#Plot3D : setting n1avg=%d n2avg=%d" % (self.n1avg, self.n2avg)

    def load_target(self, diagname, shot, sub):
        # try to get datafile from cache
        str_shot = "%d_%d" % (shot, sub)
        key = '%s_%s' % (diagname, str_shot)
        print("KEY in load_target:", key)
        cachepath = None
        filetime = self.cache.findTime(key) 
        if (self.cachebutton == 'ONLINE' and 
                egdbcommon.compareTime(diagname, shot, sub, filetime)):
            cachepath=None
            logger.debug("*** DISCARD CACHE FILE FOR %s@%s ***" % (diagname, str_shot))
        else:
            cachepath = self.cache.find(key) 
            if cachepath:
                logger.debug("*** USE CACHE FILE FOR %s@%s ***" % (diagname, str_shot))

        if not cachepath and self.cachebutton == 'ONLINE':
            # try to get datafile from server
            logger.debug("load from network")
            fname = "%s.zip" % key
            fname = self.cache.makeUniqueFilename(fname, self.temppath)
            cachepath = os.path.join(self.temppath, fname) 
            cmdstr = "igetfile -s %d -m %d -d %s -Z -o %s" % (
                        shot, sub, diagname, cachepath) 
            #cmdstr = os.path.join(self.cmdpath, cmdstr)
            #print cmdstr
            if egdbcommon.icheckfile(diagname, shot, sub):
                subprocess.check_call(cmdstr,shell=True)
                if os.path.exists(cachepath):
                    logger.debug("Register the Datafile %s" % cachepath)
                    cachepath = self.cache.registFile(key, cachepath)
                else:
                    cachepath = None
            else:
                cachepath = None
        return cachepath

    def load(self, **kwargs):

        """
        try:
            self.eg = EG_2D(self.dataid)
        except:
            self.eg = None
        """
        print("&&self.dataid=",self.dataid)
        directive, diagname, shot, sub = kaisekiInterFace.ParseDataID(self.dataid)
        print(directive, diagname, shot, sub)
        print("&&shot=%d,sub=%d" % (shot, sub))
        print("FILE:", os.path.exists(self.dataid))

        if diagname is None:
            self.eg = EG_3D(self.dataid)
        else:
            cachepath = self.load_target(diagname, shot, sub)
            print("CACHE PATH:", cachepath)
            if cachepath:
                self.eg = EG_3D(cachepath)
                self.cache.releasefile(cachepath)

        if self.eg.dimno != 3:
            self.eg = None
            self.flgloaded = False
        else:
            self.flgloaded = True
            self.init_dimensions()
        print("# **** LOAD FINISHED [%s] ***" % self.dataid)
            
    def init_dimensions(self):
        dim2data = self.eg.dims(2)
        dim1data = self.eg.dims(1)
        dim0data = self.eg.dims(0)
        dim0_indices = kaisekiInterFace.AverageGroupOfIndices(len(dim0data), self.n1avg)
        dim1_indices = kaisekiInterFace.AverageGroupOfIndices(len(dim1data), self.n2avg)
        dim2_indices = kaisekiInterFace.AverageGroupOfIndices(len(dim2data), self.n3avg)
        dim0 = [kaisekiInterFace.AveragePartOfList(dim0data, idxs) for idxs in dim0_indices]
        dim1 = [kaisekiInterFace.AveragePartOfList(dim1data, idxs) for idxs in dim1_indices]
        dim2 = [kaisekiInterFace.AveragePartOfList(dim2data, idxs) for idxs in dim2_indices]
        self.dim_indices = [dim0_indices, dim1_indices, dim2_indices]
        self.dim_list = [dim0, dim1, dim2]

    def datainfo(self, **kwargs):
        ptype = kwargs.get('type', None)
        if not self.flgloaded:
            self.load(**kwargs)

        if not self.eg:
            return "Data Loading Failed."
        return "\n".join(self.eg.headlines)

    def varsx(self, **kwargs):
        ptype = kwargs.get('type', None)
        if not self.flgloaded:
            self.load(**kwargs)

        if self.eg is None:
            return []
        return self.eg.dimnames + self.additional_axis_values

    def varsy(self, **kwargs):
        ptype = kwargs.get('type', None)
        varx = kwargs.get('varx', None)
        
        if not self.flgloaded:
            self.load(**kwargs)
        if not self.eg:
            return []
       
        if varx in self.additional_axis_values:
            varx_idx = -1
        else:
            varx_idx = self.eg.dimnames.index(varx) 

        if ptype == 'xyz':
            if varx_idx == 0:
                return self.eg.dimnames[1:3] + self.additional_axis_values
            elif varx_idx == 1:
                return [self.eg.dimnames[0], self.eg.dimnames[2]] + self.additional_axis_values
            elif varx_idx == 2:
                return self.eg.dimnames[0:2] + self.additional_axis_values
            else:
                return self.eg.dimnames # varx is in the additional axis values

        return [name for name in self.eg.valnames if name != varx]

    def varsz(self, **kwargs):
        ptype = kwargs.get('type', None)
        varx = kwargs.get('varx', None)
        vary = kwargs.get('vary', None)
        if ptype == 'xyz':
            return [name for name in self.eg.valnames if name != varx and name != vary]
        return []

    def number_not_in_list(self, lst):
        notin = [i for i in range(0, 3) if i not in lst]
        return notin[0]
       
    def indexs(self, **kwargs):
        ptype = kwargs.get('type', None)
        varx = kwargs.get('varx', None)
        vary = kwargs.get('vary', None)
        #if ptype == 'xyz':
        #    return []

        if varx in self.additional_axis_values:
            idx = self.additional_axis_values.index(varx) # find index for the name
            idxidx = self.indices_for_additional_axis_values[idx] #
        elif vary in self.additional_axis_values:
            idx = self.additional_axis_values.index(vary) # find index for the name
            idxidx = self.indices_for_additional_axis_values[idx] #
        else:
            varx_idx = self.eg.dimnames.index(varx) 
            if ptype == 'xyz':
                vary_idx = self.eg.dimnames.index(vary) 
                idxidx = self.number_not_in_list([varx_idx, vary_idx])
            else:
                if varx_idx == 0:
                    idxidx = 1
                else:
                    idxidx = 0
        dimdata = self.dim_list[idxidx]
        dimunit = self.eg.dimunits[idxidx]

        return ["%d_%g%s" % (i, v, dimunit) for i, v in enumerate(dimdata)]
    
    def refresh_index(self, kwargs):
        try:
            index = kwargs.get('index', None)
            trace = kwargs.get('trace', None)
            indexs = self.indexs(**kwargs)
            if index and trace:
                index = egdbcommon.get_index(indexs, index)
                kwargs['index'] = index
                trace = kwargs['trace']
                trace.index = index
        except:
            pass

    def dataset(self, **kwargs):
        """
        return list of dataset for varx, vary, varz, index, option.
        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog
            index: 

        dataset := [title, x, xe, y, ye, z, ze]
        x = [x1, x2, x3, ... , xn]
        y = [y1, y2, x3, ... , ym] 
        z = [[z(x1, y1), z(x2, y1), z(x3, y1),...,z(xn, y1)],
             [z(x1, y2), z(x2, y2), z(x3, y2),...,z(xn, y2)],
             [z(x1, y3), z(x2, y3), z(x3, y3),...,z(xn, y3)], 
             ......
             [z(x1, ym), z(x2, ym), z(x3, ym),...,z(xn, ym)]] 
        len(z) = len(y)
        len(z[i]) = len(x)
        """
        ptype = kwargs.get('type', None)
        varx = kwargs.get('varx', None)
        vary = kwargs.get('vary', None)

        if not self.flgloaded:
            self.load(**kwargs)

        dataset = ['%s No file' % self.dataid, [],[],[],[],[],[]]
        if not self.eg:
            return dataset

        dataset = ['%s No data' % self.dataid, [],[],[],[],[],[]]
        
        if varx in self.additional_axis_values:
            dimidx = 0
        else:
            dimidx = self.eg.dimnames.index(varx)
        dimdata = self.eg.dims(dimidx)
        if 0 == len(dimdata):
            return dataset

        self.refresh_index(kwargs)
        
        if ptype == 'xyz':
            dataset = self.mapxyz(**kwargs)
            return dataset

        dataset = self.trace(**kwargs)
        return dataset

    def mapxyz_with_additional_x_axis(self, **kwargs):
        varx, vary, varz, index = kaisekiInterFace.KeywordGet(kwargs, ('varx', 'vary', 'varz', 'index'))
        xdata, xedata, ydata, yedata, zdata, zedata = [], [], [], [], [], []
#        varx = kwargs.get('varx', None)
#        vary = kwargs.get('vary', None)
#        varz = kwargs.get('varz', None)
#        index = kwargs.get('index', None)
        idx = int(index.split('_')[0])

        didx = self.additional_axis_values.index(varx) # find dim index for the name
        dim_for_idx = self.indexs_for_additional_axis_values[didx] #

        # this plot doesn't need dim in option
        #dim_in_option = self.optdims_for_additional_axis_values[didx] #
        #dimdata_in_option = self.dim_list[dim_in_option]
        #dimunit_in_option = self.eg.dimunits[dim_in_option]
        #dimidx_in_option= FindIndexNear(self.slicedat, dimdata_in_option) 

#        xdata = []
#        xedata = []
#        ydata = []
#        yedata = []
#        zdata = []
#        zedata = []

        dimno_for_yaxis = self.eg.dimnames.index(vary)
        dimdata_for_yaxis = self.dim_list[dimno_for_yaxis]
        indices_for_yaxis = self.dim_indices[dimno_for_yaxis]

        name_pos = self.additional_axis_values.index(varx) # find dim index for the name
        dimno_for_xaxis = self.indexs_for_additional_axis_values[name_pos] #
        dimdata_for_xaxis = self.dim_list[dimno_for_xaxis]
        indices_for_xaxis = self.dim_indices[dimno_for_xaxis]

        dimno_in_index = self.number_not_in_list([dimno_for_xaxis, dimno_for_yaxis])
        dimdata_in_index = self.dim_list[dimno_in_index]
        dimunit_in_index = self.eg.dimunits[dimno_in_index]
        at = dimdata_in_index[idx]

        xidx = self.eg.vidx(varx)
        vidx = self.eg.vidx(varz)
        unit = self.eg.valunits[vidx]
        title = '%s_%s(%s)_%g%s' % (self.diagname, varz, unit, at, dimunit_in_index)

        for i, ii in enumerate(indices_for_xaxis):
            dupe_check_list = []
            #xo = xdimdata[i]
            for j, jj in enumerate(indices_for_yaxis):
                y = dimdata_for_yaxis[j]

                xpack = []
                vpack = []

                for xi in ii:
                    for yj in jj:
                        didxs = [(xdimidx, xi), (ydimidx, yi), (dim_for_idx, idx)] 
                        didxs.sort()
                        args = [k for v, k in didxs]

                        xpack.append(self.eg.vdata(xidx, *args))
                        vpack.append(self.eg.vdata(vidx, *args))
                        
                x = sum(xpack)/float(len(xpack))
                v = sum(vpack)/float(len(vpack))

                if (x, y) in dupe_check_list:
                    continue
                    
                dupe_check_list.append((x, y))
                xdata.append(x)
                ydata.append(y)
                zdata.append(v)
                
        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

    def mapxyz_with_additional_y_axis(self, **kwargs):
        varx, vary, varz, index = kaisekiInterFace.KeywordGet(kwargs, ('varx', 'vary', 'varz', 'index'))
        xdata, xedata, ydata, yedata, zdata, zedata = [], [], [], [], [], []
#        varx = kwargs.get('varx', None)
#        vary = kwargs.get('vary', None)
#        varz = kwargs.get('varz', None)
#        index = kwargs.get('index', None)
        idx = int(index.split('_')[0])

        # this plot doesn't need dim in option
        #dim_in_option = self.optdims_for_additional_axis_values[didx] #
        #dimdata_in_option = self.dim_list[dim_in_option]
        #dimunit_in_option = self.eg.dimunits[dim_in_option]
        #dimidx_in_option= FindIndexNear(self.slicedat, dimdata_in_option) 

#        xdata = []
#        xedata = []
#        ydata = []
#        yedata = []
#        zdata = []
#        zedata = []

        dimno_for_xaxis = self.eg.dimnames.index(varx)
        dimdata_for_xaxis = self.dim_list[dimno_for_xaxis]
        indices_for_xaxis = self.dim_indices[dimno_for_xaxis]

        name_pos = self.additional_axis_values.index(vary) # find dim index for the name
        dimno_for_yaxis = self.indexs_for_additional_axis_values[name_pos] #
        dimdata_for_yaxis = self.dim_list[dimno_for_yaxis]
        indices_for_yaxis = self.dim_indices[dimno_for_yaxis]

        dimno_in_index = self.number_not_in_list([dimno_for_xaxis, dimno_for_yaxis])
        dimdata_in_index = self.dim_list[dimno_in_index]
        dimunit_in_index = self.eg.dimunits[dimno_in_index]
        at = dimdata_in_index[idx]

        yidx = self.eg.vidx(vary)
        vidx = self.eg.vidx(varz)
        unit = self.eg.valunits[vidx]
        title = '%s_%s(%s)_%g%s' % (self.diagname, varz, unit, at, dimunit_in_index)

        for i, ii in enumerate(indices_for_xaxis):
            dupe_check_list = []
            x = dimdata_for_xaxis[i]
            for j, jj in enumerate(indices_for_yaxis):
                #y = ydimdata[j]
                ypack = []
                vpack = []

                for xi in ii:
                    for yj in jj:
                        argsrc = [(dimno_for_xaxis, xi), 
                                (dimno_for_yaxis, yi), 
                                (dimno_in_index, idx)] 
                        argsrc.sort()
                        arg = [k for v, k in argsrc]

                        ypack.append(self.eg.vdata(yidx, *arg))
                        vpack.append(self.eg.vdata(vidx, *arg))
                        
                y = sum(ypack)/float(len(ypack))
                v = sum(vpack)/float(len(vpack))

                if (x, y) in dupe_check_list:
                    continue
                    
                dupe_check_list.append((x, y))
                xdata.append(x)
                ydata.append(y)
                zdata.append(v)
                
        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

    def mapxyz(self, **kwargs):
        varx, vary, varz, index = kaisekiInterFace.KeywordGet(kwargs, ('varx', 'vary', 'varz', 'index'))
        xdata, xedata, ydata, yedata, zdata, zedata = [], [], [], [], [], []
#        varx = kwargs.get('varx', None)
#        vary = kwargs.get('vary', None)
#        varz = kwargs.get('varz', None)
#        index = kwargs.get('index', None)
        idx = int(index.split('_')[0])
#        xdata = []
#        xedata = []
#        ydata = []
#        yedata = []
#        zdata = []
#        zedata = []

        if varx in self.additional_axis_values:
            return self.mapxyz_with_additional_x_axis(**kwargs)

        if vary in self.additional_axis_values:
            return self.mapxyz_with_additional_y_axis(**kwargs)

        #dimidx = self.eg.dimnames.index(varx)
        #xdimdata = self.eg.dims(dimidx)
        #xdimdata = self.dim_list[dimidx]
        #xdimindices = self.dim_indices[dimidx]

        dimno_for_xaxis = self.eg.dimnames.index(varx)
        dimdata_for_xaxis = self.dim_list[dimno_for_xaxis]
        indices_for_xaxis = self.dim_indices[dimno_for_xaxis]

        dimno_for_yaxis = self.eg.dimnames.index(vary)
        dimdata_for_yaxis = self.dim_list[dimno_for_yaxis]
        indices_for_yaxis = self.dim_indices[dimno_for_yaxis]
        
        dimno_in_index = self.number_not_in_list([dimno_for_xaxis, dimno_for_yaxis])
        dimdata_in_index = self.dim_list[dimno_in_index]
        dimunit_in_index = self.eg.dimunits[dimno_in_index]

        at = dimdata_in_index[idx]

        vidx = self.eg.vidx(varz)
        unit = self.eg.valunits[vidx]
        title = '%s_%s(%s)_%g%s' % (self.diagname, varz, unit, at, dimunit_in_index)

        for i, x in enumerate(dimdata_for_xaxis):
            for j, y in enumerate(dimdata_for_yaxis):
                xdata.append(x)
                ydata.append(y)
                xindices = indices_for_xaxis[i]
                yindices = indices_for_yaxis[j]
                vpack = []
                for ii in xindices:
                    for jj in yindices:
                        argsrc = [(dimno_for_xaxis, ii), (dimno_for_yaxis, jj), 
                                (dimno_in_index, idx)] 
                        argsrc.sort()
                        arg = [k for v, k in argsrc]
                        vpack.append(self.eg.vdata(vidx, *arg))
                        
                v = sum(vpack)/len(vpack)
                zdata.append(v)
                
        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

    def trace(self, **kwargs):
        varx, vary, index = kaisekiInterFace.KeywordGet(kwargs, ('varx', 'vary', 'index'))
        xdata, xedata, ydata, yedata, zdata, zedata = [], [], [], [], [], []
#        varx = kwargs.get('varx', None)
#        vary = kwargs.get('vary', None)
#        index = kwargs.get('index', None)
#        xdata = []
#        xedata = []
#        ydata = []
#        yedata = []

        idx = int(index.split('_')[0])

        if varx in self.additional_axis_values:
            flg = True
            name_idx = self.additional_axis_values.index(varx)
            dimno_in_index = self.indices_for_additional_axis_values[name_idx]
            dimno_in_option = self.dimidxs_for_additional_axis_values[name_idx]
            dimno_for_xaxis = self.number_not_in_list([dimno_in_index, dimno_in_option])
        else:
            flg = False
            dimno_for_xaxis = self.eg.dimnames.index(varx)
            dimno_in_index = 1 if dimno_for_xaxis==0 else 0
            dimno_in_option = 1 if dimno_for_xaxis==2 else 2
            
        dimdata_in_index = self.dim_list[dimno_in_index]
        dimunit_in_index = self.eg.dimunits[dimno_in_index]
        dimdata_in_option = self.dim_list[dimno_in_option]
        dimunit_in_option = self.eg.dimunits[dimno_in_option]
        #idx = idx
        idx_in_option= kaisekiInterFace.FindIndexNear(self.slicedat, dimdata_in_option) 

        # get trace with (didx loop,  [dim_for_index:idx, dim_in_option:slicedat])
        #print '>>>', self.dim_indices
        indices_in_index = self.dim_indices[dimno_in_index][idx]
        indices_in_option = self.dim_indices[dimno_in_option][idx_in_option]

        vidx = self.eg.vidx(vary)
        unit = self.eg.valunits[vidx]

        lst_xvalues = []
        lst_yvalues = []
        lst_yevalues = []
        for ii in indices_in_index:
            for jj in indices_in_option:
                argsrc = [(dimno_in_index, ii), (dimno_in_option, jj)]
                argsrc.sort()
                arg = [i for v, i in argsrc]
                if flg:
                    xvalues = self.eg.trace_of(varx, dimno_for_xaxis, *arg)
                else:
                    xvalues = self.dim_list[dimno_for_xaxis]
                yvalues = self.eg.trace_of(vary, dimno_for_xaxis, *arg)

                lst_xvalues.append(xvalues)
                lst_yvalues.append(yvalues)

                if self.with_errbar and vidx+1 < self.eg.valno:
                    yevalues = self.eg.trace_of(vary+1, dimno_for_xaxis, *args)
                    lst_yevalues.append(yevalues)
                    
        xdata = [sum(vs)/len(vs) for vs in zip(*lst_xvalues)]
        ydata = [sum(vs)/len(vs) for vs in zip(*lst_yvalues)]
        if self.with_errbar and vidx+1 < self.eg.valno:
            yedata = [sum(vs)/len(vs) for vs in zip(*lst_yevalues)]
        else:
            yedata = []
            
        at_index = dimdata_in_index[idx]
        at_option = dimdata_in_option[idx_in_option]
        unit_index = dimunit_in_index
        unit_option = dimunit_in_option

        title = '%s_%s(%s)@%g%s_%g%s' % (self.diagname, vary, unit, 
                at_index, unit_index, at_option, unit_option)

#        zdata = []
#        zedata = []
        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

#def ReadFromZipfile(filepath):
#    zf = zipfile.ZipFile(filepath, 'r')
#    target = zf.namelist()[0]
#    cbuff = zf.read(target)
#    zf.close()
#    return cbuff

#def OpenFile(filepath):
#    if filepath[-3:].upper() == 'ZIP':
#        zf = zipfile.ZipFile(filepath, 'r')
#        target = zf.namelist()[0]
#        return zf, io.TextIOWrapper(io.BufferedReader(zf.open(target, 'r')),encoding='ascii')
#    else:
#        return None, open(filepath, "r")

#def CloseFile(zf, fp):
#    if zf:
#        fp.close()
#        zf.close()
#    else:
#        fp.close()

#def CalcWeights(lst):
#    if len(lst) == 0:
#        return [1]
#
#    subweights = CalcWeights(lst[1:])
#    return [lst[0] * subweights[0]] + subweights

class EG_3D(object):
    def __init__(self, filepath):
        """
        self.data[idx]
        iline => self.line_index() 
        size = self.dimno + self.valno
        idx = self.line_size * iline + vidx
        """
        self.filepath = filepath
        self.keyvalue = {}
        self.headlines = []
        self.load(filepath)

    def load(self, filepath):
        zf, fp = kaisekiInterFace.OpenFile(filepath)
        l = fp.readline()
        hlines = []
        while l:
            l = l.strip()
            hlines.append(l)
            if len(l) == 0:
                l = fp.readline()
                continue
            if l[0] == '#':
                clm = l[1:].split('=')
                if len(clm) == 2:
                    self.keyvalue[clm[0].strip()] = clm[1].strip()
                l = fp.readline()
                continue
            break
        self.headlines = hlines[:-1]

        cdimnames = self.keyvalue.get('DimName')
        cdimunits = self.keyvalue.get('DimUnit')
        cdimno = self.keyvalue.get('DimNo')
        cdimsize = self.keyvalue.get('DimSize')
        cvalno = self.keyvalue.get('ValNo')
        cvalnames = self.keyvalue.get('ValName')
        cvalunits = self.keyvalue.get('ValUnit')
        
        dimno = int(cdimno)
        valno = int(cvalno)
        line_size = dimno + valno
            
        dimsizes = [int(c.strip()) for c in cdimsize.split(',')]
        line_size = dimno + valno
        data_size = line_size * reduce((lambda x, y: x * y), dimsizes, 1)
        
        # allocate memory
        self.data = numpy.zeros(data_size)

        # make Headder info
        self.dimnames = [c.strip("' ") for c in cdimnames.split(',')] 
        self.dimunits = [c.strip("' ") for c in cdimunits.split(',')] 
        self.valnames = [c.strip("' ") for c in cvalnames.split(',')] 
        self.valunits = [c.strip("' ") for c in cvalunits.split(',')] 
        self.dimno = dimno
        self.valno = valno
        self.line_size = line_size
        self.dimsizes = dimsizes
        self.weights = kaisekiInterFace.CalcWeights(self.dimsizes[1:])

        idx = 0
        while l:
            l = l.strip()
            if len(l) == 0:
                l = fp.readline()
                continue
            #for c in l.split(','):
            for c in re.split(r',\s*|\s+', l):
                if c != "":
                    try:
                        self.data[idx] = float(c)
                    except ValueError:
                        self.data[idx] = numpy.nan
                    idx = idx+1
            l = fp.readline()

        kaisekiInterFace.CloseFile(zf, fp)

    def line_index(self, *dimidxs):
        weighted_indexs = [i * w for i, w in zip(dimidxs, self.weights)]
        return reduce((lambda x, y: x+y), weighted_indexs, 0)

    def get_line(self, *dimidxs):
        idx = self.line_size * self.line_index(*dimidxs)
        return self.data[idx: idx+self.line_size]
    
    def dims(self, dimidx):
        if dimidx == 0:
            pos = [[i, 0, 0] for i in range(self.dimsizes[dimidx])]
        elif dimidx == 1:
            pos = [[0, i, 0] for i in range(self.dimsizes[dimidx])]
        else:
            pos = [[0, 0, i] for i in range(self.dimsizes[dimidx])]
        return [self.get_line(*p)[dimidx] for p in pos]

    def vdata(self, idxv, *dimidxs):
        return self.get_line(*dimidxs)[self.dimno+idxv]

    def vidx(self, name):
        if name in self.valnames:
            return self.valnames.index(name)
        else:
            return -1

    def trace_of_idx(self, vidx, dimidx, idx1, idx2):
        """
        dimidx: idx of variable dims for trace
        ex)
          0: time trace
          1: radial profile
        """
        dimno = self.dimno
        dimsize = self.dimsizes[dimidx]
        if dimidx == 0:
            pos = [[i, idx1, idx2] for i in range(dimsize)]
        elif dimidx == 1:
            pos = [[idx1, i, idx2] for i in range(dimsize)]
        else:
            pos = [[idx1, idx2, i] for i in range(dimsize)]
        return [self.get_line(*p)[self.dimno + vidx] for p in pos]

    def trace_of(self, name, dimidx, idx1, idx2):
        """
        dimidx: idx of variable dims for trace
        ex)
          0: time trace
          1: radial profile
        """
        vidx = self.vidx(name)
        if vidx == -1:
            return []
        dimno = self.dimno
        dimsize = self.dimsizes[dimidx]
        if dimidx == 0:
            pos = [[i, idx1, idx2] for i in range(dimsize)]
        elif dimidx == 1:
            pos = [[idx1, i, idx2] for i in range(dimsize)]
        else:
            pos = [[idx1, idx2, i] for i in range(dimsize)]
        return [self.get_line(*p)[self.dimno + vidx] for p in pos]


if __name__ == '__main__':
    from MyViewCache import *
    #option = 'addaxv=reff;wsde;n1avg=1;n2avg=1'
    #option = 'n1avg=2;n2avg=1'
    #option = 'n1avg=2'
    #option = 'addaxv=reff;werr;n1avg=1;n2avg=1'
    option = 'slicedat=0.0'
    cache = MyViewCache('cache/cache.txt')
    #ifc = InterFace('samples\\CES2@16514.txt', temppath='temp', 
    #ifc = InterFace('samples\\thomson_109693.zip', temppath='temp', 
    #ifc = InterFace('samples\\tsmap_113287.zip', temppath='temp', 
    idstr ='samples\\CES2@19389.txt'
    idstr = 'samples\\DRMP_AC2_ece_100kHz_cave_dT_3D@1060829.dat'
    idstr = "samples\\spec2d@20231020_213121.txt"
    ifc = InterFace(idstr, temppath='temp', 
            cachebutton='ONLINE', cache=cache,
            cmdpath = 'bin', option=option) 
    ifc.load(cachebutton='ONLINE')

    print('#',ifc.varsx())
    print('#',ifc.varsy(varx='Time', type='xy'))
    print('#',ifc.varsy(varx='R', type='xyz'))
    print('#',ifc.varsy(varx='Time', type='xyz'))
    print('#',ifc.varsy(varx='Phase', type='xyz'))
    print('#', ifc.slicedat)
    print('#',ifc.varsz(varx='Time', vary='R', type='xyz'))
    print('#',ifc.indexs(varx='Time', vary='R', type='xyz'))
    print('#',ifc.indexs(varx='Time', vary='Phase', type='xyz'))
    print('#',ifc.indexs(varx='R', vary='Phase', type='xyz'))

    """
    #title, x, xe, y, ye, z, ze =ifc.dataset(varx='Time', vary='Te_0.20145s', index='15_0rad')
    title, x, xe, y, ye, z, ze =ifc.dataset(varx='R', vary='Te_0.20145s', index='100_0.24s')
    print '#', title
    for i, xv in enumerate(x):
        print("%g\t%g" % (xv, y[i]))
    """

    title, x, xe, y, ye, z, ze =ifc.dataset(varx='Time', vary='R', varz='Te_0.20145s', index='15_0rad', type='xyz')
    print('#', title)
    for i, xv in enumerate(x):
        print("%g\t%g\t%g" % (xv, y[i], z[i]))
        if (i+1) % 271==0:
            print()


