#!/usr/bin/env python
# coding: utf-8
# $Id: dm_imp02.py 411 2016-09-21 04:53:36Z emo $
import os
import subprocess
from math import *
import time
import egdbcommon
import egdb2d
from MyViewGlobal import logger
#import dataSource

def helptext():
    text = """
    <p>Here is an expanation of this module <p><br />
    $Id: dm_imp02.py 411 2016-09-21 04:53:36Z emo $ <br />
    Module name : plot_function <br />
    Synopsis    : using to plot function <br>
    Script name : dm_plot_function.py <br />
    Data source : data is created by this module itself <br>
    Plot type   : 2D data only <br>
    <br>
    option:<br>
        f=f(x) or f(x, y); :form of function like 1.0, sin(x) , cos(x), ...<br>
        n=100; :number of x series <br>
        m=100; :number of y series <br>
        xmin = -1; :min of xrange <br>
        xmax = 1; :max of xrange <br>
        ymin = -1; :min of yrange <br>
        ymax = 1; :max of yrange <br>
    """
    return text


class InterFace():
    def __init__(self, dataid, **kwargs):
        """
        kwargs:
            option: option string for trace
            cache: myview cache object
            cachebutton: state of button cache on myView.
                         "CACHE": 
                         "ONLINE":
                         "OFFLINE":
            temppath: temp path set with myView dialog.
            cmdpath: command path set with myView dialog
        if there is some troubole to make instance, raise "Error"
        """
        self.dataid = dataid
        self.option = kwargs.get('option')
        self.cache = kwargs.get('cache')
        self.cachebutton = kwargs.get('cachebutton')
        self.temppath = kwargs.get('temppath')
        self.cmdpath = kwargs.get('cmdpath')
        self.flgloaded = True
        self.diagname = 'func'
        self.cachepath = None
        (self.form, self.n, self.m,
                self.xmin, self.xmax, self.ymin, self.ymax) = self.parse_option(self.option)
        self.datacache = None

    def parse_option(self, option):
        form0 = "0.0"
        n0 = 100
        m0 = 100
        xmin0, xmax0 = -1.0, 1.0
        ymin0, ymax0 = -1.0, 1.0


        if not option:
            return (form0, n0, m0, xmin0, xmax0, ymin0, ymax0)
        
        form, n, m, xmin, xmax, ymin, ymax = form0, n0, m0, xmin0, xmax0, ymin0, ymax0
        items = [item.strip() for item in option.split(';')] # ['z =0.0', 'phi=0.0']
        for item in items:
            clm = [tok.strip() for tok in item.split('=')] # ['z', '0.0']   or  ['phi', '0.0']
            if clm[0] == 'f':
                form = clm[1]
            if clm[0] == 'n':
                try:
                    n = int(clm[1])
                except ValueError:
                    n = n0
            if clm[0] == 'm':
                try:
                    m = int(clm[1])
                except ValueError:
                    m = m0
            if clm[0] == 'xmin':
                try:
                    xmin = float(clm[1])
                except ValueError:
                    xmin = xmin0
            if clm[0] == 'xmax':
                try:
                    xmax = float(clm[1])
                except ValueError:
                    xmax = xmax0
            if clm[0] == 'ymin':
                try:
                    ymin = float(clm[1])
                except ValueError:
                    ymin = xmin0
            if clm[0] == 'ymax':
                try:
                    ymax = float(clm[1])
                except ValueError:
                    ymax = ymax0

        return (form, n, m, xmin, xmax, ymin, ymax)
 
    def load(self, **kwargs):
            return

    def varsx(self, **kwargs):
        """
        return list of candidates of variables for x-axis
        
        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog
        """
        plottype = kwargs.get('type') # xy, xy2, xyz

        if plottype == 'xyz':
            return ['x', 'y']

        return ['x', 'func']

    def varsy(self, **kwargs):
        """
        return list of candidates of variables for y-axis or y2-axis

        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog
        """
        plottype = kwargs.get('type') # xy, xy2, xyz
        varx = kwargs.get('varx', None)
        vary = kwargs.get('vary', None)
        if plottype == 'xyz':
            if varx == 'x':
                return ['y']
            else:
                return ['x']
        if varx == 'x':
            return ['func']
        else:
            return ['x']

    def varsz(self, **kwargs):
        """
        return list of candidates of variables for z-axis

        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog
        """
        plottype = kwargs.get('type') # xy, xy2, xyz
        if plottype == 'xyz':
            return ['func']
        return []

    def indexs(self, **kwargs):
        """
        return list of indexs for varx, vary, varz, option.
        
        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog
        """
        return []

#    def refresh_index(self, kwargs):
#        return
 
    def dataset(self, **kwargs):
        """
        return list of dataset for varx, vary, varz, index, option.
        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog

        dataset := [title, x, xe, y, ye, z, ze]
        x = [x1, x2, x3, ... , xn]
        y = [y1, y2, x3, ... , ym] 
        z = [[z(x1, y1), z(x2, y1), z(x3, y1),...,z(xn, y1)],
             [z(x1, y2), z(x2, y2), z(x3, y2),...,z(xn, y2)],
             [z(x1, y3), z(x2, y3), z(x3, y3),...,z(xn, y3)], 
             ......
             [z(x1, ym), z(x2, ym), z(x3, ym),...,z(xn, ym)]] 
        len(z) = len(y)
        len(z[i]) = len(x)
        """
        vary = kwargs.get('vary', None)
        plottype = kwargs.get('type') # xy, xy2, xyz
        
        
        self.refresh_index(kwargs)
            
        if self.datacache is None:
            if plottype == 'xyz':
                dataset = self.data_xyz(**kwargs)
            else:
                dataset = self.data_xy(**kwargs)
            self.datacache = dataset
            return dataset
        else:
            return self.datacache


    def data_xyz(self, **kwargs):
        varx = kwargs.get('varx', None)
        vary = kwargs.get('vary', None)
        varz = kwargs.get('varz', None)
        index = kwargs.get('index', None)
        #los = self.los_option
        xdata = []
        xedata = []
        ydata = []
        yedata = []
        zdata = []
        zedata = []
        
        title = 'f(x,y)=%s' % (self.form)

        n = self.n
        m = self.m
        xmin = self.xmin
        xmax = self.xmax
        ymin = self.ymin
        ymax = self.ymax
        dx = (xmax - xmin)/(n-1)
        dy = (ymax - ymin)/(m-1)

        lstx = [xmin + dx * i for i in range(n)]
        lsty = [ymin + dy * i for i in range(m)]

        func = self.make_func_xyz(self.form)
        xdata = sum([[x] * m for x in lstx], [])
        ydata = sum([lsty] * n, [])
        try:
            zdata = [func(x, y) for x in lstx for y in lsty]
        except:
            zdata = [0.0 for x in lstx for y in lsty]

        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

    def data_xy(self, **kwargs):
        varx = kwargs.get('varx', None)
        vary = kwargs.get('vary', None)
        index = kwargs.get('index', None)
        xdata = []
        xedata = []
        ydata = []
        yedata = []



        n = self.n
        xmin = self.xmin
        xmax = self.xmax
        dx = (xmax - xmin)/(n-1)

        func = self.make_func_xy(self.form)
        lstx = [xmin + dx*i for i in range(n)]

        if vary == 'func':
            title = 'f(x)=%s' % (self.form)
            xdata = lstx
            try:
                ydata = [func(x) for x in lstx]
            except:
                ydata = [0.0 for x in lstx]
        else:
            title = 'f(y)=%s' % (self.form)
            ydata = lstx
            try:
                xdata = [func(x) for x in lstx]
            except:
                xdata = [0.0 for x in lstx]
        
        zdata = []
        zedata = []
        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

    def make_func_xyz(self, form):
        code = '(lambda x, y: %s)' % form 
        func = eval(code)
        return func
    
    def make_func_xy(self, form):
        code = '(lambda x: %s)' % form 
        func = eval(code)
        return func

if __name__ == '__main__':
    from MyViewCache import *
    option = 'f=sin(x)*cos(y)'
    cache = MyViewCache('cache\cache.txt')
    ifc = InterFace('123130', temppath='temp', cachebutton='ONLINE', cache=cache,
            cmdpath = 'bin\win', option=option) 
    ifc.load(cachebutton='ONLINE')

    print(('#', ifc.varsx(type='xy')))
    print(('#', ifc.varsy(varx='x', type='xy')))
    print(('#', ifc.varsy(varx='func', type='xy')))
    data = ifc.dataset(varx='x', vary='y', varz='func', type='xyz')
    title, x, xe, y, ye, z, ze = data
    print(('#', title))
    for xi, yi, zi in zip(x, y, z):
        print(("%g\t%g\t%g" % (xi, yi, zi)))





