#!/usr/bin/env python
# coding: utf-8
# $Id$
import re
import numpy as np 
import kaisekiInterFace
from MyViewGlobal import logger

def helptext(title_name, script_name, data_source, synopsis, author_name, contact_to, plot_type, proc_summary, module_option):
    return kaisekiInterFace.helptext( title_name, script_name, data_source, synopsis, author_name, contact_to, plot_type, proc_summary, module_option )

class eg1dInterFace( kaisekiInterFace.kaisekiInterFace ):
    pattern   = '\Acxscti7\Z'                     # cxscti7
    pattern2  = '\Acxscti9\Z'                     # cxscti9
    pattern3  = '\Acxswpi7\Z'                     # cxswpi7
    pattern4  = '\Acxswpi9\Z'                     # cxswpi9
    pattern5  = '\ADivIis_tor\Z'                  # DivIis_tor
    pattern6  = '\ADivIis_tor_sum\Z'              # DivIis_tor_sum
    pattern7  = '\ADivTC_inner\Z'                 # DivTC_inner
    pattern8  = '\Amagnetic_axis\Z'               # magnetic_axis
    pattern9  = '\Atsmap_nel\Z'                   # tsmap_nel

    def __init__(self, data_id, title_name, data_source, tuple_ntavg, tuple_points, **kwargs):
        """
        kwargs:
            option: option string for trace
            cache: myview cache object
            cachebutton: state of button cache on myView.
                         "CACHE": 
                         "ONLINE":
                         "OFFLINE":
            temppath: temp path set with myView dialog.
            cmdpath: command path set with myView dialog
        if there is some troubole to make instance, raise "Error"
        """
        super().__init__(data_id, title_name, data_source, **kwargs)
        self.poinum                   = None
        self.ntavg                    = None
        self.time_trace_only_valnames = []
        self.poinum, self.ntavg = self.parse_option(self.option, tuple_ntavg, tuple_points )
        self.time_trace_only_valnames = []

    def parse_option(self, option, tuple_ntavg, tuple_points ):
        default_ntavg, ntavg_min, ntavg_max    = tuple_ntavg
        default_points, points_min, points_max = tuple_points
        poinum = default_points
        ntavg  = default_ntavg
        if not option:
            pass
            
        else:
            dic = kaisekiInterFace.ParseOptionToDictionary(option)
            if default_points is not None :
                vstr   = dic.get('points')
                poinum = kaisekiInterFace.GetInteger(vstr, default_points, points_min, points_max)
                
            if ntavg_default is not None :
                vstr   = dic.get('ntavg')
                ntavg  = kaisekiInterFace.GetInteger(vstr, default_ntavg, ntavg_min, ntavg_max)
                
        return poinum, ntavg

    def load(self, **kwargs):
        super().load( **kwargs )
        if self.flgloaded:
            return
            
        if self.cachepath:
            self.eg = EG_1D(self.cachepath, self.actual_name)
            self.cache.releasefile(self.cachepath)
            self.flgloaded = True
        else:
            self.eg = None
 
    def varsx(self, **kwargs):
        """
        return list of candidates of variables for x-axis
        
        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog
        """
        lst = []
        self.load(**kwargs)
        if not self.eg:
            pass
        else :
            if ( re.compile(eg1dInterFace.pattern5).search(self.actual_name) or
                 re.compile(eg1dInterFace.pattern6).search(self.actual_name) or
                 re.compile(eg1dInterFace.pattern7).search(self.actual_name) or
                 re.compile(eg1dInterFace.pattern8).search(self.actual_name) or
                 re.compile(eg1dInterFace.pattern9).search(self.actual_name) ):
                # DivIis_tor
                # DivIis_tor_sum
                # DivTC_inner
                # magnetic_axis
                # tsmap_nel
                lst = ['Time']
                
            else:
                # cxscti7
                # cxscti9
                # cxswpi7
                # cxswpi9
                plottype = kwargs.get('type') # xy, xy2, xyz
                if plottype in ['xy', 'xy2']:
                    lst = ['time', 'reff','reff/a99']
                    
        return lst
        
    def valnames2labels(self, valnames):
        """
        Take Head before '@'. 'Iis_2R@20' => 'Iis_2R' 
        """
        return [v.split('@')[0] for v in valnames]
        
    def label2valname(self, label):
        """
        label to valname 'Iis_2R' => 'Iis_2R@20'
        """
        labels = self.valnames2labels(self.eg.valnames)
        return self.eg.valnames[labels.index(label)]
        
    def varsy(self, **kwargs):
        """
        return list of candidates of variables for y-axis or y2-axis

        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog
        """
        lst = []
        self.load(**kwargs)
        if not self.eg:
            pass
        else:
            plottype = kwargs.get('type') # xy, xy2, xyz
            varx     = kwargs.get('varx') # xy, xy2, xyz
            if plottype in ['xy', 'xy2']:
                if ( re.compile(eg1dInterFace.pattern5).search(self.actual_name) or
                     re.compile(eg1dInterFace.pattern7).search(self.actual_name) or
                     re.compile(eg1dInterFace.pattern8).search(self.actual_name) or
                     re.compile(eg1dInterFace.pattern9).search(self.actual_name) ):
                    # DivIis_tor
                    # DivTC_inner
                    # magnetic_axis
                    # tsmap_nel
                    lst = self.eg.valnames
                    if type(lst) is str :
                        # If it is a string, convert it to an array.
                        lst = [lst]
                        
                elif re.compile(eg1dInterFace.pattern6).search(self.actual_name) :
                    # DivIis_tor_sum
                    lst = self.valnames2labels(self.eg.valnames)
                    if type(lst) is str :
                        # If it is a string, convert it to an array.
                        lst = [lst]
                        
            else :
                    # cxscti7
                    # cxscti9
                    # cxswpi7
                    # cxswpi9
                    if varx.casefold() == 'time'.casefold():
                        lst = self.time_trace_only_valnames
                        if type(lst) is str :
                            # If it is a string, convert it to an array.
                            lst = [lst]
                            
                    else :
                        lst = ['Ti_poly']
                        
        return lst

    def indexs(self, **kwargs):
        """
        return list of indexs for varx, vary, varz, option.
        
        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog
        """
        lst = []
        self.load(**kwargs)
        if not self.eg:
            pass
        else:
            if ( re.compile(eg1dInterFace.pattern5).search(self.actual_name) or
                 re.compile(eg1dInterFace.pattern6).search(self.actual_name) or
                 re.compile(eg1dInterFace.pattern7).search(self.actual_name) or
                 re.compile(eg1dInterFace.pattern8).search(self.actual_name) or
                 re.compile(eg1dInterFace.pattern9).search(self.actual_name) ) :
                # DivIis_tor
                # DivIis_tor_sum
                # DivTC_inner
                # magnetic_axis
                # tsmap_nel
                pass
            else :
                # cxscti7
                # cxscti9
                # cxswpi7
                # cxswpi9
                lst_time = self.eg.dims()
                if ( re.compile(eg1dInterFace.pattern).search(self.actual_name) or 
                     re.compile(eg1dInterFace.pattern3).search(self.actual_name) ) :
                    # cxscti7
                    # cxswpi7
                    lst = ['%d_%g' % (i, tm) for i, tm in enumerate(lst_time)]
                else:
                    # cxscti9
                    # cxswpi9
                    lst_times = kaisekiInterFace.Paires(lst_time, self.ntavg)
                    lst       = ['%d_%g' % (i, sum(tms)/self.ntavg) for i, tms in enumerate(lst_times)]
                    
        return lst

    def dataset(self, **kwargs):
        """
        return list of dataset for varx, vary, varz, index, option.
        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog

        dataset := [title, x, xe, y, ye, z, ze]
        x = [x1, x2, x3, ... , xn]
        y = [y1, y2, x3, ... , ym] 
        z = [[z(x1, y1), z(x2, y1), z(x3, y1),...,z(xn, y1)],
             [z(x1, y2), z(x2, y2), z(x3, y2),...,z(xn, y2)],
             [z(x1, y3), z(x2, y3), z(x3, y3),...,z(xn, y3)], 
             ......
             [z(x1, ym), z(x2, ym), z(x3, ym),...,z(xn, ym)]] 
        len(z) = len(y)
        len(z[i]) = len(x)
        """
        xdata, xedata, ydata, yedata, zdata, zedata = [], [], [], [], [], []
        ptype, varx, vary, index = kaisekiInterFace.KeywordGet(kwargs, ('type', 'varx', 'vary', 'index'))
        title = '%s@%s' % (self.diagname, self.dataid)

        self.load(**kwargs)
        if not self.eg:
            # if there is no data
            title = title+'_No Data'
            return [title, [], [], [], [], [], []]
        else:
            if ( re.compile(eg1dInterFace.pattern5).search(self.actual_name) or
                 re.compile(eg1dInterFace.pattern6).search(self.actual_name) or
                 re.compile(eg1dInterFace.pattern7).search(self.actual_name) or
                 re.compile(eg1dInterFace.pattern8).search(self.actual_name) or
                 re.compile(eg1dInterFace.pattern9).search(self.actual_name) ) :
                # DivIis_tor
                # DivIis_tor_sum
                # DivTC_inner
                # magnetic_axis
                # tsmap_nel
                if ptype == 'xyz':
                    title = title+'_Not supported'
                    return [title, [], [], [], [], [], []]
                    
                else :
                    return self.time_trace(**kwargs)
                    
            else :
                # cxscti7
                # cxscti9
                # cxswpi7
                # cxswpi9
                if ptype == 'xyz':
                    title = title+'_No 3D Data'
                    return [title, [], [], [], [], [], []]
                    
                else:
                   if varx.casefold() == 'time'.casefold():
                       return self.time_trace(**kwargs)
                       
                   else:
                        times = self.eg.dims()
                        tidx  = int(index.split('_')[0])
                        rho   = np.linspace (-1.3, 1.3, self.poinum)
                        if ( re.compile(eg1dInterFace.pattern).search(self.actual_name) or
                             re.compile(eg1dInterFace.pattern3).search(self.actual_name) ) :
                            # cxscti7
                            # cxswpi7
                            at    = times[tidx]
                            cti0  = self.eg.vdata(self.eg.vidx('cti0'), tidx)
                            cti2  = self.eg.vdata(self.eg.vidx('cti2'), tidx)
                            cti4  = self.eg.vdata(self.eg.vidx('cti4'), tidx)
                            cti6  = self.eg.vdata(self.eg.vidx('cti6'), tidx)
                            a99   = self.eg.vdata(self.eg.vidx('a99'),  tidx)
                        else:
                            # cxscti9
                            # cxswpi9
                            lst_tidxs = kaisekiInterFace.Paires([i for i, tm in enumerate(times)], self.ntavg)
                            lst_times = kaisekiInterFace.Paires(times, self.ntavg)
                            at   = sum(lst_times[tidx])/self.ntavg
                            cti0 = sum([self.eg.vdata(self.eg.vidx('cti0'), idx) for idx in lst_tidxs[tidx]])/self.ntavg
                            cti2 = sum([self.eg.vdata(self.eg.vidx('cti2'), idx) for idx in lst_tidxs[tidx]])/self.ntavg
                            cti4 = sum([self.eg.vdata(self.eg.vidx('cti4'), idx) for idx in lst_tidxs[tidx]])/self.ntavg
                            cti6 = sum([self.eg.vdata(self.eg.vidx('cti6'), idx) for idx in lst_tidxs[tidx]])/self.ntavg
                            a99  = sum([self.eg.vdata(self.eg.vidx('a99'),  idx) for idx in lst_tidxs[tidx]])/self.ntavg
                            
                        reff = [r * a99 for r in rho]
                        if varx == 'reff/a99':
                            xdata = rho
                        else:
                            xdata = reff
                            
                        if vary == 'Ti_poly':
                            ydata = [cti0+cti2*r**2+cti4*r**4+cti6*r**6 for r in rho]
                        else:
                            ydata = []
                            
                        title = '%s_%s_%ss' % (self.diagname, vary, at)
                        
        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

    def time_trace(self, **kwargs):
        xdata, xedata, ydata, yedata, zdata, zedata = [], [], [], [], [], []
        varx, vary = KeywordGet(kwargs, ('varx', 'vary'))

        vidx = self.eg.vidx(vary)
        unit = self.eg.valunits[vidx]

        title = '%s@%s_%s(%s)' % (self.diagname, self.dataid, vary, unit)

        if ( re.compile(eg1dInterFace.pattern).search(self.actual_name) or
             re.compile(eg1dInterFace.pattern3).search(self.actual_name)  ) :
            # cxscti7
            # cxswpi7
            xdata = self.eg.dims()
            ydata = self.eg.trace_of(vary)
            
        elif ( re.compile(eg1dInterFace.pattern5).search(self.actual_name) or
               re.compile(eg1dInterFace.pattern6).search(self.actual_name) or
               re.compile(eg1dInterFace.pattern7).search(self.actual_name) or
               re.compile(eg1dInterFace.pattern8).search(self.actual_name) or
               re.compile(eg1dInterFace.pattern9).search(self.actual_name) ) :
            # DivIis_tor
            # DivIis_tor_sum
            # DivTC_inner
            # magnetic_axis
            # tsmap_nel
            xdata_org = self.eg.dims()
            ydata_org = self.eg.trace_of_idx(vidx)
            if ( re.compile(eg1dInterFace.pattern5).search(self.actual_name) or
                 re.compile(eg1dInterFace.pattern7).search(self.actual_name) or
                 re.compile(eg1dInterFace.pattern8).search(self.actual_name) or
                 re.compile(eg1dInterFace.pattern9).search(self.actual_name) ) :
                # DivIis_tor
                # DivTC_inner
                # magnetic_axis
                # tsmap_nel
                xdata = xdata_org
                ydata = ydata_org
                
            else :
                # DivIis_tor_sum
                xdata = [x for x, y in zip(xdata_org, ydata_org) if y != 0.0]
                ydata = [y for x, y in zip(xdata_org, ydata_org) if y != 0.0]
                
        else :
            # cxscti9
            # cxswpi9
            xdata = [sum(v)/self.ntavg for v in kaisekiInterFace.Paires(self.eg.dims(), self.ntavg)]
            ydata = [sum(v)/self.ntavg for v in kaisekiInterFace.Paires(self.eg.trace_of(vary), self.ntavg)]
            
        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

class EG_1D(object):
    def __init__(self, filepath, actual_name):
        """
        self.data[idx]
        iline = self.line_index()
        size = self.dimno + self.valno
        idx = self.line_size*iline + vidx
        """
        self.filepath    = filepath
        self.actual_name = actual_name
        self.keyvalue    = {}
        self.load(filepath)

    def load(self, filepath):
        zf, fp = kaisekiInterFace.OpenFile(filepath)
        l = fp.readline()
        while l:
            l = l.strip()
            if len(l) == 0:
                l = fp.readline()
                continue
            if l[0] == '#':
                clm = l[1:].split('=')
                if len(clm) == 2:
                    self.keyvalue[clm[0].strip()] = clm[1].strip()
                    
                l = fp.readline()
                continue
            break
            
        cdimnames = self.keyvalue.get('DimName')
        cdimunits = self.keyvalue.get('DimUnit')
        cdimno    = self.keyvalue.get('DimNo')
        cdimsize  = self.keyvalue.get('DimSize')
        cvalno    = self.keyvalue.get('ValNo')
        cvalnames = self.keyvalue.get('ValName')
        cvalunits = self.keyvalue.get('ValUnit')
        dimno     = int(cdimno)
        valno     = int(cvalno)
        line_size = dimno + valno
        dimsizes  = [int(c.strip()) for c in cdimsize.split(',')]
        
        if ( re.compile('\ADivIis_tor(|_sum)\Z').search(self.actual_name) or 
             re.compile('\ADivTC_inner\Z').search(self.actual_name) or
             re.compile('\Amagnetic_axis\Z').search(self.actual_name) or
             re.compile('\Atsmap_nel\Z').search(self.actual_name) ) :
          self.timelist = self.times()
          data_size = line_size * (reduce((lambda x, y: x * y), dimsizes, 1) * 2)
        else :
          data_size = line_size * reduce((lambda x, y: x * y), dimsizes, 1)
          
        # allocate memory
        self.data = np.zeros(data_size)
        #
        # make Headder info
        self.dimnames  = [c.strip(" '") for c in cdimnames.split(',')]
        self.dimunits  = [c.strip(" '") for c in cdimunits.split(',')]
        self.valnames  = [c.strip(" '") for c in cvalnames.split(',')]
        self.valunits  = [c.strip(" '") for c in cvalunits.split(',')]
        self.dimno     = dimno
        self.valno     = valno
        self.line_size = line_size
        self.dimsizes  = dimsizes
        self.t_size    = dimsizes
        self.weights   = kaisekiInterFace.CalWeights(self.dimsizes[1:])
        idx = 0
        while l:
            l = l.strip()
            if len(l) == 0:
                l = fp.readline()
                continue
            for c in re.split(r',\s*|\s+',l):
                if c != "":
                    try:
                        self.data[idx] = float(c)
                    except ValueError:
                        self.data[idx] = np.nan
                    idx += 1
                    
            l = fp.readline()
            
        kaisekiInterFace.CloseFile(zf, fp)
        
    def get_line(self, idx):
        pidx = self.line_size * idx
        return self.data[pidx: pidx + self.line_size]
        
    def dims(self, dimidx=0):
        return [self.get_line(idx)[dimidx] for idx in range(self.dimsizes[dimidx])]
        
    def times(self, dimidx=0):
        return [self.get_line(idx)[dimidx] for idx in range(self.t_size)]
        
    def vdata(self, vidx, tidx):
        #print self.get_line(tidx), self.dimno, vidx
        return self.get_line(tidx)[self.dimno+vidx]
        
    def vidx(self, name):
        #print self.valnames
        if name in self.valnames:
            return self.valnames.index(name)
        else:
            return -1
            
    def trace_of_idx(self, vidx, dimidx=0):
        dimno   = self.dimno
        dimsize = self.dimsizes[dimidx]
        if ( re.compile('\ADivIis_tor(|_sum)\Z').search(self.actual_name) or 
             re.compile('\ADivTC_inner\Z').search(self.actual_name) or
             re.compile('\Amagnetic_axis\Z').search(self.actual_name) ) :
            pos = [ ([0] * dimidx) + [i] + ([0] * (dimno - dimidx -1)) for i in range(dimsize)]
            return [self.get_line(*p)[self.dimno + vidx] for p in pos]
        else :
            return [self.get_line(idx)[dimno+vidx] for idx in range(dimsize)]
            
    def trace_of(self, name):
        lst = []
        vidx = self.vidx(name)
        if vidx == -1:
            pass
        else :
#            if re.compile('\ADivIis_tor\Z').search(self.actual_name) :
#                dimno   = self.dimno
#                dimsize = self.dimsizes[dimidx]
#                pos = [ ([0] * dimidx) + [i] + ([0] * (dimno - dimidx -1)) for i in range(dimsize)]
#                lst = [self.get_line(*p)[self.dimno + vidx] for p in pos]
#                
#            else :
            lst = self.trace_of_idx(vidx)
                
        return lst

#    def getDataSources(self):
#        return [dataSource.KaisekiDataSource('ip', self.dataid)]

if __name__ == '__main__':
    from MyViewCache import *
    cache = MyViewCache('cache\cache.txt')
    option = 'points=100'
    ifc = InterFace('154900', temppath='temp', cachebutton='ONLINE', cache=cache,
            cmdpath = 'bin\win', option=option) 
    ifc.load(cachebutton='ONLINE')
    #print '#',ifc.varsx()
    print('#',ifc.varsy(varx='reff/a99', type ='xy'))
    print('#',ifc.varsx())
    #print '#',ifc.indexs(varx='reff/a99', vary='Ti_poly')

    """
    title, x, xe, y, ye, z, ze =ifc.dataset(varx='time', vary='Wdia', index='')
    print '# ', title
    for ix, iy in zip(x, y):
        print "%g\t%g" % (ix, iy)
    """

    title, x, xe, y, ye, z, ze =ifc.dataset(varx='reff/a99', vary='Ti_poly', index='60')
    title, x, xe, y, ye, z, ze =ifc.dataset(varx='reff', vary='Ti_poly', index='60')
    print('# ', title)
    """
    for ix, iy in zip(x, y):
        print "%g\t%g" % (ix, iy)
    """

    """
    import matplotlib.pyplot as plt
    plt.plot(x, y)
    plt.xlim(-1.3, 1.3)
    plt.ylim(0, 10)
    plt.show()
    """

    #print  lst_cte0 = self.eg.data[self.eg.valname2idx('cte0')]

