#!/usr/bin/env python
# coding: utf-8
# $Id$
import re
import math
import numpy as numpy
from functools import reduce
import kaisekiInterFace
from MyViewGlobal import logger

def helptext(title_name, script_name, data_source, synopsis, author_name, contact_to, plot_type, proc_summary, module_option):
    return kaisekiInterFace.helptext( title_name, script_name, data_source, synopsis, author_name, contact_to, plot_type, proc_summary, module_option )

class eg2dInterFace( kaisekiInterFace.kaisekiInterFace ):
    pattern1  = '\Alhdmse[12]_shear\Z'            # lhdmse1_shear, lhdmse2_shear
    pattern2  = '\Adytrans_ece\Z'                 # dytrans_ece
    pattern3  = '\Aece_xi\Z'                      # ece_xi
    pattern4  = '\Aece_fast_calThom\Z'            # ece_fast_calThom
    pattern5  = '\Alhdcxs7_T_V\Z'                 # lhdcxs7_T_V
    pattern6  = '\Athomson_fit\Z'                 # thomson_fit
    pattern7  = '\Atsmap_calib\Z'                 # tsmap_calib
    pattern8  = '\ADivIis_tor_add\Z'              # DivIis_tor_add
    pattern9  = '\ADivIis_tor_iso\Z'              # DivIis_tor_iso
    pattern10 = '\Amwrm_comb_envelopes_ccf\Z'     # mwrm_comb_envelopes_ccf
    pattern11 = '\Anb4_Ib_3D_5-O_LOS\Z'           # nb4_Ib_3D_5-O_LOS
    pattern12 = '\Anb4_Ib_3D_5-O_LOSint\Z'        # nb4_Ib_3D_5-O_LOSint
    pattern13 = '\ASDLloopDelPhi\Z'               # SDLloopDelPhi
    pattern14 = '\Anb4_Ib_3D_lhdcxsLOS\Z'         # nb4_Ib_3D_lhdcxsLOS

    def __init__(self, data_id, title_name, data_source, tuple_ntavg, **kwargs):
      try:
        """
        kwargs:
            option: option string for trace
            cache: myview cache object
            cachebutton: state of button cache on myView
                         "CACHE":
                         "ONLINE":
                         "OFFLINE":
            temppath: temp path set with myView dialog
            cmdpath: command path set with myView dialog
        if there is some trouble to make instance, raise "Error"
        """
        super().__init__(data_id, title_name, data_source, **kwargs)
        self.title_dataid           = None
        self.tmin                   = None
        self.tmax                   = None
        self.rmin                   = None
        self.rmax                   = None
        self.los                    = None
        self.ntavg                  = None
        self.dic_verr               = {}
        self.ignores                = []
        self.additionalnames        = []
        self.timeindices            = []
        self.timelist               = []
        self.loslist                = []
        self.pathlist               = []
        self.additional_axis_values = []
        self.theta_ignores          = []
        self.loslist1               = []
        self.loslist3               = []
        self.parse_option(self.option, tuple_ntavg)
      except Exception as e:
        print(e)

    def parse_option(self, option, tuple_ntavg):
      try:
        default_ntavg, ntavg_min, ntavg_max = tuple_ntavg
        if option :
            dic = kaisekiInterFace.ParseOptionToDictionary(option)
            vstr = dic.get('ntavg')
            self.ntavg = kaisekiInterFace.GetInteger(vstr, default_ntavg, ntavg_min, ntavg_max)
            if self.actual_name is not None :
                if ( re.compile(eg2dInterFace.pattern2).search(self.actual_name) or
                     re.compile(eg2dInterFace.pattern3).search(self.actual_name) ):
                    # dytrans_ece
                    # ece_xi
                    vstr = dic.get('tmin')
                    self.tmin = kaisekiInterFace.GetFloat(vstr, None, None, None)
                    vstr = dic.get('tmax')
                    self.tmax = kaisekiInterFace.GetFloat(vstr, None, None, None)
                    if re.compile(eg2dInterFace.pattern3).search(self.actual_name) :
                        # ece_xi
                        vstr = dic.get('rmin')
                        self.rmin = kaisekiInterFace.GetFloat(vstr, None, None, None)
                        vstr = dic.get('rmax')
                        self.rmax = kaisekiInterFace.GetFloat(vstr, None, None, None)
                        
                elif re.compile(eg2dInterFace.pattern11).search(self.actual_name) :
                    # nb4_Ib_3D_5-O_LOS
                    vstr = dic.get('los')
                    self.los = kaisekiInterFace.GetInteger(vstr, 0, None, None)
                    
                elif re.compile(eg2dInterFace.pattern13).search(self.actual_name) :
                    # SDLloopDelPhi
                    vstr = dic.get('ignores')
                    if vstr is not None :
                        self.theta_ignores = [int(c.strip()) for c in value.split(',')]
                    else :
                        self.theta_ignores = []
                        
            else :
                if re.compile(eg2dInterFace.pattern14).search(self.diagname) :
                    # nb4_Ib_3D_lhdcxsLOS
                    vstr = dic.get('los')
                    self.los = kaisekiInterFace.GetInteger(vstr, 0, None, None)
                    
        else:
            if self.actual_name is not None :
                if ( re.compile(eg2dInterFace.pattern2).search(self.actual_name) or
                     re.compile(eg2dInterFace.pattern3).search(self.actual_name) ):
                    # dytrans_ece
                    # ece_xi
                    self.ntavg = default_ntavg
                    self.tmin = None
                    self.tmax = None
                    if re.compile(eg2dInterFace.pattern3).search(self.actual_name) :
                        # ece_xi
                        self.rmin = None
                        self.rmax = None
                        
                elif re.compile(eg2dInterFace.pattern11).search(self.actual_name) :
                    # nb4_Ib_3D_5-O_LOS
                    self.los = 0
                    
                else:
                    self.ntavg = default_ntavg
                    
            else:
                if re.compile(eg2dInterFace.pattern14).search(self.diagname) :
                    # nb4_Ib_3D_lhdcxsLOS
                    self.los = 0
                    
                else:
                    self.ntavg = default_ntavg
                    
      except Exception as e:
        print(e)

    def load(self, **kwargs):
      try:
        if self.actual_name is not None :
            # lhdmse1_shear, lhdmse2_shear
            # dytrans_ece
            # ece_xi
            # ece_fast_calThom
            # lhdcxs7_T_V
            # thomson_fit
            # tsmap_calib
            # DivIis_tor_add
            # DivIis_tor_iso
            # mwrm_comb_envelopes_ccf
            # nb4_Ib_3D_5-O_LOS
            # nb4_Ib_3D_5-O_LOSint
            # SDLloopDelPhi
            super().load( **kwargs )
            if self.flgloaded:
                return
                
            if self.cachepath:
                self.eg = EG_2D(self.cachepath, self.actual_name)
                self.cache.releasefile(self.cachepath)
                self.flgloaded = True
                if ( re.compile(eg2dInterFace.pattern9).search(self.actual_name) or
                     re.compile(eg2dInterFace.pattern10).search(self.actual_name) ):
                    # DivIis_tor_iso
                    # mwrm_comb_envelopes_ccf
                    pass
                else:
                    self.init_timedimensions()
            else:
                self.eg = None
        else:
            # nb4_Ib_3D_lhdcxsLOS
            # nb4_Ib_3D_lhdcxs7LOSint
            if self.flgloaded:
                return
                
            list_cachepath = []
            for diagname in self.list_actual_name :
                cachepath = super().load_target( diagname, **kwargs )
                list_cachepath.append(cachepath)
                
            # Exclude failed loads from the list.
            list_a_filtered = list(filter(None, list_cachepath))
            if not list_a_filtered :
                for idx, diagname in enumerate(self.list_actual_name):
                    self.list_eg.append( None )
                    
            elif ( len(self.list_actual_name) == len( list_a_filtered ) ) :
                # If image data and reference data exist, an image object is created.
                for idx, diagname in enumerate(self.list_actual_name):
                    cachepath = list_a_filtered[idx]
                    self.list_eg.append( EG_2D(cachepath, diagname) )
                    self.cache.releasefile(cachepath)
                    
                self.flgloaded = True
                self.init_timedimensions()
                
            else:
                for idx, diagname in enumerate(self.list_actual_name):
                    self.list_eg.append( None )
                    
      except Exception as e:
          print(e)

    def init_timedimensions(self):
        if self.actual_name is not None :
            dim0_data = self.eg.dims(0)
            if re.compile(eg2dInterFace.pattern11).search(self.actual_name) :
                # nb4_Ib_3D_5-O_LOS
                dim0 = self.eg.dims(0)
                dim1 = self.eg.dims(1)
                dim2 = self.eg.dims(2)
                self.loslist  = dim0
                self.timelist = dim1
                self.pathlist = dim2
                
            elif re.compile(eg2dInterFace.pattern12).search(self.actual_name) :
                # nb4_Ib_3D_5-O_LOSint
                dim1data    = self.eg.dims(1)
                dim1indices = kaisekiInterFace.AverageGroupofIndices(len(dim1data), self.ntavg)
                dim1        = [kaisekiInterFace.AveragePartOfList(dim1data, idxs) for idxs in dim1indices]
                self.timeindices = dim1indices
                self.timelist    = dim1
                
            else :
                if re.compile(eg2dInterFace.pattern8).search(self.actual_name) :
                    # DivIis_tor_add
                    dim0_indices = kaisekiInterFace.AverageGroupOfIndices(len(dim0_data), self.ntavg)
                else:
                    # lhdmse1_shear, lhdmse2_shear
                    # dytrans_ece
                    # ece_xi
                    # ece_fast_calThom
                    # lhdcxs7_T_V
                    # thomson_fit
                    # tsmap_calib
                    # SDLloopDelPhi
                    idxlst = [i for i in range(len(dim0_data))]
                    if ( re.compile(eg2dInterFace.pattern2).search(self.actual_name) or
                         re.compile(eg2dInterFace.pattern3).search(self.actual_name) or
                         re.compile(eg2dInterFace.pattern5).search(self.actual_name) ):
                        # dytrans_ece
                        # ece_xi
                        pass
                    else:
                        # lhdmse1_shear, lhdmse2_shear
                        # ece_fast_calThom
                        # thomson_fit
                        # tsmap_calib
                        # SDLloopDelPhi
                        if self.ntavg > 1:
                            keydata  = self.eg.trace_of_idx(0, 0, 0)
                            idxlst = [i for i, k in zip(idxlst, keydata) if k !=0]
                            
                    dim0_indices = kaisekiInterFace.AverageGroupOfList(idxlst, self.ntavg)
                    
                dim0 = [kaisekiInterFace.AveragePartOfList(dim0_data, idxs) for idxs in dim0_indices]
                self.timeindices = dim0_indices
                self.timelist = dim0
        else :
            # nb4_Ib_3D_lhdcxsLOS
            # nb4_Ib_3D_lhdcxs7LOSint
            if re.compile(eg2dInterFace.pattern14).search(self.diagname) :
                # nb4_Ib_3D_lhdcxsLOS
                if ( 'nb4_Ib_3D_lhdcxsLOS_grp1' in self.list_actual_name and 
                     'nb4_Ib_3D_lhdcxsLOS_grp3' in self.list_actual_name ):
                    egidx1 = self.list_actual_name.index('nb4_Ib_3D_lhdcxsLOS_grp1')
                    egidx2 = self.list_actual_name.index('nb4_Ib_3D_lhdcxsLOS_grp3')
                    if self.list_eg[egidx1] :
                        dim0_1 = self.list_eg[egidx1].dims(0)
                        dim1_1 = self.list_eg[egidx1].dims(1)
                        dim2_1 = self.list_eg[egidx1].dims(2)
                        self.loslist1 = dim0_1
                        self.timelist = dim1_1
                        self.pathlist = dim2_1
                    else:
                        dim0_1 = []
                        
                    if self.list_eg[egidx2] :
                        dim0_3 = self.list_eg[egidx2].dims(0)
                    else:
                        dim0_3 = []
                        
                    self.loslist3 = dim0_3
                    self.loslist  = sorted(dim0_1+dim0_3)
                    
            else:
                # nb4_Ib_3D_lhdcxs7LOSint
                if ( 'nb4_Ib_3D_lhdcxsLOSint_grp1' in self.list_actual_name ) :
                    egidx1 = self.list_actual_name.index('nb4_Ib_3D_lhdcxsLOSint_grp1')
                    if self.list_eg[egidx1] :
                        self.loslist1 = self.list_eg[egidx1].dims(0)
                        dim1data = self.list_eg[egidx1].dims(1)
                    else:
                        dim1data = []
                        
                    dim1indices = kaisekiInterFace.AverageGroupofIndices(len(dim1data), self.ntavg)
                    dim1 = [kaisekiInterFace.AveragePartOfList(dim1data, idxs) for idxs in dim1indices]
                    self.timeindices = dim1indices
                    self.timelist = dim1
                    
                if ( 'nb4_Ib_3D_lhdcxsLOSint_grp3' in self.list_actual_name ) :
                    egidx2 = self.list_actual_name.index('nb4_Ib_3D_lhdcxsLOSint_grp3')
                    if self.list_eg[egidx2] :
                        self.loslist3 = self.list_eg[egidx2].dims(0)
                        
                self.loslist  = sorted(self.loslist1 + self.loslist3)
                
    def varsx(self, **kwargs):
      try:
        """
        return list of candidates of variables for x-axis
        kwargs:
            option: option string for trace
            varx: varx set within trace dialog
            vary: vary set within trace dialog
            varz: varz set within trace dialog
        """
        lst = []
        self.load(**kwargs)
        if self.actual_name is not None :
            if not self.eg:
                pass
            else :
                if ( re.compile(eg2dInterFace.pattern2).search(self.actual_name) or
                     re.compile(eg2dInterFace.pattern6).search(self.actual_name) ):
                    if re.compile(eg2dInterFace.pattern2).search(self.actual_name) :
                        # dytrans_ece
                        lst = ['time', 'R', 'reff'] + self.additional_axis_values
                    else:
                        # thomson_fit
                        lst = ['time', 'R', 'reff']
                        
                elif ( re.compile(eg2dInterFace.pattern3).search(self.actual_name) or
                       re.compile(eg2dInterFace.pattern4).search(self.actual_name) or
                       re.compile(eg2dInterFace.pattern5).search(self.actual_name) ):
                    # ece_xi
                    # ece_fast_calThom
                    # lhdcxs7_T_V
                    lst = ['Time', 'R']
                    
                elif re.compile(eg2dInterFace.pattern7).search(self.actual_name) :
                    # tsmap_calib
                    lst = ['time', 'R', 'reff', 'reff/a99', 'reff/a999']
                    
                elif re.compile(eg2dInterFace.pattern8).search(self.actual_name) :
                    # DivIis_tor_add
                    lst = ['Time', 'ch']
                    
                elif re.compile(eg2dInterFace.pattern9).search(self.actual_name) :
                    # DivIis_tor_iso
                    lst = ['Time', 'prb']
                    
                elif re.compile(eg2dInterFace.pattern10).search(self.actual_name) :
                    # mwrm_comb_envelopes_ccf
                    lst = ['Time', 'reff', 'reff/a99']
                    
                elif re.compile(eg2dInterFace.pattern11).search(self.actual_name) :
                    # nb4_Ib_3D_5-O_LOS
                    lst = ['Time', 'path', 'R', 'reff']
                    
                elif re.compile(eg2dInterFace.pattern12).search(self.actual_name) :
                    # nb4_Ib_3D_5-O_LOSint
                    lst = ['ilosvec', 'Time', 'reff']
                    
                elif re.compile(eg2dInterFace.pattern13).search(self.actual_name) :
                    # SDLloopDelPhi
                    lst = ['time', 'theta']
                    
                else:
                    # lhdmse1_shear, lhdmse2_shear
                    lst = ['Time', 'R', 'reff', 'reff/a99']
                    
        else :
            # nb4_Ib_3D_lhdcxsLOS
            # nb4_Ib_3D_lhdcxs7LOSint
            if re.compile(eg2dInterFace.pattern14).search(self.diagname) :
                # nb4_Ib_3D_lhdcxsLOS
                if ( 'nb4_Ib_3D_lhdcxsLOS_grp1' in self.list_actual_name ) :
                    egidx1 = self.list_actual_name.index('nb4_Ib_3D_lhdcxsLOS_grp1')
#                    egidx2 = self.list_actual_name.index('nb4_Ib_3D_lhdcxsLOS_grp3')
                    if self.list_eg[egidx1] :
                        lst = ['Time', 'path', 'R', 'reff']
                        
            else:
                # nb4_Ib_3D_lhdcxs7LOSint
                if ( 'nb4_Ib_3D_lhdcxsLOSint_grp1' in self.list_actual_name ) :
                    egidx1 = self.list_actual_name.index('nb4_Ib_3D_lhdcxsLOSint_grp1')
                    if self.list_eg[egidx1] :
                        lst = ['ilosvec', 'Time', 'reff']
                        
      except Exception as e:
          print(e)

      return lst

    def varsy(self, **kwargs):
        """
        return list of candidates of variables for y-axis or y2-axis
        kwargs:
            option: option string for trace
            varx: varx set within trace dialog
            vary: vary set within trace dialog
            varz: varz set within trace dialog
        """
        lst = []
        self.load(**kwargs)
        if self.actual_name is not None :
            if not self.eg:
                pass
            else:
                ptype = kwargs.get('type', None)
                varx  = kwargs.get('varx')
                if ptype == 'xyz':
                    if varx.casefold() == 'time'.casefold():
                        if ( re.compile(eg2dInterFace.pattern3).search(self.actual_name) or
                             re.compile(eg2dInterFace.pattern4).search(self.actual_name) or
                             re.compile(eg2dInterFace.pattern5).search(self.actual_name) ):
                            # ece_xi
                            # ece_fast_calThom
                            # lhdcxs7_T_V
                            lst = ['R']
                            
                        elif ( re.compile(eg2dInterFace.pattern2).search(self.actual_name) or
                               re.compile(eg2dInterFace.pattern6).search(self.actual_name) ):
                            # dytrans_ece
                            # thomson_fit
                            lst = ['R', 'reff']
                            
                        elif re.compile(eg2dInterFace.pattern7).search(self.actual_name) :
                            # tsmap_calib
                            lst = ['R', 'reff', 'reff/a99', 'reff/a999']
                            
                        elif re.compile(eg2dInterFace.pattern8).search(self.actual_name) :
                            # DivIis_tor_add
                            lst = ['ch']
                            
                        elif re.compile(eg2dInterFace.pattern9).search(self.actual_name) :
                            # DivIis_tor_iso
                            lst = ['prb']
                            
                        elif re.compile(eg2dInterFace.pattern10).search(self.actual_name) :
                            # mwrm_comb_envelopes_ccf
                            lst = ['reff', 'reff/a99']
                            
                        elif re.compile(eg2dInterFace.pattern11).search(self.actual_name) :
                            # nb4_Ib_3D_5-O_LOS
                            lst = ['path', 'R','reff']
                            
                        elif re.compile(eg2dInterFace.pattern12).search(self.actual_name) :
                            # nb4_Ib_3D_5-O_LOSint
                            lst = ['ilosvec', 'reff']
                            
                        elif re.compile(eg2dInterFace.pattern13).search(self.actual_name) :
                            # SDLloopDelPhi
                            lst = ['theta']
                            
                        else:
                            # lhdmse1_shear, lhdmse2_shear
                            lst = ['R', 'reff', 'reff/a99']
                            
                    else:
                        lst = ['Time']
                        
                else :
                    if ( re.compile(eg2dInterFace.pattern3).search(self.actual_name) or
                         re.compile(eg2dInterFace.pattern8).search(self.actual_name) ):
                        # ece_xi
                        # DivIis_tor_add
                        lst = self.eg.valnames
                        
                    elif re.compile(eg2dInterFace.pattern9).search(self.actual_name) :
                        # DivIis_tor_iso
                        lst = self.valnames2labels(self.eg.valnames)
                        
                    elif ( re.compile(eg2dInterFace.pattern1).search(self.actual_name) or
                           re.compile(eg2dInterFace.pattern2).search(self.actual_name) or
                           re.compile(eg2dInterFace.pattern5).search(self.actual_name) or
                           re.compile(eg2dInterFace.pattern10).search(self.actual_name) ):
                        # lhdmse1_shear, lhdmse2_shear
                        # dytrans_ece
                        # lhdcxs7_T_V
                        # mwrm_comb_envelopes_ccf
                        lst = [v for v in self.eg.valnames if v not in self.ignores]
                    else:
                        # ece_fast_calThom
                        # thomson_fit
                        # tsmap_calib
                        # nb4_Ib_3D_5-O_LOS
                        # nb4_Ib_3D_5-O_LOSint
                        # SDLloopDelPhi
                        lst = [v for v in self.eg.valnames if v not in self.ignores] + self.additionalnames
                        if re.compile(eg2dInterFace.pattern7).search(self.actual_name) :
                            # tsmap_calib
                            lst = self.vname_sorter(lst)
                            
            if type(lst) is str :
                # If it is a string, convert it to an array.
                lst = [lst]
                
        else :
            # nb4_Ib_3D_lhdcxsLOS
            # nb4_Ib_3D_lhdcxs7LOSint
            if re.compile(eg2dInterFace.pattern14).search(self.diagname) :
                # nb4_Ib_3D_lhdcxsLOS
                if ( 'nb4_Ib_3D_lhdcxsLOS_grp1' in self.list_actual_name ) :
                    egidx1 = self.list_actual_name.index('nb4_Ib_3D_lhdcxsLOS_grp1')
                    if self.list_eg[egidx1] :
                        ptype = kwargs.get('type', None)
                        if ptype == 'xyz':
                            varx  = kwargs.get('varx')
                            if varx.casefold() == 'time'.casefold():
                                lst = ['path', 'R','reff']
                            else:
                                lst = ['Time']
                                
                        else :
                            lst = [v for v in self.list_eg[egidx1].valnames if v not in self.ignores] + self.additionalnames
                            
            else:
                # nb4_Ib_3D_lhdcxs7LOSint
                if ( 'nb4_Ib_3D_lhdcxsLOSint_grp1' in self.list_actual_name ) :
                    egidx1 = self.list_actual_name.index('nb4_Ib_3D_lhdcxsLOSint_grp1')
                    if self.list_eg[egidx1] :
                        ptype = kwargs.get('type', None)
                        if ptype == 'xyz':
                            varx  = kwargs.get('varx')
                            if varx.casefold() == 'time'.casefold():
                                lst = ['ilosvec','reff']
                            else:
                                lst = ['Time']
                                
                        else :
                            lst = [v for v in self.list_eg[egidx1].valnames if v not in self.ignores] + self.additionalnames
                            
        return lst

    def varsz(self, **kwargs):
        """
        return list of candidates of variables for z-axis
        kwargs:
            option: option string for trace
            varx: varx set within trace dialog
            vary: vary set within trace dialog
            varz: varz set within trace dialog
        """
        lst = []
        self.load(**kwargs)
        if self.actual_name is not None :
            if not self.eg:
                pass
            else:
                if re.compile(eg2dInterFace.pattern10).search(self.actual_name) :
                    # mwrm_comb_envelopes_ccf
                    lst = [v for v in self.eg.valnames if v not in self.ignores]
                    
                else:
                    plottype = kwargs.get('type')
                    if plottype == 'xyz':
                        if ( re.compile(eg2dInterFace.pattern3).search(self.actual_name) or
                             re.compile(eg2dInterFace.pattern8).search(self.actual_name) ):
                            # ece_xi
                            # DivIis_tor_add
                            lst = self.eg.valnames
                            
                        elif re.compile(eg2dInterFace.pattern9).search(self.actual_name) :
                            # DivIis_tor_iso
                            lst = self.valnames2labels(self.eg.valnames)
                            
                        elif ( re.compile(eg2dInterFace.pattern1).search(self.actual_name) or
                               re.compile(eg2dInterFace.pattern2).search(self.actual_name) or
                               re.compile(eg2dInterFace.pattern5).search(self.actual_name) ):
                            # lhdmse1_shear, lhdmse2_shear
                            # dytrans_ece
                            # lhdcxs7_T_V
                            lst = [v for v in self.eg.valnames if v not in self.ignores]
                            
                        else:
                            # ece_fast_calThom
                            # thomson_fit
                            # tsmap_calib
                            # nb4_Ib_3D_5-O_LOS
                            # nb4_Ib_3D_5-O_LOSint
                            # SDLloopDelPhi
                            lst = [v for v in self.eg.valnames if v not in self.ignores] + self.additionalnames
                            if re.compile(eg2dInterFace.pattern7).search(self.actual_name) :
                                # tsmap_calib
                                lst = self.vname_sorter(lst)
                                
            if type(lst) is str :
                # If it is a string, convert it to an array.
                lst = [lst]
                
        else :
            # nb4_Ib_3D_lhdcxsLOS
            # nb4_Ib_3D_lhdcxs7LOSint
            if re.compile(eg2dInterFace.pattern14).search(self.diagname) :
                # nb4_Ib_3D_lhdcxsLOS
                if ( 'nb4_Ib_3D_lhdcxsLOS_grp1' in self.list_actual_name ) :
                    egidx1 = self.list_actual_name.index('nb4_Ib_3D_lhdcxsLOS_grp1')
                    if self.list_eg[egidx1] :
                        ptype = kwargs.get('type', None)
                        if ptype == 'xyz':
                            lst = [v for v in self.list_eg[egidx1].valnames if v not in self.ignores] + self.additionalnames
                            
            else:
                # nb4_Ib_3D_lhdcxs7LOSint
                if ( 'nb4_Ib_3D_lhdcxsLOSint_grp1' in self.list_actual_name ) :
                    egidx1 = self.list_actual_name.index('nb4_Ib_3D_lhdcxsLOSint_grp1')
                    if self.list_eg[egidx1] :
                        ptype = kwargs.get('type', None)
                        if ptype == 'xyz':
                            lst = [v for v in self.list_eg[egidx1].valnames if v not in self.ignores] + self.additionalnames
                            
        return lst

    def indexs(self, **kwargs):
        """
        return list of indices for varx, vary, varz and option.
        kwargs:
            option: option string for trace
            varx: varx set within trace dialog
            vary: vary set within trace dialog
            varz: varz set within trace dialog
        """
        lst = []
        self.load(**kwargs)
        if self.actual_name is not None :
            if not self.eg:
                pass
            else:
                plottype = kwargs.get('type', None)
                if plottype == 'xyz':
                    if re.compile(eg2dInterFace.pattern11).search(self.actual_name) :
                        # nb4_Ib_3D_5-O_LOS
                        lst = ["%d_%dlos" % (i, v) for i, v in enumerate(self.loslist)]
                        
                else:
                    list_varx = []
                    if re.compile(eg2dInterFace.pattern7).search(self.actual_name) :
                        # tsmap_calib
                        list_varx = ['R', 'reff', 'reff/a99', 'reff/a999']
                    elif ( re.compile(eg2dInterFace.pattern3).search(self.actual_name) or
                           re.compile(eg2dInterFace.pattern4).search(self.actual_name) or
                           re.compile(eg2dInterFace.pattern5).search(self.actual_name) ):
                        # ece_xi
                        # ece_fast_calThom
                        # lhdcxs7_T_V
                        list_varx = ['R']
                        
                    elif ( re.compile(eg2dInterFace.pattern2).search(self.actual_name) or
                           re.compile(eg2dInterFace.pattern6).search(self.actual_name) ):
                        # dytrans_ece
                        # thomson_fit
                        list_varx = ['R', 'reff']
                        
                    elif re.compile(eg2dInterFace.pattern8).search(self.actual_name) :
                        # DivIis_tor_add
                        list_varx = ['ch']
                        
                    elif re.compile(eg2dInterFace.pattern9).search(self.actual_name) :
                        # DivIis_tor_iso
                        list_varx = ['prb']
                        
                    elif ( re.compile(eg2dInterFace.pattern10).search(self.actual_name) or
                           re.compile(eg2dInterFace.pattern11).search(self.actual_name) ) :
                        # mwrm_comb_envelopes_ccf
                        # nb4_Ib_3D_5-O_LOS
                        list_varx = ['Time']
                        
                    elif re.compile(eg2dInterFace.pattern12).search(self.actual_name) :
                        # nb4_Ib_3D_5-O_LOSint
                        list_varx = ['ilosvec', 'reff']
                        
                    elif re.compile(eg2dInterFace.pattern13).search(self.actual_name) :
                        # SDLloopDelPhi
                        list_varx = ['theta']
                        
                    else :
                        # lhdmse1_shear, lhdmse2_shear
                        list_varx = ['R', 'reff', 'reff/a99']
                        
                    varx = kwargs.get('varx')
                    if varx in list_varx :
                        if re.compile(eg2dInterFace.pattern9).search(self.actual_name) :
                            # DivIis_tor_iso
                            lst = ["%d_%gs" % (i, v) for i, v in enumerate(self.eg.dims(0))]
                            
                        elif re.compile(eg2dInterFace.pattern10).search(self.actual_name) :
                            # mwrm_comb_envelopes_ccf
                            lst = ["%d_cf%gGHz" % (i, int(v)) for i, v in enumerate(self.eg.dims(1))]
                            
                        elif re.compile(eg2dInterFace.pattern11).search(self.actual_name) :
                            # nb4_Ib_3D_5-O_LOS
                            lst = ["%d_%gm" % (i, v) for i, v in enumerate(self.pathlist)]
                            
                        else :
                            lst = ["%d_%gs" % (i, v) for i, v in enumerate(self.timelist)]
                    else:
                        if re.compile(eg2dInterFace.pattern9).search(self.actual_name) :
                            # DivIis_tor_iso
                            lst = ["%d_prb%02d" % (i, int(v)) for i, v in enumerate(self.eg.dims(1))]
                            
                        elif re.compile(eg2dInterFace.pattern10).search(self.actual_name) :
                            # mwrm_comb_envelopes_ccf
                            lst = ["%d_%gs" % (i, v) for i, v in enumerate(self.eg.dims(0))]
                            
                        elif re.compile(eg2dInterFace.pattern11).search(self.actual_name) :
                            # nb4_Ib_3D_5-O_LOS
                            lst = ["%d_%s" % (i, v) for i, v in enumerate(self.timelist)]
                            
                        elif re.compile(eg2dInterFace.pattern12).search(self.actual_name) :
                            # nb4_Ib_3D_5-O_LOSint
                            lst = ["%d_%gchannel" % (i, v) for i, v in enumerate(self.eg.dims(0))]
                            
                        else :
                            lst = ["%d_%gm" % (i, v) for i, v in enumerate(self.eg.dims(1))]
                            
        else :
            ptype = kwargs.get('type', None)
            # nb4_Ib_3D_lhdcxsLOS
            # nb4_Ib_3D_lhdcxs7LOSint
            if re.compile(eg2dInterFace.pattern14).search(self.diagname) :
                # nb4_Ib_3D_lhdcxsLOS
                if ptype == 'xyz':
                    lst = ["%d_%dlos" % (i, v) for i, v in enumerate(self.loslist)]
                else:
                    varx = kwargs.get('varx')
                    if varx.casefold() == 'time'.casefold():
                        lst = ["%d_%gm" % (i, v) for i, v in enumerate(self.pathlist)]
                    else :
                        lst = ["%d_%gs" % (i, v) for i, v in enumerate(self.timelist)]
                        
            else:
                # nb4_Ib_3D_lhdcxs7LOSint
                if ptype == 'xyz':
                    pass
                else:
                    varx = kwargs.get('varx')
                    if varx in ['ilosvec','reff']:
                        lst = ["%d_%gs" % (i, v) for i, v in enumerate(self.timelist)]
                    else:
                        if ( 'nb4_Ib_3D_lhdcxsLOSint_grp1' in self.list_actual_name and 
                             'nb4_Ib_3D_lhdcxsLOSint_grp3' in self.list_actual_name ):
                            egidx1 = self.list_actual_name.index('nb4_Ib_3D_lhdcxsLOSint_grp1')
                            egidx2 = self.list_actual_name.index('nb4_Ib_3D_lhdcxsLOSint_grp3')
                            if self.list_eg[egidx1] and self.list_eg[egidx2] :
                                dimchs = sorted( self.list_eg[egidx1].dims(0) + self.list_eg[egidx2].dims(0) )
                                lst = ["%d_%gchannel" % (i, v) for i, v in enumerate(dimchs)]
                                
        return lst

    def valnames2labels(self, valnames):
        """
        Take Head before '@'. 'Iis_2R@20' => 'Iis_2R' 
        """
        return [v.split('@')[0] for v in valnames]

    def label2valname(self, label):
        """
        label to valname 'Iis_2R' => 'Iis_2R@20'
        """
        labels = self.valnames2labels(self.eg.valnames)
        return self.eg.valnames[labels.index(label)]

    def get_range(self, timedata, rdata):
        tmin = min(timedata) if self.tmin is None else self.tmin
        tmax = max(timedata) if self.tmax is None else self.tmax
        rmin = min(rdata) if self.rmin is None else self.rmin
        rmax = max(rdata) if self.rmax is None else self.rmax
        return tmin, tmax, rmin, rmax

    def vname_sorter(self, vnames):
        def remove_elements(src, removes):
            return [ele for ele in src if ele not in removes]
            
        vnames_src  = vnames[:]
        vnames_temp = []
        for name in vnames_src:
            if name[0:2] in ['Te']:
                vnames_temp.append(name)
                
        vnames_src = remove_elements(vnames_src, vnames_temp)
        for name in vnames_src:
            if name in ['ne_original']:
                vnames_temp.append(name)
                
        vnames_src = remove_elements(vnames_src, vnames_temp)
        for name in vnames_src:
            if name[0:2] in ['ne']:
                vnames_temp.append(name)
                
        vnames_src = remove_elements(vnames_src, vnames_temp)
        return vnames_temp + vnames_src

    def dataset(self, **kwargs):
        """
        return list of dataset for varx, vary, varz, index and option.
        kwargs:
            option: option string for trace
            varx: varx set within trace dialog
            vary: vary set within trace dialog
            varz: varz set within trace dialog
        dataset := [title, x, xe, y, ye, z, ze]
        x = [x1, x2, x3, ... , xn]
        y = [y1, y2, y3, ... , ym]
        z = [[z(x1, y1), z(x2, y1), z(x3, y1), ... , z(xn, y1)],
             [z(x1, y2), z(x2, y2), z(x3, y2), ... , z(xn, y2)],
             [z(x1, y3), z(x2, y3), z(x3, y3), ... , z(xn, y3)],
             ......
             [z(x1, ym), z(x2, ym), z(x3, ym), ... , z(xn, ym)]]
        len(z) = len(y)
        len(z[i]) = len(x)
        """
        self.load(**kwargs)

        egidx = None
        if self.actual_name is not None :
            # lhdmse1_shear, lhdmse2_shear
            # dytrans_ece
            # ece_xi
            # ece_fast_calThom
            # lhdcxs7_T_V
            # thomson_fit
            # tsmap_calib
            # DivIis_tor_add
            # nb4_Ib_3D_5-O_LOS
            # SDLloopDelPhi
            nofile = ('%s@%s No file' % (self.diagname, self.dataid), [],[],[],[],[],[])
            
            if not self.eg:
                return nofile
                
            self.refresh_index(kwargs)   # add 2023.7.21 yoshida
            ptype, varx = kaisekiInterFace.KeywordGet(kwargs, ('type', 'varx'))
            if ptype == 'xyz':
                return self.map_xyz(egidx, **kwargs)
            else:
                if varx.casefold() == 'time'.casefold():
                    return self.time_trace(egidx, **kwargs)
                    
                if re.compile(eg2dInterFace.pattern2).search(self.actual_name) :
                    # dytrans_ece
                    if varx in ['R', 'reff']:
                        return self.r_profile(egidx, **kwargs)
                    else:
                        return self.relation(**kwargs)
                        
                elif ( re.compile(eg2dInterFace.pattern8).search(self.actual_name) or
                       re.compile(eg2dInterFace.pattern10).search(self.actual_name) ):
                    # DivIis_tor_add
                    # mwrm_comb_envelopes_ccf
                    return self.ch_profile(**kwargs)
                    
                elif re.compile(eg2dInterFace.pattern9).search(self.actual_name) :
                    # DivIis_tor_iso
                    return self.prb_profile(**kwargs)
                    
                elif re.compile(eg2dInterFace.pattern13).search(self.actual_name) :
                    # SDLloopDelPhi
                    return self.theta_profile(**kwargs)
                    
                else :
                    # lhdmse1_shear, lhdmse2_shear
                    # ece_xi
                    # ece_fast_calThom
                    # lhdcxs7_T_V
                    # thomson_fit
                    # tsmap_calib
                    # nb4_Ib_3D_5-O_LOS
                    # nb4_Ib_3D_5-O_LOSint
                    return self.r_profile(egidx, **kwargs)
                    
        else :
            # nb4_Ib_3D_lhdcxsLOS
            # nb4_Ib_3D_lhdcxs7LOSint
            title = '%s@%s' % (self.diagname, self.dataid)
            if re.compile(eg2dInterFace.pattern14).search(self.diagname) :
                # nb4_Ib_3D_lhdcxsLOS
                if ( 'nb4_Ib_3D_lhdcxsLOS_grp1' in self.list_actual_name ) :
                    egidx = self.list_actual_name.index('nb4_Ib_3D_lhdcxsLOS_grp1')
            else:
                # nb4_Ib_3D_lhdcxs7LOSint
                if ( 'nb4_Ib_3D_lhdcxsLOSint_grp1' in self.list_actual_name ) :
                    egidx = self.list_actual_name.index('nb4_Ib_3D_lhdcxsLOSint_grp1')
                    
            if egidx is not None :
                if not self.list_eg[egidx] :
                    return [title + '_No data', [], [], [], [], [], []]
                else :
                    ptype = kwargs.get('type', None)
                    if ptype == 'xyz':
                        return self.map_xyz(egidx, **kwargs)
                        
                    else :
                        varx = kwargs.get('varx')
                        if varx.casefold() == 'time'.casefold():
                            return self.time_trace(egidx, **kwargs)
                        else:
                            return self.r_profile(egidx, **kwargs)
                            
            else :
                return [title + '_No data', [], [], [], [], [], []]
                

    def map_xyz(self, egidx, **kwargs):
        varx, vary, varz, index = kaisekiInterFace.KeywordGet(kwargs, ('varx', 'vary', 'varz', 'index'))
        xdata, xedata, ydata, yedata, zdata, zedata = [], [], [], [], [], []

        if egidx is None :
            vidx_H        = -1 # dytrans_ece
            vidx_D        = -1 # dytrans_ece
            vidx_reff     = -1
            vidx_reffa99  = -1
            vidx_reffa999 = -1
            idxs          = []
            reffs         = []
            reffa99s      = []
            
            if re.compile(eg2dInterFace.pattern11).search(self.actual_name) :
                # nb4_Ib_3D_5-O_LOS
                idx_x    = self.eg.vidx("x")
                idx_y    = self.eg.vidx("y")
                idx_reff = self.eg.vidx("reff")
                idx_los  = int(index.split('_')[0])
                at       = self.loslist[idx_los]
                if varz not in self.additionalnames:
                    vidx = self.eg.vidx(varz)
                    unit = self.eg.valunits[vidx]
                    title = '%s_%s(%s)_%glos' % (self.diagname, varz, unit, at)
                else:
                    title = '%s_%s' % (self.diagname,varz)
                    
            elif re.compile(eg2dInterFace.pattern12).search(self.actual_name) :
                # nb4_Ib_3D_5-O_LOSint
                idx_reff = self.eg.vidx('reff')
                if varz not in self.additionalnames:
                    vidx = self.eg.vidx(varz)
                    unit = self.eg.valunits[vidx]
                    title = '%s_%s(%s)' % (self.diagname, varz, unit)
                else:
                    title = '%s_%s' % (self.diagname,varz)
                    
            elif ( re.compile(eg2dInterFace.pattern1).search(self.actual_name) or
                   re.compile(eg2dInterFace.pattern3).search(self.actual_name) or
                   re.compile(eg2dInterFace.pattern5).search(self.actual_name) or
                   re.compile(eg2dInterFace.pattern6).search(self.actual_name) or
                   re.compile(eg2dInterFace.pattern7).search(self.actual_name) or
                   re.compile(eg2dInterFace.pattern8).search(self.actual_name) or
                   re.compile(eg2dInterFace.pattern9).search(self.actual_name) or
                   re.compile(eg2dInterFace.pattern10).search(self.actual_name) ):
                # lhdmse1_shear, lhdmse2_shear
                # ece_xi
                # lhdcxs7_T_V
                # thomson_fit
                # tsmap_calib
                # DivIis_tor_add
                # DivIis_tor_iso
                # mwrm_comb_envelopes_ccf
                if re.compile(eg2dInterFace.pattern9).search(self.actual_name) :
                    # DivIis_tor_iso : label to valname 'Iis_2R' => 'Iis_2R@20'
                    vidx = self.eg.vidx( self.label2valname(varz) )
                else:
                    # lhdmse1_shear, lhdmse2_shear
                    # ece_xi
                    # lhdcxs7_T_V
                    # thomson_fit
                    # tsmap_calib
                    # DivIis_tor_add
                    # mwrm_comb_envelopes_ccf
                    vidx = self.eg.vidx(varz)
                    
                unit = self.eg.valunits[vidx]
                title = '%s_%s(%s)' % (self.diagname, varz, unit)
            else:
                # dytrans_ece
                # ece_fast_calThom
                # SDLloopDelPhi
                if varz not in self.additionalnames:
                    vidx = self.eg.vidx(varz)
                    unit = self.eg.valunits[vidx]
                    title = '%s_%s(%s)' % (self.diagname, varz, unit)
                else:
                    if ( re.compile(eg2dInterFace.pattern4).search(self.actual_name) or
                         re.compile(eg2dInterFace.pattern13).search(self.actual_name) ):
                        # ece_fast_calThom
                        # SDLloopDelPhi
                        title = None
                    else:
                        title = '%s_%s' % (self.diagname, varz)
                        
            if ( re.compile(eg2dInterFace.pattern2).search(self.actual_name) or
                 re.compile(eg2dInterFace.pattern3).search(self.actual_name) or
                 re.compile(eg2dInterFace.pattern4).search(self.actual_name) or
                 re.compile(eg2dInterFace.pattern5).search(self.actual_name) or
                 re.compile(eg2dInterFace.pattern8).search(self.actual_name) or
                 re.compile(eg2dInterFace.pattern9).search(self.actual_name) or
                 re.compile(eg2dInterFace.pattern10).search(self.actual_name) or
                 re.compile(eg2dInterFace.pattern13).search(self.actual_name) ):
                # dytrans_ece
                # ece_xi
                # ece_fast_calThom
                # lhdcxs7_T_V
                # DivIis_tor_add
                # DivIis_tor_iso
                # mwrm_comb_envelopes_ccf
                # SDLloopDelPhi
                pass
            else:
                # lhdmse1_shear, lhdmse2_shear
                # thomson_fit
                # tsmap_calib
                vidx_reff = self.eg.vidx('reff')
                if re.compile(eg2dInterFace.pattern6).search(self.actual_name) :
                    # thomson_fit
                    pass
                elif re.compile(eg2dInterFace.pattern1).search(self.actual_name) :
                    # lhdmse1_shear, lhdmse2_shear
                    vidx_reffa99 = self.eg.vidx('reff_a99')
                elif re.compile(eg2dInterFace.pattern7).search(self.actual_name) :
                    # tsmap_calib
                    vidx_reffa99 = self.eg.vidx('reff/a99')
                    vidx_reffa999 = self.eg.vidx('reff/a999')
                    
            if ( re.compile(eg2dInterFace.pattern9).search(self.actual_name) or
                 re.compile(eg2dInterFace.pattern10).search(self.actual_name) ):
                # DivIis_tor_iso
                # mwrm_comb_envelopes_ccf
                timedata = self.eg.dims(0)
            else:
                timedata = self.timelist
                
            if re.compile(eg2dInterFace.pattern11).search(self.actual_name) :
                # nb4_Ib_3D_5-O_LOS
                pdata = self.pathlist
                
            elif re.compile(eg2dInterFace.pattern12).search(self.actual_name) :
                # nb4_Ib_3D_5-O_LOSint
                losdata = self.eg.dims(0)
                
            else:
                rdata = self.eg.dims(1) # If 'DivIis_tor_add' or 'DivIis_tor_iso' or 'mwrm_comb_envelopes_ccf', this is the channel data.
                
            if re.compile(eg2dInterFace.pattern3).search(self.actual_name) :
                # ece_xi
                # setup tmin, tmax, rmin, rmax
                tmin, tmax, rmin, rmax = self.get_range(timedata, rdata)
                # limit time and r rage
                timedata_ranged = [(tidx, tm) for tidx, tm in enumerate(timedata) if (tm >= tmin) and (tm <= tmax)]
                rdata_ranged = [(ridx, r) for ridx, r in enumerate(rdata) if (r >= rmin) and (r <= rmax)]
                sz = len(timedata_ranged) * len(rdata_ranged)
                xdata = numpy.zeros(sz)
                ydata = numpy.zeros(sz)
                zdata = numpy.zeros(sz)
                cidx = 0
                for tidx, tm in timedata_ranged:
                    idxs = self.timeindices[tidx] 
                    for ridx, r in rdata_ranged:
                        if varx.casefold() == 'time'.casefold():
                            xdata[cidx] = tm
                            ydata[cidx] = r
                        else:
                            xdata[cidx] = r
                            ydata[cidx] = tm
                            
                        vs = [self.eg.vdata(vidx, ii, ridx) for ii in idxs] # average
                        zdata[cidx] = sum(vs)/len(vs)
                        cidx += 1
                        
            else:
                # lhdmse1_shear, lhdmse2_shear
                # dytrans_ece
                # ece_fast_calThom
                # lhdcxs7_T_V
                # thomson_fit
                # tsmap_calib
                # DivIis_tor_add
                # DivIis_tor_iso
                # mwrm_comb_envelopes_ccf
                # nb4_Ib_3D_5-O_LOS
                # nb4_Ib_3D_5-O_LOSint
                # SDLloopDelPhi
                for tidx, tm in enumerate(timedata):
                    if re.compile(eg2dInterFace.pattern1).search(self.actual_name) :
                        # lhdmse1_shear, lhdmse2_shear
                        if tidx==0:
                            # skip vacuum
                            continue
                        else:
                            reffs    = self.eg.trace_of_idx(1, tidx, vidx_reff)
                            reffa99s = self.eg.trace_of_idx(1, tidx, vidx_reffa99)
                            
                    elif ( re.compile(eg2dInterFace.pattern9).search(self.actual_name) or
                           re.compile(eg2dInterFace.pattern10).search(self.actual_name) ):
                        # DivIis_tor_iso
                        # mwrm_comb_envelopes_ccf
                        if all([0.0 == self.eg.vdata(vidx, tidx, cidx) for cidx, ch in enumerate(rdata)]):
                            # All data is 0.
                            continue
                        else:
                            if re.compile(eg2dInterFace.pattern10).search(self.actual_name) :
                                # mwrm_comb_envelopes_ccf
                                reffs    = self.eg.trace_of(1, tidx, 'reff')
                                reffa99s = self.eg.trace_of(1, tidx, 'reff/a99')
                                
                    else:
                        # dytrans_ece
                        # ece_fast_calThom
                        # lhdcxs7_T_V
                        # thomson_fit
                        # tsmap_calib
                        # DivIis_tor_add
                        # nb4_Ib_3D_5-O_LOS
                        # nb4_Ib_3D_5-O_LOSint
                        # SDLloopDelPhi
                        pass
                        
                    if ( re.compile(eg2dInterFace.pattern9).search(self.actual_name) or
                         re.compile(eg2dInterFace.pattern10).search(self.actual_name) or
                         re.compile(eg2dInterFace.pattern11).search(self.actual_name) ):
                        # DivIis_tor_iso
                        # mwrm_comb_envelopes_ccf
                        # nb4_Ib_3D_5-O_LOS
                        pass
                    else:
                        idxs = self.timeindices[tidx]
                        
                    if re.compile(eg2dInterFace.pattern11).search(self.actual_name) :
                        # nb4_Ib_3D_5-O_LOS
                        for pidx, p in enumerate(pdata):
                            if varz not in self.additionalnames:
                                vs   = self.eg.vdata(vidx,     idx_los, tidx, pidx)
                                x    = self.eg.vdata(idx_x,    idx_los, tidx, pidx)
                                y    = self.eg.vdata(idx_y,    idx_los, tidx, pidx)
                                reff = self.eg.vdata(idx_reff, idx_los, tidx, pidx)
                                R    = math.sqrt(x**2+y**2)
                                zdata.append(vs)
                            else:
                                zdata.append(0.0)
                                
                            if varx == 'Time':
                                xdata.append(tm)
                                if vary == 'path':
                                    ydata.append(p)
                                elif vary == 'R':
                                    ydata.append(R)
                                else:
                                    ydata.append(reff)
                            else:
                                ydata.append(tm)
                                if varx == 'path':
                                    xdata.append(p)
                                elif varx == 'R':
                                    xdata.append(R)
                                else:
                                    xdata.append(reff)
                                    
                    elif re.compile(eg2dInterFace.pattern12).search(self.actual_name) :
                        # nb4_Ib_3D_5-O_LOSint
                        for ilidx, ilos in enumerate(losdata):
                            if varz not in self.additionalnames:
                                vs    = [self.eg.vdata(vidx,     ilidx, ii) for ii in idxs]
                                reffs = [self.eg.vdata(idx_reff, ilidx, ii) for ii in idxs]
                                reff  = sum(reffs) / len(reffs)
                                zdata.append(sum(vs) / len(vs))
                            else:
                                zdata.append(0.0)
                                reff = 0.0
                            if varx == 'Time':
                                xdata.append(tm)
                                if vary == 'ilosvec':
                                    ydata.append(ilos)
                                else:
                                    ydata.append(reff)
                            else:
                                ydata.append(tm)
                                if varx == 'ilosvec':
                                    xdata.append(ilos)
                                else:
                                    xdata.append(reff)
                                    
                    else :
                        for ridx, r in enumerate(rdata):
                            if re.compile(eg2dInterFace.pattern9).search(self.actual_name) :
                                # DivIis_tor_iso
                                vz = self.eg.vdata(vidx, tidx, ridx)
                                zdata.append(vz)
                                if varx.casefold() == 'time'.casefold():
                                    xdata.append(tm)
                                    ydata.append(r)     # this is the channel data.
                                else:
                                    xdata.append(r)     # this is the channel data.
                                    ydata.append(tm)
                                    
                            elif re.compile(eg2dInterFace.pattern10).search(self.actual_name) :
                                # mwrm_comb_envelopes_ccf
                                if varx.casefold() == 'time'.casefold():
                                    x = tm
                                    y = reffs[ridx] if vary == 'reff' else reffa99s[ridx]
                                    if y == 0.0:
                                        z = numpy.nan
                                    else:
                                        z = self.eg.vdata(vidx, tidx, ridx)
                                else:
                                    y = tm
                                    x = reffs[ridx] if varx == 'reff' else reffa99s[ridx]
                                    if x == 0.0:
                                        z = numpy.nan
                                    else:
                                        z = self.eg.vdata(vidx, tidx, ridx)
                                        
                                xdata.append(x)
                                ydata.append(y)
                                zdata.append(z)
                                
                            elif ( re.compile(eg2dInterFace.pattern5).search(self.actual_name) or
                                   re.compile(eg2dInterFace.pattern8).search(self.actual_name) ):
                                # lhdcxs7_T_V
                                # DivIis_tor_add
                                vs = [self.eg.vdata(vidx, ii, ridx) for ii in idxs] # average
                                zdata.append(sum(vs)/len(vs))
                                if varx.casefold() == 'time'.casefold():
                                    xdata.append(tm)
                                    ydata.append(r)     # If 'DivIis_tor_add', this is the channel data.
                                else:
                                    xdata.append(r)     # If 'DivIis_tor_add', this is the channel data.
                                    ydata.append(tm)
                                    
                            elif ( re.compile(eg2dInterFace.pattern4).search(self.actual_name) or
                                   re.compile(eg2dInterFace.pattern13).search(self.actual_name) ) :
                                # ece_fast_calThom
                                # SDLloopDelPhi
                                if re.compile(eg2dInterFace.pattern13).search(self.actual_name) :
                                    # SDLloopDelPhi
                                    if int(r) in self.theta_ignores:
                                        continue
                                        
                                if varz not in self.additionalnames:
                                    vs = [self.eg.vdata(vidx, ii, ridx) for ii in idxs] # average
                                    zdata.append(sum(vs)/len(vs))
                                    if varx.casefold() == 'time'.casefold():
                                        xdata.append(tm)
                                        ydata.append(r)
                                    else:
                                        ydata.append(tm)
                                        xdata.append(r)
                                        
                                else:
                                    pass
                                    
                            elif re.compile(eg2dInterFace.pattern6).search(self.actual_name) :
                                # thomson_fit
                                reffs = [self.eg.vdata(vidx_reff, ii, ridx) for ii in idxs] # average
                                reff  = sum(reffs)/len(reffs)
                                vs = [self.eg.vdata(vidx, ii, ridx) for ii in idxs] # average
                                zdata.append(sum(vs)/len(vs))
                                if varx.casefold() == 'time'.casefold():
                                    xdata.append(tm)
                                    if vary == 'reff':
                                        ydata.append(reff)
                                    else:
                                        ydata.append(r)
                                else:
                                    ydata.append(tm)
                                    if varx == 'reff':
                                        xdata.append(reff)
                                    else:
                                        xdata.append(r)
                                        
                            elif re.compile(eg2dInterFace.pattern7).search(self.actual_name) :
                                # tsmap_calib
                                reffs     = [self.eg.vdata(vidx_reff,     ii, ridx) for ii in idxs] # average
                                reffa99s  = [self.eg.vdata(vidx_reffa99,  ii, ridx) for ii in idxs] # average
                                reffa999s = [self.eg.vdata(vidx_reffa999, ii, ridx) for ii in idxs] # average
                                reff      = sum(reffs)/len(reffs)
                                reffa99   = sum(reffa99s)/len(reffa99s)
                                reffa999  = sum(reffa999s)/len(reffa999s)
                                vs = [self.eg.vdata(vidx, ii, ridx) for ii in idxs] # average
                                zdata.append(sum(vs)/len(vs))
                                if varx.casefold() == 'time'.casefold():
                                    xdata.append(tm)
                                    if vary == 'reff':
                                        ydata.append(reff)
                                    elif vary == 'reff/a99':
                                        ydata.append(reffa99)
                                    elif vary == 'reff/a999':
                                        ydata.append(reffa999)
                                    else:
                                        ydata.append(r)
                                else:
                                    ydata.append(tm)
                                    if varx == 'reff':
                                        xdata.append(reff)
                                    elif varx == 'reff/a99':
                                        xdata.append(reffa99)
                                    elif varx == 'reff/a999':
                                        xdata.append(reffa999)
                                    else:
                                        xdata.append(r)
                                        
                            elif re.compile(eg2dInterFace.pattern1).search(self.actual_name) :
                                # lhdmse1_shear, lhdmse2_shear
                                vs       = [self.eg.vdata(vidx, ii, ridx) for ii in idxs] # average
                                zdata.append(sum(vs)/len(vs))
                                if varx.casefold() == 'time'.casefold():
                                    xdata.append(tm)
                                    if vary == 'reff': 
                                        ydata.append(reffs[ridx])
                                    elif vary == 'reff/a99': 
                                        ydata.append(reffa99s[ridx])
                                    else:
                                        ydata.append(r)
                                else:
                                    ydata.append(tm)
                                    if varx == 'reff': 
                                        xdata.append(reffs[ridx])
                                    elif varx == 'reff/a99': 
                                        xdata.append(reffa99s[ridx])
                                    else:
                                        xdata.append(r)
                                        
                            elif re.compile(eg2dInterFace.pattern2).search(self.actual_name) :
                                # dytrans_ece
                                if varx.casefold() == 'time'.casefold():
                                    xdata.append(tm)
                                    ydata.append(r)
                                else:
                                    xdata.append(r)
                                    ydata.append(tm)
                                    
                                if varz not in self.additionalnames:
                                    vs = [self.eg.vdata(vidx, ii, ridx) for ii in idxs] # average
                                    zdata.append(sum(vs)/len(vs))
                                else:
                                    data_H = [self.eg.vdata(vidx_H, ii, ridx) for ii in idxs]
                                    data_D = [self.eg.vdata(vidx_D, ii, ridx) for ii in idxs]
                                    hhd = [vh/(vh+vd) for vh, vd in zip(data_H, data_D) if (vh+vd)!=0.0]
                                    dhd = [vd/(vh+vd) for vh, vd in zip(data_H, data_D) if (vh+vd)!=0.0]
                                    if varz == 'D/(H+D)':
                                        sz = len(dhd)
                                        if sz != 0:
                                            zdata.append(sum(dhd)/sz)
                                        else:
                                            zdata.append(0.0)
                                    else:
                                        sz = len(hhd)
                                        if sz != 0:
                                            zdata.append(sum(hhd)/sz)
                                        else:
                                            zdata.append(0.0)
                                            
                            else:
                                pass
        else :
            # nb4_Ib_3D_lhdcxsLOS
            # nb4_Ib_3D_lhdcxs7LOSint
            if re.compile(eg2dInterFace.pattern14).search(self.diagname) :
                # nb4_Ib_3D_lhdcxsLOS
                idx_x    = self.list_eg[egidx].vidx("x")
                idx_y    = self.list_eg[egidx].vidx("y")
                idx_reff = self.list_eg[egidx].vidx("reff")
                idx = int(index.split('_')[0])
                if idx >= len(self.loslist):
                    idx = len(self.loslist) - 1
                    
                at = self.loslist[idx]
                if at in self.loslist1:
                    idx_los = self.loslist1.index(at)
                    self.eg = self.list_eg[egidx]
                else:
                    idx_los = self.loslist3.index(at)
                    if ( 'nb4_Ib_3D_lhdcxsLOS_grp3' in self.list_actual_name ) :
                        egidx2 = self.list_actual_name.index('nb4_Ib_3D_lhdcxsLOS_grp3')
                        self.eg = self.list_eg[egidx2]
                        
                    else:
                        print("eeeeee")
                        
                if varz not in self.additionalnames:
                    vidx = self.eg.vidx(varz)
                    unit = self.eg.valunits[vidx]
                    title = '%s_%s(%s)_%glos' % (self.diagname, varz, unit, at)
                else:
                    title = '%s_%s' % (self.diagname,varz)
                    
                timedata = self.timelist
                pdata = self.pathlist
                for i, tm in enumerate(timedata):
                    for j, p in enumerate(pdata):
                        if varz not in self.additionalnames:
                            vs = self.eg.vdata(vidx, idx_los, i, j)
                            x = self.eg.vdata(idx_x, idx_los, i, j)
                            y = self.eg.vdata(idx_y, idx_los, i, j)
                            reff = self.eg.vdata(idx_reff, idx_los, i, j)
                            R = math.sqrt(x**2+y**2)
                            zdata.append(vs)
                        else:
                            zdata.append(0.0)
                            
                        if varx == 'Time':
                            xdata.append(tm)
                            if vary == 'path':
                                ydata.append(p)
                            elif vary == 'R':
                                ydata.append(R)
                            else:
                                ydata.append(reff)
                                
                        else:
                            ydata.append(tm)
                            if varx == 'path':
                                xdata.append(p)
                            elif varx == 'R':
                                xdata.append(R)
                            else:
                                xdata.append(reff)
                                
            else:
                # nb4_Ib_3D_lhdcxs7LOSint
                idx_reff = self.list_eg[egidx].vidx("reff")
                if varz not in self.additionalnames:
                    vidx = self.list_eg[egidx].vidx(varz)
                    unit = self.list_eg[egidx].valunits[vidx]
                    title = '%s_%s(%s)' % (self.diagname, varz, unit)
                else:
                    title = '%s_%s' % (self.diagname,varz)
                    
                timedata = self.timelist
                los1 = self.loslist1
                los3 = self.loslist3
                los  = sorted(los1+los3)
                if ( 'nb4_Ib_3D_lhdcxsLOSint_grp3' in self.list_actual_name ) :
                    egidx2 = self.list_actual_name.index('nb4_Ib_3D_lhdcxsLOSint_grp3')
                else:
                    egidx2 = None
                    
                for i, tm in enumerate(timedata):
                    idxs = self.timeindices[i]
                    for j, lidx in enumerate(los):
                        if varz not in self.additionalnames:
                            if lidx in los1:
                                jj    = los1.index(lidx)
                                vs    = [self.list_eg[egidx].vdata(vidx, jj, ii) for ii in idxs]
                                reffs = [self.list_eg[egidx].vdata(idx_reff, jj, ii) for ii in idxs]
                            else:
                                vs    = []
                                reffs = []
                                jj = los3.index(lidx)
                                if egidx2 is not None :
                                    if self.list_eg[egidx2] : 
                                        vs    = [self.list_eg[egidx2].vdata(vidx, jj, ii) for ii in idxs]
                                        reffs = [self.list_eg[egidx2].vdata(idx_reff, jj, ii) for ii in idxs]
                                        
                            if 0 < len(vs) :
                                zdata.append(sum(vs)/len(vs))
                            else:
                                zdata.append(0.0)
                                
                            if 0 < len(reffs) :
                                reff = sum(reffs)/len(reffs)
                            else:
                                reff = 0.0
                                
                        else:
                            zdata.append(0.0)
                            
                        if varx == 'Time':
                            xdata.append(tm)
                            if vary == 'ilosvec':
                                ydata.append(lidx)
                            else:
                                ydata.append(reff)
                                
                        else:
                            ydata.append(tm)
                            if varx == 'ilosvec':
                                xdata.append(lidx)
                            else:
                                xdata.append(reff)
                                
        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

    def time_trace(self, egidx, **kwargs):
        vary, index = kaisekiInterFace.KeywordGet(kwargs, ('vary', 'index'))
        xdata, xedata, ydata, yedata, zdata, zedata = [], [], [], [], [], []
        tmin = None
        tmax = None

        if egidx is None :
            idx = int(index.split('_')[0])
            if re.compile(eg2dInterFace.pattern12).search(self.actual_name) :
                # nb4_Ib_3D_5-O_LOSint
                at = self.eg.dims(0)[idx]
            else :
                at = self.eg.dims(1)[idx]
                
            if ( re.compile(eg2dInterFace.pattern9).search(self.actual_name) or
                 re.compile(eg2dInterFace.pattern10).search(self.actual_name) ):
                # DivIis_tor_iso
                # mwrm_comb_envelopes_ccf
                xdata = self.eg.dims(0)
            else:
                xdata = self.timelist
                
            if re.compile(eg2dInterFace.pattern11).search(self.actual_name) :
                # nb4_Ib_3D_5-O_LOS
                idx_los = self.los
                at_los  = self.loslist[idx_los]
                at_path = self.pathlist[idx]
                if vary not in self.additionalnames:
                    vidx  = self.eg.vidx(vary)
                    unit  = self.eg.valunits[vidx]
                    title = '%s@%s_%s(%s)_%glos_%gm' % (self.diagname, self.dataid, vary, unit, at_los, at_path)
                    ydata = [self.eg.vdata(vidx, idx_los, i, idx) for i in range(0,len(self.timelist))]
                else:
                    title = '%s_%s' % (self.diagname, vary)
                    ydata = [0.0]*len(self.timelist)
                    
            elif re.compile(eg2dInterFace.pattern12).search(self.actual_name) :
                # nb4_Ib_3D_5-O_LOSint
                if vary not in self.additionalnames:
                    vidx = self.eg.vidx(vary)
                    unit = self.eg.valunits[vidx]
                    title = '%s@%s_%s(%s)_%gchannel' % (self.diagname, self.dataid, vary, unit, at)
                    trace = self.eg.trace_of_idx(1,idx,vidx)
                    ydata = [kaisekiInterFace.AveragePartOfList(trace, idxs) for idxs in self.timeindices]
                else:
                    title = '%s_%s' % (self.diagname, vary)
                    ydata = [0.0]*len(self.timelist)
                    
            elif ( re.compile(eg2dInterFace.pattern1).search(self.actual_name) or
                   re.compile(eg2dInterFace.pattern2).search(self.actual_name) or
                   re.compile(eg2dInterFace.pattern3).search(self.actual_name) or
                   re.compile(eg2dInterFace.pattern5).search(self.actual_name) or
                   re.compile(eg2dInterFace.pattern6).search(self.actual_name) or
                   re.compile(eg2dInterFace.pattern7).search(self.actual_name) or
                   re.compile(eg2dInterFace.pattern8).search(self.actual_name) or
                   re.compile(eg2dInterFace.pattern9).search(self.actual_name) or
                   re.compile(eg2dInterFace.pattern10).search(self.actual_name) ):
                # lhdmse1_shear, lhdmse2_shear
                # dytrans_ece
                # ece_xi
                # lhdcxs7_T_V
                # thomson_fit
                # tsmap_calib
                # DivIis_tor_add
                # DivIis_tor_iso
                # mwrm_comb_envelopes_ccf
                if re.compile(eg2dInterFace.pattern9).search(self.actual_name) :
                    # DivIis_tor_iso
                    vidx = self.eg.vidx( self.label2valname(vary) )
                else :
                    vidx = self.eg.vidx(vary)
                    
                unit = self.eg.valunits[vidx]
                if ( re.compile(eg2dInterFace.pattern9).search(self.actual_name) ):
                    # DivIis_tor_iso
                    title = '%s@%s_%s(%s)_prb%d' % (self.diagname, self.dataid, vary, unit, int(at))
                    
                elif ( re.compile(eg2dInterFace.pattern8).search(self.actual_name) or
                       re.compile(eg2dInterFace.pattern10).search(self.actual_name) ):
                    # DivIis_tor_add
                    # mwrm_comb_envelopes_ccf
                    title = '%s@%s_%s(%s)_ch%d' % (self.diagname, self.dataid, vary, unit, int(at))
                    
                else:
                    # lhdmse1_shear, lhdmse2_shear
                    # dytrans_ece
                    # lhdcxs7_T_V
                    # ece_xi
                    # thomson_fit
                    # tsmap_calib
                    title = '%s@%s_%s(%s)_%gm' % (self.diagname, self.dataid, vary, unit, at)
                    
                if ( re.compile(eg2dInterFace.pattern9).search(self.actual_name) or
                     re.compile(eg2dInterFace.pattern10).search(self.actual_name) ):
                    # DivIis_tor_iso
                    # mwrm_comb_envelopes_ccf
                    ydata = self.eg.trace_of_idx(0, idx, vidx)
                    zero_removed_xdata = [x for x, y in zip(xdata, ydata) if y != 0.0]
                    zero_removed_ydata = [y for x, y in zip(xdata, ydata) if y != 0.0]
                    xdata = AverageListWithN(zero_removed_xdata, self.ntavg)
                    ydata = AverageListWithN(zero_removed_ydata, self.ntavg)
                    
                else:
                    # lhdmse1_shear, lhdmse2_shear
                    # dytrans_ece
                    # ece_xi
                    # lhdcxs7_T_V
                    # thomson_fit
                    # tsmap_calib
                    # DivIis_tor_add
                    trace = self.eg.trace_of_idx(0, idx, vidx)
                    ydata = [kaisekiInterFace.AveragePartOfList(trace, idxs) for idxs in self.timeindices]
                    if re.compile(eg2dInterFace.pattern2).search(self.actual_name) :
                        # dytrans_ece
                        tmin = self.tmin if self.tmin is not None else self.timelist[0]
                        tmax = self.tmax if self.tmax is not None else self.timelist[-1]
                        ydata = [v for t, v in zip(xdata, ydata) if tmin <= t <= tmax]
                        xdata = [t for t in xdata if tmin <= t <= tmax]
                        
            else:
                # ece_fast_calThom
                # SDLloopDelPhi
                if vary not in self.additionalnames:
                    vidx  = self.eg.vidx(vary)
                    unit  = self.eg.valunits[vidx]
                    if re.compile(eg2dInterFace.pattern13).search(self.actual_name) :
                        # SDLloopDelPhi
                        title = '%s@%s_%s(%s)_%gdeg.' % (self.diagname, self.dataid, vary, unit, at)
                        
                    else:
                        # ece_fast_calThom
                        title = '%s@%s_%s(%s)_%gm' % (self.diagname, self.dataid, vary, unit, at)
                        
                    trace = self.eg.trace_of_idx(0, idx, vidx)
                    ydata = [kaisekiInterFace.AveragePartOfList(trace, idxs) for idxs in self.timeindices]
                else:
                    title = None
                    
            if ( re.compile(eg2dInterFace.pattern3).search(self.actual_name) or
                 re.compile(eg2dInterFace.pattern8).search(self.actual_name) or
                 re.compile(eg2dInterFace.pattern9).search(self.actual_name) ):
                # ece_xi
                # DivIis_tor_add
                # DivIis_tor_iso
                pass
            else:
                # lhdmse1_shear, lhdmse2_shear
                # dytrans_ece
                # ece_fast_calThom
                # lhdcxs7_T_V
                # thomson_fit
                # tsmap_calib
                # nb4_Ib_3D_5-O_LOS
                # nb4_Ib_3D_5-O_LOSint
                # SDLloopDelPhi
                vename = self.dic_verr.get(vary, None)
                if vename is not None:
                    veidx  = self.eg.vidx(vename)
                    if re.compile(eg2dInterFace.pattern11).search(self.actual_name) :
                        # nb4_Ib_3D_5-O_LOS
                        yedata = [self.eg.vdata(veidx, idx_los, i, idx) for i in range(0,len(self.timelist))]
                    else :
                        if re.compile(eg2dInterFace.pattern12).search(self.actual_name) :
                            # nb4_Ib_3D_5-O_LOSint
                            etrace = self.eg.trace_of_idx(1, idx, veidx)
                        else :
                            etrace = self.eg.trace_of_idx(0, idx, veidx)
                            
                        data   = [(kaisekiInterFace.AveragePartOfList(etrace, idxs), len(idxs)) for idxs in self.timeindices]
                        yedata = [v / math.sqrt(n) for v, n in data]
                        if re.compile(eg2dInterFace.pattern2).search(self.actual_name) :
                            # dytrans_ece
                            yedata = [v for t, v in zip(xdata, yedata) if tmin <= t <= tmax]
                else:
                    if ( re.compile(eg2dInterFace.pattern1).search(self.actual_name) or
                         re.compile(eg2dInterFace.pattern2).search(self.actual_name) or
                         re.compile(eg2dInterFace.pattern4).search(self.actual_name) or
                         re.compile(eg2dInterFace.pattern5).search(self.actual_name) ):
                        # lhdmse1_shear, lhdmse2_shear
                        # dytrans_ece
                        # ece_fast_calThom
                        # lhdcxs7_T_V
                        pass
                    else:
                        # thomson_fit
                        # tsmap_calib
                        # nb4_Ib_3D_5-O_LOS
                        # nb4_Ib_3D_5-O_LOSint
                        if re.compile(eg2dInterFace.pattern7).search(self.actual_name) :
                            # tsmap_calib
                            if vary in self.additionalnames:
                                yedata = [0.0] * len(self.timelist)
                                
                        else:
                            # thomson_fit
                            # nb4_Ib_3D_5-O_LOS
                            # nb4_Ib_3D_5-O_LOSint
                            yedata = [0.0]*len(self.timelist)
                            
        else :
            # nb4_Ib_3D_lhdcxsLOS
            # nb4_Ib_3D_lhdcxs7LOSint
            idx_path = int(index.split('_')[0])
            if re.compile(eg2dInterFace.pattern14).search(self.diagname) :
                # nb4_Ib_3D_lhdcxsLOS
                idx      = self.los
                at_los   = self.loslist[idx]
                if idx_path >= len(self.pathlist):
                    idx_path = len(self.pathlist) - 1
                    
                at_path  = self.pathlist[idx_path]
                if at_los in self.loslist1:
                    idx_los = self.loslist1.index(at_los)
                    self.eg = self.list_eg[egidx]
                else:
                    idx_los = self.loslist3.index(at_los)
                    if ( 'nb4_Ib_3D_lhdcxsLOS_grp3' in self.list_actual_name ) :
                        egidx2 = self.list_actual_name.index('nb4_Ib_3D_lhdcxsLOS_grp3')
                        self.eg = self.list_eg[egidx2]
                        
                    else:
                        print("eeeeeeee")
                        
                xdata = self.timelist
                if vary not in self.additionalnames:
                    vidx = self.eg.vidx(vary)
                    unit = self.eg.valunits[vidx]
                    title = '%s@%s_%s(%s)_%glos_%gm' % (self.diagname, self.dataid, vary, unit, at_los, at_path)
                    ydata = [self.eg.vdata(vidx,idx_los, i, idx_path) for i in range(0,len(self.timelist))]
                else:
                    title = '%s_%s' % (self.diagname, vary)
                    ydata = [0.0]*len(self.timelist)
                    
                vename = self.verr.get(vary,None)
                if vename is not None:
                    veidx = self.eg.vidx(vename)
                    yedata = [self.eg.vdata(veidx,idx_los, i, idx_path) for i in range(0,len(self.timelist))]
                else:
                    yedata = [0.0]*len(self.timelist)
                    
            else:
                # nb4_Ib_3D_lhdcxs7LOSint
                if idx_path >= len(self.loslist):
                    idx_path = len(self.loslist) - 1
                    
                at = self.loslist[idx_path]
                if at in self.loslist1:
                    idx = self.loslist1.index(at)
                    self.eg = self.list_eg[egidx]
                else:
                    idx = self.loslist3.index(at)
                    if ( 'nb4_Ib_3D_lhdcxsLOSint_grp3' in self.list_actual_name ) :
                        egidx2 = self.list_actual_name.index('nb4_Ib_3D_lhdcxsLOSint_grp3')
                        self.eg = self.list_eg[egidx2]
                    else:
                        print("eeeeeeee")
                        
                xdata = self.timelist
                if vary not in self.additionalnames:
                    vidx = self.eg.vidx(vary)
                    unit = self.eg.valunits[vidx]
                    title = '%s@%s_%s(%s)_%gchannel' % (self.diagname, self.dataid, vary, unit, at)
                    trace = self.eg.trace_of_idx(1,idx,vidx)
                    ydata = [kaisekiInterFace.AveragePartOfList(trace, idxs) for idxs in self.timeindices]
                else:
                    title = '%s_%s' % (self.diagname, vary)
                    ydata = [0.0]*len(self.timelist)
                    
                vename = self.verr.get(vary,None)
                if vename is not None:
                    veidx = self.eg.vidx(vename)
                    etrace = self.eg.trace_of_idx(1, idx, veidx)
                    data = [(kaisekiInterFace.AveragePartOfList(etrace, idxs), len(idxs)) for idxs in self.timeindices]
                    yedata = [v/math.sqrt(n) for v, n in data]
                else:
                    yedata = [0.0]*len(self.timelist)
                    
        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

    def r_profile(self, egidx, **kwargs):
        varx, vary, index = kaisekiInterFace.KeywordGet(kwargs, ('varx', 'vary', 'index'))
        xdata, xedata, ydata, yedata, zdata, zedata = [], [], [], [], [], []

        if egidx is None :
            vidx_reff     = None
            vidx_reffa99  = None
            vidx_reffa999 = None
            idx = int(index.split('_')[0])
            if re.compile(eg2dInterFace.pattern11).search(self.actual_name) :
                # nb4_Ib_3D_5-O_LOS
                idx_los   = self.los
                at_los    = self.loslist[idx_los]
                vidx_x    = self.eg.vidx('x')
                vidx_y    = self.eg.vidx('y')
                vidx_reff = self.eg.vidx('reff')
                
            if re.compile(eg2dInterFace.pattern3).search(self.actual_name) :
                # ece_xi
                pass
            else:
                # lhdmse1_shear, lhdmse2_shear
                # dytrans_ece
                # ece_fast_calThom
                # lhdcxs7_T_V
                # thomson_fit
                # tsmap_calib
                # nb4_Ib_3D_5-O_LOS
                # nb4_Ib_3D_5-O_LOSint
                vename = self.dic_verr.get(vary, None)
                if vename is not None:
                    veidx = self.eg.vidx(vename)
                    if re.compile(eg2dInterFace.pattern11).search(self.actual_name) :
                        # nb4_Ib_3D_5-O_LOS
                        idx_los = self.los
                        yedata = [self.eg.vdata(veidx, idx_los, idx, i) for i in range(0,len(self.pathlist))]
                    else :
                        if re.compile(eg2dInterFace.pattern12).search(self.actual_name) :
                            # nb4_Ib_3D_5-O_LOSint
                            eprofs = [self.eg.trace_of_idx(0, i, veidx) for i in self.timeindices[idx]]
                        else :
                            eprofs = [self.eg.trace_of_idx(1, i, veidx) for i in self.timeindices[idx]]
                            
                        n      = len(eprofs)
                        yedata = [v/math.sqrt(n) for v in kaisekiInterFace.AverageLists(eprofs)]
                        
                else:
                    if ( re.compile(eg2dInterFace.pattern1).search(self.actual_name)  or
                         re.compile(eg2dInterFace.pattern2).search(self.actual_name) or
                         re.compile(eg2dInterFace.pattern4).search(self.actual_name) or
                         re.compile(eg2dInterFace.pattern5).search(self.actual_name) ):
                        # lhdmse1_shear, lhdmse2_shear
                        # dytrans_ece
                        # ece_fast_calThom
                        # lhdcxs7_T_V
                        pass
                    else:
                        # thomson_fit
                        # tsmap_calib
                        # nb4_Ib_3D_5-O_LOS
                        # nb4_Ib_3D_5-O_LOSint
                        if re.compile(eg2dInterFace.pattern11).search(self.actual_name) :
                            # nb4_Ib_3D_5-O_LOS
                            yedata = [0.0]*len(self.pathlist)
                            
                        elif re.compile(eg2dInterFace.pattern12).search(self.actual_name) :
                            # nb4_Ib_3D_5-O_LOSint
                            yedata = [0.0]*len(self.eg.dims(0))
                            
                        elif ( re.compile(eg2dInterFace.pattern6).search(self.actual_name) or
                               re.compile(eg2dInterFace.pattern7).search(self.actual_name) ):
                            # thomson_fit
                            # tsmap_calib
                            if vary in self.additionalnames:
                                yedata = [0.0]*len(self.eg.dims(1))
                                
                        else:
                            pass
                            
            if ( re.compile(eg2dInterFace.pattern1).search(self.actual_name)  or
                 re.compile(eg2dInterFace.pattern6).search(self.actual_name) or
                 re.compile(eg2dInterFace.pattern7).search(self.actual_name) or
                 re.compile(eg2dInterFace.pattern12).search(self.actual_name) ) :
                # lhdmse1_shear, lhdmse2_shear
                # thomson_fit
                # tsmap_calib
                # nb4_Ib_3D_5-O_LOSint
                vidx_reff = self.eg.vidx('reff')
                if ( re.compile(eg2dInterFace.pattern6).search(self.actual_name) or
                     re.compile(eg2dInterFace.pattern12).search(self.actual_name) ) :
                    # thomson_fit
                    # nb4_Ib_3D_5-O_LOSint
                    pass
                else :
                    # lhdmse1_shear, lhdmse2_shear
                    # tsmap_calib
                    if re.compile(eg2dInterFace.pattern1).search(self.actual_name) :
                        # lhdmse1_shear, lhdmse2_shear
                        vidx_reffa99 = self.eg.vidx('reff_a99')
                    else :
                        # tsmap_calib
                        vidx_reffa99 = self.eg.vidx('reff/a99')
                        if re.compile(eg2dInterFace.pattern7).search(self.actual_name) :
                            # tsmap_calib
                            vidx_reffa999 = self.eg.vidx('reff/a999')
                            
            at = self.timelist[idx]
            if ( re.compile(eg2dInterFace.pattern1).search(self.actual_name) or
                 re.compile(eg2dInterFace.pattern3).search(self.actual_name) or
                 re.compile(eg2dInterFace.pattern5).search(self.actual_name) or
                 re.compile(eg2dInterFace.pattern6).search(self.actual_name) or
                 re.compile(eg2dInterFace.pattern7).search(self.actual_name) ):
                # lhdmse1_shear, lhdmse2_shear
                # ece_xi
                # lhdcxs7_T_V
                # thomson_fit
                # tsmap_calib
                vidx = self.eg.vidx(vary)
                unit = self.eg.valunits[vidx]
                profs = [self.eg.trace_of_idx(1, i, vidx) for i in self.timeindices[idx]]
                ydata = kaisekiInterFace.AverageLists(profs)
                if re.compile(eg2dInterFace.pattern1).search(self.actual_name) :
                    # lhdmse1_shear, lhdmse2_shear
                    title = '%s@%s_%s(%s)_%gs' % (self.diagname, self.title_dataid, vary, unit, at)
                    if varx == 'reff': 
                        vidx_reff = self.eg.vidx('reff')
                        reffs = [self.eg.trace_of_idx(1, i, vidx_reff) for i in self.timeindices[idx]]
                        xdata = kaisekiInterFace.AverageLists(reffs)
                    elif varx == 'reff/a99':
                        # varsx function from 'reff_a99' to 'reff/a99.'
                        reffa99s = [self.eg.trace_of_idx(1, i, vidx_reffa99) for i in self.timeindices[idx]]
                        xdata    = kaisekiInterFace.AverageLists(reffa99s)
                    else:
                        xdata = self.eg.dims(1)
                        
                else :
                    # ece_xi
                    # lhdcxs7_T_V
                    # thomson_fit
                    # tsmap_calib
                    title = '%s@%s_%s(%s)_%gs' % (self.diagname, self.dataid, vary, unit, at)
                    if re.compile(eg2dInterFace.pattern6).search(self.actual_name) :
                        # thomson_fit
                        if varx == 'reff':
                            reffs = [self.eg.trace_of_idx(1, i, vidx_reff) for i in self.timeindices[idx]]
                            xdata = kaisekiInterFace.AverageLists(reffs)
                        else:
                            xdata = self.eg.dims(1)
                            
                    elif re.compile(eg2dInterFace.pattern7).search(self.actual_name) :
                        # tsmap_calib
                        if varx == 'reff':
                            reffs = [self.eg.trace_of_idx(1, i, vidx_reff) for i in self.timeindices[idx]]
                            xdata = kaisekiInterFace.AverageLists(reffs)
                        elif varx == 'reff/a99':
                            reffa99s = [self.eg.trace_of_idx(1, i, vidx_reffa99) for i in self.timeindices[idx]]
                            xdata = kaisekiInterFace.AverageLists(reffa99s)
                        elif varx == 'reff/a999':
                            reffa999s = [self.eg.trace_of_idx(1, i, vidx_reffa999) for i in self.timeindices[idx]]
                            xdata = kaisekiInterFace.AverageLists(reffa999s)
                        else:
                            xdata = self.eg.dims(1)
                            
                    else :
                        xdata = self.eg.dims(1)
                        
            elif re.compile(eg2dInterFace.pattern2).search(self.actual_name) :
                # dytrans_ece
                vidx      = self.eg.vidx(vary)
                vidx_reff = self.eg.vidx('reff')
                unit      = self.eg.valunits[vidx]
                title     = '%s@%s_%s(%s)_%gs' % (self.diagname, self.dataid, vary, unit, at)
                profs     = [self.eg.trace_of_idx(1, i, vidx) for i in self.timeindices[idx]]
                ydata     = kaisekiInterFace.AverageLists(profs)
                if varx == 'R':
                    xdata = self.eg.dims(1)
                else:
                    profs_reff = [self.eg.trace_of_idx(1, i, vidx_reff) for i in self.timeindices[idx]]
                    xdata = kaisekiInterFace.AverageLists(profs_reff)
                    
            else:
                # ece_fast_calThom
                # nb4_Ib_3D_5-O_LOS
                # nb4_Ib_3D_5-O_LOSint
                if vary not in self.additionalnames:
                    vidx = self.eg.vidx(vary)
                    unit = self.eg.valunits[vidx]
                    if re.compile(eg2dInterFace.pattern11).search(self.actual_name) :
                        # nb4_Ib_3D_5-O_LOS
                        title = '%s@%s_%s(%s)_%glos_%gs' % (self.diagname, self.dataid, vary, unit, at_los, at)
                        ydata = [self.eg.vdata(vidx,      idx_los, idx, i) for i in range(0,len(self.pathlist))]
                        x     = [self.eg.vdata(vidx_x,    idx_los, idx, i) for i in range(0,len(self.pathlist))]
                        y     = [self.eg.vdata(vidx_y,    idx_los, idx, i) for i in range(0,len(self.pathlist))]
                        reff  = [self.eg.vdata(vidx_reff, idx_los, idx, i) for i in range(0,len(self.pathlist))]
                        R     = [math.sqrt(xx**2+yy**2) for xx, yy in zip(x, y)]
                        if varx == 'path':
                            xdata = self.pathlist
                        elif varx == 'R':
                            xdata = R
                        else:
                            xdata = reff
                            
                    else :
                        # ece_fast_calThom
                        # nb4_Ib_3D_5-O_LOSint
                        title = '%s@%s_%s(%s)_%gs' % (self.diagname, self.dataid, vary, unit, at)
                        
                        if re.compile(eg2dInterFace.pattern12).search(self.actual_name) :
                            # nb4_Ib_3D_5-O_LOSint
                            profs = [self.eg.trace_of_idx(0, i, vidx) for i in self.timeindices[idx]]
                        else :
                            profs = [self.eg.trace_of_idx(1, i, vidx) for i in self.timeindices[idx]]
                            
                        # ece_fast_calThom
                        profs_nz = [prof for prof in profs if sum(prof) != 0.0]
                        if len(profs_nz) == 0:
                            if re.compile(eg2dInterFace.pattern12).search(self.actual_name) :
                                # nb4_Ib_3D_5-O_LOSint
                                profs_nz = [[0.0]*len(self.eg.dims(0))]
                            else :
                                profs_nz = [[0.0]*len(self.eg.dims(1))]
                                
                        ydata = kaisekiInterFace.AverageLists(profs_nz)
                        if re.compile(eg2dInterFace.pattern12).search(self.actual_name) :
                            # nb4_Ib_3D_5-O_LOSint
                            reffs = [self.eg.trace_of_idx(0, i, idx_reff) for i in self.timeindices[idx]]
                            reffs_nz = [r for r in reffs if sum(r) != 0.0]
                            if len(reffs_nz) == 0:
                                reffs_nz = [[0.0]*len(self.eg.dims(0))]
                                
                            reff = kaisekiInterFace.AverageLists(reffs_nz)
                            if varx == 'ilosvec':
                                xdata = self.eg.dims(0)
                            else:
                                xdata = reff
                                
                        else:
                            # ece_fast_calThom
                            xdata = self.eg.dims(1)
                            
                else:
                    if re.compile(eg2dInterFace.pattern4).search(self.actual_name) :
                        # ece_fast_calThom
                        pass
                    elif re.compile(eg2dInterFace.pattern11).search(self.actual_name) :
                        # nb4_Ib_3D_5-O_LOS
                        title = '%s@%s_%s_%glos_%gs' % (self.diagname, self.dataid, vary, at_los, at)
                        ydata = [0.0]*len(self.pathlist)
                        x = [self.eg.vdata(vidx_x, idx_los, idx, i) for i in range(0,len(self.pathlist))]
                        y = [self.eg.vdata(vidx_y, idx_los, idx, i) for i in range(0,len(self.pathlist))]
                        R = [math.sqrt(xx**2+yy**2) for xx, yy in zip(x, y)]
                        if varx == 'path':
                            xdata = self.pathlist
                        elif varx == 'R':
                            xdata = R
                        else:
                            xdata = reff
                            
                    else :
                        # nb4_Ib_3D_5-O_LOSint
                        title = '%s@%s_%s_%gs' % (self.diagname, self.dataid, vary, at)
                        if re.compile(eg2dInterFace.pattern12).search(self.actual_name) :
                            # nb4_Ib_3D_5-O_LOSint
                            profs_nz = [[0.0]*len(self.eg.dims(0))]
                            
                        else:
                            pass
                            
                        ydata = kaisekiInterFace.AverageLists(profs_nz)
                        if re.compile(eg2dInterFace.pattern12).search(self.actual_name) :
                            # nb4_Ib_3D_5-O_LOSint
                            reffs_nz = [[0.0]*len(self.eg.dims(0))]
                            if varx == 'ilosvec':
                                xdata = self.eg.dims(0)
                            else:
                                xdata = reff
                                
                        else:
                            pass
                            
        else :
            # nb4_Ib_3D_lhdcxsLOS
            # nb4_Ib_3D_lhdcxs7LOSint
            idx_time = int(index.split('_')[0])
            if idx_time >= len(self.timelist):
                idx_time = len(self.timelist) - 1
                
            if re.compile(eg2dInterFace.pattern14).search(self.diagname) :
                # nb4_Ib_3D_lhdcxsLOS
                idx      = self.los
                at_los   = self.loslist[idx]
                at_time  = self.timelist[idx_time]
                if at_los in self.loslist1:
                    idx_los = self.loslist1.index(at_los)
                    self.eg = self.list_eg[egidx]
                else:
                    idx_los = self.loslist3.index(at_los)
                    if ( 'nb4_Ib_3D_lhdcxsLOS_grp3' in self.list_actual_name ) :
                        egidx2 = self.list_actual_name.index('nb4_Ib_3D_lhdcxsLOS_grp3')
                        self.eg = self.list_eg[egidx2]
                        
                    else:
                        print("eeee")
                        
                vidx_x = self.eg.vidx('x')
                vidx_y = self.eg.vidx('y')
                vidx_reff = self.eg.vidx('reff')
                vename = self.verr.get(vary, None)
                if vename is not None:
                    veidx = self.eg.vidx(vename)
                    yedata = [self.eg.vdata(veidx, idx_los, idx_time, i) for i in range(0,len(self.pathlist))]
                else:
                    yedata = [0.0]*len(self.pathlist)
                    
                if vary not in self.additionalnames:
                    vidx = self.eg.vidx(vary)
                    unit = self.eg.valunits[vidx]
                    title = '%s@%s_%s(%s)_%glos_%gs' % (self.diagname, self.dataid, vary, unit, at_los, at_time)            
                    ydata = [self.eg.vdata(vidx, idx_los, idx_time, i) for i in range(0,len(self.pathlist))]
                    x = [self.eg.vdata(vidx_x, idx_los, idx_time, i) for i in range(0,len(self.pathlist))]
                    y = [self.eg.vdata(vidx_y, idx_los, idx_time, i) for i in range(0,len(self.pathlist))]
                    R = [math.sqrt(xx**2+yy**2) for xx, yy in zip(x, y)]
                    reff = [self.eg.vdata(vidx_reff, idx_los, idx_time, i) for i in range(0,len(self.pathlist))]
                    if varx == 'path':
                        xdata = self.pathlist
                    elif varx == 'R':
                        xdata = R
                    else:
                        xdata = reff
                else:
                    title = '%s@%s_%s_%glos_%gs' % (self.diagname, self.dataid, vary, at_los, at_time)
                    ydata = [0.0]*len(self.pathlist)
                    x = [self.eg.vdata(vidx_x, idx_los, idx_time, i) for i in range(0,len(self.pathlist))]
                    y = [self.eg.vdata(vidx_y, idx_los, idx_time, i) for i in range(0,len(self.pathlist))]
                    R = [math.sqrt(xx**2+yy**2) for xx, yy in zip(x, y)]
                    if varx == 'path':
                        xdata = self.pathlist
                    elif varx == 'R':
                        xdata = R
                    else:
                        xdata = reff
                        
            else :
                # nb4_Ib_3D_lhdcxs7LOSint
                if ( 'nb4_Ib_3D_lhdcxsLOSint_grp3' in self.list_actual_name ) :
                    egidx2 = self.list_actual_name.index('nb4_Ib_3D_lhdcxsLOSint_grp3')
                else:
                    egidx2 = None
                    
                vename = self.verr.get(vary, None)
                if vename is not None:
                    ve1idx  = self.list_eg[egidx].vidx(vename)
                    e1profs = [self.list_eg[egidx].trace_of_idx(0, i, ve1idx) for i in self.timeindices[idx_time]]
                    n1  = len(e1profs)
                    ye1 = [v/math.sqrt(n1) for v in kaisekiInterFace.AverageLists(e1profs)]

                    if egidx2 is not None :
                        ve3idx = self.self.list_eg[egidx2].vidx(vename)
                        e3profs = [self.self.list_eg[egidx2].trace_of_idx(0, i, ve3idx) for i in self.timeindices[idx_time]]
                        n3 = len(e3profs)
                        ye3 = [v/math.sqrt(n3) for v in kaisekiInterFace.AverageLists(e3profs)]
                    else:
                        print("eeeeeeee")
                        
                else:
                    ye1 = [0.0]*len(self.loslist1)
                    ye3 = [0.0]*len(self.loslist3)
                    
                at = self.timelist[idx_time]
                if vary not in self.additionalnames:
                    vidx    = self.list_eg[egidx].vidx(vary)
                    reffidx = self.list_eg[egidx].vidx('reff')
                    unit    = self.list_eg[egidx].valunits[vidx]
                    title   = '%s@%s_%s(%s)_%gs' % (self.diagname, self.dataid, vary, unit, at)
                    prof1s  = [self.list_eg[egidx].trace_of_idx(0, i, vidx) for i in self.timeindices[idx_time]]
                    r1s     = [self.list_eg[egidx].trace_of_idx(0, i, reffidx) for i in self.timeindices[idx_time]]
                    prof1s_nz = [prof for prof in prof1s if sum(prof) != 0.0]
                    if len(prof1s_nz) == 0:
                        vprofs1 = [[0.0]*len(self.loslist1)]
                    else:
                        vprofs1 = prof1s_nz
                        
                    y1 = kaisekiInterFace.AverageLists(vprofs1)
                    r1 = kaisekiInterFace.AverageLists(r1s)
                    x1 = self.loslist1

                    if egidx2 is not None :
                        prof3s = [self.list_eg[egidx2].trace_of_idx(0, i, vidx) for i in self.timeindices[idx_time]]
                        r3s = [self.list_eg[egidx2].trace_of_idx(0, i, reffidx) for i in self.timeindices[idx_time]]
                        prof3s_nz = [prof for prof in prof3s if sum(prof) != 0.0]
                        
                    if len(prof3s_nz) == 0:
                        vprofs3 = [[0.0]*len(self.loslist3)]
                    else:
                        vprofs3 = prof3s_nz
                        
                    y3 = kaisekiInterFace.AverageLists(vprofs3)
                    r3 = kaisekiInterFace.AverageLists(r3s)
                    x3 = self.loslist3
                else:
                    title = '%s@%s_%s_%gs' % (self.diagname, self.dataid, vary, at)
                    vprof1s = [[0.0]*len(self.loslist1)]
                    y1 = kaisekiInterFace.AverageLists(vprof1s)
                    r1 = [[0.0]*len(self.loslist1)]
                    x1 = self.eg.dims(0)
                    vprof3s = [[0.0]*len(self.loslist3)]
                    y3 = kaisekiInterFace.AverageLists(vprof3s)
                    r3 = [[0.0]*len(self.loslist1)]
                    x3 = self.eg3.dims(0)
                    
                x = x1+x3
                xs = sorted(x)
                lx = len(x)
                for i in range(0,lx):
                    xsi = xs[i]
                    if xsi in x1:
                        xidx = x1.index(xsi)
                        r = r1[xidx]
                        y = y1[xidx]
                        ye = ye1[xidx]
                    else:
                        xidx = x3.index(xsi)
                        r = r3[xidx]
                        y = y3[xidx]
                        ye = ye3[xidx]
                    ydata.append(y)
                    yedata.append(ye)
                    if varx == 'ilosvec':
                        xdata.append(xsi)
                    else:
                        xdata.append(r)
                        
        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

    def ch_profile(self, **kwargs):
        # DivIis_tor_add
        # mwrm_comb_envelopes_ccf
        varx, vary, index = kaisekiInterFace.KeywordGet(kwargs, ('varx', 'vary', 'index'))
        xdata, xedata, ydata, yedata, zdata, zedata = [], [], [], [], [], []
        idx   = int(index.split('_')[0])
        vidx  = self.eg.vidx(vary)
        unit  = self.eg.valunits[vidx]
        if idx >= len(self.timelist):
            idx = len(self.timelist) - 1
            
        at    = self.timelist[idx]
        title = '%s@%s_%s(%s)_%gs' % (self.diagname, self.dataid, vary, unit, at)
        if re.compile(eg2dInterFace.pattern8).search(self.actual_name) :
            # DivIis_tor_add
            xdata = self.eg.dims(1)
            profs = [self.eg.trace_of_idx(1, i, vidx) for i in self.timeindices[idx]]
            ydata = kaisekiInterFace.AverageLists(profs)
            
        else:
            # mwrm_comb_envelopes_ccf
            if varx == 'reff':
                xdata = self.eg.trace_of(1, idx, 'reff')
            else:
                xdata = self.eg.trace_of(1, idx, 'reff/a99')
                
            ydata = self.eg.trace_of_idx(1, idx, vidx)
            
        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

    def prb_profile(self, **kwargs):
        # DivIis_tor_iso
        varx, vary, index = kaisekiInterFace.KeywordGet(kwargs, ('varx', 'vary', 'index'))
        xdata, xedata, ydata, yedata, zdata, zedata = [], [], [], [], [], []
        idx   = int(index.split('_')[0])
        vary  = self.label2valname(vary)
        vidx  = self.eg.vidx(vary)
        unit  = self.eg.valunits[vidx]
        #at = self.timelist[idx]
        at    = self.eg.dims(0)[idx]
        title = '%s@%s_%s(%s)_%gs' % (self.diagname, self.dataid, vary, unit, at)
        xdata = self.eg.dims(1)
        ydata = self.eg.trace_of_idx(1, idx, vidx)
        
        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

    def relation(self, **kwargs):
        # dytrans_ece
        vary, vary, index = kaisekiInterFace.KeywordGet(kwargs, ('vary', 'vary', 'index'))
        xdata, xedata, ydata, yedata, zdata, zedata = [], [], [], [], [], []
        idx    = int(index.split('_')[0])
        at     = self.eg.dims(1)[idx]
        tmin   = self.tmin if self.tmin is not None else self.timelist[0]
        tmax   = self.tmax if self.tmax is not None else self.timelist[-1]
        vidx_x = self.eg.vidx(varx)
        vidx   = self.eg.vidx(vary)
        unit   = self.eg.valunits[vidx]
        title  = '%s@%s_%s(%s)_%gs' % (self.diagname, self.dataid, vary, unit, int(at))
        trace  = self.eg.trace_of_idx(0, idx, vidx)
        ydata  = [kaisekiInterFace.AveragePartOfList(trace, idxs) for idxs in self.timeindices]
        trace  = self.eg.trace_of_idx(0, idx, vidx_x)
        xdata  = [kaisekiInterFace.AveragePartOfList(trace, idxs) for idxs in self.timeindices]
        vename = self.verr.get(vary, None)
        if vename is not None:
            veidx  = self.eg.vidx(vename)
            etrace = self.eg.trace_of_idx(0, idx, veidx)
            data   = [(kaisekiInterFace.AveragePartOfList(etrace, idxs), len(idxs)) for idxs in self.timeindices]
            yedata = [v/math.sqrt(n) for v, n in data]
            yedata = [v for t, v in zip(self.timelist, yedata) if tmin <= t <= tmax]
            
        vename = self.verr.get(varx, None)
        if vename is not None:
            veidx  = self.eg.vidx(vename)
            etrace = self.eg.trace_of_idx(0, idx, veidx)
            data   = [(kaisekiInterFace.AveragePartOfList(etrace, idxs), len(idxs)) for idxs in self.timeindices]
            xedata = [v/math.sqrt(n) for v, n in data]
            xedata = [v for t, v in zip(self.timelist, yedata) if tmin <= t <= tmax]
            
        ydata = [v for t, v in zip(self.timelist, ydata) if tmin <= t <= tmax]
        xdata = [v for t, v in zip(self.timelist, xdata) if tmin <= t <= tmax]
        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

    def theta_profile(self, **kwargs):
        varx, vary, index = kaisekiInterFace.KeywordGet(kwargs, ('varx', 'vary', 'index'))
        xdata, xedata, ydata, yedata, zdata, zedata = [], [], [], [], [], []
        idx = int(index.split('_')[0])
        if idx >= len(self.timelist):
            idx = len(self.timelist) - 1
            
        vename = self.verr.get(vary, None)
        if vename is not None:
            veidx = self.eg.vidx(vename)
            eprofs = [self.eg.trace_of_idx(1, i, veidx) for i in self.timeindices[idx]]
            n = len(eprofs)
            yedata = [v/math.sqrt(n) for v in kaisekiInterFace.AverageLists(eprofs)]
        else:
            pass
            
        at = self.timelist[idx]
        if vary not in self.additionalnames:
            vidx = self.eg.vidx(vary)
            unit = self.eg.valunits[vidx]
            title = '%s@%s_%s(%s)_%gs' % (self.diagname, self.dataid, vary, unit, at)
            profs = [self.eg.trace_of_idx(1, i, vidx) for i in self.timeindices[idx]]
            # remove all zero profiles
            profs_nz = [prof for prof in profs if sum(prof) != 0.0]
            if len(profs_nz) == 0:
                profs_nz = [[0.0] * len(self.eg.dims(1))]
            else:
                pass
                
            ydata = kaisekiInterFace.AverageLists(profs_nz)
            xdata = self.eg.dims(1)
        else:
            pass
            
        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

class EG_2D(object):
    def __init__(self, filepath, actual_name):
      try:
        """
        self.data[idx]
        iline = self.line_index()
        size = self.dimno + self.valno
        idx = self.line_size*iline + vidx
        """
        self.keyvalue = {}
        self.filepath = filepath
        self.actual_name = actual_name
        self.load(filepath)
      except Exception as e:
        print(e)

    def load(self, filepath):
        zf, fp = kaisekiInterFace.OpenFile(filepath)
        l = fp.readline()
        while l:
            l = l.strip()
            if len(l) == 0:
                l = fp.readline()
                continue
            if l[0] == '#':
                clm = l[1:].split('=')
                if len(clm) == 2:
                    self.keyvalue[clm[0].strip()] = clm[1].strip()
                    # print '%s' % clm[1].strip()
                l = fp.readline()
                continue
            break
        cdimnames = self.keyvalue.get('DimName')
        cdimunits = self.keyvalue.get('DimUnit')
        cdimno    = self.keyvalue.get('DimNo')
        cdimsize  = self.keyvalue.get('DimSize')
        cvalno    = self.keyvalue.get('ValNo')
        cvalnames = self.keyvalue.get('ValName')
        cvalunits = self.keyvalue.get('ValUnit')
        dimno     = int(cdimno)
        valno     = int(cvalno)
        line_size = dimno + valno
        dimsizes  = [int(c.strip()) for c in cdimsize.split(',')]
        data_size = line_size*reduce((lambda x, y: x*y), dimsizes, 1)
        self.data = numpy.zeros(data_size)
        self.dimnames = [c.strip("' ") for c in cdimnames.split(',')]
        self.dimunits = [c.strip("' ") for c in cdimunits.split(',')]
        self.valnames = [c.strip("' ") for c in cvalnames.split(',')]
        self.valunits = [c.strip("' ") for c in cvalunits.split(',')]
        self.dimno = dimno
        self.valno = valno
        self.line_size = line_size
        self.dimsizes  = dimsizes
        
        if re.compile('\ASDLloopDelPhi\Z').search(self.actual_name) :
            weights_source = self.dimsizes[:]
            weights_source.reverse()
            self.weights = kaisekiInterFace.CalcWeights(weights_source[1:])
            self.weights.reverse()
        else :
            self.weights = kaisekiInterFace.CalWeights(self.dimsizes[1:])
            
        idx = 0
        while l:
            l = l.strip()
            if len(l) == 0:
                l = fp.readline()
                continue
                
            for c in re.split(r',\s*|\s+',l):
                if c != "":
                    try:
                        self.data[idx] = float(c)
                    except ValueError:
                        self.data[idx] = numpy.nan
                    idx += 1
                    
            l = fp.readline()
            
        kaisekiInterFace.CloseFile(zf, fp)

    def line_index(self, *dimidxs):
        weighted_indexs = [i * w for i, w in zip(dimidxs, self.weights)]
        return reduce((lambda x, y: x + y), weighted_indexs, 0)

    def get_line(self, *dimidxs):
        idx = self.line_size * self.line_index(*dimidxs)
        return self.data[idx:idx + self.line_size]

    def dims(self, dimidx):
        if ( re.compile('\Anb4_Ib_3D_5-O_LOS\Z').search(self.actual_name) or 
             re.compile('\Anb4_Ib_3D_lhdcxsLOS_grp[1|3]\Z').search(self.actual_name) ) :
            dimperiod = self.line_size*reduce((lambda x, y: x * y), self.dimsizes[dimidx+1:], 1)
            dimidx = [dimidx + i * dimperiod for i in range(0, self.dimsizes[dimidx])]
            return [self.data[idx] for idx in dimidx]
        else :
            if dimidx == 0:
                pos = [[i, 0] for i in range(self.dimsizes[dimidx])]
            else:
                pos = [[0, i] for i in range(self.dimsizes[dimidx])]
            return [self.get_line(*p)[dimidx] for p in pos]

    def vdata(self, idxv, *dimidxs):
        return self.get_line(*dimidxs)[self.dimno + idxv]

    def vidx(self, name):
        if name in self.valnames:
            return self.valnames.index(name)
        else:
            return -1

    def trace_of_idx(self, dimidx, idx, vidx):
        """
        dimidx: idx of variable dims for trace
        ex)
          0: time trace
          1: radial profile
        """
        dimno = self.dimno
        dimsize = self.dimsizes[dimidx]
        if dimidx == 0:
            pos = [[i, idx] for i in range(dimsize)]
        else:
            pos = [[idx, i] for i in range(dimsize)]
        return [self.get_line(*p)[dimno+vidx] for p in pos]

    def trace_of(self, dimidx, idx, name):
        """
        dimidx: idx of variable dims for trace
        ex)
          0: time trace
          1: radial profile
        """
        vals = []
        vidx = self.vidx(name)
        if vidx == -1:
            pass
        else :
            try:
                dimno   = self.dimno
                dimsize = self.dimsizes[dimidx]
                if dimidx == 0:
                    pos = [[i, idx] for i in range(dimsize)]
                else:
                    pos = [[idx, i] for i in range(dimsize)]
                    
                vals = [self.get_line(*p)[dimno + vidx] for p in pos]
                
            except Exception as e:
                print("trace_of =>", e)
                vals = []
                
        return vals


if __name__ == '__main__':
    from MyViewCache import *
    cache = MyViewCache('cache\cache.txt')
    option = 'ntavg=1'
    ifc = InterFace('150290', temppath='temp', cachebutton='ONLINE', cache=cache, cmdpath='bin\win', option=option)
    ifc.load(cachebutton='OFFLINE')
    print('#',ifc.varsx())
    print('#',ifc.varsy(varx='Time', type='xy'))
    print('#',ifc.varsy(varx='Time', type='xyz'))
    print('#',ifc.varsz(varx='Time', vary='R', type='xyz'))
    print('#',ifc.indexs(varx='Time', vary='In_G1', type='xy'))
    title, x, xe, y, ye, z, ze = ifc.dataset(varx='Time', vary='R', varz='In_G1', index='', type='xyz')
    print('#', title)
    for ix, iy, iz in zip(x, y, z):
        print("%g\t%g\t%g" % (ix, iy, iz))
        pass
