#!/usr/bin/env python
# coding: utf-8
# $Id$
import re
import numpy as np
import kaisekiInterFace
from egdb2d import egdb2d
from egdb3d import egdb3d
from MyViewGlobal import logger

def helptext(title_name, script_name, data_source, synopsis, author_name, contact_to, plot_type, proc_summary, module_option):
    return kaisekiInterFace.helptext( title_name, script_name, data_source, synopsis, author_name, contact_to, plot_type, proc_summary, module_option )

class eg3dAvgInterFace( kaisekiInterFace.kaisekiInterFace ):
    pattern   = '\AeceTe_fast\Z'                  # eceTe_fast
    pattern2  = '\AeceTe_slow\Z'                  # eceTe_slow
    pattern3  = '\Alhdcxs9_moment\Z'              # lhdcxs9_moment
    pattern4  = '\Amp_raw_[pt]_integrated\Z'      # mp_raw_p_integrated, mp_raw_t_integrated
    pattern5  = '\Axi_ece\Z'                      # xi_ece
    pattern6  = '\Afmp_psd\Z'                     # fmp_psd

    def __init__(self, data_id, title_name, data_source, default_tcal, tuple_ntavg, **kwargs):
        """
        kwargs:
            option: option string for trace
            cache: myview cache object
            cachebutton: state of button cache on myView.
                         "CACHE": 
                         "ONLINE":
                         "OFFLINE":
            temppath: temp path set with myView dialog.
            cmdpath: command path set with myView dialog
        if there is some troubole to make instance, raise "Error"
        """
        super().__init__(data_id, title_name, data_source, **kwargs)
        self.tcal, self.avgnum = self.parse_option(self.option, default_tcal, tuple_ntavg)
        self.vidxreff       = None
        self.vidxa99        = None
        self.vidxTe_ece     = None
        self.vidxdTe_calib  = None

    def parse_option(self, option, default_tcal, tuple_ntavg):
        default_ntavg, ntavg_min, ntavg_max = tuple_ntavg
        ntavg = default_ntavg
        tcal  = default_tcal
        if option:
            dic = kaisekiInterFace.ParseOptionToDictionary(option)
            if any(dic) :
                vstr  = dic.get('ntavg')
                if vstr is not None :
                    ntavg = kaisekiInterFace.GetInteger(vstr, default_ntavg, ntavg_min, ntavg_max)
                    
                vstr  = dic.get('tcal')
                if vstr is not None :
                    tcal = kaisekiInterFace.GetFloat(vstr)
                    
        return tcal, ntavg

    def load(self, **kwargs):
        if self.actual_name is not None :
            # lhdcxs9_moment
            # mp_raw_p_integrated, mp_raw_t_integrated
            # xi_ece
            # fmp_psd
            super().load( **kwargs )
            if self.flgloaded:
                return
                
            if self.cachepath:
                tdic = {}
                self.eg = egdb3d_avg(self.cachepath, tdic=tdic, avgnum=self.avgnum)
                self.cache.releasefile(self.cachepath)
                self.flgloaded = True
                if re.compile(eg3dAvgInterFace.pattern6).search(self.actual_name) :
                    # fmp_psd
                    self.vidxPDS = self.eg.valname2idx('PDS')
                    self.vidxCW  = self.eg.valname2idx('TORMODE_CW')
                    self.vidxCCW = self.eg.valname2idx('TORMODE_CCW')
                    self.vidxCOH = self.eg.valname2idx('COH')
            else:
                self.eg = None
                
        else :
            # eceTe_fast
            # eceTe_slow
            if self.flgloaded:
                return
                
            list_cachepath = []
            for diagname in self.list_actual_name :
                cachepath = super().load_target( diagname, **kwargs )
                list_cachepath.append(cachepath)
                
            # Exclude failed loads from the list.
            list_a_filtered = list(filter(None, list_cachepath))
            if not list_a_filtered :
                for idx, diagname in enumerate(self.list_actual_name):
                    self.list_eg.append( None )
                    
            elif ( len(self.list_actual_name) == len( list_a_filtered ) ) :
                # If image data and reference data exist, an image object is created.
                tdic = {}
                for idx, diagname in enumerate(self.list_actual_name):
                    cachepath = list_a_filtered[idx]
                    if diagname == 'tswpe_reff' :
                        self.list_eg.append( egdb2d(cachepath, tdic=tdic) )
                        self.list_eg[idx].readFile()
                    else :
                        self.list_eg.append( egdb3d_avg(cachepath, tdic=tdic) )
                        if diagname == 'ecemap' :
                            self.vidxreff = self.list_eg[idx].valname2idx('reff')
                            self.vidxa99  = self.list_eg[idx].valname2idx('a99')
                            if re.compile(eg3dAvgInterFace.pattern2).search(self.diagname) :
                                # eceTe_slow
                                self.vidxTe_ece    = self.list_eg[idx].valname2idx('Te')
                                self.vidxdTe_calib = self.list_eg[idx].valname2idx('calib.')
                                
                        elif diagname == 'ece_fast' :
                            self.vidxTe_ece    = self.list_eg[idx].valname2idx('Te')
                            self.vidxdTe_calib = self.list_eg[idx].valname2idx('calib.')
                            
                    self.cache.releasefile(cachepath)
                    
                self.flgloaded = True
                if ( re.compile(eg3dAvgInterFace.pattern).search(self.diagname) or
                     re.compile(eg3dAvgInterFace.pattern2).search(self.diagname) ) :
                    # eceTe_fast
                    # eceTe_slow
                    self.factors = self.calc_calibration_factor(self.tcal)
                    
            else:
                for idx, diagname in enumerate(self.list_actual_name):
                    self.list_eg.append( None )
                    
    def varsx(self, **kwargs):
        """
        return list of candidates of variables for x-axis
        
        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog
        """
        lst = []
        self.load(**kwargs)
        if self.actual_name is not None :
            if not self.eg :
                pass
            else :
                if re.compile(eg3dAvgInterFace.pattern4).search(self.diagname) :
                    # mp_raw_p_integrated, mp_raw_t_integrated
                    lst = ['Time', 'Poloidal angle']
                elif re.compile(eg3dAvgInterFace.pattern6).search(self.diagname) :
                    # fmp_psd
                    lst = self.eg.eg.DimNames
                else :
                    # lhdcxs9_moment
                    # xi_ece
                    lst = ['Time', 'R']
                    
        else :
            try:
                egidx = None
                if re.compile(eg3dAvgInterFace.pattern).search(self.diagname) :
                    # eceTe_fast
                    egidx = self.list_actual_name.index('ece_fast')
                elif re.compile(eg3dAvgInterFace.pattern2).search(self.diagname) :
                    # eceTe_slow
                    egidx = self.list_actual_name.index('ecemap')
                    
                if egidx is not None :
                    if not self.list_eg[egidx] :
                        pass
                    else :
                        lst = ['Time', 'R', 'reff', 'reff/a99']
                        
            except ValueError:
                pass
                
        return lst

    def varsy(self, **kwargs):
        """
        return list of candidates of variables for y-axis or y2-axis

        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog
        """
        lst = []
        self.load(**kwargs)
        if self.actual_name is not None :
            if not self.eg :
                pass
            else :
                ptype = kwargs.get('type', None)
                varx  = kwargs.get('varx', 'Time')
                if ptype == 'xyz':
                    if varx == 'Time':
                        if re.compile(eg3dAvgInterFace.pattern4).search(self.diagname) :
                            # mp_raw_p_integrated, mp_raw_t_integrated
                            lst = ['Poloidal angle']
                        elif re.compile(eg3dAvgInterFace.pattern6).search(self.diagname) :
                            # fmp_psd
                            lst = ['Frequency']
                        else:
                            # lhdcxs9_moment
                            # xi_ece
                            lst = ['R']
                    else:
                        lst = ['Time']
                        
                else :
                    if re.compile(eg3dAvgInterFace.pattern3).search(self.diagname) :
                        # lhdcxs9_moment
                        if varx == 'R':
                            valnames = ['0th', '1st', 'sqrt(2nd)', '3rd', '4th']
                            arys = list(set(self.get_arys()))
                            arys.sort()
                            lst = ["%s(%d)" % (vn, ary) for ary in arys for vn in valnames ]
                        else :
                            lst = ['0th', '1st', 'sqrt(2nd)', '3rd', '4th']
                            
                    elif re.compile(eg3dAvgInterFace.pattern4).search(self.diagname) :
                        # mp_raw_p_integrated, mp_raw_t_integrated
                        lst = ['B']
                        
                    elif re.compile(eg3dAvgInterFace.pattern6).search(self.diagname) :
                        # fmp_psd
                        lst = self.eg.eg.valnames()
                        
                    elif re.compile(eg3dAvgInterFace.pattern5).search(self.diagname) :
                        # xi_ece
                        lst = ['radh', 'radh_HP1kHz', 'Te', 'Te_HP1kHz', '-gradTe_LP1kHz', 'xi']
                        
                    else :
                        pass
        else :
            try:
                egidx = None
                if re.compile(eg3dAvgInterFace.pattern).search(self.diagname) :
                    # eceTe_fast
                    egidx = self.list_actual_name.index('ece_fast')
                elif re.compile(eg3dAvgInterFace.pattern2).search(self.diagname) :
                    # eceTe_slow
                    egidx = self.list_actual_name.index('ecemap')
                    
                if egidx is not None :
                    if not self.list_eg[egidx] :
                        pass
                    else :
                        ptype = kwargs.get('type', None)
                        varx  = kwargs.get('varx', 'Time')
                        if ptype == 'xyz':
                            if varx == 'Time':
                                lst = ['R']
                            else:
                                lst = ['Time']
                                
                        else :
                            lst = ['Te', 'Fece', 'Calib', 'diag_number', 'we7000_ch']
                            
            except ValueError:
                pass
                
        return lst

    def varsz(self, **kwargs):
        """
        return list of candidates of variables for z-axis

        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog
        """
        lst = []
        self.load(**kwargs)
        if self.actual_name is not None :
            if not self.eg :
                pass
            else :
                ptype = kwargs.get('type', None)
                if ptype == 'xyz':
                    if re.compile(eg3dAvgInterFace.pattern3).search(self.diagname) :
                        # lhdcxs9_moment
                        valnames = ['0th', '1st', 'sqrt(2nd)', '3rd', '4th']
                        arys = list(set(self.get_arys()))
                        arys.sort()
                        lst =  ["%s(%d)" % (vn, ary) for ary in arys for vn in valnames ]
                        
                    elif re.compile(eg3dAvgInterFace.pattern4).search(self.diagname) :
                        # mp_raw_p_integrated, mp_raw_t_integrated
                        lst = ['B']
                    elif re.compile(eg3dAvgInterFace.pattern5).search(self.diagname) :
                        # xi_ece
                        lst = ['radh', 'radh_HP1kHz', 'Te', 'Te_HP1kHz', '-gradTe_LP1kHz', 'xi']
                        
                    elif re.compile(eg3dAvgInterFace.pattern6).search(self.diagname) :
                        # fmp_psd
                        lst = self.eg.eg.valnames()
                    else :
                        pass
                        
                else :
                    pass
        else :
            try:
                egidx = None
                if re.compile(eg3dAvgInterFace.pattern).search(self.diagname) :
                    # eceTe_fast
                    egidx = self.list_actual_name.index('ece_fast')
                elif re.compile(eg3dAvgInterFace.pattern2).search(self.diagname) :
                    # eceTe_slow
                    egidx = self.list_actual_name.index('ecemap')
                    
                if egidx is not None :
                    if not self.list_eg[egidx] :
                        pass
                    else :
                        ptype = kwargs.get('type', None)
                        if ptype == 'xyz':
                            lst = ['Te', 'Fece', 'Calib', 'diag_number', 'we7000_ch']
                            
                        else :
                            if re.compile(eg3dAvgInterFace.pattern2).search(self.diagname) :
                                # eceTe_slow
                                lst = self.list_eg[egidx].eg.valnames()
                                if type(lst) is str :
                                    # If it is a string, convert it to an array.
                                    lst = [lst]
                                    
            except ValueError:
                pass
                
        return lst

    def indexs(self, **kwargs):
        """
        return list of indexs for varx, vary, varz, option.
        
        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog
        """
        lst = []
        self.load(**kwargs)
        if self.actual_name is not None :
            if not self.eg :
                pass
            else :
                ptype = kwargs.get('type', None)
                varx  = kwargs.get('varx')
                if ptype == 'xyz':
                    pass
                else :
                    if re.compile(eg3dAvgInterFace.pattern4).search(self.diagname) :
                        # mp_raw_p_integrated, mp_raw_t_integrated
                        if varx == 'Time':
                            list_r = self.eg.get_rlist()                        # R data
                            for ridx, r in enumerate(list_r):
                                c = "%d_%gdeg." % (ridx, r)
                                lst.append(c)
                                
                        elif varx == 'Poloidal angle':
                            list_time = self.eg.get_timelist()                  # Time data
                            for tidx, t in enumerate(list_time):
                                c = "%d_%gs" % (tidx, t)
                                lst.append(c)
                                
                        else :
                            pass
                    else:
                        # lhdcxs9_moment
                        # xi_ece
                        # fmp_psd
                        if varx == 'Time':
                            list_r = self.eg.get_rlist()                        # R data
                            if re.compile(eg3dAvgInterFace.pattern3).search(self.diagname) :
                                # lhdcxs9_moment
                                arys = self.get_arys()
                                for ridx, r in enumerate(list_r):
                                    c = "%d_%gm(%d)" % (ridx, r, arys[ridx])
                                    lst.append(c)
                                    
                            else:
                                # xi_ece
                                # fmp_psd
                                for ridx, r in enumerate(list_r):
                                    if re.compile(eg3dAvgInterFace.pattern5).search(self.diagname) :
                                        # xi_ece
                                        c = "%d_%gm" % (ridx, r)
                                    else :
                                        # fmp_psd
                                        c = "%d_%gHz" % (ridx, r)
                                        
                                    lst.append(c)
                                    
                        elif varx == 'R' or varx == 'Frequency':
                            # xi_ece
                            # fmp_psd
                            list_time = self.eg.get_timelist()                  # Time data
                            for tidx, t in enumerate(list_time):
                                c = "%d_%gs" % (tidx, t)
                                lst.append(c)
                                
                        else:
                            pass
        else :
            obj_eg = None
            if re.compile(eg3dAvgInterFace.pattern).search(self.diagname) :
                # eceTe_fast
#                egidx = self.list_actual_name.index('ece_fast')
                obj_eg = self.get_eg('ece_fast')
            elif re.compile(eg3dAvgInterFace.pattern2).search(self.diagname) :
                # eceTe_slow
#                egidx = self.list_actual_name.index('ecemap')
                obj_eg = self.get_eg('ecemap')
                
            if obj_eg is not None :
                ptype = kwargs.get('type', None)
                varx  = kwargs.get('varx', 'Time')
                if ptype == 'xyz':
                    pass
                else :
                    if varx == 'Time':
                        list_r = obj_eg.get_rlist()
                        for ridx, r in enumerate(list_r) :
                            c = "%d_%gm" % (ridx, r)
                            lst.append(c)
                            
                    elif varx in ['R', 'reff', 'reff/a99']:
                        list_time = obj_eg.get_timelist()
                        for tidx, t in enumerate(list_time) :
                            c = "%d_%gs" % (tidx, t)
                            lst.append(c)
                            
        return lst

    def dataset(self, **kwargs):
        """
        return list of dataset for varx, vary, varz, index, option.
        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog
            index: 

        dataset := [title, x, xe, y, ye, z, ze]
        x = [x1, x2, x3, ... , xn]
        y = [y1, y2, x3, ... , ym] 
        z = [[z(x1, y1), z(x2, y1), z(x3, y1),...,z(xn, y1)],
             [z(x1, y2), z(x2, y2), z(x3, y2),...,z(xn, y2)],
             [z(x1, y3), z(x2, y3), z(x3, y3),...,z(xn, y3)], 
             ......
             [z(x1, ym), z(x2, ym), z(x3, ym),...,z(xn, ym)]] 
        len(z) = len(y)
        len(z[i]) = len(x)
        """
        self.load(**kwargs)
        ptype, varx = kaisekiInterFace.KeywordGet(kwargs, ('type', 'varx'))
        dataset = ['%s@%s No data' % (self.diagname, self.dataid), [],[],[],[],[],[]]
        
        if self.actual_name is not None :
            if not self.eg :
                pass
            else :
                self.refresh_index(kwargs)
                if ptype == 'xyz':
                    dataset = self.data_xyz(**kwargs)
                    
                else :
                    if varx == 'Time':
                        dataset = self.time_trace(**kwargs)
                    else :
                        if re.compile(eg3dAvgInterFace.pattern4).search(self.diagname) :
                            # mp_raw_p_integrated, mp_raw_t_integrated
                            if varx == 'Poloidal angle':
                                dataset = self.r_profile(**kwargs)
                                
                        elif re.compile(eg3dAvgInterFace.pattern6).search(self.diagname) :
                            # fmp_psd
                            if varx == 'Frequency':
                                dataset = self.r_profile(**kwargs)
                                
                        else:
                            # lhdcxs9_moment
                            # xi_ece
                            if varx == 'R':
                                dataset = self.r_profile(**kwargs)
                                
        else :
            if re.compile(eg3dAvgInterFace.pattern).search(self.diagname) :
                # eceTe_fast
                obj_eg = self.get_eg('ece_fast')
            elif re.compile(eg3dAvgInterFace.pattern2).search(self.diagname) :
                # eceTe_slow
                obj_eg = self.get_eg('ecemap')
                
            if obj_eg is not None :
                self.refresh_index(kwargs)
                if ptype == 'xyz':
                    dataset = self.data_xyz(**kwargs)
                    
                else :
                    if varx== 'Time':
                        dataset = self.time_trace(**kwargs)
                        
                    elif varx == 'R':
                        dataset = self.r_profile(**kwargs)
                         
                    elif varx in ['reff', 'reff/a99']:
                        dataset = self.reff_profile(**kwargs)
                        
        return dataset

    def data_xyz(self, **kwargs):
        xdata, xedata, ydata, yedata, zdata, zedata = [], [], [], [], [], []
        varx, vary, varz, index = kaisekiInterFace.KeywordGet(kwargs, ('varx', 'vary', 'varz', 'index'))
        
        if self.actual_name is not None :
            if re.compile(eg3dAvgInterFace.pattern4).search(self.diagname) :
                # mp_raw_p_integrated, mp_raw_t_integrated
                vidx  = 0
                unit  = 'T'
                title = '%s_%s(%s)@%s' % (self.diagname, varz, unit, self.dataid)
                pos   = self.eg.position                               # Rename a function
                data  = self.eg.get()                                  # sumv
                tdata = self.eg.get_timelist()                         # Time data
                rdata = self.eg.get_rlist()                            # R data
                tidx  = int(index.split('_')[0])
                for tidx, tm in enumerate(tdata):
                    for ridx, r in enumerate(rdata):
                        if varx == 'Poloidal angle':
                            xdata.append(r)
                        else:
                            xdata.append(tm)
                            
                        if vary == 'Poloidal angle':
                            ydata.append(r)
                        else:
                            ydata.append(tm)
                            
                        v = data[pos(tidx, ridx, vidx)] 
                        zdata.append(v)
                        
            elif re.compile(eg3dAvgInterFace.pattern6).search(self.diagname) :
                # fmp_psd
                vidx = self.eg.valname2idx(varz)
                if varz == 'PSD':
                    unit = 'au'
                else:
                    unit = 'au'
                    #unit = self.eg.idx2valunit(vidx)
                title = '%s_%s(%s)@%s' % (self.diagname, varz, unit, self.dataid)
                
                pos   = self.eg.position                               # Rename a function
                data  = self.eg.get()                                  # sumv
                tdata = self.eg.get_timelist()                         # Time data
                rdata = self.eg.get_rlist()                            # R data
                for tidx, tm in enumerate(tdata):
                    for ridx, r in enumerate(rdata):
                        if varx == 'Frequency':
                            xdata.append(r)
                        else:
                            xdata.append(tm)
                            
                        if vary == 'Frequency':
                            ydata.append(r)
                        else:
                            ydata.append(tm)
                            
                        if varz in ['PSD']:
                            v = data[pos(tidx, ridx, vidx)]
                            zdata.append(v)
                        elif varz in ['TORMODE_CW']:
                            v = data[pos(tidx, ridx, vidx)]
                            zdata.append(v)
                        elif varz in ['TORMODE_CCW']:
                            v = data[pos(tidx, ridx, vidx)]
                            zdata.append(v)
                        else:
                            v = data[pos(tidx, ridx, vidx)] 
                            zdata.append(v)
                            
            else:
                if re.compile(eg3dAvgInterFace.pattern3).search(self.diagname) :
                    # lhdcxs9_moment
                    varz, ary = self.split_name_ary(varz)                  # Create a tuple
                    vidx  = self.eg.valname2idx(varz)
                    title = '%s_%s(%s)@%s' % (self.diagname, varz, ary, self.dataid)
                    arys  = self.get_arys()
                else:
                    # xi_ece
                    vidx  = self.eg.valname2idx(varz)
                    unit  = self.eg.idx2valunit(vidx)
                    title = '%s_%s(%s)@%s' % (self.diagname, varz, unit, self.dataid)
                    
                pos   = self.eg.position                               # Rename a function
                data  = self.eg.get()                                  # sumv
                tdata = self.eg.get_timelist()                         # Time data
                rdata = self.eg.get_rlist()                            # R data
                for tidx, tm in enumerate(tdata):
                    for ridx, r in enumerate(rdata):
                        if ( re.compile(eg3dAvgInterFace.pattern3).search(self.diagname) and
                             int(ary) != int(arys[ridx]) ) :
                            # lhdcxs9_moment
                                continue
                                
                        else :
                            if varx == 'R':
                                xdata.append(r)
                            else:
                                xdata.append(tm)
                                
                            if vary == 'R':
                                ydata.append(r)
                            else:
                                ydata.append(tm)
                                
                            v = data[pos(tidx, ridx, vidx)] 
                            zdata.append(v)
                            
        else :
            if re.compile(eg3dAvgInterFace.pattern).search(self.diagname) :
                # eceTe_fast
                obj_eg = self.get_eg('ece_fast')
            elif re.compile(eg3dAvgInterFace.pattern2).search(self.diagname) :
                # eceTe_slow
                obj_eg = self.get_eg('ecemap')
                
            vidx = obj_eg.valname2idx(varz)
            if varz == 'Te':
                unit = 'keV'
            else:
                unit = obj_eg.idx2valunit(vidx)
                
            title = '%s_%s(%s)@%s' % (self.diagname, varz, unit, self.dataid)
            
            cf_vidx = obj_eg.valname2idx('calib.')
            te_vidx = obj_eg.valname2idx('Te')
            pos     = obj_eg.position                     # Rename a function
            data    = obj_eg.get()                        # sumv
            tdata   = obj_eg.get_timelist()               # Time data
            rdata   = obj_eg.get_rlist()                  # R data
            for tidx, tm in enumerate(tdata):
                if re.compile(eg3dAvgInterFace.pattern).search(self.diagname) :
                    # eceTe_fast
                    if np.nan == self.reff_a99_at(tm, 0):
                        continue
                        
                for ridx, r in enumerate(rdata):
                    if re.compile(eg3dAvgInterFace.pattern2).search(self.diagname) :
                        # eceTe_slow
                        reff = data[pos(tidx, ridx, self.vidxreff)] 
                        a99  = data[pos(tidx, ridx, self.vidxa99)]
                        
                    if varx == 'R':
                        xdata.append(r)
                        
                    elif varx == 'reff':
                        if re.compile(eg3dAvgInterFace.pattern).search(self.diagname) :
                            # eceTe_fast
                            xdata.append(self.reff_at(tm, ridx))
                        else :
                            xdata.append(reff)
                            
                    elif varx == 'reff/a99':
                        if re.compile(eg3dAvgInterFace.pattern).search(self.diagname) :
                            # eceTe_fast
                            xdata.append(self.reff_a99_at(tm, ridx))
                        else :
                            if a99 != 0.0:
                                ydata.append(reff / a99)
                            else:
                                ydata.append(0.0)
                                
                    else:
                        xdata.append(tm)
                        
                    if vary == 'R':
                        ydata.append(r)
                        
                    elif vary == 'reff':
                        if re.compile(eg3dAvgInterFace.pattern).search(self.diagname) :
                            # eceTe_fast
                            ydata.append(self.reff_at(tm, ridx))
                        else :
                            ydata.append(reff)
                            
                    elif vary == 'reff/a99':
                        if re.compile(eg3dAvgInterFace.pattern).search(self.diagname) :
                            # eceTe_fast
                            ydata.append(self.reff_a99_at(tm, ridx))
                        else :
                            if a99 != 0.0:
                                ydata.append(reff / a99)
                            else:
                                ydata.append(0.0)
                    else:
                        ydata.append(tm)
                        
                    if varz == 'Te' :
                        if not self.tcal : 
                            v = data[pos(tidx, ridx, vidx)] 
                            zdata.append(v)
                            
                        else:
                            factor = self.factors[ridx]
                            v      = data[pos(tidx, ridx, te_vidx)]
                            cf     = data[pos(tidx, ridx, cf_vidx)]
                            zdata.append(v / cf * factor)
                            
                    else:
                        v = data[pos(tidx, ridx, vidx)] 
                        zdata.append(v)
                        
        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

    def time_trace(self, **kwargs):
        xdata, xedata, ydata, yedata, zdata, zedata = [], [], [], [], [], []
        vary, index = kaisekiInterFace.KeywordGet(kwargs, ('vary', 'index'))
        
        if self.actual_name is not None :
            if re.compile(eg3dAvgInterFace.pattern3).search(self.diagname) :
                # lhdcxs9_moment
                index, ary = self.split_name_ary(index)                    # Create a tuple
                
            ridx  = int(index.split('_')[0])
            rdata = self.eg.get_rlist()                                    # R data
            if ridx >= len(rdata):
                ridx = len(rdata) - 1
                
            at = rdata[ridx]
            
            if re.compile(eg3dAvgInterFace.pattern4).search(self.diagname) :
                # mp_raw_p_integrated, mp_raw_t_integrated
                vidx = 0
                unit = 'T'
                title = '%s_%s(%s)_%g%s@%s' % (self.diagname, vary, unit, at, self.eg.get_unitR(), self.dataid)
            else:
                # lhdcxs9_moment
                # xi_ece
                # fmp_psd
                vidx = self.eg.valname2idx(vary)
                
                if re.compile(eg3dAvgInterFace.pattern3).search(self.diagname) :
                    # lhdcxs9_moment
                    title = '%s_%s(%s)_%gm@%s' % (self.diagname, vary, ary, at, self.dataid)
                else:
                    # xi_ece
                    # fmp_psd
                    if re.compile(eg3dAvgInterFace.pattern5).search(self.diagname) :
                        # xi_ece
                        unit = self.eg.idx2valunit(vidx)
                    else :
                        # fmp_psd
                        if vary == 'PDS':
                            unit = 'au'
                        else:
                            unit = 'au'
                            
                    title = '%s_%s(%s)_%gm@%s' % (self.diagname, vary, unit, at, self.dataid)
                    
            pos   = self.eg.position                                   # Rename a function
            xdata = self.eg.get_timelist()                             # Time data
            data  = self.eg.get()                                      # sumv
            for tidx, t in enumerate(xdata):
                v = data[pos(tidx, ridx, vidx)]
                ydata.append(v)
                
        else :
            if re.compile(eg3dAvgInterFace.pattern).search(self.diagname) :
                # eceTe_fast
                obj_eg = self.get_eg('ece_fast')
            elif re.compile(eg3dAvgInterFace.pattern2).search(self.diagname) :
                # eceTe_slow
                obj_eg = self.get_eg('ecemap')
                
            ridx  = int(index.split('_')[0])
            rdata = obj_eg.get_rlist()                    # R data
            if ridx >= len(rdata):
                ridx = len(rdata) - 1
                
            at   = rdata[ridx]
            vidx = obj_eg.valname2idx(vary)
            
            if vary == 'Te':
                unit = 'keV'
            else:
                unit = obj_eg.idx2valunit(vidx)
                
            title = '%s_%s(%s)_%gm@%s' % (self.diagname, vary, unit, at, self.dataid)
            
            cf_vidx = obj_eg.valname2idx('calib.')
            te_vidx = obj_eg.valname2idx('Te')
            pos     = obj_eg.position                     # Rename a function
            xdata   = obj_eg.get_timelist()               # Time data
            data    = obj_eg.get()                        # sumv
            
            for tidx, t in enumerate(xdata):
                if vary == 'Te' :
                    if not self.tcal:
                        v = data[pos(tidx, ridx, vidx)] 
                        ydata.append(v)
                        
                    else:
                        factor = self.factors[ridx]
                        v      = data[pos(tidx, ridx, te_vidx)]
                        cf     = data[pos(tidx, ridx, cf_vidx)]
                        ydata.append(v / cf * factor)
                        
                else:
                    v = data[pos(tidx, ridx, vidx)]
                    ydata.append(v)
                    
        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

    def r_profile(self, **kwargs):
        xdata, xedata, ydata, yedata, zdata, zedata = [], [], [], [], [], []
        vary, index = kaisekiInterFace.KeywordGet(kwargs, ('vary', 'index'))
        
        if self.actual_name is not None :
            if re.compile(eg3dAvgInterFace.pattern3).search(self.diagname) :
                # lhdcxs9_moment
                vary, ary = self.split_name_ary(vary)                  # Create a tuple
                
            tdata = self.eg.get_timelist()                             # Time data
            tidx  = int(index.split('_')[0])
            tm    = tdata[tidx]
            
            if re.compile(eg3dAvgInterFace.pattern4).search(self.diagname) :
                # mp_raw_p_integrated, mp_raw_t_integrated
                vidx  = 0
                unit  = 'T'
                title = '%s_%s(%s)_%g%s@%s' % (self.diagname, vary, unit, tm, self.eg.get_unitT(),self.dataid)
                
            else :
                vidx = self.eg.valname2idx(vary)
                if re.compile(eg3dAvgInterFace.pattern3).search(self.diagname) :
                    # lhdcxs9_moment
                    title = '%s_%s(%s)_%gs@%s' % (self.diagname, vary, ary, tm, self.dataid)
                else :
                    # xi_ece
                    # fmp_psd
                    if re.compile(eg3dAvgInterFace.pattern5).search(self.diagname) :
                        # xi_ece
                        unit  = self.eg.idx2valunit(vidx)
                    else :
                        if vary == 'PSD':
                            unit = 'au'
                        else:
                            unit = 'au'
                            
                    title = '%s_%s(%s)_%gs@%s' % (self.diagname, vary, unit, tm, self.dataid)
                    
            pos   = self.eg.position                                   # Rename a function
            data  = self.eg.get()                                      # sumv
            if re.compile(eg3dAvgInterFace.pattern3).search(self.diagname) :
                # lhdcxs9_moment
                for ridx, r in enumerate(self.eg.get_rlist()):
                    if int(ary) != int(arys[ridx]):
                        continue
                        
                    xdata.append(r)
                    v = data[pos(tidx, ridx, vidx)] 
                    ydata.append(v)
                    
            else:
                # mp_raw_p_integrated, mp_raw_t_integrated
                # xi_ece
                # fmp_psd
                xdata = self.eg.get_rlist()                            # R data
                for ridx, r in enumerate(xdata):
                    v = data[pos(tidx, ridx, vidx)] 
                    ydata.append(v)
                    
        else :
            if re.compile(eg3dAvgInterFace.pattern).search(self.diagname) :
                # eceTe_fast
                obj_eg = self.get_eg('ece_fast')
            elif re.compile(eg3dAvgInterFace.pattern2).search(self.diagname) :
                # eceTe_slow
                obj_eg = self.get_eg('ecemap')
                
            tidx  = int(index.split('_')[0])
            tdata = obj_eg.get_timelist()                 # Time data
            at    = tdata[tidx]
            vidx  = obj_eg.valname2idx(vary)
            
            if vary == 'Te':
                unit = 'keV'
            else:
                unit = obj_eg.idx2valunit(vidx)
                
            title = '%s_%s(%s)_%gs@%s' % (self.diagname, vary, unit, at, self.dataid)
            
            cf_vidx = obj_eg.valname2idx('calib.')
            te_vidx = obj_eg.valname2idx('Te')
            pos     = obj_eg.position                     # Rename a function
            xdata   = obj_eg.get_rlist()                  # R data
            data    = obj_eg.get()                        # sumv
            
            for ridx, r in enumerate(xdata):
                if vary == 'Te' :
                    if not self.tcal:
                        v = data[pos(tidx, ridx, vidx)] 
                        ydata.append(v)
                    else:
                        factor = self.factors[ridx]
                        v      = data[pos(tidx, ridx, te_vidx)]
                        cf     = data[pos(tidx, ridx, cf_vidx)]
                        ydata.append(v/cf*factor)
                else:
                    v = data[pos(tidx, ridx, vidx)] 
                    ydata.append(v)
                    
        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

    def reff_profile(self, **kwargs):
        xdata, xedata, ydata, yedata, zdata, zedata = [], [], [], [], [], []
        varx, vary, index = kaisekiInterFace.KeywordGet(kwargs, ('varx', 'vary', 'index'))
        tidx  = int(index.split('_')[0])

        if re.compile(eg3dAvgInterFace.pattern).search(self.diagname) :
            # eceTe_fast
            obj_eg = self.get_eg('ece_fast')
        elif re.compile(eg3dAvgInterFace.pattern2).search(self.diagname) :
            # eceTe_slow
            obj_eg = self.get_eg('ecemap')
            
        tdata = obj_eg.get_timelist()                 # Time data
        if tidx >= len(tdata):
            tidx = len(tdata) - 1
            
        at   = tdata[tidx]
        vidx = obj_eg.valname2idx(vary)
        
        if vary == 'Te':
            unit = 'keV'
        else:
            unit = obj_eg.idx2valunit(vidx)
            
        title = '%s_%s(%s)_%gs@%s' % (self.diagname, vary, unit, at, self.dataid)
        
        cf_vidx = obj_eg.valname2idx('calib.')
        te_vidx = obj_eg.valname2idx('Te')
        pos     = obj_eg.position                     # Rename a function
        rdata   = obj_eg.get_rlist()                  # R data
        data    = obj_eg.get()                        # sumv
        
        for ridx, r in enumerate(rdata):
            if re.compile(eg3dAvgInterFace.pattern).search(self.diagname) :
                # eceTe_fast
                if varx == 'reff/a99':
                    xdata.append(self.reff_a99_at(at, ridx))
                else:
                    xdata.append(self.reff_at(at, ridx))
                    
            elif re.compile(eg3dAvgInterFace.pattern2).search(self.diagname) :
                # eceTe_slow
                reff = data[pos(tidx, ridx, self.vidxreff)] 
                a99  = data[pos(tidx, ridx, self.vidxa99)]
                
                if varx == 'reff':
                    xdata.append(reff)
                else:
                    if a99 != 0.0:
                        xdata.append(reff / a99)
                    else:
                        xdata.append(0.0)
                        
            if vary == 'Te' :
                if not self.tcal:
                    v = data[pos(tidx, ridx, vidx)] 
                    ydata.append(v)
                else:
                    factor = self.factors[ridx]
                    v      = data[pos(tidx, ridx, te_vidx)]
                    cf     = data[pos(tidx, ridx, cf_vidx)]
                    ydata.append(v / cf * factor)
            else :
                v = data[pos(tidx, ridx, vidx)] 
                ydata.append(v)
                
        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

    def get_arys(self):
        vidx = self.eg.valname2idx('grp')
        pos  = self.eg.position                                            # Rename a function
        data = self.eg.get()                                               # sumv
        return [data[pos(0, ridx, vidx)] for ridx, r in enumerate(self.eg.get_rlist())]

    def split_name_ary(self, name_ary):
        #
        # Make the name a tuple using a regular expression.
        na, = re.findall(r'(.*)\((.*)\)$', name_ary)                       # Create a tuple
        return na

    def reff_at(self, tm, ridx):
        reff = np.nan
        if re.compile(eg3dAvgInterFace.pattern).search(self.diagname) :
            # eceTe_fast
#            idx = self.list_actual_name.index('ecemap')
            obj_eg = self.get_eg('ecemap')
            if obj_eg is not None :
                pos           = obj_eg.position                 # Rename a function
                ece_timelist  = obj_eg.get_timelist()           # Time data
                ece_slow_data = obj_eg.get()                    # sumv
                vidx_reff     = obj_eg.valname2idx('reff')
                tidx0, tidx1  = self.searchTimeIndex(tm, ece_timelist)
                
                if tidx0 == tidx1 :
                    reff = ece_slow_data[pos(tidx0, ridx, vidx_reff)]
                else :
                    reff0 = ece_slow_data[pos(tidx0, ridx, vidx_reff)]
                    reff1 = ece_slow_data[pos(tidx1, ridx, vidx_reff)]
                    tm0 = ece_timelist[tidx0]
                    tm1 = ece_timelist[tidx1]
                    reff = reff0 + (tm-tm0)*(reff1-reff0)/(tm1-tm0)
                    
        return reff

    def reff_a99_at(self, tm, ridx):
        reff_a99 = np.nan
        if re.compile(eg3dAvgInterFace.pattern).search(self.diagname) :
            # eceTe_fast
#            idx = self.list_actual_name.index('ecemap')
            obj_eg = self.get_eg('ecemap')
            if obj_eg is not None :
                pos           = obj_eg.self.list_eg[idx].position                 # Rename a function
                ece_timelist  = obj_eg.get_timelist()           # Time data
                ece_slow_data = obj_eg.get()                    # sumv
                vidx_reff     = obj_eg.valname2idx('reff')
                vidx_a99      = obj_eg.valname2idx('a99')
                tidx0, tidx1  = self.searchTimeIndex(tm, ece_timelist)
                a990  = ece_slow_data[pos(tidx0, ridx, vidx_a99)]
                a991  = ece_slow_data[pos(tidx1, ridx, vidx_a99)]
                reff0 = ece_slow_data[pos(tidx0, ridx, vidx_reff)]
                reff1 = ece_slow_data[pos(tidx1, ridx, vidx_reff)]
                
                if tidx0 == tidx1 :
                    if a990 != 0.0 :
                        reff_a99 = reff0 / a990
                    else:
                        reff_a99 = np.nan
                        
                else :
                    tm0  = ece_timelist[tidx0]
                    tm1  = ece_timelist[tidx1]
                    reff = reff0 + (tm - tm0) * (reff1 - reff0) / (tm1 - tm0)
                    a99  = a990 + (tm - tm0) * (a991 - a990) / (tm1 - tm0)
                    if a99 != 0.0:
                        reff_a99 = reff / a99
                    else:
                        reff_a99 = np.nan
                        
        return reff_a99

    def calc_calibration_factor(self, target):
        lst_factor = []
        if ( re.compile(eg3dAvgInterFace.pattern).search(self.diagname) or
             re.compile(eg3dAvgInterFace.pattern2).search(self.diagname) ) :
            # eceTe_fast
            # eceTe_slow
#            egidx1 = self.list_actual_name.index('ecemap')
#            egidx2 = self.list_actual_name.index('tswpe_reff')
            obj_eg1 = self.get_eg('ecemap')
            obj_eg2 = self.get_eg('tswpe_reff')
            if target == 0.0:
                if obj_eg1 is not None :
                    lst_factor = [1.0 for r in obj_eg1.get_rlist()]
                    
            else :
                if obj_eg1 is not None and obj_eg2 is not None :
                    lst_cte0 = obj_eg2.data[obj_eg2.valname2idx('cte0')]
                    lst_cte2 = obj_eg2.data[obj_eg2.valname2idx('cte2')]
                    lst_cte4 = obj_eg2.data[obj_eg2.valname2idx('cte4')]
                    lst_cte6 = obj_eg2.data[obj_eg2.valname2idx('cte6')]
                    lst_coes = list(zip(lst_cte0, lst_cte2, lst_cte4, lst_cte6))
                    target_idx  = obj_eg2.value2idx(target)                    # thomson's tidx for value at tcal.
                    target_time = obj_eg2.idx2value(target_idx)
                    raw_Te       = []                                          # ece raw data for Te (non calibrated value)
                    delta_time   = 0.016
                    ece_pos      = obj_eg1.position                            # Rename a function
                    ece_rlist    = obj_eg1.get_rlist()                         # R data
                    ece_timelist = obj_eg1.get_timelist()                      # Time data
                    ece_data     = obj_eg1.get()                               # sumv
                    for ridx, r in enumerate(ece_rlist):
                        ece_target_idx = obj_eg1.eg.value2idx(target, 0)
                        reff_idx       = obj_eg1.valname2idx('reff')
                        a99_idx        = obj_eg1.valname2idx('a99')
                        vidx           = obj_eg1.valname2idx('Te')             # index for ece raw data
                        cf_vidx        = obj_eg1.valname2idx('calib')          # index for ece raw data
                        reff           = ece_data[ece_pos(ece_target_idx, ridx, reff_idx)]
                        a99            = ece_data[ece_pos(ece_target_idx, ridx, a99_idx)]
                        rho            = reff / a99 if a99 != 0.0 else 0.0
                        Te             = self.calc_Te_at(rho, lst_coes[target_idx])         # thomson Te at near the target time.
                        vlist = []
                        for tidx, tm in enumerate(ece_timelist):
                            if target_time - delta_time < tm < target_time + delta_time: 
                                v  = ece_data[ece_pos(tidx, ridx, vidx)]
                                cf = ece_data[ece_pos(tidx, ridx, cf_vidx)]
                                vlist.append(v / cf)                                        # ece rawdata
                                
                        rawTe = sum(vlist) / len(vlist)
                        lst_factor.append(Te / rawTe)
                        
        return lst_factor

    def searchTimeIndex(self, tm, timelist):
        tidx0 = 0
        for i, t in enumerate(timelist):
            tidx1 = i
            if t >= tm:
                break
                
            tidx0 = tidx1
            
        return tidx0, tidx1

    def calc_Te_at(self, rho, coes):
        #print rho, coes
        return coes[0] + coes[1] * rho ** 2 + coes[2] * rho ** 4 + coes[3] * rho ** 6

class egdb3d_avg():
    def __init__(self, filepath='temp.dat', tdic={}, avgnum=1):
        self.eg = egdb3d(filepath, tdic=tdic)
        self.eg.readFile()
        
        self.add_valnames = []
        
        self.ndatat = self.eg.getDimSize(0)
        self.ndatar = self.eg.getDimSize(1)
        self.ndatav = self.eg.getValNo() + len(self.add_valnames)
        self.tnames = []
        self.maxcount = 0
        self.sumv = []
        self.timelist = self.eg.dims(0)
        self.rlist = self.eg.dims(1)
        self.avgnum = avgnum
        self.get_AverageIntime(self.avgnum)
        
        self.unitT = self.eg.dimunit(0)
        self.unitR = self.eg.dimunit(1)

    def valname2idx(self, valname):
        if valname in self.add_valnames:
            idx = self.eg.getValNo() + self.add_valnames.index(valname)
        else:
            idx = self.eg.valname2idx(valname)
        return idx

    def idx2valunit(self, idx):
        return self.eg.idx2valunit(idx)

    def get_avgnum(self):
        return self.avgnum

    def get_time(self, c):
        return self.timelist[c]

    def get_r(self, c):
        return self.rlist[c]

    def get_timelist(self):
        return self.timelist

    def get_rlist(self):
        return self.rlist

    def get_unitR(self):
        return self.unitR

    def get_unitT(self):
        return self.unitT

    def get(self):
        return self.sumv

    def position(self, idxt, idxr, idxv):
        npos = idxv + self.ndatav * idxr + self.ndatav * self.ndatar * idxt
        return npos

    def get_unmask(self):
        maskval = False
        if maskval:
            vidx = self.eg.valname2idx(maskval)
            idxr = self.eg.value2idx(3.6, 1)
            idxlist = []
            for idxt in range(self.ndatat):
                v = self.eg.data[self.eg.position(idxt, idxr, vidx)]
                if v != 0.0:
                    idxlist.append(idxt)
        else:
            idxlist = list(range(self.ndatat))
            
        return idxlist

    def get_AverageIntime(self, avgnum=1):
        timelist = self.eg.dims(0)
        unmask   = self.get_unmask()
        nunmask  = len(unmask)
        maxcount = int(nunmask//avgnum)
        sumv  = [0.0 for c in range(self.ndatav * self.ndatar * maxcount)]
        tlist = [0.0 for c in range(maxcount)]
        
        start = 0
        stop = start + avgnum
        count = 0
        while(stop <= nunmask):
            for i in range(start, stop):
                idxt = unmask[i]
                tm = timelist[idxt]
                tlist[count] = tlist[count] + tm
            tlist[count] = tlist[count] / float(avgnum)
            
            start = stop
            stop = start + avgnum
            count = count + 1

        start = 0
        stop = start + avgnum
        count = 0
        data = self.eg.data
        pos = self.eg.position
        while(stop <= nunmask):
            for idxr in range(self.ndatar):
                for vidx in range(self.ndatav):
                    npos = self.position(count, idxr, vidx)
                    idx = vidx - self.eg.getValNo()
                    for i in range(start, stop):
                        idxt = unmask[i]
                        if idx < 0: # from original eg
                            v = data[pos(idxt, idxr, vidx)]
                        else:
                            pass
                        sumv[npos] = sumv[npos] + v
                    sumv[npos] = sumv[npos]/float(avgnum)
            start = stop
            stop = start + avgnum
            count = count + 1
            
        self.sumv = sumv
        self.timelist = tlist
        self.maxcount = maxcount
