#!/usr/bin/env python
# coding: utf-8
import re
import math
import numpy as np
from ftplib import FTP
import zipfile
import os
from decimal import Decimal

def ftpGet(targetpath, outputpath):
    ftp = FTP('egftp1.lhd.nifs.ac.jp')
    ftp.login()
    cmd = 'RETR %s' % targetpath
    fpo = open(outputpath, 'wb')
    ftp.retrbinary(cmd, fpo.write)
    fpo.close()
    ftp.quit()

def ftpList(targetpath):
    ftp = FTP('egftp1.lhd.nifs.ac.jp')
    ftp.login()
    flist = ftp.nlst(targetpath)
    ftp.quit()
    return flist

def unzip(targetfile):
    zf = zipfile.ZipFile(targetfile, 'r')
    flist = []
    for target in zf.namelist():
        try:
            data = zf.read(target)
            fpo = open(target, 'wb')
            fpo.write(data)
            fpo.close()
            flist.append(target)
        except:
            data = None
    zf.close()
    os.remove(targetfile)
    return flist
    

def igetfile(diagname, shotno, subshot, outputname):
    targetfolderpath = 'data/'
    targetfolderpath = targetfolderpath + diagname + '/'
    firstshotno = int(shotno / 1000) * 1000
    firstfolder = '%06d/' % firstshotno
    targetfolderpath = targetfolderpath + firstfolder
    secondfolder = '%06d/' % shotno
    targetfolderpath = targetfolderpath + secondfolder
    subfolder = '%06d' % subshot
    targetfolderpath = targetfolderpath + subfolder
    #print targetfolderpath
    filelist = ftpList(targetfolderpath)
    if 0 == len(filelist):
        return None
    for fn in filelist:
        if fn[-4:].upper() == '.ZIP':
            targetpath = fn
            targetfile = targetpath.split('/')[-1]
            print(targetpath)
            print(targetfile)
            ftpGet(targetpath, targetfile)
            files = unzip(targetfile)
            os.rename(files[0], outputname) 
    return outputname


class egdata():
    """
    egdata() is pure python module for parsing EGDB formatted file.
    egdata()はEGDB形式のファイルを分解して読み込むためのモジュールです。

    if you want to open 'temp.dat', make instance of the file with edgb(), 
    and then call readFile() method.
    'temp.dat'というファイルを開きたい場合は、egdata()を使ってインスタンスを作成し、
    readFile()メソッドを実行します。
    ex)
        egfile = egdata('temp.dat')
        egfile.readFile()

    """
    def __init__(self, fname=None, delim=',', tdic={}):
        """
        fname    full path of the file you want to open.
                 開きたいファイルのフルパスを設定します。
        delim    delimiter of the data.
                 データの区切り文字を指定します。
        tdic     the dectionary for translation from non-numeric value 
                 to the value in the dictionary.
                 非数値の文字を、辞書を使って変換します。
                 ex) tdic = {'+Inf': '99999.9', '-Inf': '-99999.9'}
        """
        self.ana_file_name      = fname
        self.delim              = delim
        self.parameters         = ''
        self.comments           = ''
        self.ana_name           = ''
        self.ana_shot_no        = ''
        self.ana_date           = ''
        self.ana_dim_no         = 0
        self.arry_ana_dim_size  = []
        self.arry_ana_dim_name  = []
        self.arry_ana_dim_unit  = []
        self.ana_val_no         = 0
        self.arry_ana_val_name  = []
        self.arry_ana_val_unit  = []
        self.parametersDict     = {}
        self.parametersFlg      = False
        self.commentsFlg        = False
        self.dataFlg            = False
        self.arry_ana_dim_data  = []      # Dim Data配列
        self.arry_ana_val_data  = []      # Val Data配列
        self.ana_data_size      = 0       # 解析データサイズ
        self.transdic           = tdic    # 辞書
        self.reobjTitle         = re.compile(r'\s*\[(.+)\]\s*')
        self.reobjValue         = re.compile(r'(.+?)\s*=\s*(.+)')

    def parseLine(self, line):
        """
        parsing a line of headder and getting the size of the data.
        ヘッダを解析して、メモリサイズを得て、適切なサイズのリストを作成する
        """
        #reobjTitle = re.compile(r'\s*\[(.+)\]\s*')
        m = self.reobjTitle.match(line)
        if m:
            idx = m.groups()[0].upper()
            if 'PARAMETERS' == idx:
                self.parametersFlg = True
                self.commentsFlg = False
                self.dataFlg = False 
            if 'COMMENTS' == idx:
                self.parametersFlg = False
                self.commentsFlg = True
                self.dataFlg = False 
            if 'DATA' == idx:
                self.parametersFlg = False
                self.commentsFlg = False 
                self.dataFlg = True
            return

        #reobjValue = re.compile(r'(.+)\s*=\s*(.+)')
        m = self.reobjValue.match(line)
        if m:
            #print m.groups()
            nm = m.groups()[0].upper()
            nm = nm.strip()
            vl = m.groups()[1]
            if 'NAME' == nm:
                clm = vl.strip()
                clm = clm.strip('\'')
                self.ana_name = clm
            if 'SHOTNO' == nm:
                clm = vl.strip()
                self.ana_shot_no = clm
            if 'DATE' == nm:
                clm = vl.strip()
                clm = clm.strip('\'')
                self.ana_date = clm
            if 'DIMNO' == nm:
                self.ana_dim_no = int(vl)
#                for i in range(self.ana_dim_no):
#                    self.idx.append({})
#                    self.invidx.append({})
#                    self.idxn.append(-1)
#                    self.dimbackup.append('')
            if 'DIMSIZE' == nm:
                for i in range(self.ana_dim_no):
                    clm = vl.split(',')
                    self.arry_ana_dim_size.append(int(clm[i]))
                    
                self.ana_data_size = sum(self.arry_ana_dim_size)
                #
                # DIMデータの初期データを作成する
#                for i in range(self.ana_dim_no):
#                    self.arry_ana_dim_data[i] = [0.0] * self.ana_data_size
                    
            if 'DIMNAME' == nm:
                for i in range(self.ana_dim_no):
                    clm = vl.split(',')
                    clmd = clm[i].strip()
                    clmd = clmd.strip('\'')
                    clmd = clmd.strip()
                    self.arry_ana_dim_name.append(clmd)
                    self.arry_ana_dim_data.append([])
            if 'DIMUNIT' == nm:
                for i in range(self.ana_dim_no):
                    clm = vl.split(',')
                    clmd = clm[i].strip()
                    clmd = clmd.strip('\'')
                    clmd = clmd.strip()
                    self.arry_ana_dim_unit.append(clmd)
            if 'VALNO' == nm:
                self.ana_val_no = int(vl)
                for i in range(self.ana_val_no):
                    self.arry_ana_val_data.append([])
                    
                #
                # VALデータの初期データを作成する
#                for i in range(self.ana_val_no):
#                    self.arry_ana_val_data[i] = [0.0] * self.ana_data_size
                    
            if 'VALNAME' == nm:
                for i in range(self.ana_val_no):
                    clm = vl.split(',')
                    clmd = clm[i].strip()
                    clmd = clmd.strip('\'')
                    clmd = clmd.strip()
                    self.arry_ana_val_name.append(clmd)
            if 'VALUNIT' == nm:
                for i in range(self.ana_val_no):
                    clm = vl.split(',')
                    clmd = clm[i].strip()
                    clmd = clmd.strip('\'')
                    clmd = clmd.strip()
                    self.arry_ana_val_unit.append(clmd)
                    
            self.parametersDict[nm] = vl

    def makeIndex(self, idxlist):
        """
        calculate the position in the list data stored from the list of indexs.
        各変数に対するインデックス番号のリストからメモリ位置を示すインデックスを得る
        ex) get value at [5, 10, 1]
            pos = makeIndex([5, 10])
            v = egfile.data[1][pos]

        """
        pos = 0
        for i in range(self.ana_dim_no):
            weight = 1
            for j in range(i):
                weight = weight * self.arry_ana_dim_size[j]
            pos = pos + weight * idxlist[i]
        return pos

    def readFile(self):
        """
        reading file.
        ファイルの読み込みを実行する
        """
        """
        reading file.
        ファイルの読み込みを実行する
        """
        lines = []
        if self.ana_file_name[-3:].upper() == 'ZIP':
            # open zip file
            #zf = zipfile.ZipFile(self.ana_file_name, 'r')
            #targetname = zf.namelist()[0]
            #buff = zf.read(targetname)
            #zf.close()
            #lines = buff.splitlines()
            with zipfile.ZipFile(self.ana_file_name, 'r') as zip_data:
                # ファイルリスト取得
                infos = zip_data.infolist()
                for info in infos:
                    # zipからファイルデータを読み込む
                     with zip_data.open(info.filename, 'r') as fp:
                         for line_data in fp:
                             if type(line_data) is str :
                                 lines.append(line_data)
                             else :
                                 lines.append(line_data.decode())
                                 
        else:
            fp = open(self.ana_file_name, 'r')
            lines = fp.readlines()
            fp.close()
            
        for l_data in lines:
            l_data = l_data.strip('\n')
            if len(l_data) == 0: continue
            if l_data[0] == '#':
                self.parseLine(l_data[1:])
                if self.parametersFlg:
                    self.parameters = self.parameters + l_data +'\n'
                if self.commentsFlg:
                    self.comments = self.comments + l_data +'\n'
            else:
                clm_data = l_data.split(self.delim) # データを区切り文字で分割する
                #
                # Dimデータを格納する
                for idx in range(self.ana_dim_no):
                    vm   = np.nan
                    vstr = clm_data[idx].strip(' ')
                    if ( vstr != '' ) :
                        try:
                            vm = float(vstr)
                        except ValueError:
                            vm = no.nan
                            
                    self.arry_ana_dim_data[idx].append(vm)
                #
                # Valデータを格納する
                for idx in range(self.ana_val_no):
                    vm   = np.nan
                    vstr = clm_data[(idx + self.ana_dim_no)].strip(' ')
                    if ( vstr != '' ) :
                        try:
                            vm = float(vstr)
                        except ValueError:
                            vm = np.nan
                            
                    self.arry_ana_val_data[idx].append(vm)
                    

    def getName(self):
        """
        return the name of diagnostic
        """
        return self.ana_name

    def getShotNo(self):
        """
        return the shot number written in the file.
        """
        return self.ana_shot_no

    def getDate(self):
        """
        return the DATE written in the file.
        """
        return self.ana_date

    def getDimNo(self):
        """
        return the number of dimensions.
        """
        return self.ana_dim_no

    def getDimNo(self, dim):
        """
        return the size of dimensions.
        """
        return self.arry_ana_dim_size[dim]

    def getValNo(self):
        """
        return the number of variables (Values).
        """
        return self.ana_dim_no
    
    def getParameters(self):
        """
        return the block of the parameters.
        パラメータブロックを得る
        """
        return self.parameters

    def getComments(self):
        """
        return the block of the comments.
        コメントを得る
        """
        return self.comments

    def getParameterByName(self, name):
        """
        return the parameter in the comment block indicated by 'name ='.
        """
        nameu = name.upper()
        return self.parametersDict.get(nameu)

    def getDimNames(self):
        """
        get the list of the dimenstion names. 
        次元の名前のリストを得る
        """
        return self.arry_ana_dim_name

    def getDimName2Data(self, dim_name):
        """
        Get a data array for a dim name.
        次元名からそのデータ配列を取得する
        """
        idx = self.dimname2idx( dim_name )
        if ( -1 < idx ):
            arry_data = self.arry_ana_dim_data[idx]
        else:
            arry_data = []
        return arry_data

    def getDimNearestValue(self, dim_name, search_val):
        """
        Returns the value of the Dim name that is closest to the specified value.
        次元名に対するデータの内、指定した値に最も近い値を返す
          @param dim_name   : 次元名
          @param search_val : 対象値
          @return           : 対象値に最も近い値
        """
        return self.getNearestValue( self.getDimName2Data(dim_name), search_val )

    def getDimSizes(self):
        """
        get the list of the dimenstion sizes. 
        次元のサイズのリストを得る
        """
        return self.arry_ana_dim_size


    def getValNames(self):
        """
        get the list of the variable (Value) names. 
        変数の名前のリストを得る
        """
        return self.arry_ana_val_name

    def getValName2Data(self, val_name):
        """
        Get a data array for a val name.
        変数(値)名からそのデータ配列を取得する
        """
        idx = self.valname2idx( val_name )
        if ( -1 < idx ):
            arry_data = self.arry_ana_val_data[idx]
        else:
            arry_data = []
        return arry_data

    def getValNearestValue(self, val_name, search_val):
        """
        Get a data array for a val name.
        変数(値)名に対するデータの内、指定した値に最も近い値を返す
          @param dim_name   : 変数(値)名
          @param search_val : 対象値
          @return           : 対象値に最も近い値
        """
        return self.getNearestValue( self.getValName2Data(dim_name), search_val )


    def getNearestValue(self, arry_data, search_val):
        """
        Returns the value of the Val name that is closest to the specified value.
        データ配列の要素の内、指定した値に最も近い値を返す
          @param arry_data  : データ配列
          @param search_val : 対象値
          @return           : 対象値に最も近い値
        """
        # リスト要素と対象値の差分を計算し最小値のインデックスを取得
        idx = np.abs(np.asarray(arry_data) - search_val).argmin()
        return arry_data[idx]

    def dimname2idx(self, dim_name):
        """
        get the index of the dimension by the name.
        次元名から、インデックスを得る
        次元名は大文字小文字を区別しない
        """
        idx = -1
        try:
            if ( 0 < len(self.arry_ana_dim_name) ):
                for didx, item in enumerate( self.arry_ana_dim_name ):
                    if dim_name.casefold() == item.casefold():
                        idx = didx
                        break
        except:
            idx = -1
            
        return idx

    def idx2dimname(self, idx):
        """
        get the dimension name by the index.
        インデックスから次元名を得る
        """
        return self.arry_ana_dim_name[idx]

    def idx2dimunit(self, idx):
        """
        get the unit of the dimension by the index.
        インデックスから次元単位を得る
        """
        return self.arry_ana_dim_unit[idx]

    def idx2dimtitle(self, idx):
        """
        get the title of the dimension by the index.
        the title is formatted as 'name(unit)'.
        インデックスから次元タイトルを得る
        """
        return self.idx2dimname(idx) + ' (' + self.idx2dimunit(idx) + ')'

    def valname2idx(self, val_name):
        """
        get the index of the variable (Value) by the name.
        変数(値)名から、インデックスを得る
        """
        try:
            idx = self.arry_ana_val_name.index(val_name)
        except:
            idx = -1
        return idx

    def idx2valname(self, idx):
        """
        get the variable (Value) name by the index.
        インデックスから変数(値)名を得る
        """
        return self.arry_ana_val_name[idx]

    def idx2valunit(self, idx):
        """
        get the unit of the variable (Value) by the index.
        インデックスから変数(値)の単位を得る
        """
        return self.arry_ana_val_unit[idx]

    def idx2valtitle(self, idx):
        """
        get the title of the variable (Value) by the index.
        the title is formatted as 'name(unit)'.
        インデックスから変数(値)タイトルを得る
        """
        return self.idx2valname(idx)+' ('+self.idx2valunit(idx)+')'

if __name__ == '__main__':
    egfile = egdata('temp.dat')
    egfile.readFile()
    print(egfile.ana_name)
    print(egfile.ana_shot_no)
    print(egfile.ana_date)
    print(egfile.ana_dim_no)
    print(egfile.arry_ana_dim_size)
    print(egfile.arry_ana_dim_name)
    print(egfile.arry_ana_dim_unit)
    print(egfile.ana_val_no)
    print(egfile.arry_ana_val_name)
    print(egfile.arry_ana_val_unit)
    print(egfile.parametersDict)
    print(egfile.getParameterByName('grating_mum'))
