#!/usr/bin/env python
# coding: utf-8
# $Id$

import re
import math
from ftplib import FTP
import zipfile
import os
from MyViewGlobal import logger

def ftpGet(targetpath, outputpath):
    ftp = FTP('egftp1.lhd.nifs.ac.jp')
    ftp.login()
    cmd = 'RETR %s' % targetpath
    fpo = open(outputpath, 'wb')
    ftp.retrbinary(cmd, fpo.write)
    fpo.close()
    ftp.quit()

def ftpList(targetpath):
    #print 'In ftpList %s' % targetpath
    ftp = FTP('egftp1.lhd.nifs.ac.jp')
    ftp.login()
    flist = ftp.nlst(targetpath)
    ftp.quit()
    return flist

def unzip(targetfile):
    zf = zipfile.ZipFile(targetfile, 'r')
    flist = []
    for target in zf.namelist():
        try:
            data = zf.read(target)
            fpo = open(target, 'wb')
            fpo.write(data)
            fpo.close()
            flist.append(target)
        except:
            data = None
    zf.close()
    os.remove(targetfile)
    return flist

def icheckfile(diagname, shotno, subshot):
    targetfolderpath = 'data/'
    targetfolderpath = targetfolderpath + diagname + '/'
    firstshotno = int(shotno / 1000) * 1000
    firstfolder = '%06d/' % firstshotno
    targetfolderpath = targetfolderpath + firstfolder
    secondfolder = '%06d/' % shotno
    targetfolderpath = targetfolderpath + secondfolder
    subfolder = '%06d' % subshot
    targetfolderpath = targetfolderpath + subfolder
    #print targetfolderpath
    filelist = ftpList(targetfolderpath)
    if 0 == len(filelist):
        return False
    return True
    

def igetfile(diagname, shotno, subshot, outputname):
    targetfolderpath = 'data/'
    targetfolderpath = targetfolderpath + diagname + '/'
    firstshotno = int(shotno / 1000) * 1000
    firstfolder = '%06d/' % firstshotno
    targetfolderpath = targetfolderpath + firstfolder
    secondfolder = '%06d/' % shotno
    targetfolderpath = targetfolderpath + secondfolder
    subfolder = '%06d' % subshot
    targetfolderpath = targetfolderpath + subfolder
    #print targetfolderpath
    filelist = ftpList(targetfolderpath)
    if 0 == len(filelist):
        return None
    for fn in filelist:
        if fn[-4:].upper() == '.ZIP':
            targetpath = fn
            targetfile = targetpath.split('/')[-1]
            print(targetpath)
            print(targetfile)
            ftpGet(targetpath, targetfile)
            files = unzip(targetfile)
            os.rename(files[0], outputname) 
            break
    return outputname

def igetfilez(diagname, shotno, subshot, outputname):
    targetfolderpath = 'data/'
    targetfolderpath = targetfolderpath + diagname + '/'
    firstshotno = int(shotno / 1000) * 1000
    firstfolder = '%06d/' % firstshotno
    targetfolderpath = targetfolderpath + firstfolder
    secondfolder = '%06d/' % shotno
    targetfolderpath = targetfolderpath + secondfolder
    subfolder = '%06d' % subshot
    targetfolderpath = targetfolderpath + subfolder
    #print targetfolderpath
    filelist = ftpList(targetfolderpath)
    if 0 == len(filelist):
        return None
    for fn in filelist:
        if fn[-4:].upper() == '.ZIP':
            targetpath = fn
            targetfile = targetpath.split('/')[-1]
            print(targetpath)
            print(targetfile)
            ftpGet(targetpath, targetfile)
            os.rename(targetfile, outputname) 
            break
    return outputname

class egdb():
    """
    egdb() is pure python module for parsing EGDB formatted file.
    egdb()はEGDB形式のファイルを分解して読み込むためのモジュールです。

    if you want to open 'temp.dat', make instance of the file with edgb(), 
    and then call readFile() method.
    'temp.dat'というファイルを開きたい場合は、egdb()を使ってインスタンスを作成し、
    readFile()メソッドを実行します。
    ex)
        egfile = egdb('temp.dat')
        egfile.readFile()

    """
    def __init__(self, fname=None, delim=',', tdic={}):
        """
        fname    full path of the file you want to open.
                 開きたいファイルのフルパスを設定します。
        delim    delimiter of the data.
                 データの区切り文字を指定します。
        tdic     the dectionary for translation from non-numeric value 
                 to the value in the dictionary.
                 非数値の文字を、辞書を使って変換します。
                 ex) tdic = {'+Inf': '99999.9', '-Inf': '-99999.9'}
        """
        self.fileName = fname
        self.delim = delim
        self.parameters = ''
        self.comments = ''
        self.Name = ''
        self.ShotNo = ''
        self.Date = ''
        self.DimNo=0
        self.DimSize=[]
        self.DimName=[]
        self.DimUnit=[]
        self.ValNo=0
        self.ValName=[]
        self.ValUnit=[]
        self.parametersDict = {}
        self.parametersFlg = False
        self.commentsFlg = False
        self.dataFlg = False
        self.idx = []
        self.invidx = []
        self.idxn = [] #各Dimのインデックスカウンタ
        self.data = []
        self.dataSize = 0
        #self.dimbackup = [] # 前の行のDimを保存しておく
        self.transdic = tdic
        self.reobjTitle = re.compile(r'\s*\[(.+)\]\s*')
        self.reobjValue = re.compile(r'(.+?)\s*=\s*(.+)')
        self.linecount = 0

    def parseLine(self, line):
        """
        parsing a line of headder and getting the size of the data.
        ヘッダを解析して、メモリサイズを得て、適切なサイズのリストを作成する
        """
        #reobjTitle = re.compile(r'\s*\[(.+)\]\s*')
        m = self.reobjTitle.match(line)
        if m:
            idx = m.groups()[0].upper()
            if 'PARAMETERS' == idx:
                self.parametersFlg = True
                self.commentsFlg = False
                self.dataFlg = False 
            if 'COMMENTS' == idx:
                self.parametersFlg = False
                self.commentsFlg = True
                self.dataFlg = False 
            if 'DATA' == idx:
                self.parametersFlg = False
                self.commentsFlg = False 
                self.dataFlg = True
            return

        #reobjValue = re.compile(r'(.+)\s*=\s*(.+)')
        m = self.reobjValue.match(line)
        if m:
            #print m.groups()
            nm = m.groups()[0].upper()
            nm = nm.strip()
            vl = m.groups()[1]
            if 'NAME' == nm:
                clm = vl.strip()
                clm = clm.strip('\'')
                self.Name = clm
            if 'SHOTNO' == nm:
                clm = vl.strip()
                self.ShotNo = clm
            if 'DATE' == nm:
                clm = vl.strip()
                clm = clm.strip('\'')
                self.Date = clm

            if 'DIMNO' == nm:
                self.DimNo = int(vl)
                for i in range(self.DimNo):
                    self.idx.append({})
                    self.invidx.append({})
                    self.idxn.append(0)
                    #self.dimbackup.append('')
            if 'DIMSIZE' == nm:
                for i in range(self.DimNo):
                    clm = vl.split(',')
                    if len(clm) != self.DimNo:
                        clm = re.split(' +?', vl)
                    self.DimSize.append(int(clm[i]))
            if 'DIMNAME' == nm:
                for i in range(self.DimNo):
                    clm = vl.split(',')
                    clmd = clm[i].strip()
                    clmd = clmd.strip('\'')
                    clmd = clmd.strip()
                    self.DimName.append(clmd)
            if 'DIMUNIT' == nm:
                for i in range(self.DimNo):
                    clm = vl.split(',')
                    clmd = clm[i].strip()
                    clmd = clmd.strip('\'')
                    clmd = clmd.strip()
                    self.DimUnit.append(clmd)
            if 'VALNO' == nm:
                self.ValNo = int(vl)
                for i in range(self.ValNo):
                    self.data.append([])
                    self.dataSize = 1
                    for j in range(self.DimNo):
                        self.dataSize = self.dataSize * self.DimSize[j]
                    for j in range(self.dataSize):
                        self.data[i].append(0.0)
            if 'VALNAME' == nm:
                for i in range(self.ValNo):
                    clm = vl.split(',')
                    clmd = clm[i].strip()
                    clmd = clmd.strip('\'')
                    clmd = clmd.strip()
                    self.ValName.append(clmd)
            if 'VALUNIT' == nm:
                for i in range(self.ValNo):
                    clm = vl.split(',')
                    clmd = clm[i].strip()
                    clmd = clmd.strip('\'')
                    clmd = clmd.strip()
                    self.ValUnit.append(clmd)
            self.parametersDict[nm] = vl

    def makeIndex(self, idxlist):
        """
        calculate the position in the list data stored from the list of indexs.
        各変数に対するインデックス番号のリストからメモリ位置を示すインデックスを得る
        ex) get value at [5, 10, 1]
            pos = makeIndex([5, 10])
            v = egfile.data[1][pos]

        """
        pos = 0
        for i in range(self.DimNo):
            weight = 1
            for j in range(i):
                weight = weight * self.DimSize[j]
            pos = pos + weight * idxlist[i]
        return pos

        
    def parseData(self, line):
        """
        parsing a line of data and set the value at the proper postion in the list.
        1行分のデータを変数の適切なメモリ位置に登録していく
        """
        clm = line.split(self.delim)
        idxl = []
        #dimbackup = []
        linecount = self.linecount

        for i in range(self.DimNo):
            if i < self.DimNo-1:
                blocksize = self.DimSize[i-1]
            else:
                blocksize = 1

            idxl.append(self.idxn[i])
            label = "%s_%d" % (clm[i], self.idxn[i])
            self.idx[i][label] = self.idxn[i] #辞書に登録
            self.invidx[i][self.idxn[i]] = clm[i] #逆引き辞書
            if 0 == ((linecount+1) % blocksize): #つぎのブロックに移動したら
                self.idxn[i] = self.idxn[i]+1
                if self.idxn[i] >= self.DimSize[i]: #カウンターがオーバーしたら
                    self.idxn[i] = 0

        #self.dimbackup = dimbackup
       
        pos = self.makeIndex(idxl)

        for i in range(self.ValNo):
            vstr = clm[i + self.DimNo].strip(' ')
            #vstr = vstr.strip('\n')
            #if vstr[-1] == '\n':
            #    vstr = vstr[:-1]
            #if self.transdic.has_key(vstr):
            #    vstr = self.transdic[vstr]
            vstr = self.transdic.get(vstr, vstr)
            try:
                d = float(vstr)
            except(ValueError):
                logger.error("Error(%d): Could not convert string to float '%s'" % (linecount+1, vstr))
                d = 0.0
            self.data[i][pos] = d

        self.linecount = self.linecount + 1
        

    def readFile(self):
        """
        reading file.
        ファイルの読み込みを実行する
        """
        if self.fileName[-3:].upper() == 'ZIP':
            # open zip file
            zf = zipfile.ZipFile(self.fileName, 'r')
            targetname = zf.namelist()[0]
            buff = zf.read(targetname)
            zf.close()
            lines = buff.split('\n')
        else:
            fp = open(self.fileName, 'r')
            lines = fp.readlines()
            fp.close()

        self.linecount = 0

        for l in lines:
            l = l.strip('\n')
            if len(l) == 0: continue
            if l[0] == '#':
                self.parseLine(l[1:])
                if self.parametersFlg:
                    self.parameters = self.parameters+l+'\n'
                if self.commentsFlg:
                    self.comments = self.comments+l+'\n'
            else:
                self.parseData(l)

    def getName(self):
        """
        return the name of diagnostic
        """
        return self.Name

    def getShotNo(self):
        """
        return the shot number written in the file.
        """
        return self.ShotNo

    def getDate(self):
        """
        return the DATE written in the file.
        """
        return self.Date

    def getDimNo(self):
        """
        return the number of dimensions.
        """
        return self.DimNo

    def getDimSize(self, dim):
        """
        return the size of dimensions.
        """
        return self.DimSize[dim]

    def getValNo(self):
        """
        return the number of variables (Values).
        """
        return self.ValNo
    
    def getParameters(self):
        """
        return the block of the parameters.
        パラメータブロックを得る
        """
        return self.parameters

    def getComments(self):
        """
        return the block of the comments.
        コメントを得る
        """
        return self.comments

    def getParameterByName(self, name):
        """
        return the parameter in the comment block indicated by 'name ='.
        """
        nameu = name.upper()
        return self.parametersDict.get(nameu)
        #if self.parametersDict.has_key(nameu):
        #    return self.parametersDict[nameu]
        #return None

    def value2idx(self, dim, v):
        """
        get an index value of the dimension at the value near the argument value.
        ある次元dimの値vに近い場所のインデックス値を得る

        ex) idx = value2idx(0, 3.4)

        """
        keylist = list(self.idx[dim].keys())
        klist = [k.split('_')[0] for k in keylist]
        ilist = [int(k.split('_')[1]) for k in keylist]
        idx = []
        dlist = []
        min = math.sqrt((v - float(klist[0]))**2)
        dlist.append(min)
        for i in range(1, len(klist)):
            d = math.sqrt((v - float(klist[i]))**2)
            dlist.append(d)
            if d < min:
                min = d
        i = 0
        for d in dlist:
            if d == min:
                idx.append(ilist[i])
            i = i + 1
        return idx
    
    def idx2value(self, dim, idx):
        """
        get the value at the idx of the dimmension.
        ある次元dimの実際の値をインデックス値から得る

        ex) time = idx2value(self, 0, 10)
        """
        v = self.invidx[dim][idx]
        return float(v)

    def dims(self, dim):
        """
        get the list of value in the dimension.
        次元dimの値のリストをインデックス順に得る
        """
        lst = []
        for i in range(len(self.invidx[dim])):
            lst.append(float(self.invidx[dim][i]))
        return lst

    def DimNames(self):
        """
        get the list of the dimenstion names. 
        次元の名前のリストを得る
        """
        return self.DimName

    def DimUnits(self):
        """
        get the list of the dimenstion names. 
        次元の単位のリストを得る
        """
        return self.DimUnit

    def ValNames(self):
        """
        get the list of the variable (Value) names. 
        変数の名前のリストを得る
        """
        return self.ValName

    def ValUnits(self):
        """
        get the list of the units of variable (Value). 
        変数の単位のリストを得る
        """
        return self.ValUnit

    def dimname2idx(self, name):
        """
        get the index of the dimension by the name.
        次元名から、インデックスを得る
        """
        for i in range(len(self.DimName)):
            dn = self.DimName[i].upper()
            name = name.upper()
            if dn == name:
                return i
        return -1
    
    def valname2idx(self, name):
        """
        get the index of the variable (Value) by the name.
        変数(値)名から、インデックスを得る
        """
        for i in range(len(self.ValName)):
            vn = self.ValName[i].upper()
            name = name.upper()
            if vn == name:
                return i
        return -1

    def idx2dimname(self, idx):
        """
        get the dimension name by the index.
        インデックスから次元名を得る
        """
        return self.DimName[idx]

    def idx2dimunit(self, idx):
        """
        get the unit of the dimension by the index.
        インデックスから次元単位を得る
        """
        return self.DimUnit[idx]

    def idx2dimtitle(self, idx):
        """
        get the title of the dimension by the index.
        the title is formatted as 'name(unit)'.
        インデックスから次元タイトルを得る
        """
        return self.idx2dimname(idx)+' ('+self.idx2dimunit(idx)+')'

    def idx2valname(self, idx):
        """
        get the variable (Value) name by the index.
        インデックスから変数(値)名を得る
        """
        return self.ValName[idx]

    def idx2valunit(self, idx):
        """
        get the unit of the variable (Value) by the index.
        インデックスから変数(値)の単位を得る
        """
        return self.ValUnit[idx]

    def idx2valtitle(self, idx):
        """
        get the title of the variable (Value) by the index.
        the title is formatted as 'name(unit)'.
        インデックスから変数(値)タイトルを得る
        """
        return self.idx2valname(idx)+' ('+self.idx2valunit(idx)+')'

    def getData(self, idxlist):
        """
        get the data at the indexs in the idxlist.
        idxlistで指定されたデータを取得する
        ex) getting the data of variable (Value) 1 at (dim1, dim2) = (5, 10).
            v = egfile.getData([5, 10, 1])
        """
        if self.DimNo+1 != len(idxlist):
            return None
        pos = self.makeIndex(idxlist[:-1])
        val = idxlist[-1]
        return self.data[val][pos]


if __name__ == '__main__':
    egfile = egdb('temp.dat')
    egfile.readFile()
    print((egfile.Name))
    print((egfile.ShotNo))
    print((egfile.Date))
    print((egfile.DimNo))
    print((egfile.DimSize))
    print((egfile.DimName))
    print((egfile.DimUnit))
    print((egfile.ValNo))
    print((egfile.ValName))
    print((egfile.ValUnit))
    print((egfile.parametersDict))
    print((egfile.getParameterByName('grating_mum')))


    print((egfile.idxn))
    tlist = egfile.dims(0)

    for idxt, t in enumerate(tlist): 
        idx = [idxt, 5]
        pos = egfile.makeIndex(idx)
        ti = egfile.data[1][pos]
        print((idxt, t, pos, ti))


    #idxr = egfile.value2idx(1, 3.6)
    #print idxr
    #print len(egfile.idx[0].keys())
    #print len(egfile.idx[1].keys())
    #print len(egfile.invidx[0].keys())
    #print len(egfile.invidx[1].keys())
    #print egfile.invidx[1][idxr[0]]

    
    """
    print tsmap.idx2value(1, idxr)
    for i in range(tsmap.DimSize[0]):
        idxl = [i, idxr]
        pos = tsmap.makeIndex(idxl)
        print "%d\t%8.5f\t%8.5f" % (i, tsmap.data[0][pos], tsmap.data[1][pos])

    print tsmap.dimname2idx('r')
    print tsmap.valname2idx('reff')
    print tsmap.idx2valtitle(6)
    print tsmap.idx2dimtitle(0)
    print tsmap.idx2dimtitle(1)

    print tsmap.DimNames()
    print tsmap.ValNames()
    print tsmap.dims(0)
    print tsmap.dims(1)
    """




