#!/usr/bin/env python
# coding: utf-8
# $Id$
from __future__ import print_function
import re
import math
from ftplib import FTP
import zipfile
import os
from MyViewGlobal import logger, globalError, variables
import datetime
import time
import numpy
import pg8000
import urllib
import warnings
warnings.filterwarnings("ignore",
                        "DB-API extension")

dimno_p = re.compile(r'#\s*DIMNO\s*=\s*(\d+)',re.IGNORECASE)
valno_p = re.compile(r'#\s*VALNO\s*=\s*(\d+)',re.IGNORECASE)
dimname_p = re.compile(r'#\s*DIMNAME\s*=\s*(.*)\s*',re.IGNORECASE)
valname_p = re.compile(r'#\s*VALNAME\s*=\s*(.*)\s*',re.IGNORECASE)

def getConnection():
    #config = variables['%CONFIG%']
    config = variables['%CONFIG%']
    # DEEP COPY REQUIRED
    dbhost=str(config['kserverdbhost'])
    dbname=str(config['kserverdbname'])
    dbuser=str(config['kserverdbuser'])
    dbpass=str(config['kserverdbpass'])
    dbport=config['kserverdbport']
    if  not dbport:
        dbport=5432
    #print "dbconfig : ('%s', '%s', '%s', '%s')" % (dbhost, dbname, dbuser, dbpass)

    conn = None
    try:
        conn = pg8000.connect(host=dbhost, user=dbuser, password=dbpass, database=dbname, port=dbport, timeout=10)
    except:
        import sys,traceback
        traceback.print_exc()
    return conn

def checkDBConnection():
    conn = None
    try:
        conn = getConnection()
        if ( conn ):
            ret = True
        else:
            ret = False
    finally:
        if conn:
            conn.close()
    return ret
        
def icheckfileWeb(api, diagname, shotno, subshot):
    url="%s?cmd=filecheck&diag=%s&shotno=%d&subno=%d" % (api, diagname, shotno, subshot)
    req = urllib.request.Request(url)
    try:
        res = urllib.request.urlopen(req)
        return True
    except:
        return False

def icheckfile(diagname, shotno, subshot):
    config = variables['%CONFIG%']

    if ( "webapi" in config ):
        return icheckfileWeb(config['webapi'], diagname, shotno, subshot)
        
    conn = None
    try:
        conn = getConnection()
        cursor = conn.cursor()
        sql = "SELECT shotno from _locationfileinfo_ where diagnostics = '%s' and shotno = %d and subshotno = %d" % (diagname, shotno, subshot)
        cursor.execute(sql)
        #print  'cursor.rowcont=',cursor.rowcount
        if cursor.rowcount > 0:
            ret = True
        else:
            ret = False
        cursor.close()
    except:
        ret = False
        globalError.set("DB Error")
    finally:
        if conn:
            conn.close()
    return ret

def registeredTimeWeb(api, diagname, shotno, subshot):
    url="%s?cmd=gettime&diag=%s&shotno=%d&subno=%d" % (api, diagname, shotno, subshot)
    try:
        with urllib.request.urlopen(url) as response:
            s = response.read().decode('utf8').strip()
            t=datetime.datetime.strptime(s,'%Y/%m/%d %H:%M:%S %Z')
            return t
    except:
        return False
    
def registeredTime(diagname, shotno, subshot):
    """
    解析サーバに登録された時刻を取得する。
    """
    config = variables['%CONFIG%']
    if ( "webapi" in config ):
        return registeredTimeWeb(config['webapi'], diagname, shotno, subshot)
        
    conn = None
    try:
        conn = getConnection()
        cursor = conn.cursor()
        sql = "SELECT registered FROM _locationfileinfo_ WHERE diagnostics = '%s' and shotno = %d and subshotno = %d" % (diagname, shotno, subshot)
        cursor.execute(sql)
        row = cursor.fetchone()
        (ret,) = row
    except:
        import sys, traceback
        logger.error(traceback.print_exc())
        ret = False
        globalError.set("DB Error")
    finally:
        if conn:
            conn.close()
    return ret

def convert_timestamp(dt):
    t = time.mktime(dt.timetuple()) + 3600 * 9
    return t

def compareTime(diagname, shotno, subshot, t_file):
    """
    ファイルのタイムスタンプとデータベース上の登録時刻を比較して、
    データベースの方が新しければ 真
    """
    if ( not t_file ):
        logger.debug("*** CACHE FILE DOES NOT EXIST ***")
        return False
    try:
        t_db = registeredTime(diagname, shotno, subshot)
        if ( not t_db ):
            logger.debug("*** CANNOT CONNECT TO THE DATABASE ***")
            return False
        #print 't_db=',t_db
        t_db = convert_timestamp(t_db)
        #print 't_file=',datetime.datetime.fromtimestamp(t_file)

        if ( t_db > t_file ):
            logger.debug("*** THE CACHE IS OLDER THAN THAT IN DATABASE ***")
            return True
        else:
            logger.debug("*** THE CACHE IS NEWER THAN THAT IN DATABASE ***")
            return False
    except:
        import sys, traceback
        logger.error(traceback.print_exc())
        return False

def ftpGet(targetpath, outputpath):
    ftp = FTP('egftp1.lhd.nifs.ac.jp')
    ftp.login()
    cmd = 'RETR %s' % targetpath
    fpo = open(outputpath, 'wb')
    ftp.retrbinary(cmd, fpo.write)
    fpo.close()
    ftp.quit()

def ftpList(targetpath):
    #print 'In ftpList %s' % targetpath
    ftp = FTP('egftp1.lhd.nifs.ac.jp')
    ftp.login()
    flist = ftp.nlst(targetpath)
    ftp.quit()
    return flist

def unzip(targetfile):
    zf = zipfile.ZipFile(targetfile, 'r')
    flist = []
    for target in zf.namelist():
        try:
            data = zf.read(target)
            fpo = open(target, 'wb')
            fpo.write(data)
            fpo.close()
            flist.append(target)
        except:
            data = None
    zf.close()
    os.remove(targetfile)
    return flist

def icheckfile_(diagname, shotno, subshot):
    targetfolderpath = 'data/'
    targetfolderpath = targetfolderpath + diagname + '/'
    firstshotno = int(shotno / 1000) * 1000
    firstfolder = '%06d/' % firstshotno
    targetfolderpath = targetfolderpath + firstfolder
    secondfolder = '%06d/' % shotno
    targetfolderpath = targetfolderpath + secondfolder
    subfolder = '%06d' % subshot
    targetfolderpath = targetfolderpath + subfolder
    #print targetfolderpath
    filelist = ftpList(targetfolderpath)
    if 0 == len(filelist):
        return False
    return True
    

def igetfile(diagname, shotno, subshot, outputname):
    cmd = 'igetfile -s %d -m %d -d %s -o %s' % (shotno, subshot, diagname, outputname)
    logger.debug(cmd)
    os.system(cmd)
    return outputname

def igetfile_(diagname, shotno, subshot, outputname):
    targetfolderpath = 'data/'
    targetfolderpath = targetfolderpath + diagname + '/'
    firstshotno = int(shotno / 1000) * 1000
    firstfolder = '%06d/' % firstshotno
    targetfolderpath = targetfolderpath + firstfolder
    secondfolder = '%06d/' % shotno
    targetfolderpath = targetfolderpath + secondfolder
    subfolder = '%06d' % subshot
    targetfolderpath = targetfolderpath + subfolder
    #print targetfolderpath
    filelist = ftpList(targetfolderpath)
    if 0 == len(filelist):
        return None
    for fn in filelist:
        if fn[-4:].upper() == '.ZIP':
            targetpath = fn
            targetfile = targetpath.split('/')[-1]
            print(targetpath)
            print(targetfile)
            ftpGet(targetpath, targetfile)
            files = unzip(targetfile)
            os.rename(files[0], outputname) 
            break
    return outputname

def igetfilez(diagname, shotno, subshot, outputname):
    targetfolderpath = 'data/'
    targetfolderpath = targetfolderpath + diagname + '/'
    firstshotno = int(shotno / 1000) * 1000
    firstfolder = '%06d/' % firstshotno
    targetfolderpath = targetfolderpath + firstfolder
    secondfolder = '%06d/' % shotno
    targetfolderpath = targetfolderpath + secondfolder
    subfolder = '%06d' % subshot
    targetfolderpath = targetfolderpath + subfolder
    #print targetfolderpath
    filelist = ftpList(targetfolderpath)
    if 0 == len(filelist):
        return None
    for fn in filelist:
        if fn[-4:].upper() == '.ZIP':
            targetpath = fn
            targetfile = targetpath.split('/')[-1]
            print(targetpath)
            print(targetfile)
            ftpGet(targetpath, targetfile)
            os.rename(targetfile, outputname) 
            break
    return outputname

def sort(x, xe, y, ye):
    """
    x の昇順にソートし、同じ順序で xe, y, ye を並べ替える
    """
    x = numpy.array(x)
    xe = numpy.array(xe)
    y = numpy.array(y)
    ye = numpy.array(ye)
    idx = x.argsort()
    x = x[idx]
    y = y[idx]
    if ( len(xe) > 0):
        xe = xe[idx]
    if ( len(ye) > 0 ):
        ye = ye[idx]
    return [x, xe, y, ye]

def get_index(indexes, index):
    """
    index の配列 indexes の中から val に近い要素を返す
    (例)
    indexes = ["0_0.0 s", "1_1.0 s", "2_2.0 s", "3_3.0 s"]
    index = "1_1.2 s"

    返値: '1_1.0 s"
    """
    try:
        midx = indexes.index(index)
        return indexes[midx]
    except:
        pass

    fmt = re.compile(r'(.*)_([\+|\-]{0,1}[0-9]*\.*[0-9]*e*[\+|\-]{0,1}[0-9]*)(.*)')
    m = fmt.match(index)
    if not m:
        return None
    _index, _val, _unit = m.groups()
    val = float(_val)
    list_idxf = []
    list_idxv = []
    for idx in indexes:
        m = fmt.match(idx)
        if m:
            list_idxf.append(m.groups()[0])
            try:
                vd = float(m.groups()[1])
                list_idxv.append(vd)
            except ValueError:
                pass
#    if ( val < list_idxv[0] ):
#        return '%d_%s%s' % (int(list_idxf[-1])+1, _val, _unit)
#    if ( val > list_idxv[-1] ):
#        return '%d_%s%s' % (int(list_idxf[-1])+1, _val, _unit)

    list_d = [abs(val - v) for v in list_idxv]
    midx = list_d.index(min(list_d))
    return indexes[midx]

def parseSummary(summary):
    dimno, valno, dimname, valname = None, None, None, None
    for line in summary.split("\n"):
        m = re.search(dimno_p,line)
        if ( m ):
            dimno = int(m.group(1))
            next
        m = re.search(valno_p,line)
        if ( m ):
            valno = int(m.group(1))
            next
        m = re.search(dimname_p,line)
        if ( m ):
            dimname = m.group(1)
            next
        m = re.search(valname_p,line)
        if ( m ):
            valname = m.group(1)
            next
    return dimno, valno, dimname, valname

def getEGDBinfo():
    dic = {}
    try:
        conn = getConnection()
        cursor = conn.cursor()
        sql = """
select
  d.name,
  d.diagcomment,
  l1.shotno,
  l1.summary
from
  datalocation l1
join (
   select
     did,
     max(shotno) as max_shotno
   from
     datalocation l2
   group by
      did) as l3
  on l1.did = l3.did and l1.shotno = l3.max_shotno
join diagnostics d on (l1.did = d.id)
"""
        cursor.execute(sql)
        results = cursor.fetchall()
        for row in results:
            name, diagcomment, shotno, summary = row
            dimno, valno, dimname, valname = parseSummary(summary)
            if not diagcomment:
                diagcomment = ""
            if not summary:
                summry = ""
            diagcomment = diagcomment.replace('\n',' ')
            diagcomment = diagcomment.replace('\r',' ')
            dic[name] = {'DIMNO':dimno, 'VALNO':valno, 'DIMNAME':dimname, 'VALNAME':valname, 'COMMENT':diagcomment}
    except:
        import sys,traceback
        traceback.print_exc()
    return dic

if __name__ == '__main__':
    from MyViewConfig import *
    config = MyViewConfig('myView2.cnf')
    variables['%CONFIG%']=config
    h = getEGDBinfo()
    for key, val in h.items():
        print(key, val['COMMENT'], val['DIMNO'], val['VALNO'])

    
