#!/usr/bin/env python
# coding: utf-8
# $Id$
import os
import subprocess
import math
import re
import numpy as np
import scipy.interpolate
import zipfile
import io
import egdbcommon
import egdb2d
import egdb3d
from util import ParseShotNo
from MyViewGlobal import logger
#import dataSource

def helptext(title_name, script_name, data_source, synopsis, author_name, contact_to, plot_type, proc_summary, module_option):
    text = """
    <p>Here is an expanation of this module <p><br>
    Module name : %module_name% <br>
    Synopsis    : %synopsis% <br>
    Script name : %script_name% <br>
    Data source : %data_source% on EG server <br>
    Author name : %author_name% <br>
    Contact to  : %contact_to% <br>
    Plot type   : %plot_type%<br> 
    <br>
    %proc_summary%
    <p>Options<p><br>
    %module_option%
    """
    text = text.replace('%module_name%', title_name)
    text = text.replace('%script_name%', script_name)
    if type(data_source) is str :
        text = text.replace('%data_source%', data_source)
    elif  type(data_source) is list :
        text = text.replace('%data_source%', data_source[0])
    else :
        text = text.replace('%data_source%', title_name)
        
    text = text.replace('%synopsis%', synopsis)
    text = text.replace('%author_name%', author_name)
    text = text.replace('%contact_to%', contact_to)
    text = text.replace('%plot_type%', plot_type)
    text = text.replace('%proc_summary%', proc_summary)
    text = text.replace('%module_option%', module_option)
    return text

class kaisekiInterFace():
    def __init__(self, data_id, title_name, data_source, **kwargs):
        """
        dataid:          data id(shot number)
        diagname:        Measurement name for display.
        actual_name:     Measurement name for data access.
        kwargs:
            option:      option string for trace
            cache:       myview cache object
            cachebutton: state of button cache on myView.
                         "CACHE": 
                         "ONLINE":
                         "OFFLINE":
            temppath: temp path set with myView dialog.
            cmdpath: command path set with myView dialog
        if there is some troubole to make instance, raise "Error"
        """
        self.dataid          = data_id
        self.diagname        = title_name
        if type(data_source) is str :
            self.actual_name      = data_source
            self.list_actual_name = None
        elif  type(data_source) is list :
            self.actual_name      = None
            self.list_actual_name = data_source
            
        self.option          = kwargs.get('option')
        self.cache           = kwargs.get('cache')
        self.cachebutton     = kwargs.get('cachebutton')
        self.temppath        = kwargs.get('temppath')
        self.cmdpath         = kwargs.get('cmdpath')
        self.flgloaded       = False
        self.cachepath       = None
        self.eg              = None
        self.list_eg         = []
        self.avgnum          = 1

    def load(self, **kwargs):
        """
        Read the specified analysis data.
        """
        if self.flgloaded:
            return
            
        self.cachebutton = kwargs.get('cachebutton')
        shot, subshot = ParseShotNo(self.dataid)
        key = '%s_%s' % (self.actual_name, self.dataid)

        self.cachepath = None
        #self.cache.opencache()

        # try to get datafile from cache
        filetime = self.cache.findTime(key) 
        if self.cachebutton == 'ONLINE' and egdbcommon.compareTime(self.actual_name, shot, subshot, filetime):
            self.cachepath = None
            logger.debug("*** DISCARD CACHE FILE FOR %s@%s ***" % (self.actual_name, self.dataid))
        else:
            if self.flgloaded:
                return
                
            logger.debug("*** USE CACHE FILE FOR %s@%s ***" % (self.actual_name, self.dataid))
            self.cachepath = self.cache.find(key)
            
        if not self.cachepath and self.cachebutton == 'ONLINE':
            # try to get datafile from server
            logger.debug("load from network")
            fname = "%s.zip" % key
            fname = self.cache.makeUniqueFilename(fname, self.temppath)
            self.cachepath = os.path.join(self.temppath, fname) 
            cmdstr = "igetfile -s %d -m %d -d %s -Z -o %s" % (
                    shot, subshot, self.actual_name, self.cachepath) 
            #cmdstr = os.path.join(self.cmdpath, cmdstr)
            #print cmdstr
            if egdbcommon.icheckfile(self.actual_name, shot, subshot):
                subprocess.check_call(cmdstr,shell=True)
                if os.path.exists(self.cachepath):
                    logger.debug("Register the Datafile %s" % self.cachepath)
                    self.cachepath = self.cache.registFile(key, self.cachepath)
                else:
                    self.cachepath = None
            else:
                self.cachepath = None

    def load_target(self, target_source, **kwargs):
        """
        Read the specified analysis data (auxiliary data such as masks).
        """
        self.cachebutton = kwargs.get('cachebutton')
        shot, subshot = ParseShotNo(self.dataid)
        key = '%s_%s' % (target_source, self.dataid)
        cachepath = None
        # try to get datafile from cache
        filetime = self.cache.findTime(key) 
        if self.cachebutton == 'ONLINE' and egdbcommon.compareTime(target_source, int(self.dataid), 1, filetime):
            cachepath = None
            logger.debug("*** DISCARD CACHE FILE FOR %s@%s ***" % (target_source, self.dataid))
        else:
            logger.debug("*** USE CACHE FILE FOR %s@%s ***" % (target_source, self.dataid))
            cachepath = self.cache.find(key) 
            
        if not cachepath and self.cachebutton == 'ONLINE':
            # try to get datafile from server
            logger.debug("load from network")
            fname     = "%s.zip" % key
            fname     = self.cache.makeUniqueFilename(fname, self.temppath)
            cachepath = os.path.join(self.temppath, fname) 
            cmdstr = "igetfile -s %d -m %d -d %s -Z -o %s" % (
                    shot, subshot, target_source, cachepath) 
            #cmdstr = os.path.join(self.cmdpath, cmdstr)
            #print cmdstr
            if egdbcommon.icheckfile(target_source, shot, subshot):
                subprocess.check_call(cmdstr,shell=True)
                if os.path.exists( cachepath ):
                    logger.debug("Register the Datafile %s" % cachepath)
                    cachepath = self.cache.registFile(key, cachepath)
                else:
                    cachepath = None
            else:
                cachepath = None
                
        return cachepath

    def varsx(self, **kwargs):
        """
        return list of candidates of variables for x-axis
        
        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog
        """
        lst = []
        self.load(**kwargs)
        if not self.eg:
            pass
        else:
            plottype = kwargs.get('type') # xy, xy2, xyz
            if plottype in ['xy', 'xy2']:
                lst = self.eg.dimname()
                if type(lst) is str :
                    # If it is a string, convert it to an array.
                    lst = [lst]
                    
            else:
                pass
                
        return lst

    def varsy(self, **kwargs):
        """
        return list of candidates of variables for y-axis or y2-axis

        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog
        """
        lst = []
        self.load(**kwargs)
        if not self.eg:
            pass
        else:
            plottype = kwargs.get('type') # xy, xy2, xyz
            if plottype in ['xy', 'xy2']:
                lst = self.eg.valnames()
                if type(lst) is str :
                    # If it is a string, convert it to an array.
                    lst = [lst]
                    
            else:
                pass
                
        return lst


    def varsz(self, **kwargs):
        """
        return list of candidates of variables for z-axis

        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog
        """
        return []

    def indexs(self, **kwargs):
        """
        return list of indexs for varx, vary, varz, option.
        
        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog
        """
        return []

    def refresh_index(self, kwargs):
        try:
            index = kwargs.get('index', None)
            trace = kwargs.get('trace', None)
            indexs = self.indexs(**kwargs)
            if index and trace:
                index = egdbcommon.get_index(indexs, index)
                kwargs['index'] = index
                trace = kwargs['trace']
                trace.index = index
        except:
            pass

    def dataset(self, **kwargs):
        """
        return list of dataset for varx, vary, varz, index, option.
        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog

        dataset := [title, x, xe, y, ye, z, ze]
        x = [x1, x2, x3, ... , xn]
        y = [y1, y2, x3, ... , ym] 
        z = [[z(x1, y1), z(x2, y1), z(x3, y1),...,z(xn, y1)],
             [z(x1, y2), z(x2, y2), z(x3, y2),...,z(xn, y2)],
             [z(x1, y3), z(x2, y3), z(x3, y3),...,z(xn, y3)], 
             ......
             [z(x1, ym), z(x2, ym), z(x3, ym),...,z(xn, ym)]] 
        len(z) = len(y)
        len(z[i]) = len(x)
        """
        xdata, xedata, ydata, yedata, zdata, zedata = [], [], [], [], [], []
        plottype, vary = KeywordGet(kwargs, ('type', 'vary'))
        title = '%s_%s@%s' % (self.diagname, vary, self.dataid)
        self.load(**kwargs)
        if not self.eg:
            title = title+'_No Data'
            return [title, [], [], [], [], [], []]
            
        if plottype == 'xyz':
            title = title+'_No 3D Data'
            return [title, [], [], [], [], [], []]
            
        vidx  = self.eg.valname2idx(vary)
        unit  = self.eg.idx2valunit(vidx)
        title = '%s_%s(%s)@%s' % (self.diagname, vary, unit, self.dataid)

        xdata = self.eg.dimdata
        ydata = self.eg.data[vidx]
        if ( re.compile('\Amp_amp\Z').search(self.actual_name) or
             re.compile('\ARF_Spec_(broad|low)\Z').search(self.actual_name) ) :
            xdata, ydata = self.avg_data(xdata, ydata, self.avgnum)
        else:
            pass
        if re.compile('\Ati_crystal\Z').search(self.actual_name) :
            yedata = self.eg.data[vidx+1]
            
        if ( re.compile('\AFC\Z').search(self.actual_name) or
             re.compile('\Afc_eth_test\Z').search(self.actual_name) or
             re.compile('\Aipd_spec\Z').search(self.actual_name) or
             re.compile('\Amapping_(ts|cxs7)\Z').search(self.actual_name) or
             re.compile('\Anfm[|1]\Z').search(self.actual_name) or
             re.compile('\Aqmas\Z').search(self.actual_name) or
             re.compile('\ASciFi\Z').search(self.actual_name) or
             re.compile('\ASDLloopMode\Z').search(self.actual_name) ) :
            if all([y < 0 for y in ydata]):
                title = title+'_No Valid Data'
                
        return [title, xdata, xedata, ydata, yedata, zdata, zedata]

    def packs(self, lst, n):
        return [lst[i:i+n] for i in range(0, len(lst)-n+1, n)]

    def avg_data(self, xdata, ydata, ntavg=1):
        xpacks = self.packs(xdata, ntavg)
        ypacks = self.packs(ydata, ntavg)
        xavg = [sum(v)/len(v) for v in xpacks]
        yavg = [sum(v)/len(v) for v in ypacks]
        return xavg, yavg

    def get_eg(self, diagname):
        eg = None
        if diagname :
            if not self.list_actual_name or not self.list_eg :
                eg = None
                
            else :
                try :
                    egidx = self.list_actual_name.index(diagname)
                    eg = self.list_eg[egidx]
                    
                except ValueError as e:
                    print(e)
        return eg

    def failed_eg(self):
        if any([(v is None) for v in self.list_eg]):
            return True
            
        return False


#
# Utility Functions
def ParseOptionToDictionary(option):
    """
    option : ';' separated string
    syntax :
        keyword1 = value; keyword2 = value; keyword3
    return dict
        value = dict.get(keyword)
        if there is keyword but no value set True
    """
    lines = [l.strip() for l in option.split(';')]
    dic = {}
    for l in lines:
        clm = l.split('=')
        if len(clm) == 0: continue
        if len(clm) == 1:
            dic[clm[0]] = True
        elif len(clm) == 2:
            dic[clm[0]] = clm[1]
    return dic

def GetInteger(value, default=0, min_val=None, max_val=None):
    """
    string value to integer value
    """
    try:
        i_val = int(value)
        if min_val and i_val <= min_val:
            i_val = default
            
        if max_val and i_val > max_val:
            i_val = default
            
    except ValueError:
        i_val = default
        
    return i_val

def GetFloat(value, default=0.0, min_val=None, max_val=None):
    """
    string value to float value
    """
    try:
        f_val = float(value)
        if min_val and f_val < min_val:
            f_val =default
            
        if max_val and f_val > max_val:
            f_val = default
            
    except ValueError:
        f_val = default
        
    return f_val

def LimitValue(target_val, min_val, max_val):
    """
    Keep the input value within the limit.
        in 
           target_val : Target Value
           min_val    : Minimum Value
           max_val    : Maximum Value
        out
           ret_val    : Return value
    """
    ret_val = target_val
    if min_val > target_val:
        ret_val = min_val
        
    if max_val < target_val:
        ret_val = max_val
        
    return ret_val

def AverageGroupOfIndices(sz, avgn):
    """
    Creates a new array by extracting the element indices from a two-dimensional array at specified intervals.
        in 
           sz   : Number of elements in the array
           avgn : step number
        out
           Array of element indices
    """
    lst = [i for i in range(sz)]
    paires = [lst[i:i+avgn] for i in range(0, sz, avgn)]
    return paires

def AverageGroupOfList(lst, avgn):
    """
    Creates a new two-dimensional array by extracting elements from a two-dimensional array at specified intervals.
        in 
           lsts : two-dimensional array
           avgn : step number
        out
           new two-dimensional array
    """
    sz = len(lst)
    paires = [lst[i:i+avgn] for i in range(0, sz, avgn)]
    return paires

def AveragePartOfList(lst, indices):
    """
    Find the average of a specified array.
        in 
           lsts    : two-dimensional array
           indices : array of array indices
        out
           array of mean values
    """
    return sum([lst[i] for i in indices]) / len(indices)

def AverageLists(lsts):
    """
    To obtain an average value of each element of a plurality of arrays.
        in 
           lsts : two-dimensional array
        out
           array of mean values
    ex) [[1,2,3],[4,5,6]] => [2.5, 3.5, 4.5]
    """
    n = len(lsts)
    return [sum(values)/float(n) for values in zip(*lsts)]

def AverageListWithN(lst, avgn):
    """
    Average N sequences.
        in 
           lsts : two-dimensional array
           avgn : step number
        out
           array of mean values
    """
    indices = AverageGroupOfIndices(len(lst), avgn)
    return [AveragePartOfList(lst, idx) for idx in indices]

def FindNear(val, lst):
    """
    Finds the nearest value and its index in an array.
        in 
           val : search value
           lst : array
        out
           Index and value.
    """
    idx = np.argmin(np.abs(np.array(lst) - val))
    return idx, lst[idx]

def FindNearIndex(val, lst):
    """
    Finds the index of the nearest value in an array.
        in 
           val : search value
           lst : array
        out
           Index
    """
    idx = np.argmin(np.abs(np.array(lst) - val))
    return idx

def RemoveData(lst, lst_key, func):
    """
    """
    return [v for v, k in zip(lst, lst_key) if not func(k)]

def KeywordGet(dic, keys, defaults=()):
    if len(defaults) < len(keys):
        defs = defaults + (None,)*(len(keys) - len(defaults))
        
    else:
        defs = defaults 
        
    return [dic.get(k, d) for k, d in zip(keys, defs)]

def Paires(lst, n):
    paired_list = [lst[i:i+n] for i in range(0, len(lst)-1, n)]
    return paired_list

def AverageByIndices(lst, indices):
    averaged_list = [sum([lst[i] for i in idx]) / len(idx) for idx in indices]
    return averaged_list

def AveragedList(lst, n):
    indices = Paires([i for i in range(len(lst))], n)
    averaged_list = AverageByIndices(lst, indices)
    return averaged_list, indices

def OpenFile(filepath):
    if filepath[-3:].upper() == 'ZIP':
        zf = zipfile.ZipFile(filepath, 'r')
        target = zf.namelist()[0]
        return zf, io.TextIOWrapper(io.BufferedReader(zf.open(target, 'r')),encoding='ascii')
    else:
        return None, open(filepath, "r")

def CloseFile(zf, fp):
    if zf:
        fp.close()
        zf.close()
    else:
        fp.close()


def CalWeights(lst):
    if len(lst) == 0:
        return [1]
        
    subweights = CalWeights(lst[1:])
    return [lst[0]*subweights[0]] + subweights

def StringToInteger(vstr):
    try:
        return int(vstr)
    except ValueError:
        return 0

def ParseDataID(dataid):
    if -1 == dataid.find(">"):
        # not found
        return None, None, 0, 0

    clms = dataid.split(">")
    name_shot = clms[1]
    name, shot = name_shot.split("@")
    print("in Kaiseki : ", shot)
    shot, sub = ParseShotNo(shot)
    return clms[0], name, shot, sub

#def MakeMeshed(xn, yn, x_org, y_org, z_org):
#    xi = np.linspace(min(x_org), max(x_org), xn)
#    yi = np.linspace(min(y_org), max(y_org), yn)
#    zi = matplotlib.mlab.griddata(
#            np.array(x_org), np.array(y_org), np.array(z_org), xi, yi, interp='linear')
#    xm, ym, zm = [], [], []
#    for i, x in enumerate(xi):
#        for j, y in enumerate(yi):
#            xm.append(x)
#            ym.append(y)
#            zm.append(zi[j][i])
#    return xm, ym, zm

def MakeMeshedData(x, y, z):
  try :
    xlst, ylst, zlst = [], [], []
    x = np.array(x)
    y = np.array(y)
    z = np.array(z)
    
    xsize = len(set(x))
    ysize = len(set(y))
    
    xi = np.linspace(min(x), max(x), xsize)
    yi = np.linspace(min(y), max(y), ysize)
    #
    # Deprecated since version 2.2: The griddata function was deprecated in Matplotlib 2.2 and will be removed in 3.1. Use scipy.interpolate.griddata instead.
#        zi = matplotlib.mlab.griddata(x, y, z, xi, yi, interp='linear')
    mesh_xi, mesh_yi = np.meshgrid(xi, yi)
    zi = scipy.interpolate.griddata((x, y), z, (mesh_xi, mesh_yi), method='linear')
#    zi = scipy.interpolate.griddata((x, y), z, (mesh_xi, mesh_yi), method='cubic')
    #print("#Finished !")
    zi[np.isnan(zi)] = 0
    for i, xv in enumerate(xi):
        for j, yv in enumerate(yi):
            xlst.append(xv)
            ylst.append(yv)
            zlst.append(zi[j][i])
            
  except Exception as e:
    print("MakeMeshedData =>", e)

  return xlst, ylst, zlst

def Sort3D(xn, yn, x_org, y_org, z_org, key_is_x):
    if key_is_x: # sort with xdata  x:r  y:time
        rs = x_org[:xn]
        r_idx = [(r, i) for i, r in enumerate(rs)]
        r_idx.sort()
        indices = [i for r, i in r_idx]
        xdata = [x_org[indices[j]] for i in range(yn) for j in range(xn)]
        ydata = [y_org[i*xn] for i in range(yn) for j in range(xn)]
        zdata = [z_org[indices[j]+i*xn] for i in range(yn) for j in range(xn)]

    else: # sort with ydata x:time y:r
        rs = y_org[:yn]
        r_idx = [(r, i) for i, r in enumerate(rs)]
        r_idx.sort()
        indices = [i for r, i in r_idx]
        xdata = [x_org[i*yn] for i in range(xn) for j in range(yn)]
        ydata = [y_org[indices[j]] for i in range(xn) for j in range(yn)]
        zdata = [z_org[indices[j]+i*yn] for i in range(xn) for j in range(yn)]
    return xdata, ydata, zdata

def SearchIndex(z, zdata, phi, pdata):
    sorted_dz = sorted([(abs(z-zi), i) for i, zi in enumerate(zdata)])
    zidx = sorted_dz[0][1]
    
    sorted_dphi = sorted([(abs(phi-phii), i) for i, phii in enumerate(pdata)])
    pidx = sorted_dphi[0][1]
    return zidx, pidx

def CalcWeights(lst):
    if len(lst) == 0:
        return [1]

    subweights = CalcWeights(lst[1:])
    return [lst[0] * subweights[0]] + subweights

if __name__ == '__main__':
    from MyViewCache import *
    option = 'ntavg=1'
    cache = MyViewCache('cache\cache.txt')
    ifc = kaisekiInterFace('101911', temppath='temp', cachebutton='ONLINE', cache=cache,
            cmdpath = 'bin\win', option=option) 
    #ifc.load(cachebutton='ONLINE')
    ifc.load(cachebutton='ONLINE')
    #logger.debug('#',ifc.varsx())
    #logger.debug('#',ifc.varsy(varx='time'))
    #logger.debug('#',ifc.indexs(varx='time', vary='Ti'))
    #logger.debug('#',ifc.indexs(varx='R', vary='Ti'))
    #title, x, xe, y, ye, z, ze =ifc.dataset(varx='time', vary='Ti', index='28')
    #title, x, xe, y, ye, z, ze =ifc.dataset(varx='R', vary='Ti', index='26')
    #for i, xv in enumerate(x):
    #    print "%g\t%g\t%g" % (xv, y[i], ye[i])
    #title, x, xe, y, ye, z, ze =ifc.dataset(varx='time', vary='R', varz='Ti', index='1', type='xyz')
    #for i, xv in enumerate(x):
    #    logger.debug("%g\t%g\t%g" % (xv, y[i], z[i]))


