#!/usr/bin/env python
# coding: utf-8
# $Id$
import os
import re
import pickle
import numpy
import labcom
import kaisekiInterFace
from MyViewGlobal import logger

def helptext(title_name, script_name, data_source, synopsis, author_name, contact_to, plot_type, proc_summary, module_option):
    return kaisekiInterFace.helptext( title_name, script_name, data_source, synopsis, author_name, contact_to, plot_type, proc_summary, module_option )

class labInterFace( kaisekiInterFace.kaisekiInterFace ):
    pattern  = '\ACTRLDISP[123]\Z'                       # CTRLDISP1, 2, 3
    pattern2 = '\AImpMon\Z'                              # fe(ImpMon)
    pattern3 = '\AFIG\Z'                                 # FIG
    pattern4 = '\AFPellet\Z'                             # FPellet
    pattern5 = '\AGASPUFF\Z'                             # GASPUFF
    pattern6 = '\AICHVOLT\Z'                             # ICHVOLT
    pattern7 = '\AIPD\Z'                                 # IPD

    def __init__(self, data_id, title_name, data_source, dic_channels, num_points, **kwargs):
        """
        kwargs:
            option: option string for trace
            cache: myview cache object
            cachebutton: state of button cache on myView.
                         "CACHE": 
                         "ONLINE":
                         "OFFLINE":
            temppath: temp path set with myView dialog.
            cmdpath: command path set with myView dialog
        if there is some troubole to make instance, raise "Error"
        """
        super().__init__(data_id, title_name, data_source, **kwargs)
        self.list_alldata = []
        self.dic_channel  = dic_channels
        self.dic_valname  = {}
        self.num_points   = self.parse_option(self.option, num_points)
        for k, v in list(self.dic_channel.items()):
            self.dic_valname[v] = k

    def parse_option(self, option, num_points):
        numpoints = num_points
        try:
            if not option:
                pass
                
            else :
                dic = kaisekiInterFace.ParseOptionToDictionary(option)
                if any(dic) :
                    if numpoints is not None:
                        vstr = dic.get('NUMP')
                        if vstr is not None :
                            numpoints = kaisekiInterFace.GetInteger(vstr, numpoints, None, None)
                            
                        else :
                            vstr = dic.get('nump')
                            if vstr is not None :
                                numpoints = kaisekiInterFace.GetInteger(vstr, numpoints, None, None)
                                
                else:
                    pass
                    
        except Exception as e:
            print(e)
            pass
            
        return numpoints

    def retrieve(self):
        if ( not labcom.icheckfile(self.actual_name, int(self.dataid), 1) ):
            return []
        retriever = labcom.Retriever()
        all_ch    = list( self.dic_channel.keys() )
        all_ch.sort()
        if re.compile(labInterFace.pattern7).search(self.actual_name) :
            # IPD
            ret = retriever.retrieve(self.actual_name, int(self.dataid), 1, all_ch, timeRead=True, numPoints=self.num_points, convertVolt=True)
            
        elif re.compile(labInterFace.pattern).search(self.actual_name) :
            # CTRLDISP1, 2, 3
            ret = retriever.retrieve(self.actual_name, int(self.dataid), 1, all_ch, numPoints=self.num_points)
            
        else :
            # fe(ImpMon)
            # FIG
            # FPellet
            # GASPUFF
            # ICHVOLT
            ret = retriever.retrieve(self.actual_name, int(self.dataid), 1, all_ch, timeRead=True, numPoints=self.num_points )
            
        return ret
        
    def restore(self, path):
        fp   = open(path,'rb')
        data = pickle.load(fp)
        fp.close()
        return data
        
    def save(self, path):
        fp = open(path,'wb')
        pickle.dump(self.list_alldata, fp)
        fp.close()
        
    def load(self, **kwargs):
        # キャッシュを開く
        #cache = myViewCache.cache()
        key = '%s_%s' % (self.actual_name, self.dataid)
        self.cachepath = None

        # try to get datafile from cache
        filetime = self.cache.findTime(key) 
        if self.flgloaded:
            return
            
        if not self.cachepath and self.cachebutton == 'ONLINE':
            self.list_alldata = self.retrieve()
            fname = "%s.zip" % key
            fname = self.cache.makeUniqueFilename(fname, self.temppath)
            self.cachepath = os.path.join(self.temppath, fname)
            if ( len(self.list_alldata) > 0 ):
                self.save(self.cachepath)
                self.cache.registFile(key, self.cachepath)
        else:
            self.list_alldata = self.restore(self.cachepath)
            
        self.loadflg = True


    def varsx(self, **kwargs):
        """
        return list of candidates of variables for x-axis
        
        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog
        """
        lst = []
        self.load(**kwargs)
        if len(self.list_alldata) == 0 :
            pass
        else :
            plottype = kwargs.get('type') # xy, xy2, xyz
            if plottype in ['xy', 'xy2']:
                lst = ['time']
            else:
                pass
                
        return lst

    def varsy(self, **kwargs):
        """
        return list of candidates of variables for y-axis or y2-axis

        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog
        """
        lst = []
        self.load(**kwargs)
        if len(self.list_alldata) == 0:
            pass
        else :
            plottype = kwargs.get('type') # xy, xy2, xyz
            if plottype in ['xy', 'xy2']:
                lst = list(self.dic_channel.values())
            else:
                pass
                
        return lst

    def dataset(self, **kwargs):
        """
        return list of dataset for varx, vary, varz, index, option.
        kwargs:
            option: option string for trace
            varx: varx set with in trace dialog
            vary: varx set with in trace dialog
            varz: varx set with in trace dialog

        dataset := [title, x, xe, y, ye, z, ze]
        x = [x1, x2, x3, ... , xn]
        y = [y1, y2, x3, ... , ym] 
        z = [[z(x1, y1), z(x2, y1), z(x3, y1),...,z(xn, y1)],
             [z(x1, y2), z(x2, y2), z(x3, y2),...,z(xn, y2)],
             [z(x1, y3), z(x2, y3), z(x3, y3),...,z(xn, y3)], 
             ......
             [z(x1, ym), z(x2, ym), z(x3, ym),...,z(xn, ym)]] 
        len(z) = len(y)
        len(z[i]) = len(x)
        """
        xdata, xedata, ydata, yedata, zdata, zedata = [], [], [], [], [], []
        plottype, vary = kaisekiInterFace.KeywordGet(kwargs, ('type', 'vary'))
        title = '%s_%s@%s' % (self.diagname, vary, self.dataid)
        self.load(**kwargs)
        if len(self.list_alldata) == 0:
            # if there is no data
            title = title+'_No Data'
            return [title, [], [], [], [], [], []]
            
        if plottype == 'xyz':
            title = title+'_No 3D Data'
            return [title, [], [], [], [], [], []]
            
        if re.compile(labInterFace.pattern2).search(self.actual_name) :
            # fe(ImpMon)
            unit = 'arb'
        else :
            # CTRLDISP1, 2, 3
            # FIG
            # FPellet
            # GASPUFF
            # ICHVOLT
            unit  = 'V'
            
        title = '%s_%s(%s)@%s' % (self.diagname, vary, unit, self.dataid)
        if ( vary in self.dic_valname ) :
            # Get channel number from name.
            vchno  = self.dic_valname[vary]
            all_ch = list( self.dic_channel.keys() )
            all_ch.sort()
            try :
                # The index of the array storing the data is obtained from the channel number.
                vidx = all_ch.index(vchno)
                data  = self.list_alldata[vidx]
                xdata = data.get_time()
                ydata = data.get_vals()
                if re.compile(labInterFace.pattern4).search(self.actual_name) :
                    # FPellet
                    idxs = numpy.where(ydata > 0.1)[0]
                    lstidxs = list(idxs)
                    if len(lstidxs) == 0:
                        xdata = xdata[:600]
                        ydata = ydata[:600]
                    else:
                        lststart = []
                        idx0 = lstidxs[0]
                        lststart.append(idx0)
                        for idx in lstidxs[1:]:
                            if 1 < idx - idx0:
                                lststart.append(idx)
                                
                            idx0 = idx
                            
                        expands = []
                        for i in lststart:
                            for j in range(600):
                                idx = i + j - 300
                                if idx not in expands and idx > 0:
                                    expands.append(idx)
                        idxs = numpy.array(expands)
                        xdata = xdata[(idxs)]
                        ydata = ydata[(idxs)]
                        
            except ValueError:
                pass
                
        return [title, xdata, xedata, ydata, yedata, zdata, zedata]
