#!/usr/bin/env python
# coding: utf-8
# $Id$
import numpy
import os
import sys
sys.path.append('.')
import subprocess
import util
import pg8000
import warnings
warnings.filterwarnings("ignore",
                        "DB-API extension")

def deleteIfExists(filename):
    if ( os.path.exists(filename) ):
        os.unlink(filename)

def icheckfile(diagname, shotno, subshot):
    try:
        conn = pg8000.connect(host="dasindex.lhd.nifs.ac.jp", user="eg_md5", password="e130906g", database="index")
        cursor = conn.cursor()
        sql = "select real_arcshot diag_name from shot s join diag d on (s.diag_id=d.diag_id) where diag_name = '%s' and real_arcshot = %d and s.site_id = %d " % (diagname, shotno, subshot)
        #print "sql=",sql
        cursor.execute(sql)
        print(('cursor.rowcont=',cursor.rowcount))
        if cursor.rowcount > 0:
            ret = True
        else:
            ret = False
        cursor.close()
    except:
        ret = False
    finally:
        conn.close()
    return ret

class RawData():
    def __init__(self, vals, params={}, tvals = None, skip = 1):
        self.vals   = vals
        self.params = params
        self.tvals  = tvals
        self.skip   = skip

    def get_vals(self):
        return self.vals

    def get_time(self):
        if ( self.tvals is not None ):
            return self.tvals
        #print('params=',self.params)
        clk = self.params['SamplingClock']
        interval = self.params['SamplingInterval']
        # Make an array of the same length as the voltage data and divide by the sampling clock (to get an array of sampling times)
        if ( 0 < self.skip ) :
            tvals = numpy.arange(0.0, len(self.vals)) / clk * interval * self.skip
        else:
            tvals = numpy.arange(0.0, len(self.vals)) / clk * interval
            
        print(tvals)
        return tvals
        
class BinaryReader():
    def __init__(self):
        pass
    
    def read_param_file(self, paramFile):
        fp=open(paramFile)
        lines = fp.readlines()
        fp.close()
        params = {}
        for line in lines:
            (module, key, val, t) = line.split(',')
            t = int(t)
            if ( t == 1):
                pass
            elif (t == 2 or t==3 or t==4):
                val = int(val)
            elif (t == 5 or t==6):
                val = float(val)
            else:
                print(("ERROR type=%d" % ( t )))
            params[key]=val
        return params
        
    def read(self, dataFile, paramFile = None, timeFile = None, numPoints=None, convertVolt=True):
        print("BinaryReader.read()")
        if ( paramFile ):
            params = self.read_param_file(paramFile)
            #print 'params=',params
        if ( convertVolt ):
            vals = numpy.fromfile(dataFile, dtype=numpy.float32)
        else:
            try:
                bits = params['Resolution(bit)']
                if ( bits <= 8 ):
                    dtype = numpy.int8
                elif ( bits <= 16 ):
                    dtype = numpy.int16
                elif ( bits <= 32 ):
                    dtype = numpy.int32
            except:
                dtype = numpy.int16
            vals = numpy.fromfile(dataFile, dtype=dtype)

        if ( numPoints ):
            skip = int(len(vals) / numPoints)
            if ( skip == 0 ):
                skip = 1
            vals = vals[::skip]
        else :
            skip = 1
            
        tvals = None
        if ( timeFile ):
            tvals = numpy.fromfile(timeFile,dtype=numpy.float32)
            #print "len(tvals)=",len(tvals)
            if ( numPoints ):
                tvals = tvals[::skip]
        #print 'params=',params,'tvals=',tvals
        return RawData(vals, params, tvals, skip)

class Retriever():
    def __init__(self):
        pass
    

        
    def retrieve(self, diag, shotNo, subNo, chList, timeRead = False, numPoints=None, convertVolt=True):
        """
        Convert Voltage
        """
        ret = []
        reader = BinaryReader()
        for ch in chList:
            random = util.random_str(15)
            basename = "%s-%d-%d-%d" % ( random, shotNo, subNo, ch )
            filename = basename + '.dat'
            paramfile = basename + '.prm'
            filename_t = basename + '.time'
            paramfile_t = basename + '.tprm'
            silent = '--silent'
            silent = ''
            volt=''
            try:
                if ( convertVolt ):
                    volt = '-V4'
                cmd=["retrieve", diag, str(shotNo), str(subNo), str(ch), random]
                #if [ silent != '' ]:
                #    cmd.append(silent)
                if [ volt != '' ]:
                    cmd.append(volt)
                print(cmd)
                subprocess.check_call(cmd)
                if ( timeRead ):
                    cmd=["retrieve_t", diag, str(shotNo), str(subNo), str(ch), random]
                    #cmd = "retrieve_t %s %d %d %d %s %s -V4" % ( diag, shotNo, subNo, ch, random, silent)
                    subprocess.check_call(cmd)
                    data = reader.read(filename, paramFile = paramfile, timeFile = filename_t, numPoints=numPoints, convertVolt=convertVolt)
                else:
                    data = reader.read(filename, paramFile = paramfile, numPoints=numPoints, convertVolt=convertVolt)
                ret.append(data)
            except:
                import sys,traceback
                traceback.print_exc(file=sys.stdout)
            finally:
                deleteIfExists(filename)
                deleteIfExists(filename_t)
                deleteIfExists(paramfile)
                deleteIfExists(paramfile_t)
            #print 'ret=',ret
        return ret

        

if __name__ == '__main__':
#    reader = BinaryReader()
#    data = reader.read('CTRLDISP1-111111-1-1.dat',paramFile = 'CTRLDISP1-111111-1-1.prm')
#    vals = data.get_vals()
#    tvals = data.get_time()
#    for i in xrange(0,10):
#        print tvals[i],' ',vals[i]


    retriever = Retriever()
    allData = retriever.retrieve('CTRLDISP1', 111111, 1, [1,2,3,4,5,6])
    for data in allData:
        print("===========================")
        vals = data.get_vals()
        tvals = data.get_time()
        for i in range(0,10):
            print((tvals[i],' ',vals[i]))
        print("===========================")
        
    import pickle
    fp=open('CTRLDISP1-111111.dat','w')
    pickle.dump(allData,fp)
    fp.close()

    fp=open('CTRLDISP1-111111.dat')
    allData = pickle.load(fp)
    for data in allData:
        print("===========================")
        vals = data.get_vals()
        tvals = data.get_time()
        for i in range(0,10):
            print((tvals[i],' ',vals[i]))
        print("===========================")
    

    allData =retriever.retrieve('ImpMon',111111,1,[1], convertVolt=False)
    print(('allData=',allData))

    print((icheckfile('CTRLDISP1', 111111, 1)))
