from __future__ import print_function
import os
import sys
import time


encoding = sys.getfilesystemencoding()
sys.path.append(os.path.join('.','DataModules'))
from DataModules.egdbcommon import getEGDBinfo

class EG_List(object):
    def __init__(self, filepath):
        self.filepath = filepath
        self.diagnames = {}
        if ( os.path.exists(filepath)):
            self.load()

    def load(self):
        try:
            with open(self.filepath, "r", encoding='utf-8') as fp:
                lines = [l.strip() for l in fp.readlines()]
        except UnicodeDecodeError:
            with open(self.filepath, "r", encoding='cp932') as fp:
                lines = [l.strip() for l in fp.readlines()]


        for l in lines:
#            l = l.decode('utf-8')
            if l[0] == '#':
                continue
            clms = l.split('\t')
            if len(clms) == 4:
                name, desc, s_dimno, s_weight = clms
            elif len(clms) == 3:
                name, desc, s_dimno = clms
                s_weight = "0"
            dimno = int(s_dimno)
            weight = int(s_weight)
            self.diagnames[name] = (desc, dimno, weight)

    def countup(self, diagname):
        desc, dimno, weight = self.diagnames.get(diagname)
        self.diagnames[diagname] = (desc, dimno, weight+1)
    
    def count(self, diagname):
        v = self.diagnames.get(diagname, None)
        if v is not None:
            desc, dimno, weight = v
        else:
            desc, dimno, weight = "", 0, 0
        return weight

    def save(self):
        lines = ["#name\tdesc\tdimno\tweight"]
        for name, v in self.diagnames.items():
            desc, dimno, weight = v
            l = "%s\t%s\t%d\t%d" % (name, desc, dimno, weight)
#            l = l.encode('utf-8')
            lines.append(l)

        with open(self.filepath, "w", encoding='utf-8') as fp:
            fp.write("\n".join(lines)+'\n')

    def getlist(self):
        lst = []
        for name, v in self.diagnames.items():
            desc, dimno, weight = v
            if 0 < dimno <= 3: 
                lst.append((1.0/(float(weight)+1), name, desc, dimno))
        lst.sort()
        return [(n, de) for w, n, de, di in lst]

def Rebuild_EGList(filepath, flg_count_reset=False):
    print('Rebuilding [%s] ...' % (filepath))
    egl = EG_List(filepath)
    
    while(True):
        h = []
        try:
            h = getEGDBinfo()
        except:
            import sys,traceback
            traceback.print_exc()
#            pass
        if len(h) != 0:
            break
        print("Connection busy ! Retrying fetch EG info")
        time.sleep(2)

    diagnames = {}
    for key, val in h.items():
        name, desc, dimno, valno = (key, val['COMMENT'], val['DIMNO'], val['VALNO'])
        if dimno is None:
            dimno = 0
        if flg_count_reset:
            count = 0
        else:
            count = egl.count(name)
        diagnames[name] = (desc, dimno, count)
    egl.diagnames = diagnames
    egl.save()
    print('Finish Rebuilding [%s]' % (filepath))

def update():
    from MyViewGlobal import logger, globalError, variables 
    import MyViewConfig
    config = MyViewConfig.MyViewConfig('myView2.cnf')
    variables['%CONFIG%'] = config

    #import egdbcommon
    #egl = EG_List('eglist.txt')
    #for n, de in egl.getlist():
    #    print "%s\t%s" % (n, de)
    Rebuild_EGList('eglist.txt', True)


    #egl.countup('bolo')
    #egl.save()

def clear():
    with open('eglist.txt','w') as fp:
        pass

if __name__ =='__main__':
    update()
