#!/usr/bin/env python
# coding: utf-8
from __future__ import print_function
import wx

import os
import re
import sys

import EGList
import SearchListPanel

if wx.__version__ < '3.0':
    wx.Color = wx.Colour

class EGSelectDlg(wx.Dialog):
    def __init__(self, parent, record):
        wx.Dialog.__init__(self, parent, -1, 'EG Selector', 
                style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER|wx.STAY_ON_TOP)
        #self.SetBackgroundColour(wx.Colour(200, 200, 200))
        self.font = wx.Font(11, wx.MODERN, wx.NORMAL, wx.NORMAL, False)

        self.cwd = os.getcwd()
        
        self.panel = SearchListPanel.SearchListPanel(self, 
                record_sources=record,
                titles=["diagname", "desc"],
                widths=[0.4, 0.6],
                default_key="",
                default_keys=['cxs', 'thomson', 'tsmap'])
        
        #------------ 
        btnOk = wx.Button(self, wx.ID_OK, "OK")
        btnOk.SetDefault()
        btnCancel= wx.Button(self, wx.ID_CANCEL, "Cancel")
        btnsizer = wx.BoxSizer(wx.HORIZONTAL)
        btnsizer.Add(btnOk, 1, wx.EXPAND)
        btnsizer.Add(btnCancel, 1, wx.EXPAND)
        
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.panel, 1, wx.EXPAND|wx.ALL, 0)
        sizer.Add(btnsizer, 0, wx.EXPAND|wx.ALL, 5)
        
        self.SetSizer(sizer)
        self.Fit()

        self.panel.Bind(self.panel.LIST_ITEM_ACTIVATED, self.OnListItemActivated)

    def OnListItemActivated(self, event):
        self.EndModal(wx.ID_OK)

    def GetResult(self):
        results = self.panel.get_values()
        if len(results) == 0:
            return None
        return results[0]

    def OnQuit(self, evt=None):
        self.Close()

class EGSelector(object):
    def __init__(self):
        self.setEGList()

    def setEGList(self):
        self.eglist = EGList.EG_List('eglist.txt')

    def getName(self):
        return self.selected_name

    def choiceModuleAndDiagname(self, parent=None):
        name = self.askUser(parent)
        if name is None:
            return None
        desc, dimno, weight = self.eglist.diagnames.get(name)
        self.eglist.countup(name)
        self.eglist.save()
        if dimno == 1:
            return "plot_eg_1D", name
        elif dimno == 2:
            return "plot_eg_2D", name
        else:
            return "plot_eg_3D", name

    def askUser(self, parent=None):
        dialog = EGSelectDlg(parent, self.eglist.getlist())
        result = dialog.ShowModal()
        name = None
        if result == wx.ID_OK:
            name = dialog.GetResult()
        dialog.Destroy()
        return name

if __name__ == '__main__':
    #from MyViewConsoleOut import *
    app = wx.App()
    egsel = EGSelector()
    #cout(egsel.askUser(None))
    #print egsel.askUser(None)
    print(egsel.choiceModuleAndDiagname(None))



