#!/usr/bin/env python
# coding: utf-8
# $Id$

import os
import math
import time
import subprocess
from shotinfo import ShotInfo
from MyViewGlobal import debug, logger

def helptext():
    print("shotinfo.helptext()")
    text = """
    <p>Here is an expanation of this module <p><br />
    $Id$ <br />
    Module name : shotinfo <br />
    Script name : lm_shotinfo.py <br />
    Data source : kaiseki database <br />
    Author      : emo@LHD.NIFS.AC.JP <br />
    Contact to  : emo@lhd.nifs.ac.jp <br>
    Variables   : the following variables are available.<br />
       %bt%, %rax%, %gamma%, %bq% <br />
       %year%, %month%, %day%, %hour%, %minute% <br />
       %gas%   <br />
       %theme% <br />
    """
    return text

class InterFace():
    """
    ショット情報を取得するクラス
    """
    def __init__(self, dataid, **kwargs):
        #print dataid
        self.dataid = dataid
        self.cmdpath = kwargs.get('cmdpath')
        self.cache = kwargs.get('cache')
        self.flgloaded = False
        pass

        
        
    def load(self, **kwargs):
        self.cachebutton = kwargs.get('cachebutton')
        option = kwargs.get('option', None)
        labelname = 'shotinfo'
        key = '%s_%s' % (labelname, self.dataid)
        self.cache.opencache()
        flg = False # flg for success or not
        data = None
        
        if self.cachebutton != 'ONLINE':
            data = self.cache.restoreData(key)
            if data:
                parm, date, s1, s2 = data
                bt, rax, bq, gamma = parm
                year, month, day, hour, minute = date
                gas = s1
                theme = s2
#                gas=unicode(gas,'utf8')
#                theme=unicode(theme,'utf8')
                flg = True

        if not data and self.cachebutton != 'OFFLINE':
            shotInfo = ShotInfo(int(self.dataid), 1)
            try:
                h = shotInfo.getData()
                bt = h['b']
                rax = h['rax']
                bq = h['bq']
                gamma = h['gamma']
                date = h['date']
                gas = h['gas']
                theme = h['theme']
                flg = True 
            except:
                import sys,traceback
                logger.error(traceback.print_exc())

            # 0123456789012345
            # 2014/12/24 11:36
            if flg:
                year = int(date[:4])
                month = int(date[5:7])
                day = int(date[8:10])
                hour = int(date[11:13])
                minute = int(date[14:16])
            else:
                bt = 0.0
                rax = 0.0
                bq = 0.0
                gamma = 0.0
                year  = 0
                month = 0
                day = 0
                hour = 0
                minute = 0
                gas = b''
                theme = b''
            self.cache.registData(key, 
                    [['double', [bt, rax, bq, gamma]],
                     ['int', [year, month, day, hour, minute]],
                     ['string',gas.encode('utf8')],
                     ['string',theme.encode('utf8')]])
        self.bt = bt
        self.rax = rax
        self.bq = bq
        self.gamma = gamma
        self.year = year
        self.month = month
        self.day = day
        self.hour = hour
        self.minute = minute
        self.gas = gas
        self.theme = theme

    def labeltext(self, **kwargs):
        """
        キーワードを値に置換して返す
        
        """
        if not self.flgloaded:
            self.load(**kwargs)

        arg = kwargs.get('arg', "")
        arg = arg.replace('%rax%', "%g" % self.rax)
        arg = arg.replace('%bt%', "%g" % self.bt)
        arg = arg.replace('%bq%', "%g" % self.bq)
        arg = arg.replace('%gamma%', "%g" % self.gamma)
        arg = arg.replace('%year%', "%04d" % self.year)
        arg = arg.replace('%month%', "%02d" % self.month)
        arg = arg.replace('%hour%', "%02d" % self.hour)
        arg = arg.replace('%minute%', "%02d" % self.minute)
        arg = arg.replace('%day%', "%02d" % self.day)
        arg = arg.replace('%gas%',"%s" % self.gas)
        arg = arg.replace('%theme%',"%s" % self.theme)

        return arg

def doCommand(cmdstr):
    p = subprocess.Popen(cmdstr, shell=True, cwd=os.getcwd(), stdin=subprocess.PIPE,
            stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    (stdout, stdin) = (p.stdout, p.stdin)  # file object
    buff = []
    while True:
        line = stdout.readline()  # leadlines with return code
        if not line:
            break
        buff.append(line.strip())
    ret = p.wait()  # ret has return value
    return buff

if __name__ == '__main__':
    from MyViewCache import *
    import sys
    shotno = 123130
    if ( len(sys.argv) > 1 ):
        shotno = int(sys.argv[1])
    cache = MyViewCache('cache/cache.txt')
    ifc = InterFace(str(shotno), temppath='temp', cachebutton='ONLINE', cache=cache,
            cmdpath = 'bin\win') 
    print(ifc.labeltext(arg='(%bt%, %rax%, %gamma%, %bq%)'))
    print(ifc.labeltext(arg='%year%/%month%/%day% %hour%:%minute%'))
#    print ifc.labeltext(arg='%date%')
    print(ifc.labeltext(arg='GAS=%gas%'))

