#!/usr/bin/env python
# coding: shift_jis
# $Id$
import sys
import os
import json
import pytz
from MyViewGlobal import logger, globalError, variables
import pg8000 
import urllib.request
import re

pa_header = r'^#\s+(.*)\s*=\s*(.*)$'
pa_val = r"\s*'(.*)'\s*"
re_header=re.compile(pa_header)
re_val=re.compile(pa_val)

def get_single_val(str):
   val=re.sub(re_val, r'\1',str)
   return val

def get_array_val(str):
   vals = str.split(',')
   ary = []
   for val in vals:
      ary.append(get_single_val(val))
   return ary

class ShotInfo():
    """
    Vbg̏擾NX
    
    """
    def __init__(self, sn, online):
        self.sn = sn
        self.online = online
        
    def getExpInfo(self, cursor, h):
        sql = "select magneticfield, magneticaxis, quadruple, gamma,ddatacreationtime  from explog2 where nshotnumber = %d" % (self.sn)
#        sql = "select magneticstrength, axis,bq,gamma,l4ontime from seigyo.ps_shotlog T1 join seigyo.plc_shotlog T2 on (T1.shotno = T2.shotno) where T1.shotno = %d" % ( self.sn );
        #print 'sql=',sql
        try:
            cursor.execute(sql)
            row = cursor.fetchone()
        except:
            globalError.set('DB Error')
            row = None
        if ( row != None ):
            (b,rax,bq,gamma,date) = row
        else:
            (b,rax,bq,gamma, date) = (None, None, None, None, None)
        #logger.debug('date=',date)
        h['b'] = b
        h['rax'] = rax
        h['bq'] = bq
        h['gamma'] =gamma
        if ( date != None ):
            date = date.astimezone(pytz.timezone('Asia/Tokyo'))
            h['date'] = date.strftime('%Y/%m/%d %H:%M')
        else:
            h['date'] = '0000/00/00 00:00'
        
    def getTheme(self, cursor,h):
        sql = "select shotno,theme from funaba.exp_comment where shotno <= %d and theme is not null order by shotno desc limit 1;" % self.sn
        try:
            cursor.execute(sql)
            row = cursor.fetchone()
        except:
            globalError.set('DB Error')
            row = None
        if ( row != None ):
            (shotno,theme,) = row
        else:
            theme = ''
        h['theme'] = theme

    def getGasname(self, cursor, h):
        sql = "SELECT distinct gas_name.gas_name as gas_name FROM seigyo.gas_shotlog gas_shotlog, seigyo.gas_name gas_name  WHERE gas_shotlog.shotno=%d and gas_name.gas_name_id=gas_shotlog.gas_name_id " % (self.sn)
        try:
            cursor.execute(sql)
            row = cursor.fetchone()
        except:
            globalError.set('DB Error')
            row = None
        if ( row != None ):
            (gas_name,) = row
        else:
            gas_name = ''
        h['gas'] = gas_name

    def getLIDInfo(self, cursor, h):
        sql = "SELECT current1,current2,current3 FROM seigyo.lid_shotlog, seigyo.lid_point WHERE seigyo.lid_shotlog.shotno= %d AND seigyo.lid_shotlog.point_id =  seigyo.lid_point.point_id " % (self.sn)
        try:
            cursor.execute(sql)
            row = cursor.fetchone()
        except:
            globalError.set('DB Error')
            row = None
        if ( row != None ):
            (ilid1,ilid2,ilid3,) = row
        else:
            (ilid1,ilid2,ilid3,) = (None, None, None)
        h['ilid1'] = ilid1
        h['ilid2'] = ilid2
        h['ilid3'] = ilid3

    def getData(self):
        config = variables['%CONFIG%']
        h = {}

        if ( "webapi" in config ):
           url="%s?cmd=getfile&diag=shotinfo&shotno=%d&subno=1" % (config['webapi'], self.sn)
           self.getDataByWeb(url, h)

           return h

        try:
           dbhost=str(config['kserverdbhost'])
           dbname=str(config['kserverdbname'])
           dbuser=str(config['kserverdbuser'])
           dbpass=str(config['kserverdbpass'])
           conn = None
           conn = pg8000.connect(host=dbhost, user=dbuser, password=dbpass, database=dbname)
           cursor = conn.cursor()
           self.getExpInfo(cursor,h)
           self.getTheme(cursor,h)
           self.getGasname(cursor,h)
           self.getLIDInfo(cursor,h)
           cursor.close()
            
        except:
            logger.error('Error!!')
            import sys, traceback
            logger.error(traceback.print_exc())
            globalError.set('DB Error')
        finally:
            if conn:
                conn.close()
            
        return h

    def getDataByWeb(self, url, hh):
        with urllib.request.urlopen(url) as response:
           s = response.read().decode('utf8').strip()
           lines = s.split("\n")
           h={}
           data=[]
           for l in lines:
              l = l.strip()
              m = re.search(re_header,l)
              if ( m ):
                 key = m.group(1).strip().upper()
                 val = m.group(2)
                 if ( key == 'VALNAME' or  key == 'VALUNIT'):
                    h[key]=get_array_val(val)
                 else:
                    h[key]=get_single_val(val)
              elif ( len(l) > 0 and l[0] != '#'):
                 data = l.split(',')
                 h['DIMDATA']=float(data[0])
           dh = {}
           for i,k in enumerate(h['VALNAME']):
              dh[k] = data[i+1]
           h['VALDATA']=dh
           hh['date'] = "%04d/%02d/%02d %02d:%02d" % ( int(dh['year']), int(dh['month']), int(dh['day']), int(dh['hour']), int(dh['minute']))
           hh['b']=float(dh['B'])
           hh['rax']=float(dh['Rax_vac'])
           hh['bq']=float(dh['Bq'])
           hh['gamma']=float(dh['gamma'])
           hh['gas']=''
           hh['theme']=''
           hh['ilid1'] = None
           hh['ilid2'] = None
           hh['ilid3'] = None

if __name__ == '__main__':
    shotno = 111111
    if ( len(sys.argv) > 1 ):
        shotno = int(sys.argv[1])
    print('shotno=',shotno)
    shotinfo = ShotInfo(shotno,1)
    h = shotinfo.getData()
    print('h=',h)
    
