#!/usr/bin/env python
# coding: utf-8
# $Id$

import wx
import wx.lib.newevent
import os
import time
import re
import sys
import zipfile
import urllib.request, urllib.error, urllib.parse
import MyViewUpdateDlg 

from MyViewGlobal import debug, logger, variables

if wx.__version__ >= "4.1.0":
    import wx.adv
    HyperlinkCtrl = wx.adv.HyperlinkCtrl
else:
    HyperlinkCtrl = wx.HyperlinkCtrl
#HOMEPAGE="http://kaiseki-web.lhd.nifs.ac.jp/software/myView2/"
#DOCPAGE="http://kaiseki-web.lhd.nifs.ac.jp/software/myView2/myView2-Eng.pdf"
#STATPAGE="http://egdb.lhd.nifs.ac.jp/dataexplog/registered_data.html"
#REGCHECKPAGE="http://kaiseki-dev.lhd.nifs.ac.jp/rails/dataexplog/dataexplog/index2"
#SKIP_FILES = ['myView2.cnf', 'plot_default.txt', 'myView2.zip']
SKIP_FILES = ['myView2.cnf', 'myView2.zip']

def UrlGet(url):
    request = urllib.request.Request(url)
    request.add_header('User-Agent', 'Mozilla/5.0')
    request.add_header('Pragma', 'no-cache')
    content = urllib.request.build_opener().open(request)
    return content.read().strip()

class AboutDlg(wx.Dialog):
    """
    About ウィンドウ
    """
    def __init__(self, parent):
        #wx.Dialog.__init__(self, parent, -1, 'About myView', size=(480, 320))
        wx.Dialog.__init__(self, parent, -1, 'About myView')
        self.SetBackgroundColour('#dcdda8') #dcdda8
        #config = variables.get('%CONFIG%', "{}")
        config = variables['%CONFIG%']
        try:
            self.HOMEPAGE=config['homepage']
            self.DOCPAGE=config['docpage']
            self.STATPAGE=config['statpage']
            self.REGCHECKPAGE=config['regcheckpage']
            self.DESCRIPTION=config['datadescription']
        except:
            self.HOMEPAGE=""
            self.DOCPAGE=""
            self.STATPAGE=""
            self.REGCHECKPAGE=""
            self.DESCRIPTION=""

        #self.Refresh()
        button = wx.Button(self, wx.ID_OK, "OK")
#        img = wx.Image("myViewAbout.png")
#        img.ConvertToBitmap()
        #panel = wx.Panel
        #sbmp = wx.StaticBitmap(panel, -1, img)
        hc_url = HyperlinkCtrl(self, wx.ID_ANY, self.HOMEPAGE, self.HOMEPAGE)
        st_help=wx.StaticText(self, -1, "Home Page :")

        doc_url = HyperlinkCtrl(self, wx.ID_ANY, "MyView2 Manual", self.DOCPAGE)
        st_doc = wx.StaticText(self, -1, "Document :")

        stat_url = HyperlinkCtrl(self, wx.ID_ANY, "Search for Registered Data", self.STATPAGE)
        st_stat = wx.StaticText(self, -1, "Available Data:")

        desc_url = HyperlinkCtrl(self, wx.ID_ANY, label="Data Description", url=self.DESCRIPTION)
        st_desc = wx.StaticText(self, -1, "Data Description:")

        reg_url = HyperlinkCtrl(self, wx.ID_ANY, "AutoAna Status Check", self.REGCHECKPAGE)
        st_reg = wx.StaticText(self, -1, "AutoAna Satus:")

        try:
            self.version = open('version.txt').read().strip()
            self.version_no = int(self.version)
        except:
            self.version = "Unknown"
            self.version_no = 0

        st_version_label=wx.StaticText(self, -1, "Version: ")
        st_version_value=wx.StaticText(self, -1, self.version)
        btn_check=wx.Button(self, -1, "Check for Update")
        
        sizer = wx.BoxSizer(wx.VERTICAL)
        fgsizer = wx.FlexGridSizer(rows=6, cols=2, hgap=5, vgap=2)

        # 1
        fgsizer.Add(st_help, 0, wx.EXPAND)
        fgsizer.Add(hc_url, 0, wx.EXPAND)

        # 2
        fgsizer.Add(st_doc, 0, wx.EXPAND)
        fgsizer.Add(doc_url, 0, wx.EXPAND)

        # 3
        fgsizer.Add(st_stat, 0, wx.EXPAND)
        fgsizer.Add(stat_url, 0, wx.EXPAND)

        # 4
        fgsizer.Add(st_desc, 0, wx.EXPAND)
        fgsizer.Add(desc_url, 0, wx.EXPAND)

        # 5
        fgsizer.Add(st_reg, 0, wx.EXPAND)
        fgsizer.Add(reg_url, 0, wx.EXPAND)

        # 6
        fgsizer.Add(st_version_label, 0, wx.EXPAND)
        fgsizer.Add(st_version_value, 0, wx.EXPAND)

        # 6
        #fgsizer.Add(wx.Size(20,20), 0, wx.EXPAND)

        # 7
#        fgsizer.Add(wx.Size(20,5), 0, wx.EXPAND)
#        fgsizer.Add(wx.Size(20,5), 0, wx.EXPAND)

        sizer.Add(fgsizer, 0, wx.EXPAND|wx.ALL, 5)
        sizer.Add(btn_check, 0, wx.ALIGN_CENTER)
        sizer.Add(button, 0, wx.ALIGN_CENTER)

        self.Bind(wx.EVT_BUTTON, self.OnCheck, btn_check)

        self.SetSizer(sizer)
        self.Layout()
        
    def OnCheck(self, event=None):
        url = self.HOMEPAGE + "/version.txt"
        try:
            html = UrlGet(url)
            latest = int(html)
        except:
            wx.MessageBox("Fail to check","ERROR")
            return

        if ( latest > self.version_no ):
            diag = MyViewUpdateDlg.MyViewUpdateDlg(self)
            ret = diag.ShowModal()
            if ret == wx.ID_YES:
                choice = diag.GetSelection()
                self.Download(choice)
        else:
            wx.MessageBox("You are using the lastest version")

    def unzip(self, targetfile, skip=SKIP_FILES):
        zf = zipfile.ZipFile(targetfile, 'r')
        flist = []
        topDir = None
        for info in zf.infolist():
            try:
                name = info.filename
                if ( not topDir ):
                    topDir = name
                    topDir_pat = re.compile("^%s" % (info.filename))
                    continue
                name = re.sub(topDir_pat,'',name)
                if ( name in skip ):
                    continue
                attr = info.external_attr
                time_s = "%04d/%02d/%02d %02d:%02d:%02d" % info.date_time
                mtime = time.mktime(time.strptime(time_s,'%Y/%m/%d %H:%M:%S'))
                perm = (attr >> 16 ) & 0o777
                if name.endswith('/'):
                    if ( not os.path.exists(name)):
                        os.mkdir(name, perm)
                else:
                    with zf.open(info.filename) as fpi:
                        data = fpi.read()
                    with open(name, 'wb') as fpo:
                        fpo.write(data)            
                    os.chmod(name, perm)
                    os.utime(name, (mtime, mtime))
            except:
                import sys,traceback
                logger.error(traceback.print_exc())
        zf.close()

    def Download(self, choice):
        url = self.HOMEPAGE + "/myView2.zip"
        wildcard = "zip file (*.zip)|*.zip" 
        dialog = wx.FileDialog(None, "Input Save filename", os.getcwd(),
                "myView2.zip", wildcard, wx.SAVE)
        result = dialog.ShowModal()
        if result == wx.ID_OK:
            filepath = dialog.GetPath()
        dialog.Destroy()
        if result == wx.ID_OK:
            try:
                res = urllib.request.urlopen(url)
                zipFile = res.read()
                f=open(filepath,'wb')
                f.write(zipFile)
                f.close()
                if ( choice == MyViewUpdateDlg.DOWNLOAD_AND_UPDATE):
                    self.unzip(filepath)
                    import sys
                    sys.exit(0)
                
            except Exception as e:
                import sys,traceback
                logger.error(traceback.print_exc())
                wx.MessageBox("Fail to save","ERROR")
            

if __name__ == '__main__':
    app = wx.App()

    dialog = AboutDlg(None)
    result = dialog.ShowModal()
    addlist = []

    dialog.Destroy()

