#!/usr/bin/env python
# coding: utf-8
# $Id$
import matplotlib
matplotlib.use('WXAgg')
from matplotlib.figure import Figure
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
import scipy.stats
import matplotlib.cm as cm
import numpy as np
import wx
import os
import re
import sys

try:
    _dummy = wx.Colour
except NameError:
    wx.Colour = wx.Color
    
        
COLORMAPS = ['jet','gist_rainbow','Spectral','hot','gray', 'coolwarm', 'bwr','seismic', 'none']

COLORMAPS_DIC = {k:i for i,k in enumerate(COLORMAPS)}

class ColorMapPickerDlg(wx.Dialog):
    def __init__(self, parent, mapname='jet'):
        wx.Dialog.__init__(self, parent, -1, 'Color Map Picker',
                style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER|wx.STAY_ON_TOP)
        #self.SetBackgroundColour(wx.Colour(200, 200, 200))
        self.font = wx.Font(11, wx.MODERN, wx.NORMAL, wx.NORMAL, False)
      
#        self.colpanel = ColorPanel(self, self.colorpicker, maps, idx)

        #self.SetBackgroundColour(wx.NamedColour("WHITE"))
        self.SetBackgroundColour(wx.Colour("WHITE"))

        self.cmapid = COLORMAPS_DIC.get(mapname,0)

        self.figure = Figure()
        self.axes = self.figure.add_subplot(111)
        self.canvas = FigureCanvas(self, -1, self.figure)

        cmsizer = wx.BoxSizer(wx.HORIZONTAL)
        stCmap = wx.StaticText(self, -1, 'ColorMap: ')
        self.choiceCmap = wx.Choice(self, -1, choices=COLORMAPS)
        cmsizer.Add(stCmap, 0, 0)
        cmsizer.Add(self.choiceCmap, 0, wx.EXPAND)
        btnOk = wx.Button(self, wx.ID_OK, "OK")
        btnOk.SetDefault()
        btnCancel= wx.Button(self, wx.ID_CANCEL, "Cancel")
        btnsizer = wx.BoxSizer(wx.HORIZONTAL)
        btnsizer.Add(btnOk, 1, wx.EXPAND)
        btnsizer.Add(btnCancel, 1, wx.EXPAND)
        
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.canvas, 1, wx.EXPAND, 0)
        sizer.Add(cmsizer, 0, wx.EXPAND, 0)
        sizer.Add(btnsizer, 0, wx.EXPAND|wx.ALL, 5)
        
        self.SetSizer(sizer)
        self.plot()
        self.Fit()
        #self.colpanel.redraw()
        self.Bind(wx.EVT_CHOICE, self.OnCmapChanged, self.choiceCmap)
        self.Bind(wx.EVT_SIZE, self.OnResize, self)
        
        self.choiceCmap.SetSelection(self.cmapid)

    def bivariate_normal(self, x, y, covx, covy, mux, muy):
        rv = scipy.stats.multivariate_normal([mux, muy], [[covx, 0.0], [0.0, covy]])
        pos = np.zeros(x.shape + (2,))
        pos[:, :, 0] = x
        pos[:, :, 1] = y
        return rv.pdf(pos)
        
    def plot(self):
        delta = 0.025
        x = y = np.arange(-3.0, 3.01, delta)
        X, Y = np.meshgrid(x, y)
        #Z1 = plt.mlab.bivariate_normal(X, Y, 1.0, 1.0, 0.0, 0.0)
        Z1 = self.bivariate_normal(X, Y, 1.0, 1.0, 0.0, 0.0)
        #Z2 = plt.mlab.bivariate_normal(X, Y, 1.5, 0.5, 1, 1)
        Z2 = self.bivariate_normal(X, Y, 1.5, 0.5, 1, 1)
        Z = 10 * (Z1 - Z2)
        
        Z = np.ma.array(Z)
        cmap_name = COLORMAPS[self.cmapid]
        if ( cmap_name != 'none' ):
            cmap = cm.get_cmap(cmap_name)
            CS = self.axes.contourf(X, Y, Z, 10, # [-1, -0.1, 0, 0.1],
                                    #alpha=0.5,
                                    cmap=cmap)
            # Make a colorbar for the ContourSet returned by the contourf call.
            #        cbar = self.figure.colorbar(CS)
            self.cbar = self.figure.colorbar(CS)
        else:
            pass
        

        self.canvas.Refresh()
        self.canvas.draw()

    def OnResize(self, event):
        event.Skip()

    def GetResult(self):
        #return self.colpanel.getColorAtCursorRGB()
        return COLORMAPS[self.cmapid]

    def OnQuit(self, evt=None):
        print('OnQuit')
        self.Close()

    def OnCmapChanged(self, event):
        n = self.choiceCmap.GetSelection()
        self.cmapid = n
        self.figure.clear()
        self.axes = self.figure.add_subplot(111)
        self.plot()
        
if __name__ == '__main__':
    app = wx.App()
    dialog = ColorMapPickerDlg(None,'hot')
    result = dialog.ShowModal()
    print(('result=',result))
    if result == wx.ID_OK:
        dialogdata = dialog.GetResult()
        print(dialogdata)
    dialog.Destroy()

