#!/usr/bin/env python
# coding: utf-8
# $Id$

import wx
import os
import re
import sys

if wx.__version__ < '3.0':
    wx.Color = wx.Colour

def EmptyBitmap(w, h):
    if wx.__version__ < '4.0':
        return wx.EmptyBitmap(w, h)
    return wx.Bitmap(w, h)

class ColorButton(wx.BitmapButton):
    def __init__(self, parent, color):
        bmp = self.MakeBitmap(color)
        wx.BitmapButton.__init__(self, parent, -1, bmp)
   
    def set_color(self, color):
        bmp = self.MakeBitmap(color)
        self.SetBitmapLabel(bmp)

    def MakeBitmap(self, color):
        #bmp = wx.EmptyBitmap(200, 15)
        bmp = wx.Bitmap(200, 15)
        dc = wx.MemoryDC()
        dc.SelectObject(bmp)
        dc.SetBackground(wx.Brush(color))
        dc.Clear()
        dc.SelectObject(wx.NullBitmap)
        return bmp 

class ColorBmp(object):
    def __init__(self, red=0, green=0, blue=0):
        self.setRGB(red, green, blue)
    
    def getRGB(self):
        return self.red, self.green, self.blue
    
    def setRGB(self, r, g, b):
        self.red, self.green, self.blue = r, g, b

    def getColor(self):
        #print "getColor"
        return "#%02x%02x%02x" % (self.red, self.green, self.blue)

    def getbmp(self, width, height, flgcursor=False):
        #bmp = wx.EmptyBitmap(width, height)
        bmp = wx.Bitmap(int(width), int(height))
        dc = wx.MemoryDC()
        dc.SelectObject(bmp)
        dc.SetBackground(wx.Brush(self.getColor()))
        dc.Clear()
        if flgcursor:
            #col = "#%02x%02x%02x" % (~self.green, ~self.red, ~self.green)
            col = wx.Colour(255, 255, 255)
            dc.SetPen(wx.Pen(col, 3))
            dc.SetBrush(wx.Brush("Black", style=wx.TRANSPARENT))
            dc.DrawRectangle(0, 0, int(width), int(height))
            col = wx.Colour(0, 0, 0)
            dc.SetPen(wx.Pen(col, 1))
            dc.SetBrush(wx.Brush("Black", style=wx.TRANSPARENT))
            dc.DrawRectangle(0+1, 0+1, int(width)-2, int(height)-2)

        dc.SelectObject(wx.NullBitmap)
        return bmp 


class ColorPanel(wx.Panel):
    def __init__(self, parent, colorpicker, rows, cols, idx=0):
        wx.Panel.__init__(self, parent)
        self.parent = parent
        self.colorpicker = colorpicker
        self.colors = self.colorpicker.getColors()
        self.colors = self.colors[:rows*cols]
        
        self.rows = rows
        self.cols = cols
        if idx >= rows*cols:
            self.idx = 0
        else:
            self.idx = idx

        self.nullcolor = ColorBmp(0xFF, 0xFF, 0xFF)
        self.color_bmps = []

        for col in self.colors:
            bmp = col.getbmp(20, 20)
            self.color_bmps.append(wx.StaticBitmap(self, -1, bmp))

        for i in range(self.rows * self.cols - len(self.color_bmps)):
            bmp = self.nullcolor.getbmp(20, 20)
            self.color_bmps.append(wx.StaticBitmap(self, -1, bmp))


        fgsizer = wx.FlexGridSizer(rows=self.rows, cols=self.cols, hgap=1, vgap=1)
        for j in range(self.cols):
            fgsizer.AddGrowableCol(j)
        for i in range(self.rows):
            for j in range(self.cols):
                idx = j + i * self.cols
                fgsizer.Add(self.color_bmps[idx], 1, wx.EXPAND|wx.ALL, 0)
        self.SetSizer(fgsizer)
    
        for item in self.color_bmps:
            item.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown, item)
        self.Bind(wx.EVT_SIZE, self.OnResize, self)
        

    def OnResize(self, event=None):
        event.Skip()
        self.redraw()

    def setColorAtCursor(self, r, g, b):
        if self.idx < len(self.colors):
            self.colors[self.idx].setRGB(r, g, b)
            self.drawCursor(self.idx)

    def getColorAtCursor(self):
        if self.idx < len(self.colors):
            col = self.colors[self.idx].getColor()
        else:
            col = self.nullcolor.getColor()
        return col
    
    def getColorAtCursorRGB(self):
        if self.idx < len(self.colors):
            col = self.colors[self.idx].getRGB()
        else:
            col = self.nullcolor.getRGB()
        return col

    def redraw(self):
        bmpw, bmph = self.boxSize()

        for i in range(self.rows):
            for j in range(self.cols):
                idx = j + i * self.cols
                if idx < len(self.colors):
                    bmp = self.colors[idx].getbmp(bmpw, bmph, idx==self.idx)
                else:
                    bmp = self.nullcolor.getbmp(bmpw, bmph, idx==self.idx) 
                self.color_bmps[idx].SetBitmap(bmp)

    def boxSize(self):
        x, y, width, height = self.GetClientRect()
        bmpw = width / float(self.cols) -1
        bmph = height / float(self.rows) -1
        return bmpw, bmph

    def drawCursor(self, idx):
        bmpw, bmph = self.boxSize()
        if idx < len(self.colors):
            bmp = self.colors[idx].getbmp(bmpw, bmph, True)
        else:
            bmp = self.nullcolor.getbmp(bmpw, bmph, True) 
        self.color_bmps[idx].SetBitmap(bmp)

    def clearCursor(self, idx):
        bmpw, bmph = self.boxSize()
        if self.idx < len(self.colors):
            bmp = self.colors[self.idx].getbmp(bmpw, bmph, False)
        else:
            bmp = self.nullcolor.getbmp(bmpw, bmph, False) 
        self.color_bmps[self.idx].SetBitmap(bmp)

    def setCursor(self, idx):
        self.clearCursor(self.idx)
        self.idx = idx
        self.drawCursor(self.idx)

    def OnLeftDown(self, event=None):
        obj = event.GetEventObject()
        idx = self.color_bmps.index(obj)
        if idx < len(self.colors):
            self.setCursor(idx)
            r, g, b = self.colors[self.idx].getRGB()
            self.parent.setSlider(r, g, b)


class ColorPickerDlg(wx.Dialog):
    def __init__(self, parent, colorpicker=None, rows=6, cols=6, idx=0):
        wx.Dialog.__init__(self, parent, -1, 'Color Picker', 
                style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER|wx.STAY_ON_TOP)
        self.colorpicker = colorpicker
        self.colors = self.colorpicker.getColors()
        r, g, b = self.colors[idx].getRGB()
       
        #self.SetBackgroundColour(wx.Colour(200, 200, 200))
        self.font = wx.Font(11, wx.MODERN, wx.NORMAL, wx.NORMAL, False)
      
        self.colpanel = ColorPanel(self, self.colorpicker, rows, cols, idx)

        self.st_r = wx.StaticText(self, -1, 'R(%03d)' % r)
        self.st_g = wx.StaticText(self, -1, 'G(%03d)' % g)
        self.st_b = wx.StaticText(self, -1, 'B(%03d)' % b)
        self.slider_r = wx.Slider(self, -1, r, 0, 255, style=wx.SL_HORIZONTAL, name='R')
        self.slider_g = wx.Slider(self, -1, g, 0, 255, style=wx.SL_HORIZONTAL, name='G')
        self.slider_b = wx.Slider(self, -1, b, 0, 255, style=wx.SL_HORIZONTAL, name='B')
        slider_sizer = wx.FlexGridSizer(rows=3, cols=2, hgap=0, vgap=0)
        slider_sizer.AddGrowableCol(1)
        slider_sizer.Add(self.st_r, 0, wx.ALL, 0)
        slider_sizer.Add(self.slider_r, 1, wx.EXPAND, 0)
        slider_sizer.Add(self.st_g, 0, wx.ALL, 0)
        slider_sizer.Add(self.slider_g, 1, wx.EXPAND, 0)
        slider_sizer.Add(self.st_b, 0, wx.ALL, 0)
        slider_sizer.Add(self.slider_b, 1, wx.EXPAND, 0)

        btnOk = wx.Button(self, wx.ID_OK, "OK")
        btnOk.SetDefault()
        btnCancel= wx.Button(self, wx.ID_CANCEL, "Cancel")
        btnsizer = wx.BoxSizer(wx.HORIZONTAL)
        btnsizer.Add(btnOk, 1, wx.EXPAND)
        btnsizer.Add(btnCancel, 1, wx.EXPAND)
        
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.colpanel, 1, wx.EXPAND, 0)
        sizer.Add(slider_sizer, 0, wx.EXPAND, 0)
        sizer.Add(btnsizer, 0, wx.EXPAND|wx.ALL, 5)
        
        self.SetSizer(sizer)
        self.Fit()

        #self.colpanel.redraw()

        self.Bind(wx.EVT_SLIDER, self.OnSlider, self.slider_r)
        self.Bind(wx.EVT_SLIDER, self.OnSlider, self.slider_g)
        self.Bind(wx.EVT_SLIDER, self.OnSlider, self.slider_b)

    def getColorFromSlider(self):
        r = self.slider_r.GetValue()
        g = self.slider_g.GetValue()
        b = self.slider_b.GetValue()
        return r, g, b

    def setSlider(self, r, g, b):
        self.slider_r.SetValue(r)
        self.slider_g.SetValue(g)
        self.slider_b.SetValue(b)
        self.setLabel(r, g, b)

    def setLabel(self, r, g, b):
        self.st_r.SetLabel("R(%03d)" % r)
        self.st_g.SetLabel("G(%03d)" % g)
        self.st_b.SetLabel("B(%03d)" % b)

    def OnSlider(self, event=None):
        r, g, b = self.getColorFromSlider()
        self.setLabel(r, g, b)
        self.colpanel.setColorAtCursor(r, g, b)

    def GetResult(self):
        #return self.colpanel.getColorAtCursorRGB()
        return self.colpanel.getColorAtCursor()

    def OnQuit(self, evt=None):
        self.Close()


class ColorPicker(object):
    def __init__(self):
        self.setColors()

    def setColors(self):
        self.colors = []
        vlist = list(range(0, 255, int(255/6)))
        vlist.append(255)
        state = 0
        cols = []
        red = []
        green = []
        blue = []
        mono = []

        while(state < 3):
            if state == 0:
                for v in vlist:
                    cols.append((v, 0, 0))
                state = 1
            elif state == 1:
                for v in vlist:
                    cols.append((255, v, 0))
                state = 2 
            elif state == 2:
                for v in vlist:
                    cols.append((255, 255, v))
                state = 3 
        for r, g, b in cols:
            red.append((r, g, b))
            green.append((g, b, r))
            blue.append((b, r, g))
        
        vlist = list(range(0, 255, int(255/6)))
        vlist.append(255)
        for v in vlist:
            mono.append((v, v, v))

        for r, g, b in red+green+blue+mono:
            c = ColorBmp(r, g, b)
            self.colors.append(c)

    def getColors(self):
        return self.colors

    def getIndex(self, col):
        for i, c in enumerate(self.colors):
            if col.upper() == c.getColor().upper():
                return i
        return 0

    def getRGB(self, col):
        """
        col = '#rrggbb'
        """
        try:
            r = int(col[1:3], 16)
            g = int(col[3:5], 16)
            b = int(col[5:7], 16)
        except:
            r, g, b = 0, 0, 0
        return r, g, b

    def askUser(self, parent=None, col=None):
        if col is not None:
            idx = self.getIndex(col)
            r, g, b = self.getRGB(col)
        else:
            r, g, b = 0, 0, 0
            idx = 0
        self.colors[idx] = ColorBmp(r, g, b) 
        dialog = ColorPickerDlg(parent, self, rows=10, cols=8, idx=idx)
        result = dialog.ShowModal()
        col = False
        if result == wx.ID_OK:
            col = dialog.GetResult()
        dialog.Destroy()
        return col

if __name__ == '__main__':
    app = wx.App()
    cp = ColorPicker()
    print((cp.askUser(None, '#00FF00')))



