#!/usr/bin/env python
# coding:utf-8
# $Id$

import json
import os

DEFAULT = {'config_version':'1.1.1',
           'datamodulefilename':'datamodules.txt',
           'datamodulefoldername':'DataModules',
           'presetfilename':'presets.txt',
           'presetfoldername':'Presets',
           'enabledmodulefilename':'enabled.txt',
           'labelmodulefoldername':'LabelModules',
           'labelmodulefilename':'labelmodules.txt',
           'cachefoldername':'cache',
           'cachefilename':'cache.txt',
           'commandfoldername':'bin',
           'exportfoldername':'user/export',
           'printfoldername':'user/print',
           'tempfoldername':'temp',
           'notifyfoldername':'.',
           'homepage':'http://kaiseki-web.lhd.nifs.ac.jp/software/myView2/',
           'docpage':'http://kaiseki-web.lhd.nifs.ac.jp/software/myView2/myView2-Eng.pdf',
           'statpage':'http://egdb.lhd.nifs.ac.jp/dataexplog/registered_data.html',
           'regcheckpage':'http://kaiseki-dev.lhd.nifs.ac.jp/rails/dataexplog/dataexplog/index2',
           'datadescription':'http://kaiseki-dev.lhd.nifs.ac.jp/documents/diagnostics/dataname.shtml',
           'kserverini':'client2-lhd.ini',
           'kserverhost':'egftp1.lhd.nifs.ac.jp',
           'kserverdbhost':'egdb.lhd.nifs.ac.jp',
           'kserverdbname':'db1',
           'kserverdbuser':'anonymous',
           'kserverdbpass':'',
           'profile':'lhd'
           }
           
           
class MyViewConfig(object):
    """
    キャッシュディレクトリ等、設定情報を保持するクラス
    """
    def __init__(self, filename = None):
        self.filename = filename
        self.values = DEFAULT
        newval = {}
        if ( filename != None ):
            try:
                f = open(self.filename)
                newval = json.load(f)
                f.close()
            except:
                import sys,traceback
                traceback.print_exc()
        for k, v in list(newval.items()):
            self.values[k] = v

        self.createFolders()

    def dump(self):
        return json.dumps(self.values, sort_keys = True, indent = 4)

    def save(self, filename=None):
        if ( filename != None ):
            self.filename = filename
        f = open(self.filename,'w')
        f.write(self.dump())
        f.close()

    def createFolders(self):
        for k in ['cachefoldername', 'exportfoldername','exportfoldername', 'printfoldername','tempfoldername']:
            v = self.values[k]
            if not os.path.exists(v):
                os.makedirs(v)

    def __getitem__(self, key):
        if ( key in self.values ):
            return self.values[key]
        else:
            return None
    
    def __setitem__(self, key, val):
        if ( self.values[key] != val):
            self.values[key] = val
            self.save()

    def __contains__(self, key):
        return key in self.values
        
if __name__ == '__main__':
    config = MyViewConfig('myView2.cnf')
    #config = MyViewConfig()
    print((config['config_version']))
    print((config['homepage']))
    #config['config_version'] = '2.0.0'
    config.save('myView2-new.conf')
