#!/usr/bin/env python
# coding: utf-8
# $Id$
import os
import sys
import re
import threading
#import Queue
import time
import numpy
from MyViewGlobal import debug, logger
import imp
import random
import util

FlgDebug = debug
#FlgDebug = False

LockDataBusy = threading.Lock()
LockLabelBusy = threading.Lock()
LockPlotData = threading.Lock()
LockLabelData = threading.Lock()
LockDisplayCache = threading.Lock()
LockFetchLockSetup = threading.Lock()
INF = float('inf')
_INF = float('-inf')

class DataModules(object):
    """
    データモジュール管理クラス
    """
    def __init__(self, filepath, enabledpath):
        """
        Data Module
        """
        self.dict_datamodules = {}
        self.dict_enabledmodules = {}
        self.keylist = []
        self.interfaces = []
        self.cachesize = 100
        self.busy = []
        self.filepath = filepath
        self.enabledpath = enabledpath
        self.folderpath, self.filename = os.path.split(self.filepath)
        if self.folderpath == "":
            self.folderpath = '.'
        sys.path.insert(0, self.folderpath)
        self.create()
        self.ModuleLoad(self.filepath)
        self.EnabledModuleLoad(self.enabledpath)

    def changePath(self, filepath, enabledpath):
        self.ClearInstances()
        self.filepath = filepath
        self.enabledpath = enabledpath
        self.folderpath, self.filename = os.path.split(self.filepath)
        if self.folderpath == "":
            self.folderpath = '.'
        #sys.path.insert(0, self.folderpath)
        sys.path[0] = self.folderpath
        self.create()
        self.ModuleLoad(self.filepath)
        self.EnabledModuleLoad(self.enabledpath)

    def create(self):
        # check the folder or create
        if os.path.exists(self.folderpath):
            pass
        else:
            os.makedirs(self.folderpath, 0o644)

        # check the dictionary file or create
        if os.path.exists(self.filepath):
            pass
        else:
            fpo = open(self.filepath, 'w')
            fpo.close()

    def ResetModulePath(self, filepath): 
        self.ClearInstances()
        self.filepath = filepath
        self.folderpath, self.filename = os.path.split(self.filepath)
        sys.path.insert(0, self.folderpath)
        self.create()
        self.dict_datamodules = {}
        self.dict_enabledmodules = {}
        self.ModuleLoad(self.filepath)
        
    def SetBusy(self, ifc):
        """
        BUSY セット
        """
        LockDataBusy.acquire()
        self.busy.append(ifc)
        LockDataBusy.release()

    def IsBusy(self, ifc):
        return ifc in self.busy
    
    def UnsetBusy(self, ifc):
        """
        BUSY 解除
        """
        LockDataBusy.acquire()
        self.busy.remove(ifc)
        LockDataBusy.release()

    def ModuleLoad(self, filepath):
        """
        データモジュールのロード
        """
        self.ClearInstances()
        fpi = open(filepath, 'r')
        lines = fpi.readlines()
        fpi.close()
       
        loaded = list(self.dict_datamodules.keys())
        for i, l in enumerate(lines):
            l = l.strip('\n')
            if 0 == len(l): continue
            if l[0] == '#': continue
            if debug:
                clm = l.split('\t')
                keyname = clm[0]
                scriptname = clm[1].split('.')[0]
                #print keyname, scriptname
                if keyname in loaded:
                    mod = self.dict_datamodules[keyname]
                    imp.reload(mod)
                    logger.info("%s.py for %s reloaded." % (scriptname, keyname))
                else:
                    self.dict_datamodules[keyname] = __import__(scriptname)
                    logger.info("%s.py for %s imported." % (scriptname, keyname))
            else:
                try:
                    clm = l.split('\t')
                    keyname = clm[0]
                    scriptname = clm[1].split('.')[0]
                    if keyname in loaded:
                        mod = self.dict_datamodules[keyname]
                        imp.reload(mod)
                        logger.info("%s.py for %s reloaded." % (scriptname, keyname))
                    else:
                        self.dict_datamodules[keyname] = __import__(scriptname)
                        logger.info("%s.py for %s imported." % (scriptname, keyname))
                except:
                    import sys,traceback
                    logger.error(traceback.print_exc())
                    logger.error("Error: in line %d for Data Module." % (i + 1))

    def EnabledModuleLoad(self, filepath):
        """
        無効なデーターモジュール設定ファイルの読み込み
        """
        if ( os.path.exists(filepath) ):
            fpi = open(filepath, 'r')
            lines = fpi.readlines()
            fpi.close()
            self.dict_enabledmodules = {}
            loaded = list(self.dict_datamodules.keys())
            for i, l in enumerate(lines):
                key = l.strip('\n')
                if key in loaded:
                    self.dict_enabledmodules[key] = self.dict_datamodules[key]
        else:
            self.SetAllDisabled()
            self.EnabledModuleSave(filepath)

    def SetAllEnabled(self):
        self.dict_enabledmodules = self.dict_datamodules
        
    def SetAllDisabled(self):
        self.dict_enabledmodules = {}

    def SetEnabledModules(self, l):
        self.dict_enabledmodules = {}
        for e in l:
            self.dict_enabledmodules[e] = self.dict_datamodules[e]
        
    def AddEnabledModules(self, l):
        for e in l:
            if ( e in self.dict_datamodules ):
                self.dict_enabledmodules[e] = self.dict_datamodules[e]
        
    def EnabledModuleSave(self, filepath=None):
        """
        無効なデーターモジュール設定ファイルの書き込み
        """
        if ( not filepath ):
            filepath = self.enabledpath
        fpo = open(filepath, 'w')
        list = self.GetEnabledModules()
        for e in list:
            fpo.write(e+'\n')
        fpo.close()
    
    def RegistInstanceOfInterface(self, key, interface):
        """
        キーとモジュールを登録
        """
        if key not in self.keylist:
            self.keylist.append(key)
            self.interfaces.append(interface)

    def ShowKeys(self):
        """
        キー一覧を表示
        """
        logger.debug("==== INSTANCES ON MEMORY ====")
        for i, item in enumerate(self.keylist):
            logger.debug("%3d:\t%s" % (i, item))

    def TouchInstance(self, key):
        """
        Key で参照されるデータを末尾に移動
        """
        # change position to be last if you accesss.
        if key in self.keylist:
            idx = self.keylist.index(key)
            # Swap it to last one
            self.keylist[idx], self.keylist[-1] = self.keylist[-1], self.keylist[idx]
            self.interfaces[idx], self.interfaces[-1] = \
                    self.interfaces[-1], self.interfaces[idx]
            #print "touched data instance %s" % key

    def CheckCacheLimit(self):
        """
        Data Module
        """
        #delete if the size is over.
        if self.cachesize < len(self.keylist):
            del self.keylist[0]
            del self.interfaces[0]

    def MakeKey(self, modulename, dataid, option):
        """
        キーの文字列を作成
        """
        key = "%s_%s_%s" % (modulename, dataid, option)
        return key

    def IsExistInstance(self, key):
        """
        キーのモジュールが登録されているかチェック
        """
        return (key in self.keylist)
    
    def ClearInstances(self):
        """
        登録されているモジュールを全消去
        """
        self.keylist = []
        self.interfaces = []

    def GetDataModules(self):
        """
        全モジュール名を返す
        """
        l = list(self.dict_datamodules.keys())
        l.sort()
        return l

    def GetEnabledModules(self):
        """
        有効なモジュール名を返す
        """
        l = list(self.dict_enabledmodules.keys())
        l.sort()
        return l

    def SetEnabledModules(self, modules):
        """
       有効なモジュールの設定
        """
        self.dict_enabledmodules = {}
        loaded = list(self.dict_datamodules.keys())
        for key in modules:
            if key in loaded:
                self.dict_enabledmodules[key] = self.dict_datamodules[key]
            
    def GetDisabledModules(self):
        """
        無効なモジュール名を返す
        """
        l1 = self.GetDataModules()
        l2 = self.GetEnabledModules()
        return [e for e in l1 if not (e in l2)]
    
    def GetInstances(self):
        """
        キー一覧を返す
        """
        return self.keylist

    def GetInstance(self, modulename, dataid, option, args):
        """
        Data Module
        """
        """
        dict_prms use dictionary for give the parameter to data module. 
        """
        key = self.MakeKey(modulename, dataid, option)
        if self.IsExistInstance(key):
            idx = self.keylist.index(key)
            ifc = self.interfaces[idx]
            self.TouchInstance(key)
        else:
            if debug:
                ifc = self.dict_datamodules[modulename].InterFace(dataid, option=option, **args)
            else:
                try:
                    ifc = self.dict_datamodules[modulename].InterFace(dataid, option=option, **args)
                except:
                    ifc = None
            if ifc:
                self.RegistInstanceOfInterface(key, ifc)
                self.CheckCacheLimit()
        return ifc

class LabelModules(object):
    """
    アクティブラベル管理クラス
    """
    def __init__(self, filepath):
        """
        Label Module
        """
        self.dict_labelmodules = {}
        self.keylist = []
        self.interfaces = []
        self.cachesize = 100
        self.busy = []

        self.filepath = filepath
        self.folderpath, self.filename = os.path.split(self.filepath)
        if self.folderpath == "":
            self.folderpath = '.'
        sys.path.insert(0, self.folderpath)
        self.create()
        self.ModuleLoad(self.filepath)
    
    def changePath(self, filepath):
        self.ClearInstances()
        self.filepath = filepath
        self.folderpath, self.filename = os.path.split(self.filepath)
        if self.folderpath == "":
            self.folderpath = '.'
        sys.path[0] = self.folderpath
        self.create()
        self.ModuleLoad(self.filepath)

    def create(self):
        # check the folder or create
        if os.path.exists(self.folderpath):
            pass
        else:
            os.makedirs(self.folderpath, 0o644)

        # check the dictionary file or create
        if os.path.exists(self.filepath):
            pass
        else:
            fpo = open(self.filepath, 'w')
            fpo.close()

    def ResetModulePath(self, filepath): 
        self.ClearInstances()
        self.filepath = filepath
        self.folderpath, self.filename = os.path.split(self.filepath)
        sys.path.insert(0, self.folderpath)
        self.create()
        self.dict_labelmodules = {}
        self.ModuleLoad(self.filepath)

    def SetBusy(self, ifc):
        LockLabelBusy.acquire()
        self.busy.append(ifc)
        LockLabelBusy.release()

    def IsBusy(self, ifc):
        return ifc in self.busy
    
    def UnsetBusy(self, ifc):
        LockLabelBusy.acquire()
        self.busy.remove(ifc)
        LockLabelBusy.release()

    def ModuleLoad(self, filepath):
        """
        モジュールのロード
        """
        self.ClearInstances()
        fpi = open(filepath, 'r')
        lines = fpi.readlines()
        fpi.close()
        
        loaded = list(self.dict_labelmodules.keys())
        for i, l in enumerate(lines):
            l = l.strip('\n')
            if 0 == len(l): continue
            if l[0] == '#': continue
            if debug:
                logger.degug(l)
                clm = l.split('\t')
                keyname = clm[0]
                scriptname = clm[1].split('.')[0]
                #print keyname, scriptname
                if keyname in loaded:
                    mod = self.dict_labelmodules[keyname]
                    imp.reload(mod)
                    logger.info("%s.py for %s reloaded." % (scriptname, keyname))
                else:
                    self.dict_labelmodules[keyname] = __import__(scriptname)
                    logger.info("%s.py for %s imported." % (scriptname, keyname))
            else:
                try:
                    clm = l.split('\t')
                    keyname = clm[0]
                    scriptname = clm[1].split('.')[0]
                    if keyname in loaded:
                        mod = self.dict_labelmodules[keyname]
                        imp.reload(mod)
                        logger.info("%s.py for %s reloaded." % (scriptname, keyname))
                    else:
                        self.dict_labelmodules[keyname] = __import__(scriptname)
                        logger.info("%s.py for %s imported." % (scriptname, keyname))
                except:
                    logger.error("Error: in line %d for Label Module." % (i + 1))
                    if debug:
                        import sys,traceback
                        logger.error(traceback.print_exc())
        
    def RegistInstanceOfInterface(self, key, interface):
        """
        キーとインタフェイスオブジェクトを格納
        """
        if key not in self.keylist:
            self.keylist.append(key)
            self.interfaces.append(interface)

    def ShowKeys(self):
        """
        キーの一覧表示
        """
        logger.debug("==== INSTANCES ON MEMORY ====")
        for i, item in enumerate(self.keylist):
            print(("%3d:\t%s" % (i, item)))

    def TouchInstance(self, key):
        """
        指定したキーを最後に移動する
        """
        # change position to be last if you accesss.
        if key in self.keylist:
            idx = self.keylist.index(key)
            # Swap it to last one
            self.keylist[idx], self.keylist[-1] = self.keylist[-1], self.keylist[idx]
            self.interfaces[idx], self.interfaces[-1] = \
                    self.interfaces[-1], self.interfaces[idx]
            #print "touched label instance %s" % key

    def CheckCacheLimit(self):
        """
        Label Module
        """
        #delete if the size is over.
        if self.cachesize < len(self.keylist):
            del self.keylist[0]
            del self.interfaces[0]

    def MakeKey(self, modulename, dataid):
        """
        キーを生成
        """
        key = "%s_%s" % (modulename, dataid)
        return key

    def IsExistInstance(self, key):
        """
        キーが存在するかチェック
        """
        return (key in self.keylist)
    
    def ClearInstances(self):
        """
        Label Module
        """
        self.keylist = []
        self.interfaces = []

    def GetLabelModules(self):
        """
        Label Module
        """
        l = list(self.dict_labelmodules.keys())
        l.sort()
        return l
    
    def GetInstances(self):
        """
        Label Module
        """
        return self.keylist

    def GetInstance(self, modulename, dataid, args):
        """
        Label Module
        """
        """
        dict_prms use dictionary for give the parameter to data module. 
        """
        key = self.MakeKey(modulename, dataid)
        if self.IsExistInstance(key):
            idx = self.keylist.index(key)
            ifc = self.interfaces[idx]
            self.TouchInstance(key)
        else:
            if debug:
                ifc = self.dict_labelmodules[modulename].InterFace(dataid, **args)
            else:
                try:
                    ifc = self.dict_labelmodules[modulename].InterFace(dataid, **args)
                except:
                    ifc = None
            if ifc:
                self.RegistInstanceOfInterface(key, ifc)
                self.CheckCacheLimit()
        return ifc

DISPLAY_CACHE_DATA = {}
DISPLAY_CACHE_LABEL = {}
def ClearDisplayCache():
    global DISPLAY_CACHE_DATA
    global DISPLAY_CACHE_LABEL
    LockDisplayCache.acquire()
    DISPLAY_CACHE_DATA = {}
    DISPLAY_CACHE_LABEL = {}
    LockDisplayCache.release()


def ClearDisplayDataCache(idx, modulename, dataid, option, args):
    global DISPLAY_CACHE_DATA
    key = ProduceKeyForDisplayCache(modulename, dataid, option, args)
    try:
        del DISPLAY_CACHE_DATA[key]
    except:
        pass

def ShowDisplayCache():
    print("****** DISPLAY CACHE DATA ******")
    for i, k in enumerate(DISPLAY_CACHE_DATA.keys()):
        v = DISPLAY_CACHE_DATA.get(k)
        title = v[0]
        sz = len(v[1])
        print(("%d: %s:" % (i, k), (v is not None), title, sz))
    print("********************************")

def SerializeArgs(args):
    targets = ('varx', 'vary', 'varz', 'type', 'index')
    s = ["%s%s" % (k, v) for k, v in list(args.items()) if k in targets]
    s.sort()
    return "".join(s)

def ProduceKeyForDisplayCache(modulename, dataid, option_or_arg, args):
    key = "%s%s%s%s" % (modulename, dataid, option_or_arg, SerializeArgs(args))
    return key

def ElementCopy(d):
    if type(d) is numpy.ndarray:
        return d.copy()
    if type(d) is list:
        return d[:]
    return d

def DataCopy(data):
    return [ElementCopy(d) for d in data]

def FetchData(dm, que):
    """
    データ取得関数
    """
    idx, modulename, dataid, option, args = que
    ifc = dm.GetInstance(modulename, dataid, option, args)
    
    if ifc:
        # Make sure that the interface has a fetch lock
        with LockFetchLockSetup:
            if '_myview2_fetch_lock' not in dir(ifc):
                ifc._myview2_fetch_lock = threading.Lock()
        
        # Lock interface
        with ifc._myview2_fetch_lock:
            print(("Data Read start for %s %s %s" % (modulename, dataid, option)))
            if FlgDebug:
                data = ifc.dataset(option=option, **args)
            else:
                try:
                    data = ifc.dataset(option=option, **args)
                except:
                    import sys,traceback
                    logger.debug(traceback.print_exc())
                    data = ["Error in %s@%s" % (modulename, dataid), [], [], [], [], [], []]
            print(("Finish Data Read for %s %s %s" % (modulename, dataid, option)))
            title, x, xe, y, ye, z, ze = data
            xe = [abs(v) for v in xe]
            ye = [abs(v) for v in ye]
            ze = [abs(v) for v in ze]
            data = title, x, xe, y, ye, z, ze
            return data

class FetchDataThread(threading.Thread):
    """
    データ取得用のスレッド
    """
    def __init__(self, dm, traces, func_fetched):
        threading.Thread.__init__(self)
        self.datamodules = dm
        self.traces = traces
        self.func_fetched = func_fetched
        self.stop  = False
        self.fmt = re.compile(r'(.*)_([\+|\-]{0,1}[0-9]*\.*[0-9]*e*[\+|\-]{0,1}[0-9]*).*')

    def get_index_values(self, indexs):
        int_indexs = []
        double_indexs = []
        for i, sidx in enumerate(indexs):
            m = self.fmt.match(sidx)
            if m:
                try:
                    v = int(m.groups()[0])
                    int_indexs.append([i, v])
                except ValueError:
                    pass
                try:
                    v = float(m.groups()[1])
                    double_indexs.append([i, v])
                except ValueError:
                    pass
        return [int_indexs, double_indexs]

    def run(self):
        """
        メインループ
        """
        logger.debug("FetchDataThread.run()")
        global DISPLAY_CACHE_DATA
        new_cache = {}
        for trace in self.traces:
            logger.debug("FetchDataThread.run() trace="+str(trace))
            if ( self.stop ):
                break
            idx, modulename, dataid, option, args = trace
            que = (idx, modulename, dataid, option, args)
            key = ProduceKeyForDisplayCache(modulename, dataid, option, args)

            # check and read from display cache
            data = DISPLAY_CACHE_DATA.get(key)
            logger.debug("FetchDataThread.run() key=" + key + " data="+str(data))
            if data is None:
                data = FetchData(self.datamodules, que)
            else:
                data = DataCopy(data)
            self.func_fetched((idx, data))
            if data is not None:
                new_cache[key] = DataCopy(data)
            else:
                logger.debug("Error: NoData for the Key:[%s]" % key)
        LockDisplayCache.acquire()
        DISPLAY_CACHE_DATA = new_cache
        LockDisplayCache.release()

    def abort(self):
        self.stop = True

def FetchLabel(lm, que):
    """
    アクティブラベル取得関数
    """
    idx, modulename, dataid, arg, args = que
    ifc = lm.GetInstance(modulename, dataid, args)
        
    if ifc:
        # Make sure interface has a fetch lock
        with LockFetchLockSetup:
            if '_myview2_fetch_lock' not in dir(ifc):
                ifc._myview2_fetch_lock = threading.Lock()
            
        with ifc._myview2_fetch_lock:
            print(("Label Read start for %s %s %s" % (modulename, dataid, arg)))
            if FlgDebug:
                text = ifc.labeltext(arg=arg, **args)
            else:
                try:
                    text = ifc.labeltext(arg=arg, **args)
                except:
                    import sys,traceback
                    logger.debug(traceback.print_exc())
                    text = "Error in %s@%s" % (modulename, dataid)
            print(("Finish Label Read for %s %s %s" % (modulename, dataid, arg)))
            return text

class FetchLabelThread(threading.Thread):
    """
    ラベル取得用スレッド
    """
    def __init__(self, lm, activelabels, func_fetched):
        threading.Thread.__init__(self)
        self.labelmodules = lm
        self.stop  = False
        self.activelabels = activelabels
        self.func_fetched = func_fetched

    def run(self):
        global DISPLAY_CACHE_LABEL
        new_cache = {}
        for activelabel in self.activelabels:
            if ( self.stop ):
                return
            idx, modulename, dataid, arg, args = activelabel
            que = (idx, modulename, dataid, arg, args)
            key = ProduceKeyForDisplayCache(modulename, dataid, arg, args)
            
            # check and read from display cache
            text = DISPLAY_CACHE_LABEL.get(key)
            if text is None:
                text = FetchLabel(self.labelmodules, que)
            self.func_fetched((idx, text))
            new_cache[key] = text
        LockDisplayCache.acquire()
        DISPLAY_CACHE_LABEL = new_cache
        LockDisplayCache.release()

    def abort(self):
        self.stop = True

class DataInfo(object):
    """
    プロット用のデータを格納するクラス
    """
    def __init__(self, graphinfo, datamodulefilepath, enabledmodulefilepath, labelmodulefilepath, n_threads = 12):
        self.graphinfo = graphinfo
        self.datamodules = DataModules(datamodulefilepath, enabledmodulefilepath)
        self.labelmodules = LabelModules(labelmodulefilepath)
        self.labeldata = []
        self.labelthreads = [] 
        self.plotdata = []
        self.datathreads = []
        self.datacount = 0
        self.labelcount = 0
        self.n_threads = n_threads

    def clearInstances(self):
        self.datamodules.ClearInstances()
        self.labelmodules.ClearInstances()

    def getDataModuleSynopsis(self, modulename):
        pat = re.compile('\s*Synopsis\s*:\s*(.*)')
        tag_pat = re.compile(r"<[^>]*?>")
        try:
            htmltext = self.datamodules.dict_datamodules[modulename].helptext()
            for l in htmltext.splitlines():
                m = pat.match(l)
                if ( m ):
                    synopsis = tag_pat.sub("",m.group(1))
                    return synopsis
            return ''
        except:
            return ''

    def getDataModuleReference(self, modulename):
        pat=re.compile('Reference\s*:\s*<a\s+href=[\'"](.*)[\'"]>(.*)</a>', flags=re.IGNORECASE)
        try:
            htmltext = self.datamodules.dict_datamodules[modulename].helptext()
            for l in htmltext.splitlines():
                lst = pat.findall(l)
                if ( len(lst) > 0 ):
                    return lst[0]
            return None
        except:
            return None
        

    def getDataModuleHelp(self, modulename):
        """
        ヘルプの内容を返す
        """
        try:
            htmltext = self.datamodules.dict_datamodules[modulename].helptext()
            
        except:
            htmltext = None
        return htmltext
    
    def getLabelModuleHelp(self, modulename):
        logger.debug("MyViewDataInfo.getlabeModuleHelp("+modulename+")")
        try:
            htmltext = self.labelmodules.dict_labelmodules[modulename].helptext()
        except:
            htmltext = None
        return htmltext

    def getLabelModuleChoices(self):
        lst = self.labelmodules.GetLabelModules()
        return lst

    def getDataModuleChoices(self):
        lst = self.datamodules.GetDataModules()
        return lst

    def getEnabledDataModuleChoices(self):
        lst = self.datamodules.GetEnabledModules()
        return lst

    def getDataInfo(self, modulename, dataid, option, **args):
        """
        データ情報(ヘッダ等)を返す
        """
        ifc = self.datamodules.GetInstance(modulename, dataid, option, args)
        logger.debug("Getting datainfo for %s %s %s" % (modulename, dataid, option))
        if not "datainfo" in dir(ifc):
            return "This feature of this module is currently unsupported."
        if ifc:
            if debug:
                retv = ifc.datainfo(option=option, **args)
            else:
                try:
                    retv = ifc.datainfo(option=option, **args)
                except:
                    retv = ""
            return retv
        else:
            return ""

    def getDataVarsX(self, modulename, dataid, option, **args):
        """
        X 座標の値を返す
        """
        ifc = self.datamodules.GetInstance(modulename, dataid, option, args)
        logger.debug("Getting varx for %s %s %s" % (modulename, dataid, option))
        if ifc:
            if debug:
                retv = ifc.varsx(option=option, **args)
            else:
                try:
                    retv = ifc.varsx(option=option, **args)
                except:
                    retv = []
            return retv
        else:
            return []
        
    def getDataVarsY(self, modulename, dataid, option, **args):
        """
        Y 座標の値を返す
        """
        ifc = self.datamodules.GetInstance(modulename, dataid, option, args)
        if ifc:
            if debug:
                retv = ifc.varsy(option=option, **args)
            else:
                try:
                    retv = ifc.varsy(option=option, **args)
                except:
                    retv = []
            return retv
        else:
            return []

    def getDataVarsZ(self, modulename, dataid, option, **args):
        """
        Z 座標の値を返す
        """
        ifc = self.datamodules.GetInstance(modulename, dataid, option, args)
        if ifc:
            if debug:
                retv = ifc.varsz(option=option, **args)
            else:
                try:
                    retv = ifc.varsz(option=option, **args)
                except:
                    retv = []
            return retv
        else:
            return []

    def getDataIndexs(self, modulename, dataid, option, **args):
        ifc = self.datamodules.GetInstance(modulename, dataid, option, args)
        if ifc:
            if debug:
                retv = ifc.indexs(option=option, **args)
            else:
                try:
                    retv = ifc.indexs(option=option, **args)
                except:
                    retv = []
            return retv
        else:
            return []

    def getIndexs(self, trace):
        """
        trace のインデックスを返す
        """
        modulename = trace.modulename
        dataid = self.graphinfo.dataid
        option = trace.option
        args = self.graphinfo.args

        dataidc = trace.dataidc
        if dataidc.upper() == 'C':
            dataid = self.graphinfo.dataid
        elif dataidc.upper() == 'R':
            dataid = self.graphinfo.dataid_ref
        else:
            shot, sub, rshot, rsub = util.ShotStringToInteger(
                    self.graphinfo.dataid,
                    self.graphinfo.dataid_ref)
            dataid = util.ReplaceShotnumberInFilename(
                    dataidc, shot,sub, rshot, rsub)
        
        args['varx'] = trace.varx
        args['vary'] = trace.vary
        args['varz'] = trace.varz
        args['type'] = trace.plottype
        ifc = self.datamodules.GetInstance(modulename, dataid, option, args)
        #fmt = re.compile(r'.*_([0-9]*\.*[0-9]*).*')
        fmt = re.compile(r'.*_([\+|\-]{0,1}[0-9]*\.*[0-9]*e*[\+|\-]{0,1}[0-9]*).*')
        if ifc:
            if debug:
                indexs = ifc.indexs(option=option, **args)
            else:
                try:
                    indexs = ifc.indexs(option=option, **args)
                except:
                    indexs= []
            values = []
            for idx in indexs:
                m = fmt.match(idx)
                if m:
                    vstr = m.groups()[0]
                    try:
                        values.append(float(vstr))
                    except ValueError:
                        values = None
                        break
                else:
                    values = None
                    break
            return [indexs, values]
        else:
            return [None, None]

    def FetchData(self, traces):
        """
        データを取得するスレッドを開始する
        """
        logger.debug("DataInfo.FetchData")
        self.datathreads = []
        self.datacount = 0
        self.plotdata = []
        
        # Shuffle and chunk active traces to distribute them across threads
        traces = list(traces)
        random.shuffle(traces)
        
        def chunk(i):
            for k in range(i, len(traces), self.n_threads):
                yield traces[k]
        
        # Launch fetch threads
        for i in range(self.n_threads):
            subtraces = chunk(i)
            thread = FetchDataThread(self.datamodules, subtraces, self.funcDataFetched)
            thread.start()
            self.datathreads.append(thread)

    def funcDataFetched(self, idx_data):
        """
        取得したデータを plotdata 格納(スレッドから呼ばれる)
        """
        LockPlotData.acquire()
        self.plotdata.append(idx_data)
        self.datacount += 1
        LockPlotData.release()

    def AbortFetching(self):
        """
        スレッドを停止する。
        """
        for thread in self.datathreads:
            thread.abort()
        for thread in self.labelthreads:
            thread.abort()
        self.datathreads = []
        self.datacount = 0
        self.plotdata = []
        self.labeldata = []
        self.labelcount = 0
        self.labeldata = []

    def GetCount(self):
        """
        取得したデータ数を返す
        アクティブラベル数＋トレース数
        """
        LockPlotData.acquire()
        datacount = self.datacount
        LockPlotData.release()
        
        LockLabelData.acquire()
        labelcount = self.labelcount
        LockLabelData.release()

        return datacount+labelcount

    def IsFetchDataFinished(self):
        """
        データの取得が終わったか？
        """
        aliveThreads = [t.isAlive() for t in self.datathreads]
        if 0 == aliveThreads.count(True):
            self.datathreads = []
            return True
        return False

    def GetFetchDataCount(self):
        """
        データ取得用スレッドのアクティブなスレッド数と全スレッド数の組を返す
        """
        aliveThreads = [t.isAlive() for t in self.datathreads]
        return (aliveThreads.count(True), len(self.datathreads))

    def FetchLabels(self, activelabels):
        """
        alable = [idx, modulename, dataid, arg]
        を要素とする Active Label の配列の各要素について、
        アクティブラベルを取得するスレッドを作成し、配列 labelthreads
        に格納する
        """
        self.labeldata = []
        self.labelcount = 0
        self.labelthreads = []
        
        # Shuffle and chunk active labels to distribute them across threads
        activelabels = list(activelabels)
        random.shuffle(activelabels)
        
        def chunk(i):
            for k in range(i, len(activelabels), self.n_threads):
                yield activelabels[k]
        
        # Launch n_threads fetching threads
        for i in range(self.n_threads):
            sublabels = chunk(i)
            thread = FetchLabelThread(self.labelmodules, sublabels, self.funcLabelFetched)
            thread.start()
            self.labelthreads.append(thread)
        

    def funcLabelFetched(self, idx_data):
        """
        取得したラベルを labeldata に格納(スレッドから呼ばれる)
        """
        LockLabelData.acquire()
        self.labeldata.append(idx_data)
        LockLabelData.release()

    def IsFetchLabelFinished(self):
        aliveThreads = [t.isAlive() for t in self.labelthreads]
        if 0 == aliveThreads.count(True):
            self.labelthreads = []
            return True
        return False
    
    def GetFetchLabelCount(self):
        aliveThreads = [t.isAlive() for t in self.labelthreads]
        return (aliveThreads.count(True), len(self.labelthreads))

    def save(self, filepath, flggp=False):
        """
        テキストファイルにデータを出力
        """
        retv = []
        buff = []
        plotdata = self.plotdata[:]
        plotdata.sort()
        if flggp:
            nullchar = '-'
        else:
            nullchar = ''
        
        #output title
        traces = self.graphinfo.traces
        titles = []
        flgs = []
        clms = []
        clm = 0
        for idx, data in plotdata:
            clmx = clmxe = clmy = clmye = clmz = clmze = -1
            flgx = flgxe = flgy = flgye = flgz = flgze = False
            trace = traces[idx]
            panel = trace.panel
            group = panel.group
            xtitle = group.labelx.text
            ytitle = group.labely.text
            if xtitle == "":
                xtitle = 'x'
            if ytitle == "":
                ytitle = 'y'
            title, xdata, xedata, ydata, yedata, zdata, zedata = data
            if len(xdata) != 0:
                titles.append(xtitle)
                flgx = True
                clmx = clm
                clm = clm + 1
            if len(xedata) != 0:
                titles.append('xerr')
                flgxe = True
                clmxe = clm
                clm = clm + 1
            if trace.plottype == 'xyz':
                if len(ydata) != 0:
                    titles.append(ytitle)
                    flgy = True
                    clmy = clm
                    clm = clm + 1
                if len(yedata) != 0:
                    titles.append('yerr')
                    flgye = True
                    clmye = clm
                    clm = clm + 1
                if len(zdata) != 0:
                    titles.append(title)
                    flgz = True
                    clmz = clm
                    clm = clm + 1
                if len(zedata) != 0:
                    titles.append('err')
                    flgze = True
                    clmze = clm
                    clm = clm + 1
            else:
                if len(ydata) != 0:
                    titles.append(title)
                    flgy = True
                    clmy = clm
                    clm = clm + 1
                if len(yedata) != 0:
                    titles.append('err')
                    flgye = True
                    clmye = clm
                    clm = clm + 1
            flgs.append([flgx, flgxe, flgy, flgye, flgz, flgze])
            clms.append([clmx, clmxe, clmy, clmye, clmz, clmze])
        buff.append("#" + "\t".join(titles))
        
        for i, dataset in enumerate(plotdata):
            idx, data = dataset
            retv.append([idx, clms[i], flgs[i]])

        #output headder
        endflg = False 
        i = 0
        while not endflg:
            values = []
            for j, pdata in enumerate(plotdata):
                idx, data = pdata
                title, xdata, xedata, ydata, yedata, zdata, zedata = data
                flgx, flgxe, flgy, flgye, flgz, flgze = flgs[j]
                if i < len(xdata):
                    values.append("%g" % xdata[i])
                else:
                    if flgx:
                        values.append(nullchar)
                if i < len(xedata):
                    values.append("%g" % xedata[i])
                else:
                    if flgxe:
                        values.append(nullchar)
                if i < len(ydata):
                    values.append("%g" % ydata[i])
                else:
                    if flgy:
                        values.append(nullchar)
                if i < len(yedata):
                    values.append("%g" % yedata[i])
                else:
                    if flgye:
                        values.append(nullchar)
                if i < len(zdata):
                    values.append("%g" % zdata[i])
                else:
                    if flgz:
                        values.append(nullchar)
                if i < len(zedata):
                    values.append("%g" % zedata[i])
                else:
                    if flgze:
                        values.append(nullchar)
            if len(values) == values.count(nullchar):
                endflg = True
            else:
                buff.append("\t".join(values))
            i = i + 1

        fpo = open(filepath, "w")
        for l in buff:
            fpo.write(l+'\n')
        fpo.close()
        
        return retv
        
    def getdata(self, targetidx, plotdata):
        """
        plotdata の中から targetindex に一致するデータを返す
        """
        for idx, data in plotdata:
            if targetidx == idx:
                return data
        return None
    
    def save_compact(self, filename, flggp=False):
        """
        テキストファイルにデータを出力(コンパクト)
        """
        plotdata = self.plotdata[:]
        plotdata.sort()

        
        group_trace = []
        groups = sorted(self.graphinfo.groups, key=lambda g:g.groupid)

        for x in range(len(groups)):
            group_trace.append([])
        trace_data = {}
        for idx, trace in enumerate(self.graphinfo.traces):
            if not trace.flgshow: continue
            gid = trace.panel.group.groupid
            group_trace[gid].append(trace)
            data = self.getdata(idx, plotdata)
            trace_data[trace]=data
        
        try:
            basename, ext = os.path.splitext(filename)
            if ( len(self.graphinfo.groups) > 1 ):
                filename = "%s-g1%s" % ( basename, ext)
            for gid, group in enumerate(groups):
                dataAll = ()
                header1 = ""
                header2 = ""
                lastX = numpy.array([])
                maxLen = 0
                #print "** group=",group
                xtitle = group.labelx.text
                ytitle = group.labely.text
                if xtitle == "":
                    xtitle = 'x'
                #if ytitle == "":
                #    ytitle = 'y'
                for trace in sorted(group_trace[gid], key=lambda t:t.panel.panelid):
                    #print "** trace=",trace
                    if ( trace not in trace_data):
                        #print trace, "is not found in trace2data"
                        continue
                    data = trace_data[trace]
                    if ( data is None ):
                        continue
                    if ( trace.plottype == 'xyz' ):
                        self.save3D(trace, data, basename, ext)
                        continue
                    t, x, xe, y, ye, z, ze = data
                    if ( x is None ):
                        continue
                    if ( numpy.array_equal(lastX, x)):
                        header1 += ("%s\t" % trace.modulename)
                        header2 += ("%s\t" % t )
                        dataAll += (y,)
                    else:
                        header1 += ("%s\t" % trace.modulename)
                        header2 += ("%s\t%s\t" % (xtitle,t) )
                        dataAll += (x,y)
                    if ( len(xe) == len(x)):
                        header1 += "\t"
                        header2 += "Xerr\t"
                        dataAll+=(xe,)
                        
                    if ( len(ye) == len(x)):
                        header1 += "\t"
                        header2 += "Yerr\t"
                        dataAll+=(ye,)
                    if ( len(x) > maxLen):
                        maxLen = len(x)
                    lastX = x
                if ( gid > 0 ):
                    filename = "%s-g%d%s" % ( basename, (gid+1), ext)
                
                fp = open(filename,'w')
                fp.write(header1[:-1]+"\n")
                fp.write(header2[:-1]+"\n")
                cols = len(dataAll)
                for i in range(maxLen):
                    line = ""
                    for j in range(cols):
                        e = dataAll[j]
                        if ( i < len(e)):
                            if ((e[i] == INF) or (e[i] == _INF) or (e[i] != e[i])): # if e[i] is infinte or NaN
                                v = "\t"
                            else:
                                v = "%g\t" % e[i]
                        else:
                            v = "\t"
                        line += v
                    fp.write(line[:-1]+"\n")
                fp.close()
                
        except:
            import sys,traceback
            logger.error(traceback.print_exc())

    def save3D(self, trace, data, basename, ext):
        try:
            name = "%s_%s" % ( trace.modulename, trace.varz)
            filename = "%s-%s%s" % ( basename, name , ext)
            filename.replace('/',' ')
            fp = open(filename,'w')
            fp.write("%s, %s, %s\n" % (trace.varx, trace.vary, trace.varz))
            t, x, xe, y, ye, z, ze = data
            for i in range(len(x)):
                fp.write("%g, %g, %g\n" % (x[i],y[i],z[i]))
            fp.close()
        except:
            import sys,traceback
            logger.error(traceback.print_exc())
        

if __name__ == '__main__':
    from MyViewGraphInfo import GraphInfo
    sn = 106174
    graphinfo = GraphInfo()
    graphinfo.load('plot_default.txt')
#    datainfo = DataInfo(graphinfo, graphinfo.datamodulefilepath, graphinfo.labelmodulefilepath)
    datainfo = DataInfo(graphinfo, 'DataModules/datamodules.txt', 'DataModules/enabled.txt', 'LabelModules/labelmodules.txt')
    #datainfo = DataInfo()
    datamodules = datainfo.datamodules
    labelmodules = datainfo.labelmodules
    flg = True
    while (flg):
        s = eval(input('>'))
        clm = s.split(' ')
        if len(clm)==0: continue
        cmd = clm[0].lower()
        if cmd == 'quit':
            flg = False
        elif cmd == 'reload':
            datamodules.ModuleLoad('DataModules/datamodules.txt')
            labelmodules.ModuleLoad('LabelModules/labelmodules.txt')

        elif cmd == 'list':
            lst = datamodules.GetDataModules()
            print("------ Data Modules ------")
            for i, l in enumerate(lst):
                print(("%d: %s" % (i+1, l)))
            lst = labelmodules.GetLabelModules()
            print("------ Label Modules ------")
            for i, l in enumerate(lst):
                print(("%d: %s" % (i+1, l)))

        elif cmd == 'instances':
            lst = datamodules.GetInstances()
            print("------ Data Module Instances ------")
            for i, l in enumerate(lst):
                print(("%d: %s" % (i+1, l)))
            lst = labelmodules.GetInstances()
            print("------ Label Module Instances ------")
            for i, l in enumerate(lst):
                print(("%d: %s" % (i+1, l)))

        elif cmd == 'activelabel':
            if len(clm) < 3: 
                print("activelabel modulename dataid arg")
                continue
            modulename = clm[1]
            dataid = clm[2]
            if len(clm) == 4:
                arg = clm[3]
            else:
                arg = ""
            ifc = labelmodules.GetInstance(modulename, dataid)
            print((ifc.labeltext(arg=arg)))

        elif cmd == 'testlabel':
            lst = []
            for i in range(10):
                lst.append((i, 'directtext', '1234', 'Hello%d' % (i+1)))
            datainfo.FetchLabels(lst)
            while not datainfo.IsFetchLabelFinished():
                #while 10 != len(datainfo.labeldata):
                pass
            print((datainfo.labeldata))

        elif cmd == 'varsx':
            if len(clm) < 3: 
                print("varsx datamodule dataid [option]")
                continue
            datamodule = clm[1]
            dataid = clm[2]
            if len(clm) == 4:
                option = clm[3]
            else:
                option = ""
            ifc = datamodules.GetInstance(datamodule, dataid, option)
            print((ifc.varsx(dataid, option)))



