#/usr/bin/env python
# coding: utf-8
# $Id$

import os
import sys

class SaveGnuplot():
    def __init__(self, datafilepath, graphinfo, clminfo, plotdata):
        self.datafilepath = datafilepath
        self.graphinfo = graphinfo
        self.clminfo = clminfo
        self.plotdata = plotdata

        self.symboldic = { 
                'o_':6, 's_':4, 
                '^_':8, 'V_':10,
                'D_':12,  
                'o':7, 's':5, 
                '^':9, 'V':11,
                'D':13, '+':1,
                'x':2, '*':3,
                }
        
        self.linedic = {
                '-':1,
                '--':2,
                '-.':6,
                ':':0,
                }
        self.fontsizedic = {
                'xx-small':6, 
                'x-small':8, 
                'small':10, 
                'medium':12, 
                'large':16, 
                'x-large':20, 
                'xx-large':24}
        
        self.fontdic = {
                ('serif', 'normal', 'normal'):'Times',
                ('serif', 'bold', 'normal'):'Times-Bold',
                ('serif', 'normal', 'italic'):'Times-Italic',
                ('serif', 'bold', 'italic'):'Times-Bold-Italic',
                ('sans-serif', 'normal', 'normal'):'Arial',
                ('sans-serif', 'bold', 'normal'):'Arial-Bold',
                ('sans-serif', 'normal', 'italic'):'Arial-Italic',
                ('sans-serif', 'bold', 'italic'):'Arial-Bold-Italic',}

        self.vadic = {
                'top':1,
                'center':0.5,
                'bottom':0,
                'baseline':0,}

    def output(self, filepath, epsfile=None):
        fpo = open(filepath, "w")
        self.output_init(fpo, epsfile=epsfile)
        self.output_layout(fpo)
        self.output_label(fpo)
        for group in self.graphinfo.groups:
            for panel in group.panels:
                self.output_panel(fpo, panel)
        self.output_end(fpo, epsfile=epsfile)
        fpo.close()

    def output_init(self, fpo, epsfile=None):
        buff = []
        line = "# Gnuplot Script Produced with MyView"
        buff.append(line)
        line = "clear"
        buff.append(line)
        line = "reset\n"
        buff.append(line)
        line = 'set datafile separator "\\t"\n'
        buff.append(line)
        if epsfile:
            line = "set term postscript eps enhanced color size 16cm,10cm"
            buff.append(line)
            line = "set output '%s'\n" % epsfile
            buff.append(line)
        fpo.write("\n".join(buff))

    def output_layout(self, fpo):
        buff = []
        line = "set multiplot"
        buff.append(line)
        line = "set bmargin 0"
        buff.append(line)
        line = "set tmargin 0"
        buff.append(line)
        line = "set lmargin 0"
        buff.append(line)
        line = "set rmargin 0\n"
        buff.append(line)
        fpo.write("\n".join(buff))

    def output_end(self, fpo, epsfile=None):
        buff = []
        line = "unset multiplot\n"
        buff.append(line)
        if epsfile:
            line = "set output\n"
            buff.append(line)
        fpo.write("\n".join(buff))

    def getdata(self, targetidx):
        for idx, data in self.plotdata:
            if targetidx == idx:
                return data
        return None
    
    def getinfo(self, targetidx):
        for idx, clms, flgs in self.clminfo:
            if targetidx == idx:
                return [clms, flgs]
        return None
   
    def plot(self, panel):
        buff = []
        traces = [(i, t) for i, t in enumerate(self.graphinfo.traces) 
                if t.panel is panel and t.plottype!='xyz']  
        l = ""
        dict_plotwith = {}
        for i, t in traces:
            lstyle = self.linedic.get(t.linestyle, 1) 
            try:
                lwidth = int(t.linewidth)
            except ValueError:
                lwidth = 0
            color = t.color
            symbol = self.symboldic.get(t.symbol, 64)
            try:
                symbolsize = int(t.symbolsize)
            except ValueError:
                symbolsize = 0
            if lwidth == 0:
                plotwith = 'points'
                lw = 1
            elif symbolsize == 0:
                plotwith = 'lines'
                lw = lwidth
            else:
                plotwith = 'linespoints'
                lw = lwidth
            symbolsize = symbolsize / 6.0
            dict_plotwith[i] = plotwith
            
            l = l + "set style line %d lt %d lw %g lc rgb '%s' pt %d ps %g \n" % (i+1,
                    lstyle, lw, color, symbol, symbolsize)
        buff.append(l)
        
        #buff = buff + self.label(graph)
 
        if 0 != len(traces):
            l = "plot "
            for i, t in traces:
                data = self.getdata(i)
                info = self.getinfo(i)
                if not data: continue
                if not info: continue
                plotwith = dict_plotwith[i]
                title, x, xe, y, ye, z, ze = data
                clms, flgs = info
                # plot error
                flgerr = False
                datafile = "'%s' \\\n" % self.datafilepath
                l = l + datafile
                if clms[1] != -1 and clms[3] != -1:
                    flgerr = True
                    l = l + " u %d:%d:%d:%d with xyerr ls %d " % (
                            clms[0]+1, clms[2]+1, clms[1]+1, clms[3]+1, i+1)
                elif clms[1] == -1 and clms[3] != -1:
                    flgerr = True
                    l = l + " u %d:%d:%d with yerr ls %d " % (
                            clms[0]+1, clms[2]+1, clms[3]+1, i+1)
                elif clms[1] != -1 and clms[3] == -1:
                    flgerr = True
                    l = l + " u %d:%d:%d with xerr ls %d " % (
                            clms[0]+1, clms[2]+1, clms[1]+1, i+1)
                else:
                    flgerr = False
                    l = l + " u %d:%d " % (clms[0]+1, clms[2]+1)

                if t.plottype == 'xy2':
                    l = l + "axis x1y2 "
                elif t.plottype == 'xy':
                    l = l + "axis x1y1 "
                elif t.axis == 'xyz':
                    pass
                if flgerr:
                    l = l + "ti '' ,\\\n" 
                    # plot data
                    l = l + "'' u %d:%d " % (clms[0]+1, clms[2]+1)
                    l = l + "with %s ls %d " % (plotwith, i+1)
                    if title == '':
                        l = l + "ti '%s',\\\n" % ("%s %d:%d" % (
                            self.datafilepath, clms[0]+1, clms[2]+1))
                    else:
                        l = l + "ti '%s',\\\n" % (title)
                else:
                    l = l + "with %s ls %d " % (plotwith, i+1)
                    if title == '':
                        l = l + "ti '%s',\\\n" % ("%s %d:%d" % (
                            self.datafilepath, clms[0]+1, clms[2]+1))
                    else:
                        l = l + "ti '%s',\\\n" % (title)
            buff.append(l)
            buff[-1] = buff[-1][:-3]
        else:
            l = 'set yrange [0.0:1.0]\n'
            l = l + 'unset key\n'
            l = l + "plot 1/0\n"
            buff.append(l)
        return buff

    def output_label(self, fpo):
        buff = []
        l= ""
        for i, label in enumerate(self.graphinfo.staticlabels):
            text = label.label.text
            atx = label.posx
            aty = label.posy
            ha = label.label.halignment
            va = self.vadic.get(label.label.valignment, 0)
            key = (label.label.fontfamily, label.label.fontweight, label.label.fontstyle)
            font = self.fontdic.get(key, 'Times')
            try:
                size = int(label.label.fontsize)
            except:
                size = self.fontsizedic.get(label.label.fontsize, 12)
            color = label.label.fontcolor
            rotation = label.label.rotation
            l = l + 'set label %d "%s" at %s %g, %s %g %s rotate by %s offset 0,%d\n' % (i+1,
                    text, 'screen', atx, 'screen', aty,
                    ha, rotation, va)
            l = l + 'set label %d font "%s,%d"\n' % (i+1, font, size)
            l = l + "set label %d textcolor rgb '%s'\n" % (i+1, color)
        buff.append(l)
        count = i+1
        l = ""
        for i, label in enumerate(self.graphinfo.activelabels):
            text = label.label.text
            atx = label.posx
            aty = label.posy
            ha = label.label.halignment
            va = self.vadic.get(label.label.valignment, 0)
            key = (label.label.fontfamily, label.label.fontweight, label.label.fontstyle)
            font = self.fontdic.get(key, 'Times')
            try:
                size = int(label.label.fontsize)
            except:
                size = self.fontsizedic.get(label.label.fontsize, 12)
            color = label.label.fontcolor
            rotation = label.label.rotation
            l = l + 'set label %d "%s" at %s %g, %s %g %s rotate by %s offset 0,%d\n' % (
                    i+1+count,
                    text, 'screen', atx, 'screen', aty,
                    ha, rotation, va)
            l = l + 'set label %d font "%s,%d"\n' % (i+1+count, font, size)
            l = l + "set label %d textcolor rgb '%s'\n" % (i+1+count, color)
        buff.append(l)
        fpo.write("\n".join(buff))
        return buff

    def output_panel(self, fpo, panel):
        buff = []
        group = panel.group
        panelid = panel.panelid
        groupid = group.groupid
        left, bottom, width, height = group.getPanelsRect(panel)
        line = "\n# graph %d-%d" % (groupid+1, panelid+1)
        buff.append(line)
        line = "set size %f, %f" % (width, height)
        buff.append(line)
        line = "set origin %f, %f" % (left, bottom)
        buff.append(line)
        if group.xmin == group.xmax:
            line = "set xrange [*:*]"
            buff.append(line)
        else:
            line = "set xrange [%g:%g]" % (group.xmin, group.xmax)
            buff.append(line)
        
        if group.flglogx:
            line = "set logscale x"
            buff.append(line)

        if panel.flggrid:
            line = "set grid xtics mxtics"
            buff.append(line)
            line = "set grid ytics mytics"
            buff.append(line)

        if group.flgcommony:
            ymin = group.ymin
            ymax = group.ymax
            flglogy = group.flglogy
            labely = group.labely
            flgshowlegy = group.flgshowlegy
        else:
            ymin = panel.ymin
            ymax = panel.ymax
            flglogy = panel.flglogy
            labely = panel.labely
            flgshowlegy = group.flgshowlegy

        if ymin == ymax:
            line = "set yrange [*:*]"
        else:
            line = "set yrange [%g:%g]" % (ymin, ymax)
        buff.append(line)

        if flglogy:
            line = "set logscale y"
            buff.append(line)


        if panel.ptype == 1: #xy-y2
            line = "set ytics nomirror"
            buff.append(line)
            line = "set y2tics"
            buff.append(line)
            if panel.smin == panel.smax:
                line = "set y2range [*:*]"
                buff.append(line)
            else:
                line = "set y2range [%g:%g]" % (panel.smin, panel.smax)
                buff.append(line)
            
            if panel.flglogs:
                line = "set logscale y2"
                buff.append(line)
        else:
            line = "set ytics mirror"
            buff.append(line)
            line = "unset y2tics"
            buff.append(line)


            
        numcols = group.numcols
        numrows = group.numrows
        r = panelid / numcols
        c = panelid % numcols

        if r != numrows-1:
            line = 'set format x ""'
            buff.append(line)
        else:
            line = 'set format x "%g"'
            buff.append(line)

        if c != 0:
            line = 'set format y ""'
            buff.append(line)
        else:
            line = 'set format y "%g"'
            buff.append(line)
        
        if c != numcols -1:
            line = 'set format y2 ""'
            buff.append(line)
        elif panel.ptype == 1:
            line = 'set format y2 "%g"'
            buff.append(line)
        else:
            line = 'set format y2 ""'
            buff.append(line)

        if r == numrows-1 and c == (numcols)/2:
            key = (group.labelx.fontfamily, 
                   group.labelx.fontweight, 
                   group.labelx.fontstyle)
            font = self.fontdic.get(key, 'Times')
            try:
                size = int(group.labelx.fontsize)
            except:
                size = self.fontsizedic.get(group.labelx.fontsize, 12)
            va = self.vadic.get(group.labelx.valignment, 0)
            color = group.labelx.fontcolor
            line = 'set xlabel "%s" font "%s,%d" offset 0,1' % (
                    group.labelx.text, font, size)
            buff.append(line)
            line = "set xlabel textcolor rgb '%s'" % color
            buff.append(line)
        else:
            line = 'unset xlabel'
            buff.append(line)

        if group.flgcommony and r == (numrows)/2 and c == 0:
            key = (group.labely.fontfamily, 
                   group.labely.fontweight, 
                   group.labely.fontstyle)
            font = self.fontdic.get(key, 'Times')
            try:
                size = int(group.labely.fontsize)
            except:
                size = self.fontsizedic.get(group.labely.fontsize, 12)
            va = self.vadic.get(group.labely.valignment, 0)
            color = group.labely.fontcolor
            line = 'set ylabel "%s" font "%s,%d" offset 1,0' % (
                    group.labely.text, font, size)
            buff.append(line)
            line = "set ylabel textcolor rgb '%s'" % color
            buff.append(line)
        elif c == 0:
            key = (panel.labely.fontfamily, 
                   panel.labely.fontweight, 
                   panel.labely.fontstyle)
            font = self.fontdic.get(key, 'Times')
            try:
                size = int(panel.labely.fontsize)
            except:
                size = self.fontsizedic.get(panel.labely.fontsize, 12)
            va = self.vadic.get(panel.labely.valignment, 0)
            color = panel.labely.fontcolor
            line = 'set ylabel "%s" font "%s,%d" offset 1,0' % (
                    panel.labely.text, font, size)
            buff.append(line)
            line = "set ylabel textcolor rgb '%s'" % color
            buff.append(line)
        else:
            line = 'unset ylabel'
            buff.append(line)

        if c == numcols-1 and panel.ptype==1: #xy-y2
            key = (panel.labels.fontfamily, 
                   panel.labels.fontweight, 
                   panel.labels.fontstyle)
            font = self.fontdic.get(key, 'Times')
            try:
                size = int(panel.labels.fontsize)
            except:
                size = self.fontsizedic.get(panel.labels.fontsize, 12)
            va = self.vadic.get(panel.labels.valignment, 0)
            color = panel.labels.fontcolor
            line = 'set y2label "%s" font "%s,%d" offset -1,0' % (
                    panel.labels.text, font, size)
            buff.append(line)
            line = "set y2label textcolor rgb '%s'" % color
            buff.append(line)
        else:
            line = 'unset y2label'
            buff.append(line)

        if r == 0 and c == (numcols)/2:
            key = (group.labelt.fontfamily, 
                   group.labelt.fontweight, 
                   group.labelt.fontstyle)
            font = self.fontdic.get(key, 'Times')
            try:
                size = int(group.labelt.fontsize)
            except:
                size = self.fontsizedic.get(group.labelt.fontsize, 12)
            va = self.vadic.get(group.labelt.valignment, 0)
            color = group.labelt.fontcolor
            line = 'set title "%s" font "%s,%d" offset 0,-1' % (
                    group.labelt.text, font, size)
            buff.append(line)
            line = "set title textcolor rgb '%s'" % color
            buff.append(line)
        else:
            line = 'unset title'
            buff.append(line)

        line = 'set tics font "Times,8"'
        buff.append(line)
        line = 'set xtics offset 0,0.5'
        buff.append(line)
        line = 'set ytics offset 0.5,0'
        buff.append(line)
        if panel.ptype == 1: #xy-y2
            line = 'set y2tics offset -0.5,0'
            buff.append(line)

        # -----------------------------------------------------------------
        line = '# setup border line bottom 1 + left 2 + top 4 + right 8'
        buff.append(line)
        line = 'set border 15 lw 1'
        buff.append(line)
        if flgshowlegy or panel.flgshowlegs:
            line = 'set key right font "Times,8"'
            #line = 'set key right font "Times,8" opaque'
            buff.append(line)
        else:
            line = 'unset key'
            buff.append(line)

        buff = buff + self.plot(panel)

        line = 'unset label\n'
        buff.append(line)
        line = 'unset grid\n'
        buff.append(line)
        fpo.write("\n".join(buff))
    
