#!/usr/bin/env python
# coding: utf-8
# $Id$

from __future__ import print_function
import os
import re
import wx
import MyViewEnv

SUPPORTED_VERSION = "2.0.0"
DEFAULT_VERSION = "2.3.2"

LineStyleSet = ['-', '--', '-.', ':']
LineWidthSet = ['0', '1', '2', '3', '4', '5']
SymbolSet = ['o', '^', 'V', 's', 'D', '*', '+', 'x', '.',
    'o_', '^_', 'V_', 's_', 'D_', '*_']
SymbolSizeSet = ['0', '2', '4', '6', '8', '10', '12']


#FontFamilySet = ['serif', 'sans-serif', 'cursive', 'fantasy', 'monospace']
FontFamilySet = ['serif', 'sans-serif']
#FontWeightSet = ['light', 'normal', 'medium', 'semibold', 'bold', 'heavy', 'black']
FontWeightSet = ['normal', 'bold']
#FontStyleSet  = ['normal', 'italic', 'oblique']
FontStyleSet  = ['normal', 'italic']
FontSizeSet = ['xx-small', 'x-small', 'small', 'medium', 'large', 'x-large', 'xx-large']
HAlignmentSet = ['center', 'right', 'left']
VAlignmentSet = ['center', 'top', 'bottom', 'baseline']
RotationSet = ['0', '90', '180', '-90', '-180']

class Group():
    """
    グループを管理するクラス
    """
    def __init__(self, **args):
        self.groupid = args.get('groupid', None)
        self.numrows = args.get('numrows', 1)
        self.numcols = args.get('numcols', 1)
        self.left = args.get('left', 0.1)  
        self.bottom = args.get('bottom', 0.1)
        self.width = args.get('width', 0.8)
        self.height = args.get('height', 0.8)
        self.gap = args.get('gap', 0.0)
        self.xmin = args.get('xmin', 0.0)
        self.xmax = args.get('xmax', 0.0)
        self.flglogx = args.get('flglogx', False)
        self.labelx = Label("", valignment='center', halignment='center')
        self.ticklabelx = TickLabelStyle(valignment='top', halignment='center')
        self.ymin = args.get('ymin', 0.0)
        self.ymax = args.get('ymax', 0.0)
        self.flglogy = args.get('flglogy', False)
        self.flgshowlegy = args.get('flgshowlegy', True)
        self.labely = Label("", rotation="90", valignment='bottom', halignment='center')
        self.ticklabely = TickLabelStyle(valignment='center', halignment='right')
        self.flgcommony = args.get('flgcommony', False)
        self.linkx = args.get('linkx', None)
        self.linky = args.get('linky', None)
        self.labelt = Label("", valignment='center', halignment='center')
        self.panels = []
        self.panelsrect = []
        self.resetPanels()

    def isInside(self, x, y, rect):
        l, b, w, h = rect
        if x > l and x < l + w and y > b and y < b + h:
            return True
        else:
            return False

    def getPanelsAt(self, x, y):
        """
        get Panel at x, y
        """
        panels = []
        for panel in self.panels:
            rect = self.panelsrect[panel.panelid]
            if self.isInside(x, y, rect):
                panels.append(panel)
        return panels

    def getPanelsRect(self, panel):
        if panel in self.panels:
            idx = self.panels.index(panel)
            rect = self.panelsrect[idx]
        else:
            rect = None
        return rect

    def resetPanels(self):
        """
        Re-arrange panels when the num rows and num columns are changed.
        """
        newpanels = []
        newpanelsrect = []
        gapx = gapy = self.gap
        gl, gb, gw, gh = self.left, self.bottom, self.width, self.height
        for r in range(self.numrows):
            for  c in range(self.numcols):
                panelid = c + r * self.numcols
                panel = self.getPanel(panelid)
                if panel:
                    newpanels.append(panel)
                else:
                    panel = Panel(self, panelid = panelid)
                    newpanels.append(panel)
                w = (gw - gapx * (self.numcols-1)) / float(self.numcols)
                h = (gh - gapy * (self.numrows-1)) / float(self.numrows)
                l = gl + (w + gapx) * c
                b = (gb + gh) - h * (r+1) - gapy * r 
                newpanelsrect.append([l, b, w, h])
        self.panels = newpanels
        self.panelsrect = newpanelsrect

    def swapPanel(self, panel1, panel2):
        idx1 = self.panels.index(panel1)
        idx2 = self.panels.index(panel2)
        self.panels[idx1], self.panels[idx2] = self.panels[idx2], self.panels[idx1]
        panel1.panelid = idx2
        panel2.panelid = idx1

    def getPanel(self, panelid):
        for panel in self.panels:
            if panel.panelid == panelid:
                return panel
        return None

    def setData(self, **args):
        self.groupid = args.get('groupid', self.groupid)
        self.numrows = args.get('numrows', self.numrows)
        self.numcols = args.get('numcols', self.numcols)
        self.left = args.get('left', self.left)  
        self.bottom = args.get('bottom', self.bottom)
        self.width = args.get('width', self.width)
        self.height = args.get('height', self.height)
        self.gap = args.get('gap', self.gap)
        self.xmin = args.get('xmin', self.xmin)
        self.xmax = args.get('xmax', self.xmax)
        self.flglogx = args.get('flglogx', self.flglogx)
        self.labelx = args.get('labelx', self.labelx)
        self.ticklabelx = args.get('ticklabelx', self.ticklabelx)
        self.ymin = args.get('ymin', self.ymin)
        self.ymax = args.get('ymax', self.ymax)
        self.flglogy = args.get('flglogy', self.flglogy)
        self.flgshowlegy = args.get('flgshowlegy', self.flgshowlegy)
        self.labely = args.get('labely', self.labely)
        self.ticklabely = args.get('ticklabely', self.ticklabely)
        self.flgcommony = args.get('flgcommony', self.flgcommony)
        self.linkx = args.get('linkx', self.linkx) 
        self.linky = args.get('linky', self.linky) 
        self.labelt = args.get('labelt', self.labelt)
        if not self.checkGapValue():
            self.gap = 0.0
        self.resetPanels()

    def copy(self):
        group = Group()
        group.setData(
            groupid = self.groupid,
            numrows = self.numrows,
            numcols = self.numcols,
            left = self.left,
            bottom = self.bottom,
            width = self.width,
            height = self.height,
            gap = self.gap,
            xmin = self.xmin,
            xmax = self.xmax,
            flglogx = self.flglogx,
            labelx = self.labelx.copy(),
            ticklabelx = self.ticklabelx.copy(),
            ymin = self.ymin,
            ymax = self.ymax,
            flglogy = self.flglogy,
            flgshowlegy = self.flgshowlegy,
            labely = self.labely.copy(),
            ticklabely = self.ticklabely.copy(),
            flgcommony = self.flgcommony,
            linkx = self.linkx,
            linky = self.linky,
            labelt = self.labelt.copy())
        return group

    def checkGapValue(self):
        self.minimumSize = 0.01
        gapx = gapy = self.gap
        gl, gb, gw, gh = self.left, self.bottom, self.width, self.height
        w = (gw - gapx * (self.numcols-1)) / float(self.numcols)
        h = (gh - gapy * (self.numrows-1)) / float(self.numrows)
        if w >= self.minimumSize and h >= self.minimumSize:
            return True
        else:
            return False


class TickLabelStyle():
    """
    軸のラベルを管理するクラス
    """
    def __init__(self, **args):
        self.fmt = args.get('fmt', "")
        self.fontfamily = args.get('fontfamily', "sans-serif")
        self.fontsize = args.get('fontsize', "small")
#        self.fontsize = args.get('fontsize', "large")
        self.fontweight = args.get('fontweight', "normal")
        self.fontstyle = args.get('fontstyle', "normal")
        self.fontcolor = args.get('fontcolor', "#000000")
        self.halignment = args.get('halignment', "center")
        self.valignment = args.get('valignment', "center")
        self.rotation = args.get('rotation', '0')
        self.flgtrimfirst = False
        self.flgtrimlast = True

    def setData(self, **args):
        self.fmt = args.get('fmt', self.fmt)
        self.fontfamily = args.get('fontfamily', self.fontfamily)
        self.fontsize = args.get('fontsize', self.fontsize)
        self.fontweight = args.get('fontweight', self.fontweight)
        self.fontstyle = args.get('fontstyle', self.fontstyle)
        self.fontcolor = args.get('fontcolor', self.fontcolor)
        self.halignment = args.get('halignment', self.halignment)
        self.valignment = args.get('valignment', self.valignment)
        self.rotation = args.get('rotation', self.rotation)
        self.flgtrimfirst = args.get('flgtrimfirst', self.flgtrimfirst)
        self.flgtrimlast = args.get('flgtrimlast', self.flgtrimlast)

    def copy(self):
        ticklabelstyle = TickLabelStyle()
        ticklabelstyle.setData(
            fmt = self.fmt,
            fontfamily = self.fontfamily,
            fontsize = self.fontsize,
            fotweight = self.fontweight,
            fontstyle = self.fontstyle,
            fontcolor = self.fontcolor,
            halignment = self.halignment,
            valignment = self.valignment,
            rotation = self.rotation,
            flgtrimfirst = self.flgtrimfirst,
            flgtrimlast = self.flgtrimlast
            )
        return ticklabelstyle

class LabelStyle():
    """
    軸のラベルスタイルを管理するクラス
    """
    def __init__(self, **args):
        self.fontfamily = args.get('fontfamily', "sans-serif")
        self.fontsize = args.get('fontsize', "small")
        self.fontweight = args.get('fontweight', "normal")
        self.fontstyle = args.get('fontstyle', "normal")
        self.fontcolor = args.get('fontcolor', "#000000")
        self.halignment = args.get('halignment', "center")
        self.valignment = args.get('valignment', "center")
        self.rotation = args.get('rotation', '0')

    def setData(self, **args):
        self.fontfamily = args.get('fontfamily', self.fontfamily)
        self.fontsize = args.get('fontsize', self.fontsize)
        self.fontweight = args.get('fontweight', self.fontweight)
        self.fontstyle = args.get('fontstyle', self.fontstyle)
        self.fontcolor = args.get('fontcolor', self.fontcolor)
        self.halignment = args.get('halignment', self.halignment)
        self.valignment = args.get('valignment', self.valignment)
        self.rotation = args.get('rotation', self.rotation)

    def copy(self):
        labelstyle = LabelStyle()
        labelstyle.setData(
            fontfamily = self.fontfamily,
            fontsize = self.fontsize,
            fotweight = self.fontweight,
            fontstyle = self.fontstyle,
            fontcolor = self.fontcolor,
            halignment = self.halignment,
            valignment = self.valignment,
            rotation = self.rotation,
            )
        return labelstyle

class Label():
    """
    ラベルを管理するクラス
    """
    def __init__(self, text, **args):
        self.text = text
        self.fontfamily = args.get('fontfamily', "sans-serif")
        self.fontsize = args.get('fontsize', "medium")
        self.fontweight = args.get('fontweight', "normal")
        self.fontstyle = args.get('fontstyle', "normal")
        self.fontcolor = args.get('fontcolor', "#000000")
        self.halignment = args.get('halignment', "left")
        self.valignment = args.get('valignment', "bottom")
        self.rotation = args.get('rotation', '0')

    def setData(self, **args):
        self.text = args.get('text', self.text)
        self.fontfamily = args.get('fontfamily', self.fontfamily)
        self.fontsize = args.get('fontsize', self.fontsize)
        self.fontweight = args.get('fontweight', self.fontweight)
        self.fontstyle = args.get('fontstyle', self.fontstyle)
        self.fontcolor = args.get('fontcolor', self.fontcolor)
        self.halignment = args.get('halignment', self.halignment)
        self.valignment = args.get('valignment', self.valignment)
        self.rotation = args.get('rotation', self.rotation)

    def copy(self):
        label = Label(self.text)
        label.setData(
            fontfamily = self.fontfamily,
            fontsize = self.fontsize,
            fotweight = self.fontweight,
            fontstyle = self.fontstyle,
            fontcolor = self.fontcolor,
            halignment = self.halignment,
            valignment = self.valignment,
            rotation = self.rotation
            )
        return label
        
class Panel():
    """
    パネル（グラフ上の区画化された領域）を管理するクラス
    """
    def __init__(self, group, **args):
        self.group = group
        self.panelid = args.get('panelid', None)
        self.ymin = args.get('ymin', 0.0)
        self.ymax = args.get('ymax', 0.0)
        self.flglogy = args.get('flglogy', False)
        self.flgshowlegy = args.get('flgshowlegy', True)
        self.labely = Label("", rotation="90", valignment='bottom', halignment='center')
        self.ticklabely = TickLabelStyle(valignment='center', halignment='right')
        self.flgsubax = args.get('flgsubax', False)
        self.ptype = args.get('ptype', 0) #0:xy 1:xy-y2 2:xy-z
        self.smin = args.get('smin', 0.0)
        self.smax = args.get('smax', 0.0)
        self.flglogs = args.get('flglogs', False)
        self.flgshowlegs = args.get('flgshowlegs', True)
        self.labels = Label("", rotation="90", valignment='top', halignment='center')
        self.ticklabels = TickLabelStyle(valignment='center', halignment='left')
        self.flggrid = args.get('flggrid', False)
        self.linky = None # touple of (group, panel)
        self.interval = args.get('interval',0.0)
        self.interval2 = args.get('interval2',0.0)
        self.legendstyley = LabelStyle(fontsize='small', valignment='center', halignment='left')
        self.legendstyles = LabelStyle(fontsize='small', valignment='center', halignment='left')

    def setData(self, **args):
        self.group = args.get('group', self.group)
        self.panelid = args.get('panelid', self.panelid)
        self.ymin = args.get('ymin', self.ymin)
        self.ymax = args.get('ymax', self.ymax)
        self.flglogy = args.get('flglogy', self.flglogy)
        self.flgshowlegy = args.get('flgshowlegy', self.flgshowlegy)
        self.labely = args.get('labely', self.labely)
        self.ticklabely = args.get('ticklabely', self.ticklabely)
        self.flgsubax = args.get('flgsubax', self.flgsubax) 
        self.ptype = args.get('ptype', self.ptype) #0:xy 1:xy-y2 2:xy-z
        self.smin = args.get('smin', self.smin)
        self.smax = args.get('smax', self.smax)
        self.flglogs = args.get('flglogs', self.flglogs)
        self.flgshowlegs = args.get('flgshowlegs', self.flgshowlegs)
        self.labels = args.get('labels', self.labels)
        self.ticklabels = args.get('ticklabely', self.ticklabels)
        self.flggrid = args.get('flggrid', self.flggrid)
        self.linky = args.get('linky', self.linky)
        self.interval = args.get('interval',self.interval)
        self.interval2 = args.get('interval2',self.interval2)
        self.legendstyley = args.get('legendstyley', self.legendstyley)
        self.legendstyles = args.get('legendstyles', self.legendstyles)

    def copy(self):
        panel = Panel(self.group)
        panel.setData(
            panelid = self.panelid,
            ymin = self.ymin,
            ymax = self.ymax,
            flglogy = self.flglogy,
            flgshowlegy = self.flgshowlegy,
            labely = self.labely.copy(),
            ticklabely = self.ticklabely.copy(),
            flgsubax = self.flgsubax,
            ptype = self.ptype,
            smin = self.smin,
            smax = self.smax,
            flglogs = self.flglogs,
            flgshowlegs = self.flgshowlegs,
            labels = self.labels.copy(),
            ticklabels = self.ticklabels.copy(),
            flggrid = self.flggrid,
            linky = self.linky,
            interval = self.interval,
            interval2 = self.interval2,
            legendstyley = self.legendstyley.copy(),
            legendstyles = self.legendstyles.copy(),
            )
        return panel

    def getUpperPanel(self):
        if self.panelid == 0:
            return None
        upper_panelid = self.panelid -1
        upper_panel = self.group.panels[upper_panelid]
        return upper_panel

class Trace():
    """
    グラフの一本の線に対応するクラス
    """
    def __init__(self, panel,  **args):
        self.panel = panel
        self.modulename = args.get('modulename', "")
        self.dataidc = args.get('dataidc', "C")
        self.varx = args.get('varx', "")
        self.vary = args.get('vary', "")
        self.varz = args.get('varz', "")
        self.index = args.get('index', "")
        self.option = args.get('option', "")
        self.plottype = args.get('plottype', "xy") # xy2, xy2, xyz
        self.linestyle = args.get('linestyle', "-")
        self.linewidth = args.get('linewidth', "1")
        self.symbol = args.get('symbol', "o")
        self.symbolsize = args.get('symbolsize', "4")
        self.color = args.get('color', "#FF0000")
        self.factor = args.get('factor', 1.0)
        self.offset = args.get('offset', 0.0)
        self.flgshow = args.get('flgshow', True)
        self.cmap = args.get("cmap","")
        self.xfactor = args.get('xfactor', 1.0)
        self.xoffset = args.get('xoffset', 0.0)
        self.zfactor = args.get('zfactor', 1.0)
        self.zoffset = args.get('zoffset', 0.0)

    def setData(self, **args):
        self.panel = args.get('panel', self.panel)
        self.modulename = args.get('modulename', self.modulename)
        self.dataidc = args.get('dataidc', self.dataidc)
        self.varx = args.get('varx', self.varx)
        self.vary = args.get('vary', self.vary)
        self.varz = args.get('varz', self.varz)
        self.index = args.get('index', self.index)
        self.option = args.get('option', self.option)
        self.plottype = args.get('plottype', self.plottype)
        self.linestyle = args.get('linestyle', self.linestyle)
        self.linewidth = args.get('linewidth', self.linewidth)
        self.symbol = args.get('symbol', self.symbol)
        self.symbolsize = args.get('symbolsize', self.symbolsize)
        self.color = args.get('color', self.color)
        self.factor = args.get('factor', self.factor)
        self.offset = args.get('offset', self.offset)
        self.flgshow = args.get('flgshow', self.flgshow)
        self.cmap = args.get("cmap", self.cmap)
        self.xfactor = args.get('xfactor', self.xfactor)
        self.xoffset = args.get('xoffset', self.xoffset)
        self.zfactor = args.get('zfactor', self.xfactor)
        self.zoffset = args.get('zoffset', self.xoffset)

    def copy(self):
        trace = Trace(self.panel)
        trace.setData(
            modulename = self.modulename,
            dataidc = self.dataidc,
            varx = self.varx,
            vary = self.vary,
            varz = self.varz,
            index = self.index,
            option = self.option,
            plottype = self.plottype,
            linestyle = self.linestyle,
            linewidth = self.linewidth,
            symbol = self.symbol,
            symbolsize = self.symbolsize,
            color = self.color,
            factor = self.factor,
            offset = self.offset,
            flgshow = self.flgshow,
            cmap = self.cmap,
            xfactor = self.xfactor,
            xoffset = self.xoffset,
            zfactor = self.zfactor,
            zoffset = self.zoffset,
            )
        return trace

class ActiveLabel():
    """
    データベース等から取得し、内容の変更されるラベル
    """
    def __init__(self, **args):
        self.modulename = args.get('modulename', "")
        self.dataidc = args.get('dataidc', "C")
        self.arg = args.get('arg', "")
        self.posx = args.get('posx', 0.0)
        self.posy = args.get('posy', 0.0)
        self.label = Label("")

    def setData(self, **args):
        self.modulename = args.get('modulename', self.modulename)
        self.dataidc = args.get('dataidc', self.dataidc)
        self.arg = args.get('arg', self.arg)
        self.posx = args.get('posx', self.posx)
        self.posy = args.get('posy', self.posy)
        self.label = args.get('label', self.label)

    def copy(self):
        alabel = ActiveLabel()
        alabel.setData(
            modulename = self.modulename,
            dataidc = self.dataidc,
            arg = self.arg,
            posx = self.posx,
            posy = self.posy,
            label = self.label.copy())
        return alabel

class StaticLabel():
    def __init__(self, **args):
        self.posx = args.get('posx', 0.0)
        self.posy = args.get('posy', 0.0)
        self.label = Label("")

    def setData(self, **args):
        self.posx = args.get('posx', self.posx)
        self.posy = args.get('posy', self.posy)
        self.label = args.get('label', self.label)

    def copy(self):
        slabel = StaticLabel()
        slabel.setData(
            posx = self.posx,
            posy = self.posy,
            label = self.label.copy())
        return slabel

class GraphInfo():
    """
    グラフの描画属性
    """
    def __init__(self):
        self.canvassize = (1280, 960)
        self.dataid = "90000"
        self.dataid_ref = "90000"
        self.groups = []
        self.traces = []
        self.activelabels = []
        self.staticlabels = []
        self.dict_panelrect = {}
        self.positions = []
        #------ MyView Body Data -----
        self.args = {}
        #------ Color Controls -----
        self.facecolor = '#e9ddaf'
        self.facecolor = '#dddddd'
        self.realtime_printer = 'lastshot'
        self.realtime_delay = 1
        self.realtime_duration = 150
    def setData(self, *args):
        self.canvassize = args.get('canvassize', self.canvassize)
        self.dataid = args.get('dataid', self.dataid) 
        self.dataid_ref = args.get('dataid_ref', self.dataid_ref)
        self.groups = args.get('groups', self.groups)
        self.traces = args.get('traces', self.traces)
        self.activelabels = args.get('activelabels', self.activelabels)
        self.staticlabels = args.get('staticlabels', self.staticlabels)
        self.realtime_printer = args.get('realtime_printer', self.realtime_printer)
        self.realtime_delay = args.get('realtime_delay', self.realtime_delay)
        self.realtime_duration = args.get('realtime_duration', self.realtime_duration)

    def clear(self):
        self.groups = []
        self.traces = []
        self.activelabels = []
        self.staticlabels = []

    def moveUpTrace(self, trace):
        idx = self.traces.index(trace)
        if idx == 0: return
        self.traces[idx], self.traces[idx-1] = self.traces[idx-1], self.traces[idx]
    
    def swapTrace(self, idx0, idx1):
        """
        swap traces[idx0] and traces[idx1]
        """
        self.traces[idx0], self.traces[idx1] = self.traces[idx1], self.traces[idx0]

    def moveUpGroup(self, group):
        idx = self.groups.index(group)
        if idx == 0: return
        self.groups[idx], self.groups[idx-1] = self.groups[idx-1], self.groups[idx]
        self.groups[idx].groupid = idx
        self.groups[idx-1].groupid = idx-1

    def moveUpPanel(self, targetpanel):
        upper_panel = targetpanel.getUpperPanel()
        if upper_panel is None:
            return False
        # swap two panels        
        targetpanel.group.swapPanel(targetpanel, upper_panel)
        return True
    
    def swapPanel(self, targetpanel1, targetpanel2):
        group1 = targetpanel1.group
        group2 = targetpanel2.group
        if group1 is not group2:
            return False
        group1.swapPanel(targetpanel1, targetpanel2)
        return True

    def getIndexValue(self, trace):
        index = trace.index
        #fmt = re.compile(r'.*_([0-9]*\.*[0-9]*).*')
        fmt = re.compile(r'.*_([0-9]*\.*[0-9]*e*[\+|\-]{0,1}[0-9]*).*')
        value = None
        m = fmt.match(index)
        if m:
            vstr = m.groups()[0]
            try:
                value = float(vstr)
            except ValueError:
                value = None
        return value
        
    def correctLink(self):
        """
        group が削除された時の処理
        """
        for group in self.groups:
            if group.linkx not in self.groups:
                group.linkx = None
            if group.linky not in self.groups:
                group.linky = None
            for panel in group.panels:
                if panel.linky is not None:
                    g, p = panel.linky
                    if g not in self.groups:
                        panel.linky = None
                    else:
                        if p not in g.panels:
                            panel.linky = None

    def correctPanelInTraces(self):
        """
        trace が削除された時 panel を 再設定する。
        """
        removes = []
        for trace in self.traces:
            group = trace.panel.group
            panelid = trace.panel.panelid
            panel = group.getPanel(panelid)
            if panel is None:
                removes.append(trace)
        self.removeTraces(removes)

    def setGroupLink(self, group):
        """
        Set x and y range with link start from Group Change 
        """
        # X link loop
        for g in self.groups:
            g.flg = False
        group.flg = True
        xmin = group.xmin
        xmax = group.xmax
        flglogx = group.flglogx
        target = group.linkx
        while target and (not target.flg):
            target.xmin = xmin
            target.xmax = xmax
            target.flglogx = flglogx
            target.flg = True
            target = target.linkx

        # Y link loop
        if not group.flgcommony: return
        
        for g in self.groups:
            g.flg = False
        group.flg = True
        ymin = group.ymin
        ymax = group.ymax
        flglogy = group.flglogy
        target = group.linky
        while target and (not target.flg):
            target.flgcommony = True
            target.ymin = ymin
            target.ymax = ymax
            target.flglogy = flglogy
            target.flg = True
            target = target.linky

    def setPanelLink(self, panel):
        """
        Set y range with link start from Panel Change 
        """
        # for x-axis use group link
        group = panel.group
        for g in self.groups:
            g.flg = False
        group.flg = True
        xmin = group.xmin
        xmax = group.xmax
        flglogx = group.flglogx
        target = group.linkx
        while target and (not target.flg):
            target.xmin = xmin
            target.xmax = xmax
            target.flglogx = flglogx
            target.flg = True
            target = target.linkx

        # for y-axis
        for g in self.groups:
            for p in g.panels:
                p.flg = False
        panel.flg = True
        ymin = panel.ymin
        ymax = panel.ymax
        flglogy = panel.flglogy

        target = panel.linky
        while target and (not target[1].flg):
            #print target[0].group.groupid, target[1].panelid
            gp, pnl = target
            if gp.flgcommony:
                gp.ymin = ymin
                gp.ymax = ymax
                gp.flglogy = flglogy
                self.setGroupLink(gp) # => change to commony propagation
                break
            else:
                pnl.ymin = ymin 
                pnl.ymax = ymax
                pnl.flglogy = flglogy
                pnl.flg = True
                target = pnl.linky

    def getGroupChoices(self):
        """
        return list for group choices
        """
        choices = []
        for i, group in enumerate(self.groups):
            choices.append("Group No.%d(%d)" % (i+1, group.groupid+1))
        return choices

    def getPanelChoices(self, group):
        """
        return list for panel choices
        """
        choices = []
        for i, panel in enumerate(group.panels):
            choices.append("Panel No.%d(%d)" % (i+1, panel.panelid+1))
        return choices

    def getGroupList(self):
        """
        return data for list controlls
        """
        listdata = []
        for group in self.groups:
            groupid = "%d" % (group.groupid + 1)
            left = "%g" % group.left
            bottom = "%g" % group.bottom
            width = "%g" % group.width
            height = "%g" % group.height
            listdata.append([groupid, left, bottom, width, height])
        return listdata
            
    def getPanelList(self, groups):
        """
        return data for list controlls
        """
        listdata = []
        for group in groups:
            groupid = "%d" % (group.groupid + 1)
            for panel in group.panels:
                panelid = "%d" % (panel.panelid + 1)
                ymin = "%g" % panel.ymin
                ymax = "%g" % panel.ymax
                smin = "%g" % panel.smin
                smax = "%g" % panel.smax
                listdata.append([groupid, panelid, ymin, ymax, smin, smax])
        return listdata

    def getLegendList(self):
        """
        return data for list controlls
        """
        listdata = []
        ltype = 'Static Label'
        for i, slabel in enumerate(self.staticlabels):
            idx = "%d" % (i + 1)
            pos1 = "(%g, %g)" % (slabel.posx, slabel.posy)
            pos2 = "(-, -)"
            data = slabel.label.text
            arg = "-"
            color = slabel.label.fontcolor
            idc = ""
            listdata.append([idx, ltype, data, arg, idc, pos1, pos2, color])
        ltype = 'Active Label'
        for i, alabel in enumerate(self.activelabels):
            idx = "%d" % (i + 1)
            pos1 = "(%g, %g)" % (alabel.posx, alabel.posy)
            pos2 = "(-, -)"
            data = alabel.modulename
            arg = alabel.arg
            idc = alabel.dataidc
            color = alabel.label.fontcolor
            listdata.append([idx, ltype, data, arg, idc, pos1, pos2, color])
        #print listdata
        return listdata

    def getTraceList(self, panels):
        """
        return data for list controlls
        """
        listdata = []
        for i, trace in enumerate(self.traces):
            if trace.panel in panels:
                traceid = "%d" % (i + 1)
                groupid = "%d" % (trace.panel.group.groupid + 1)
                panelid = "%d" % (trace.panel.panelid + 1)
                modulename = "%s" % trace.modulename
                dataidc = "%s" % trace.dataidc
                varx = "%s" % trace.varx
                vary = "%s" % trace.vary
                varz = "%s" % trace.varz
                index = "%s" % trace.index
                option = "%s" % trace.option
                factor = "%s" % trace.factor
                offset = "%s" % trace.offset
                xfactor = "%s" % trace.xfactor
                xoffset = "%s" % trace.xoffset
                zfactor = "%s" % trace.zfactor
                zoffset = "%s" % trace.zoffset
                plottype = "%s" % trace.plottype
                color = "%s" % trace.color
                if not trace.flgshow:
                    color = "#CCCCCC"
                listdata.append([traceid, groupid, panelid, 
                    modulename, dataidc, varx, vary, varz, index, option, 
                    xfactor, xoffset, factor, offset, zfactor, zoffset, plottype, color])
        return listdata

    def addGroup(self, group):
        """
        グループを追加
        """
        self.groups.append(group)
        return len(self.groups) - 1

    def addTrace(self, trace):
        """
        トレースを追加
        """
        self.traces.append(trace)
        return len(self.traces) - 1

    def addActiveLabel(self, alabel):
        """
        アクティブラベルの追加
        """
        self.activelabels.append(alabel)
        return len(self.activelabels) - 1
    
    def addStaticLabel(self, slabel):
        """
        静的ラベルの追加
        """
        self.staticlabels.append(slabel)
        return len(self.staticlabels) - 1
    
    def removeGroups(self, removetargets):
        """
        グループの削除
        """
        newgroups = []
        for group in self.groups[:]:
            if group not in removetargets:
                group.groupid = len(newgroups) # renumber the groupid
                newgroups.append(group)
            else:
                targets = []
                for trace in self.traces: 
                    if trace.panel in group.panels:
                        targets.append(trace)
                self.removeTraces(targets)
        self.groups = newgroups

    def removeTraces(self, removetargets):
        """
        トレースの削除
        """
        newtraces = []
        for trace in self.traces[:]:
            if trace not in removetargets:
                newtraces.append(trace)
        self.traces = newtraces

    def removeActiveLabels(self, removetargets):
        newactivelabels = []
        for label in self.activelabels[:]:
            if label not in removetargets:
                newactivelabels.append(label)
        self.activelabels = newactivelabels
        
    def removeStaticLabels(self, removetargets):
        newstaticlabels = []
        for label in self.staticlabels[:]:
            if label not in removetargets:
                newstaticlabels.append(label)
        self.staticlabels = newstaticlabels
    
    def getModules(self):
        modules = set([])
        for t in self.traces:
            modules.add(t.modulename)
        return list(modules)

    def version_val(self, version):
        f = version.split('.') + ['0','0','0']
        major = int(f[0])
        minor = int(f[1])
        sub = int(f[2])
        val = major * 10000 + minor * 100 + sub
        return val

    def check_version(self, version):
        if ( self.version_val(version) < self.version_val(SUPPORTED_VERSION)):
            raise Exception("Unsupprted Version")

    def save(self, filepath):
        """
        設定ファイルの内容をセーブ
        """
        buff = ""

        grouptitle = "#cmd\tgroupid\tnrows\tncols\tleft\tbottom\twidth\theight\tgap\t" + \
                     "xmin\txmax\tlogx\tcomy\tymin\tymax\tlogy\tlegy\tlinkx\tlinky"
        paneltitle = "#cmd\tgroupid\tpanelid\tymin\tymax\tlogy\tlegy\t" + \
                     "type\tsmin\tsmax\tlogs\tlegs\tgrid\tlinkyg\tlinkyp\tinterval\tinterval2"
        glabeltitle = "#cmd\tgroupid\ttext\tat\tfamily\tstyle\tweight\tsize\tcolor\trotation\tha\tva\t"
        plabeltitle = "#cmd\tgroupid\tpanelid\tat\ttext\tfamily\tstyle\tweight\tsize\tcolor\trotation\tha\tva\t"
        tracetitle = "#cmd\tgroupid\tpanelid\tname\tidc\tvx\tvy\tvz\tidx\toption\t" + \
                     "type\ts\tss\tls\tlw\tcolor\tfactor\toffset\tshow\tcmap\t" + \
                     "xfactor\txoffset\tzfactor\tzoffset\t" 
        alabeltitle = "#cmd\tname\tidc\tposx\tposy\targ\tfamily\tstyle\tweight\tsize\tcolor\trotation\tha\tva\t"
        slabeltitle = "#cmd\tposx\tposy\ttext\tfamily\tstyle\tweight\tsize\tcolor\trotation\tha\tva\t"

        gticklabeltitle = "#cmd\tgroupid\tat\tfmt\tfamily\tstyle\tweight\tsize\tcolor\trotation\tha\tva\tft\tlt"
        pticklabeltitle = "#cmd\tgroupid\tpanelid\tat\tfmt\tfamily\tstyle\tweight\tsize\tcolor\trotation\tha\tva\tft\tlt"
        legendstyletitle = "#cmd\tgroupid\tpanelid\tat\tfamily\tstyle\tweight\tsize\tcolor\trotation\tha\tva"
#        version = "layout_version\t%s" % self.version
        version = "layout_version\t%s" % DEFAULT_VERSION
        buff = buff + version + "\n"
        canvassize = "canvassize\t%d\t%d" % self.canvassize
        buff = buff + canvassize + "\n"
        shotno = "dataid\t%s\t%s" % (self.dataid, self.dataid_ref)
        buff = buff + shotno + "\n"

        fcolor = "facecolor\t%s" % (self.facecolor)
        buff = buff + fcolor + "\n"
        realtimer_printer = "realtime_printer\t%s" % (self.realtime_printer)
        buff = buff + realtimer_printer + "\n"
        realtimer_delay = "realtime_delay\t%d" % (self.realtime_delay)
        buff = buff + realtimer_delay + "\n"
        realtimer_duration = "realtime_duration\t%d" % (self.realtime_duration)
        buff = buff + realtimer_duration + "\n"

        # Output Group
        buff = buff + '\n'
        buff = buff + grouptitle + "\n"
       
        panels = []
        output = []
        for group in sorted(self.groups, key=lambda g:g.groupid):
            panels = panels + sorted(group.panels, key=lambda p:p.panelid)
            groupid = group.groupid
            numrows = group.numrows
            numcols = group.numcols
            left = group.left  
            bottom = group.bottom
            width = group.width
            height = group.height
            gap = group.gap
            xmin = group.xmin
            xmax = group.xmax
            flglogx = int(group.flglogx)
            ymin = group.ymin
            ymax = group.ymax
            flglogy = int(group.flglogy)
            flgshowlegy = int(group.flgshowlegy)
            flgcommony = int(group.flgcommony)
            if group.linkx:
                linkxid = group.linkx.groupid 
            else:
                linkxid = -1
            if group.linky:
                linkyid = group.linky.groupid 
            else:
                linkyid = -1
            ot = "group\t%d\t%d\t%d\t%g\t%g\t%g\t%g\t%g\t%g\t%g\t%d\t%g\t%g\t%d\t%d\t%d\t%d\t%d" % (
                groupid, numrows, numcols, left, bottom, width, height, gap, 
                xmin, xmax, flglogx, ymin, ymax, flglogy, flgshowlegy, flgcommony, linkxid, linkyid)
            output.append(ot)
        line = "\n".join(output)
        buff = buff + line + '\n'

        # Output Group Label
        buff = buff + '\n'
        buff = buff + glabeltitle + "\n"
        output = []
        for group in sorted(self.groups, key=lambda g:g.groupid):
            labelx = group.labelx
            labely = group.labely
            labelt = group.labelt
            groupid = group.groupid
            for at, label in [['x', labelx], ['y', labely], ['t', labelt]]:
                text = label.text
                family = label.fontfamily
                style = label.fontstyle
                weight = label.fontweight
                size = label.fontsize
                color = label.fontcolor
                rotation = label.rotation
                ha = label.halignment
                va = label.valignment
                ot = "glabel\t%d\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s" % (
                    groupid, at, text, family, style, weight, size, color, rotation,ha, va)
                output.append(ot)
        line = "\n".join(output)
        buff = buff + line + '\n'

        # Output Group Tick Label
        buff = buff + '\n'
        buff = buff + gticklabeltitle + "\n"
        output = []
        for group in sorted(self.groups, key=lambda g:g.groupid):
            ticklabelx = group.ticklabelx
            ticklabely = group.ticklabely
            groupid = group.groupid
            for at, label in [['x', ticklabelx], ['y', ticklabely]]:
                fmt = label.fmt
                family = label.fontfamily
                style = label.fontstyle
                weight = label.fontweight
                size = label.fontsize
                color = label.fontcolor
                rotation = label.rotation
                ha = label.halignment
                va = label.valignment
                tf = int(label.flgtrimfirst)
                tl = int(label.flgtrimlast)
                ot = "gticklabel\t%d\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%d\t%d" % (
                    groupid, at, fmt, family, style, weight, size, color, rotation,
                    ha, va, tf, tl)
                output.append(ot)
        line = "\n".join(output)
        buff = buff + line + '\n'

        # Output Panel
        buff = buff + '\n'
        buff = buff + paneltitle + "\n"
        output = []
        for panel in panels:
            groupid = panel.group.groupid
            panelid = panel.panelid
            ymin = panel.ymin
            ymax = panel.ymax
            flglogy = int(panel.flglogy)
            flgshowlegy = int(panel.flgshowlegy)
            flgsubax = int(panel.flgsubax) 
            ptype = panel.ptype
            smin = panel.smin
            smax = panel.smax
            flglogs = int(panel.flglogs)
            flgshowlegs = int(panel.flgshowlegs)
            flggrid = int(panel.flggrid)
            interval = panel.interval
            interval2 = panel.interval2
            if panel.linky:
                linkygid = panel.linky[0].groupid 
                linkypid = panel.linky[1].panelid 
            else:
                linkygid = -1
                linkypid = -1
            ot = "panel\t%d\t%d\t%g\t%g\t%d\t%d\t%d\t%g\t%g\t%d\t%d\t%d\t%d\t%d\t%g\t%g" % (
                groupid, panelid, ymin, ymax, flglogy, flgshowlegy, 
                ptype, smin, smax, flglogs, flgshowlegs, flggrid, linkygid, linkypid, interval, interval2)
            output.append(ot)
        line = "\n".join(output)
        buff = buff + line + '\n'

        # Output Panel Label
        buff = buff + '\n'
        buff = buff + glabeltitle + "\n"
        output = []
        for panel in panels:
            labely = panel.labely
            labels = panel.labels
            groupid = panel.group.groupid
            panelid = panel.panelid
            for at, label in [['y', labely], ['s', labels]]:
                text = label.text
                family = label.fontfamily
                style = label.fontstyle
                weight = label.fontweight
                size = label.fontsize
                color = label.fontcolor
                rotation = label.rotation
                ha = label.halignment
                va = label.valignment
                ot = "plabel\t%d\t%d\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s" % (
                    groupid, panelid, 
                    at, text, family, style, weight, size, color, rotation,ha, va)
                output.append(ot)
        line = "\n".join(output)
        buff = buff + line + '\n'

        # Output Panel Tick Label
        buff = buff + '\n'
        buff = buff + pticklabeltitle + "\n"
        output = []
        for panel in panels:
            ticklabely = panel.ticklabely
            ticklabels = panel.ticklabels
            groupid = panel.group.groupid
            panelid = panel.panelid
            for at, label in [['y', ticklabely], ['s', ticklabels]]:
                fmt = label.fmt
                family = label.fontfamily
                style = label.fontstyle
                weight = label.fontweight
                size = label.fontsize
                color = label.fontcolor
                rotation = label.rotation
                ha = label.halignment
                va = label.valignment
                tf = int(label.flgtrimfirst)
                tl = int(label.flgtrimlast)
                ot = "pticklabel\t%d\t%d\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%d\t%d" % (
                    groupid, panelid, 
                    at, fmt, family, style, weight, size, color, rotation,ha, va, tf, tl)
                output.append(ot)
        line = "\n".join(output)
        buff = buff + line + '\n'

        # Output Panel Legend Style
        buff = buff + '\n'
        buff = buff + legendstyletitle + "\n"
        output = []
        for panel in panels:
            legendstyley = panel.legendstyley
            legendstyles = panel.legendstyles
            groupid = panel.group.groupid
            panelid = panel.panelid
            
            for at, lstyle in [['y', legendstyley], ['s', legendstyles]]:
                family = lstyle.fontfamily
                style = lstyle.fontstyle
                weight = lstyle.fontweight
                size = lstyle.fontsize
                color = lstyle.fontcolor
                rotation = lstyle.rotation
                ha = lstyle.halignment
                va = lstyle.valignment
                ot = "legendstyle\t%d\t%d\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s" % (
                        groupid, panelid, 
                        at, family, style, weight, size, color, rotation,ha, va)
                output.append(ot)
        line = "\n".join(output)
        buff = buff + line + '\n'

        # Output Trace
        buff = buff + '\n'
        buff = buff + tracetitle + "\n"
        output = []
        for trace in sorted(self.traces, key=lambda t: [t.panel.group.groupid, t.panel.panelid]):
            groupid = trace.panel.group.groupid
            panelid = trace.panel.panelid
            modulename = trace.modulename
            dataidc = trace.dataidc
            varx = trace.varx
            vary = trace.vary
            varz = trace.varz
            index = trace.index
            plottype = trace.plottype
            symbol = trace.symbol
            symbolsize = trace.symbolsize
            linestyle = trace.linestyle
            linewidth = trace.linewidth
            color = trace.color
            factor = trace.factor
            offset = trace.offset
            option = trace.option
            flgshow = int(trace.flgshow)
            cmap = trace.cmap
            xfactor = trace.xfactor
            xoffset = trace.xoffset
            zfactor = trace.zfactor
            zoffset = trace.zoffset
            ot = "trace\t%d\t%d\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%g\t%g\t%d\t%s\t%g\t%g\t%g\t%g" % (
                groupid, panelid, modulename, dataidc, varx, vary, varz, index, option,
                plottype, symbol, symbolsize, linestyle, linewidth, color, 
                factor, offset, flgshow, cmap, xfactor, xoffset, zfactor, zoffset)
            output.append(ot)
        line = "\n".join(output)
        buff = buff + line + '\n'

        # Output Active Label
        buff = buff + '\n'
        buff = buff + alabeltitle + "\n"
        output = []
        for alabel in self.activelabels:
            modulename = alabel.modulename 
            dataidc = alabel.dataidc
            arg = alabel.arg
            posx = alabel.posx 
            posy = alabel.posy 
            family = alabel.label.fontfamily
            style = alabel.label.fontstyle
            weight = alabel.label.fontweight
            size = alabel.label.fontsize
            color = alabel.label.fontcolor
            rotation = alabel.label.rotation
            ha = alabel.label.halignment
            va = alabel.label.valignment
            ot = "alabel\t%s\t%s\t%g\t%g\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s" % (
                modulename, dataidc, posx, posy, arg, 
                family, style, weight, size, color, rotation, ha, va)
            output.append(ot)
        line = "\n".join(output)
        buff = buff + line + '\n'

        # Output Static Label
        buff = buff + '\n'
        buff = buff + slabeltitle + "\n"
        output = []
        for slabel in self.staticlabels:
            posx = slabel.posx 
            posy = slabel.posy 
            text = slabel.label.text
            family = slabel.label.fontfamily
            style = slabel.label.fontstyle
            weight = slabel.label.fontweight
            size = slabel.label.fontsize
            color = slabel.label.fontcolor
            rotation = slabel.label.rotation
            ha = slabel.label.halignment
            va = slabel.label.valignment
            ot = "slabel\t%g\t%g\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s" % (
                posx, posy, text, 
                family, style, weight, size, color, rotation, ha, va)
            output.append(ot)
        line = "\n".join(output)
        buff = buff + line + '\n'

        self.safety_write(filepath, buff)

    def safety_write(self, filepath, buff):
        base, ext = os.path.splitext(filepath)
        temppath = base + '.tmp'
        count = 1
        while os.path.exists(temppath):
            temppath = base + '_%d.tmp' % count
            count = count + 1
        try:
            with open(temppath, 'w') as fpo:
                fpo.write(buff)
        except:
            os.unlink(temppath)
            wx.MessageBox('Failed saving Layout file.', 'Error')
            return
        self.file_copy(temppath, filepath)
        os.unlink(temppath)

    def file_copy(self, file_from, file_to):
        fsize = os.stat(file_from).st_size
        fsrc = open(file_from, "rb")
        fdst = open(file_to, "wb")
        while True: 
            buf = fsrc.read(2**20)
            if not buf:
                break
            fdst.write(buf)
        if fdst:
            fdst.close()
        if fsrc:
            fsrc.close()
            
    def load(self, filepath):
        self.clear()
        fpi = open(filepath, 'r')
        lines = fpi.readlines()
        fpi.close()

        errflg = False
        self.version = DEFAULT_VERSION
        linkxgroups = []
        linkygroups = []
        linkypanels = []
        for c, l in enumerate(lines):
            l = l.strip('\n')
            l = l.strip('\r')
            if len(l) == 0:
                continue
            if l[0] == '#':
                continue
            try:
                clm = l.split('\t')
                if 'LAYOUT_VERSION' == clm[0].upper():
                    version = clm[1]
                    self.check_version(version)
                    self.version = version

                if 'DATAID' == clm[0].upper():
                    self.dataid = clm[1]
                    self.dataid_ref = clm[2]

                if 'CANVASSIZE' == clm[0].upper():
                    width = int(clm[1])
                    height = int(clm[2])
                    self.canvassize = (width, height)
                
                if 'FACECOLOR' == clm[0].upper():
                    self.facecolor = clm[1]

                if 'REALTIME_PRINTER' == clm[0].upper():
                    self.realtime_printer = clm[1]

                if 'REALTIME_DELAY' == clm[0].upper():
                    self.realtime_delay = int(clm[1])

                if 'REALTIME_DURATION' == clm[0].upper():
                    self.realtime_duration = int(clm[1])

                if 'GROUP' == clm[0].upper():
                    groupid = int(clm[1])
                    numrows = int(clm[2])
                    numcols = int(clm[3])
                    left = float(clm[4])
                    bottom = float(clm[5])
                    width = float(clm[6])
                    height = float(clm[7])
                    gap = float(clm[8])
                    xmin = float(clm[9])
                    xmax = float(clm[10])
                    flglogx = bool(int(clm[11]))
                    ymin = float(clm[12])
                    ymax = float(clm[13])
                    flglogy = bool(int(clm[14]))
                    flgshowlegy = bool(int(clm[15]))
                    flgcommony = bool(int(clm[16]))
                    linkxid = int(clm[17])
                    linkyid = int(clm[18])
                    group = Group()
                    group.setData(
                        groupid = groupid,
                        numrows = numrows,
                        numcols = numcols,
                        left = left,
                        bottom = bottom,
                        width = width,
                        height = height,
                        gap = gap,
                        xmin = xmin,
                        xmax = xmax,
                        flglogx = flglogx,
                        ymin = ymin,
                        ymax = ymax,
                        flglogy = flglogy,
                        flgshowlegy = flgshowlegy,
                        flgcommony = flgcommony,
                        linkx = None, 
                        linky = None)
                    if linkxid != -1:
                        linkxgroups.append([groupid, linkxid])
                    if linkyid != -1:
                        linkygroups.append([groupid, linkyid])
                    self.addGroup(group)
                
                if 'GLABEL' == clm[0].upper():
                    groupid = int(clm[1])
                    at = clm[2]
                    text = clm[3]
                    family = clm[4]
                    style = clm[5]
                    weight = clm[6]
                    size = clm[7]
                    color = clm[8]
                    rotation = clm[9]
                    ha = clm[10]
                    va = clm[11]
                    group = self.groups[groupid]
                    if at == 'x': 
                        label = group.labelx
                    elif at == 'y':
                        label = group.labely
                    else: 
                        label = group.labelt
                    label.setData(
                        text = text,
                        fontfamily = family,
                        fontstyle = style,
                        fontweight = weight,
                        fontsize = size,
                        fontcolor = color,
                        rotation = rotation,
                        halignment = ha,
                        valignment = va)

                if 'GTICKLABEL' == clm[0].upper():
                    groupid = int(clm[1])
                    at = clm[2]
                    if len(clm)==13:
                        fmt = ""
                        family = clm[3]
                        style = clm[4]
                        weight = clm[5]
                        size = clm[6]
                        color = clm[7]
                        rotation = clm[8]
                        ha = clm[9]
                        va = clm[10]
                        tf = bool(int(clm[11]))
                        tl = bool(int(clm[12]))
                    else:
                        # new format with fmt(ticklabel)
                        fmt = clm[3]
                        family = clm[4]
                        style = clm[5]
                        weight = clm[6]
                        size = clm[7]
                        color = clm[8]
                        rotation = clm[9]
                        ha = clm[10]
                        va = clm[11]
                        tf = bool(int(clm[12]))
                        tl = bool(int(clm[13]))

                    group = self.groups[groupid]
                    if at == 'x': 
                        label = group.ticklabelx
                    else:
                        label = group.ticklabely
                    label.setData(
                        fmt = fmt,
                        fontfamily = family,
                        fontstyle = style,
                        fontweight = weight,
                        fontsize = size,
                        fontcolor = color,
                        rotation = rotation,
                        halignment = ha,
                        valignment = va,
                        flgtrimfirst = tf,
                        flgtrimlast = tl)

                if 'PANEL' == clm[0].upper():
                    groupid = int(clm[1])
                    panelid = int(clm[2])
                    ymin = float(clm[3])
                    ymax = float(clm[4])
                    flglogy = bool(float(clm[5]))
                    flgshowlegy = bool(float(clm[6]))
                    ptype = int(clm[7])
                    smin = float(clm[8])
                    smax = float(clm[9])
                    flglogs = bool(int(clm[10]))
                    flgshowlegs = bool(float(clm[11]))
                    flggrid = bool(int(clm[12]))
                    linkygid = int(clm[13])
                    linkypid = int(clm[14])
                    if ( self.version_val(self.version) < self.version_val("2.2.0")):
                        interval = 0.0
                    else:
                        interval = float(clm[15])
                    if ( self.version_val(self.version) < self.version_val("2.3.0")):
                        interval2 = 0.0
                    else:
                        interval2 = float(clm[16])

                    group = self.groups[groupid]
                    panel = group.panels[panelid]
                    if linkygid != -1:
                        linkypanels.append([groupid, panelid, linkygid, linkypid])

                    panel.setData(
                       panelid = panelid,
                       ymin = ymin,
                       ymax = ymax,
                       flglogy = flglogy,
                       flgshowlegy = flgshowlegy,
                       ptype = ptype,
                       smin = smin,
                       smax = smax,
                       flglogs = flglogs,
                       flgshowlegs = flgshowlegs,
                       flggrid = flggrid,
                       linky = None,
                       interval=interval,
                       interval2=interval2)
                    

                if 'PLABEL' == clm[0].upper():
                    groupid = int(clm[1])
                    panelid = int(clm[2])
                    at = clm[3]
                    text = clm[4]
                    family = clm[5]
                    style = clm[6]
                    weight = clm[7]
                    size = clm[8]
                    color = clm[9]
                    rotation = clm[10]
                    ha = clm[11]
                    va = clm[12]
                    group = self.groups[groupid]
                    panel = group.panels[panelid]
                    if at == 'y': 
                        label = panel.labely
                    else: 
                        label = panel.labels
                    label.setData(
                        text = text,
                        fontfamily = family,
                        fontstyle = style,
                        fontweight = weight,
                        fontsize = size,
                        fontcolor = color,
                        rotation = rotation,
                        halignment = ha,
                        valignment = va)

                if 'PTICKLABEL' == clm[0].upper():
                    groupid = int(clm[1])
                    panelid = int(clm[2])
                    at = clm[3]
                    if len(clm)==14:
                        fmt = ""
                        family = clm[4]
                        style = clm[5]
                        weight = clm[6]
                        size = clm[7]
                        color = clm[8]
                        rotation = clm[9]
                        ha = clm[10]
                        va = clm[11]
                        tf = bool(int(clm[12]))
                        tl = bool(int(clm[13]))
                    else:
                        # new format with fmt(ticklabel)
                        fmt = clm[4]
                        family = clm[5]
                        style = clm[6]
                        weight = clm[7]
                        size = clm[8]
                        color = clm[9]
                        rotation = clm[10]
                        ha = clm[11]
                        va = clm[12]
                        tf = bool(int(clm[13]))
                        tl = bool(int(clm[14]))
                    group = self.groups[groupid]
                    panel = group.panels[panelid]
                    if at == 'y': 
                        label = panel.ticklabely
                    else: 
                        label = panel.ticklabels
                    label.setData(
                        fmt = fmt,
                        fontfamily = family,
                        fontstyle = style,
                        fontweight = weight,
                        fontsize = size,
                        fontcolor = color,
                        rotation = rotation,
                        halignment = ha,
                        valignment = va,
                        flgtrimfirst = tf,
                        flgtrimlast = tl)
                
                if 'LEGENDSTYLE' == clm[0].upper():
                    groupid = int(clm[1])
                    panelid = int(clm[2])
                    at = clm[3]

                    family = clm[4]
                    style = clm[5]
                    weight = clm[6]
                    size = clm[7]
                    color = clm[8]
                    rotation = clm[9]
                    ha = clm[10]
                    va = clm[11]

                    group = self.groups[groupid]
                    panel = group.panels[panelid]
                    if at == 'y': 
                        lstyle = panel.legendstyley
                    else:
                        lstyle = panel.legendstyles
                    lstyle.setData(
                        fontfamily = family,
                        fontstyle = style,
                        fontweight = weight,
                        fontsize = size,
                        fontcolor = color,
                        rotation = rotation,
                        halignment = ha,
                        valignment = va,)

                if 'TRACE' == clm[0].upper():
                    groupid = int(clm[1])
                    panelid = int(clm[2])
                    modulename = clm[3]
                    dataidc = clm[4].upper() if clm[4] in ['c', 'r'] else clm[4]
                    varx = clm[5]
                    vary = clm[6]
                    varz = clm[7]
                    index = clm[8]
                    option = clm[9]
                    plottype = clm[10]
                    symbol = clm[11]
                    symbolsize = clm[12]
                    linestyle = clm[13]
                    linewidth = clm[14]
                    color = clm[15]
                    factor = float(clm[16])
                    offset = float(clm[17])
                    flgshow = bool(int(clm[18]))
                    if ( self.version_val(self.version) < self.version_val("2.3.0")):
                        cmap = ''
                    else:
                        cmap = clm[19]
                    if len(clm) > 20:
                        xfactor = float(clm[20])
                        xoffset = float(clm[21])
                    else:
                        xfactor = 1.0
                        xoffset = 0.0
                    if len(clm) > 22:
                        zfactor = float(clm[22])
                        zoffset = float(clm[23])
                    else:
                        zfactor = 1.0
                        zoffset = 0.0

                    panel = self.groups[groupid].panels[panelid]
                    trace = Trace(panel)
                    trace.setData(
                        modulename = modulename,
                        dataidc = dataidc,
                        varx = varx,
                        vary = vary,
                        varz = varz,
                        index = index,
                        option = option,
                        plottype = plottype,
                        symbol = symbol,
                        symbolsize = symbolsize,
                        linestyle = linestyle,
                        linewidth = linewidth,
                        color = color,
                        factor = factor,
                        offset = offset,
                        flgshow = flgshow,
                        interval = interval,
                        interval2 = interval2,
                        cmap = cmap,
                        xfactor=xfactor,
                        xoffset=xoffset,
                        zfactor=zfactor,
                        zoffset=zoffset,
                        )
                    self.addTrace(trace)

                if 'ALABEL' == clm[0].upper():
                    modulename = clm[1]
                    dataidc = clm[2]
                    posx = float(clm[3])
                    posy = float(clm[4])
                    arg = clm[5]
                    family = clm[6]
                    style = clm[7]
                    weight = clm[8]
                    size = clm[9]
                    color = clm[10]
                    rotation = clm[11]
                    ha = clm[12]
                    va = clm[13]
                    alabel = ActiveLabel(
                        modulename = modulename,
                        dataidc = dataidc,
                        posx = posx,
                        posy = posy,
                        arg = arg)
                    alabel.label.setData(
                        text = "",
                        fontfamily = family,
                        fontstyle = style,
                        fontweight = weight,
                        fontsize = size,
                        fontcolor = color,
                        rotation = rotation,
                        halignment = ha,
                        valignment = va)
                    self.addActiveLabel(alabel)

                if 'SLABEL' == clm[0].upper():
                    posx = float(clm[1])
                    posy = float(clm[2])
                    text = clm[3]
                    family = clm[4]
                    style = clm[5]
                    weight = clm[6]
                    size = clm[7]
                    color = clm[8]
                    rotation = clm[9]
                    ha = clm[10]
                    va = clm[11]
                    slabel = StaticLabel(
                        posx = posx,
                        posy = posy)
                    slabel.label.setData(
                        text = text,
                        fontfamily = family,
                        fontstyle = style,
                        fontweight = weight,
                        fontsize = size,
                        fontcolor = color,
                        rotation = rotation,
                        halignment = ha,
                        valignment = va)
                    self.addStaticLabel(slabel)
            except:
                print("Error: in line %d." % (c+1))
                import sys, traceback
                traceback.print_exc(file=sys.stdout)
                sys.exit(-1)
                errflg = True

        for groupid, linkxid in linkxgroups:
            self.groups[groupid].setData(linkx = self.groups[linkxid])
        for groupid, linkyid in linkygroups:
            self.groups[groupid].setData(linky = self.groups[linkyid])
        for groupid, panelid, linkygid, linkypid in linkypanels:
            self.groups[groupid].panels[panelid].setData(
                linky = (self.groups[linkygid], self.groups[linkygid].panels[linkypid])) 

    def nextSize(self, sizename, dir):
        nextsize = sizename
        
        for i, name in enumerate(FontSizeSet):
            if ( name == sizename ):
                ni = i + dir
                if ( 0 <= ni  and ni < len(FontSizeSet)):
                    nextsize = FontSizeSet[ni]
                break
        return nextsize
             
                
    def changeFontSize(self, direction=+1):
        """フォントサイズの変更"""

        if ( direction > 0 ):
            direction=1
        else:
            direction=-1
            
        # Group Label
        panels = []
        for group in sorted(self.groups, key=lambda g:g.groupid):
            panels = panels + sorted(group.panels, key=lambda p:p.panelid)
#            print('group.labelx.fontsize=',group.labelx.fontsize)
#            print('group.labely.fontsize=',group.labely.fontsize)
#            print('group.labelt.fontsize=',group.labelt.fontsize)
            group.labelx.fontsize=self.nextSize(group.labelx.fontsize, direction)
            group.labely.fontsize=self.nextSize(group.labely.fontsize, direction)
            group.labelt.fontsize=self.nextSize(group.labelt.fontsize, direction)
            group.ticklabelx.fontsize=self.nextSize(group.ticklabelx.fontsize, direction)
            group.ticklabely.fontsize=self.nextSize(group.ticklabely.fontsize, direction)
            
        
        for panel in panels:
            #print('panel.ticklabely.fontsize=',panel.ticklabely.fontsize)
            #print('panel.ticklabels.fontsize=',panel.ticklabels.fontsize)
            panel.labely.fontsize=self.nextSize(panel.labely.fontsize, direction)
            panel.ticklabely.fontsize=self.nextSize(panel.ticklabely.fontsize, direction)
            panel.ticklabels.fontsize=self.nextSize(panel.ticklabels.fontsize, direction)
            panel.legendstyley.fontsize=self.nextSize(panel.legendstyley.fontsize, direction)
            panel.legendstyles.fontsize=self.nextSize(panel.legendstyles.fontsize, direction)

        for alabel in self.activelabels:
            alabel.label.fontsize = self.nextSize(alabel.label.fontsize, direction)

        for slabel in self.staticlabels:
            slabel.label.fontsize = self.nextSize(slabel.label.fontsize, direction)

if __name__ == '__main__':
    import sys
    if ( len(sys.argv) == 3 ):
        infile = sys.argv[1]
        outfile = sys.argv[2]
    else:
        infile = 'plot_default.txt'
        outfile = 'test_plot_default.txt'
    graphinfo = GraphInfo()
    graphinfo.load(infile)
    graphinfo.save(outfile)

