#!/usr/bin/env python
# coding: utf-8
# $Id$

import wx
import wx.html
import os
import re
import sys

if wx.__version__ < '3.0':
    wx.Color = wx.Colour

class MyViewRecorderFrame(wx.Frame):
    """
    マウスをクリックした位置を記録するクラス
    """
    def __init__(self, parent, id):
        wx.Frame.__init__(self, parent, id, 'Recording...')

        self.tcRecord = wx.TextCtrl(self,-1,style=wx.TE_MULTILINE, size=(200,400))
        self.tcRecord.Disable()
        self.Running = False
        self.parent = parent

        menu_file = wx.Menu()
        menu_edit = wx.Menu()

        item_save = menu_file.Append(1,"Save")
        item_close = menu_file.Append(2,"Close")

        item_clear = menu_edit.Append(3,"Clear")

        menu_bar = wx.MenuBar()
        menu_bar.Append(menu_file,"File")
        menu_bar.Append(menu_edit,"Edit")

        self.SetMenuBar(menu_bar)

        vsizer = wx.BoxSizer(wx.VERTICAL)
        vsizer.Add(self.tcRecord,1, wx.EXPAND)
        self.SetSizer(vsizer)
        self.Fit()

        self.Bind(wx.EVT_MENU, self.OnClear, item_clear)
        self.Bind(wx.EVT_MENU, self.OnClose, item_close)
        self.Bind(wx.EVT_MENU, self.OnSave, item_save)
        self.Bind(wx.EVT_CLOSE, self.OnClose)

    def OnClear(self, event=None):
        """
        ClEAR が選択された
        """
        self.tcRecord.SetValue("")
        
    def OnSave(self, event=None):
        """
        SAVE が選択された
        """
        default_filename = "cursor_record.txt"
        wildcard = "text file (*.txt)|*.txt" 
        dialog = wx.FileDialog(None, "Input Export filename", self.parent.config['exportfoldername'],
                default_filename, wildcard, wx.SAVE)
        result = dialog.ShowModal()
        if result == wx.ID_OK:
            filepath = dialog.GetPath()
            self.parent.config['exportfoldername'], filename = os.path.split(filepath)
        dialog.Destroy()
        if result == wx.ID_OK:
            self.Save(filepath)

    def Save(self, filepath):
        """
        記録内容のセーブ
        """
        txt = self.tcRecord.GetValue()
        fp = open(filepath,"w")
        fp.write(txt)
        fp.close()

    def OnClose(self, event=None):
        """
        CLOSEの選択またはウィンドウのクローズ
        """
        self.parent.menuDict['Recorder*'].Check(False)
        self.Show(False)

    def Add(self, xydata):
        """
        座標の追加
        """
        txt = self.tcRecord.GetValue()
        if ( len(xydata) > 2 ):
            s = "%g, %g, %g\n" % ( xydata[0], xydata[1], xydata[2])
        else:
            s = "%g, %g\n" % ( xydata[0], xydata[1])
        txt = txt + s
        self.tcRecord.SetValue(txt)
        
if __name__ == '__main__':
    app = wx.PySimpleApp()
    recorder = MyViewRecorderFrame(None, -1)
    recorder.Show()
    app.MainLoop()

