#!/usr/bin/env python
# coding: utf-8
# $Id$

import wx
import wx.html
import os
import re
import sys

from MyViewGraphInfo import *

if wx.__version__ < '3.0':
    wx.Color = wx.Colour

class SelectGroupAndPanelDlg(wx.Dialog):
    def __init__(self, parent, trace=None, graphinfo=None):
        wx.Dialog.__init__(self, parent, -1, 'MyView Select Group and Panel', 
                style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER |wx.STAY_ON_TOP)
       
        #self.SetBackgroundColour(wx.Colour(200, 200, 200))
        self.font = wx.Font(11, wx.MODERN, wx.NORMAL, wx.NORMAL, False)

        self.trace = trace
        self.graphinfo = graphinfo

        stGroup = wx.StaticText(self, -1, "Group: ")
        stPanel = wx.StaticText(self, -1, "Panel: ")

        self.choiceGroup = wx.Choice(self, -1, choices=[])
        self.choicePanel = wx.Choice(self, -1, choices=[])

        fgsizer = wx.FlexGridSizer(rows=2, cols=2, hgap=5, vgap=5)
        fgsizer.AddGrowableCol(1)

        fgsizer.Add(stGroup, 0, wx.ALIGN_CENTER_VERTICAL) 
        fgsizer.Add(self.choiceGroup, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)
        fgsizer.Add(stPanel, 0, wx.ALIGN_CENTER_VERTICAL) 
        fgsizer.Add(self.choicePanel, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)

        btnOk = wx.Button(self, wx.ID_OK, "OK")
        btnOk.SetDefault()
        btnCancel= wx.Button(self, wx.ID_CANCEL, "Cancel")
        btnsizer = wx.BoxSizer(wx.HORIZONTAL)
        btnsizer.Add(btnOk, 1, wx.EXPAND)
        btnsizer.Add(btnCancel, 1, wx.EXPAND)
        
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(fgsizer, 1, wx.EXPAND|wx.ALL)
        sizer.Add(btnsizer, 0, wx.EXPAND|wx.ALL, 5)
        
        self.SetSizer(sizer)
        self.Fit()

        self.Bind(wx.EVT_SIZE, self.OnResize, self)
        self.Bind(wx.EVT_CHOICE, self.OnChoiceGroup, self.choiceGroup)
        
        self.SetStateFirst()

    def SetStateFirst(self):
        """
        Setup the value of widgets on this dialog 
        by referencing with self.group and self.graphinfo.
        """
        groupid = self.trace.panel.group.groupid
        group = self.graphinfo.groups[groupid]
        choices = self.graphinfo.getGroupChoices()
        self.choiceGroup.SetItems(choices)
        self.choiceGroup.SetSelection(groupid)

        panelid = self.trace.panel.panelid
        choices = self.graphinfo.getPanelChoices(group)
        self.choicePanel.SetItems(choices)
        self.choicePanel.SetSelection(panelid)

    def OnResize(self, event):
        event.Skip()

    def OnChoiceGroup(self, event):
        #print "OnChoiceGroup"
        event.Skip()
        groupid = self.choiceGroup.GetSelection()
        group = self.graphinfo.groups[groupid]
        choices = self.graphinfo.getPanelChoices(group)
        self.choicePanel.SetItems(choices)
        self.choicePanel.SetSelection(0)

    def OnQuit(self, evt=None):
        self.Close()

    def GetResult(self):
        groupid = self.choiceGroup.GetSelection()
        panelid = self.choicePanel.GetSelection()
        
        return groupid, panelid

if __name__ == '__main__':
    app = wx.App()

    graphinfo = GraphInfo()
    graphinfo.load('plot_default.txt')
#    datainfo = DataInfo(graphinfo, 'DataModules/datamodules.txt', 'LabelModules/labelmodules.txt')
    group = graphinfo.groups[0]
    panel = group.panels[0]
    trace = Trace(panel)
    trace.modulename = 'testdata'
    graphinfo.addTrace(trace)

    dialog = SelectGroupAndPanelDlg(None, trace, graphinfo)
    result = dialog.ShowModal()
    if result == wx.ID_OK:
        dialogdata = dialog.GetResult()
        print(dialogdata)
    dialog.Destroy()

