#!/usr/bin/env python
# coding: utf-8
# $Id$

import wx
import wx.lib.newevent
import os
try:
    _dummy = wx.Colour
except NameError:
    wx.Colour = wx.Color


class SetIndexStartStepDlg(wx.Dialog):
    """
    インデックス設定用ダイアログ
    """
    def __init__(self, parent, start=0.0, step=0.0):
        wx.Dialog.__init__(self, parent, -1, 'MyView Set Index Start and Step', 
                style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER)

        self.start = start
        self.step = step

        stStart = wx.StaticText(self, -1, "Start: ")
        stStep = wx.StaticText(self, -1, "Step: ")
        self.tcStart = wx.TextCtrl(self, -1, '', size=(80, 20), 
                style=wx.TE_PROCESS_ENTER)
        self.tcStep = wx.TextCtrl(self, -1, '', size=(80, 20),
                style=wx.TE_PROCESS_ENTER)

        fgsizer = wx.FlexGridSizer(rows=2, cols=2, hgap=5, vgap=5)
        fgsizer.AddGrowableCol(0)
        fgsizer.AddGrowableCol(1)
        fgsizer.Add(stStart, 1, wx.EXPAND)
        fgsizer.Add(stStep, 1, wx.EXPAND)
        
        fgsizer.Add(self.tcStart, 1, wx.EXPAND)
        fgsizer.Add(self.tcStep, 1, wx.EXPAND)

        self.okButton = wx.Button(self, wx.ID_OK, "OK")
        self.cancelButton = wx.Button(self, wx.ID_CANCEL, "Cancel")
        btnsizer = wx.BoxSizer(wx.HORIZONTAL)
        btnsizer.Add(self.okButton, 1, wx.EXPAND)
        btnsizer.Add(self.cancelButton, 1, wx.EXPAND)
        
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(fgsizer, 0, wx.EXPAND|wx.ALL)
        sizer.Add(btnsizer, 0, wx.EXPAND|wx.ALL, 5)
        self.SetSizer(sizer)
        self.Fit()

        self.tcStart.Bind(wx.EVT_TEXT_ENTER, self.OnStart, self.tcStart)
        self.tcStart.Bind(wx.EVT_KILL_FOCUS, self.OnStart, self.tcStart)
        self.tcStep.Bind(wx.EVT_TEXT_ENTER, self.OnStep, self.tcStep)
        self.tcStep.Bind(wx.EVT_KILL_FOCUS, self.OnStep, self.tcStep)

        self.SetStateFirst()

    def OnStart(self, event=None):
        vstr = self.tcStart.GetValue()
        try:
            v = float(vstr)
        except ValueError:
            v = self.start
        self.start = v
        self.tcStart.SetValue("%g" % self.start)

    def OnStep(self, event=None):
        vstr = self.tcStep.GetValue()
        try:
            v = float(vstr)
        except ValueError:
            v = self.step
        self.step = v
        self.tcStep.SetValue("%g" % self.step)

    def SetStateFirst(self):
        self.tcStart.SetValue("%g" % self.start)
        self.tcStep.SetValue("%g" % self.step)

    def GetResult(self):
        """
        設定した内容を配列で返す
        """
        return (float(self.tcStart.GetValue()), float(self.tcStep.GetValue()))


if __name__ == '__main__':
    app = wx.App()

    dialog = SetIndexStartStepDlg(None, 0.3, 0.1)
    result = dialog.ShowModal()
    if result == wx.ID_OK:
        print((dialog.GetResult()))
    dialog.Destroy()

