#!/usr/bin/env python
# coding: utf-8
# $Id$

import wx
from MyViewGraphInfo import *
from MyViewColorButton import *

try:
    _dummy = wx.Colour
except NameError:
    wx.Colour = wx.Color

class SetLabelDlg(wx.Dialog):
    """
    ラベル設定用ダイアログ
    """
    def __init__(self, parent, label=None, graphinfo=None, colorpicker=None):
        wx.Dialog.__init__(self, parent, -1, 'MyView2 Setup Label', 
                style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER)
        
        #self.SetBackgroundColour(wx.Colour(200, 200, 200))
        self.font = wx.Font(11, wx.MODERN, wx.NORMAL, wx.NORMAL, False)

        self.label = label
        self.graphinfo = graphinfo
        self.colorpicker = colorpicker
        """
        do not change the values in self.panel and self.graphinfo inside this dialog
        after back to parent, the values in the panel should be changed if the dialog 
        return the wx.ID_OK.
        You can get the values in this dialog with calling dialog.getResult()  
        """

        stText = wx.StaticText(self, -1, "Text: ")
        stFamily = wx.StaticText(self, -1, "Family: ")
        stStyle = wx.StaticText(self, -1, "Style: ")
        stWeight = wx.StaticText(self, -1, "Weight: ")
        stSize = wx.StaticText(self, -1, "Size: ")
        stColor = wx.StaticText(self, -1, "Color: ")
        stRotation = wx.StaticText(self, -1, "Rotation: ")
        stHAlign = wx.StaticText(self, -1, "HAlign: ")
        stVAlign = wx.StaticText(self, -1, "VAlign: ")

        self.tcText = wx.TextCtrl(self, -1, '', style=wx.TE_PROCESS_ENTER)
        self.choiceFamily = wx.Choice(self, -1, choices=[])
        self.choiceStyle = wx.Choice(self, -1, choices=[])
        self.choiceWeight = wx.Choice(self, -1, choices=[])
        self.comboSize = wx.ComboBox(self, -1, choices=[])
        self.comboRotation = wx.ComboBox(self, -1, choices=[])
        self.choiceHAlign = wx.Choice(self, -1, choices=[])
        self.choiceVAlign = wx.Choice(self, -1, choices=[])
        color = "#%02X%02X%02X" % (255, 0, 0)
        self.btnColor = ColorButton(self, color, self.colorpicker)
       
        fgsizer = wx.FlexGridSizer(rows=9, cols=2, hgap=5, vgap=5)
        fgsizer.AddGrowableCol(1)
        fgsizer.Add(stText, 0, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)
        fgsizer.Add(self.tcText, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)
        fgsizer.Add(stFamily, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)
        fgsizer.Add(self.choiceFamily, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)
        fgsizer.Add(stStyle, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)
        fgsizer.Add(self.choiceStyle, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)
        fgsizer.Add(stWeight, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)
        fgsizer.Add(self.choiceWeight, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)
        fgsizer.Add(stSize, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)
        fgsizer.Add(self.comboSize, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)
        fgsizer.Add(stColor, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)
        fgsizer.Add(self.btnColor, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)
        fgsizer.Add(stRotation, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)
        fgsizer.Add(self.comboRotation, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)
        fgsizer.Add(stHAlign, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)
        fgsizer.Add(self.choiceHAlign, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)
        fgsizer.Add(stVAlign, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)
        fgsizer.Add(self.choiceVAlign, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)

        btnOk = wx.Button(self, wx.ID_OK, "OK")
        btnOk.SetDefault()
        btnCancel= wx.Button(self, wx.ID_CANCEL, "Cancel")
        btnsizer = wx.BoxSizer(wx.HORIZONTAL)
        btnsizer.Add(btnOk, 1, wx.EXPAND)
        btnsizer.Add(btnCancel, 1, wx.EXPAND)
        
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(fgsizer, 0, wx.EXPAND|wx.ALL)
        sizer.Add(btnsizer, 0, wx.EXPAND|wx.ALL, 5)
        
        self.SetSizer(sizer)
        self.Fit()

        self.Bind(wx.EVT_SIZE, self.OnResize, self)
        
        self.SetStateFirst()

    def SetStateFirst(self):
        """
        Setup the value of widgets on this dialog 
        by referencing with self.group and self.graphinfo.
        """
        text = self.label.text
        self.tcText.SetValue("%s" % text)
        #---------------------
        choices = FontFamilySet
        self.choiceFamily.SetItems(choices)
        family = self.label.fontfamily
        if family in choices:
            idx = choices.index(family)
            self.choiceFamily.SetSelection(idx)
        #---------------------
        choices = FontStyleSet
        self.choiceStyle.SetItems(choices)
        style = self.label.fontstyle
        if style in choices:
            idx = choices.index(style)
            self.choiceStyle.SetSelection(idx)
        #---------------------
        choices = FontWeightSet
        self.choiceWeight.SetItems(choices)
        weight = self.label.fontweight
        if weight in choices:
            idx = choices.index(weight)
            self.choiceWeight.SetSelection(idx)
        #---------------------
        choices = FontSizeSet
        self.comboSize.SetItems(choices)
        self.comboSize.SetValue(self.label.fontsize)
        #---------------------
        choices = RotationSet
        self.comboRotation.SetItems(choices)
        self.comboRotation.SetValue(self.label.rotation)
        #---------------------
        choices = HAlignmentSet
        self.choiceHAlign.SetItems(choices)
        halign = self.label.halignment
        if halign in choices:
            idx = choices.index(halign)
            self.choiceHAlign.SetSelection(idx)
        #---------------------
        choices = VAlignmentSet
        self.choiceVAlign.SetItems(choices)
        valign = self.label.valignment
        if valign in choices:
            idx = choices.index(valign)
            self.choiceVAlign.SetSelection(idx)
        #---------------------
        color = self.label.fontcolor
        self.btnColor.set_color(color)

    def OnResize(self, event):
        event.Skip()

    def GetResult(self):
        """
        設定した内容を配列で返す
        """
        text = self.tcText.GetValue()
        choices = self.choiceFamily.GetItems()
        family = choices[self.choiceFamily.GetSelection()]
        choices = self.choiceStyle.GetItems()
        style = choices[self.choiceStyle.GetSelection()]
        choices = self.choiceWeight.GetItems()
        weight = choices[self.choiceWeight.GetSelection()]
        choices = self.choiceHAlign.GetItems()
        halign = choices[self.choiceHAlign.GetSelection()]
        choices = self.choiceVAlign.GetItems()
        valign = choices[self.choiceVAlign.GetSelection()]
        size = self.comboSize.GetValue()
        rotation = self.comboRotation.GetValue()
        color = self.btnColor.get_color()
        return [text, family, style, weight, size, color, rotation, halign, valign]

    def OnQuit(self, evt=None):
        self.Close()

if __name__ == '__main__':
    app = wx.App()

    graphinfo = GraphInfo()
    graphinfo.load('plot_default.txt')

    group = graphinfo.groups[0]
    panel = group.panels[0]

    dialog = SetLabelDlg(None, group.labelx, graphinfo)
    result = dialog.ShowModal()
    if result == wx.ID_OK:
        dialogdata = dialog.GetResult()
        print(dialogdata)
    dialog.Destroy()


