#!/usr/bin/env python
# coding: utf-8
# $Id$

import wx
import wx.grid
import wx.lib.newevent
import os
import re
import sys
import copy
import MyViewUpdateDlg 
from MyViewGlobal import debug, logger
from MyViewDataInfo import DataInfo

class MyViewSetModuleDlg(wx.Dialog):
    """
    Mocule ウィンドウ
    """
    def SetModules(self, datamodules, enabled, synopses, urls):
        """
        データモジュールのロード
        """
        self.grid.AppendRows(len(datamodules))
        self.datamodules = datamodules
        for i, m in enumerate(self.datamodules):
            self.grid.SetCellValue(i,1,m)
            if ( urls[i] ):
                link, text = urls[i]
                self.grid.SetCellValue(i,2,link)
            else:
                self.grid.SetCellValue(i,2,synopses[i])
            if ( m in  enabled ):
                self.grid.SetCellValue(i,0,'1')
            self.grid.SetReadOnly(i,0)
            self.grid.SetReadOnly(i,1)
            self.grid.SetReadOnly(i,2)
        self.grid.AutoSizeColumns()
#        self.Fit()
        s = self.grid.GetBestSize()
#        sz = self.GetSize()
#        print 'size=',self.GetBestSize()
        self.SetClientSize((s.width+20, 250))

    def Clicked(self, event):
        r, c = event.Row, event.Col
        if ( c == 0 ):
            v = self.grid.GetCellValue(r,0)
            if ( v == '1' ):
                self.grid.SetCellValue(r,0,'0')
            else:
                self.grid.SetCellValue(r,0,'1')
        event.Skip()

    def GetEnabled(self):
        selected = []
        for i, m in enumerate(self.datamodules):
            v = self.grid.GetCellValue(i,0)
            if ( v == '1' ):
                selected.append(m)
        return selected

    def __init__(self, parent):
        wx.Dialog.__init__(self, parent, -1, 'MyView2 Custom Modules', size=(650, 250),
                           style=wx.DEFAULT_DIALOG_STYLE|wx.THICK_FRAME|wx.RESIZE_BORDER|
                           wx.TAB_TRAVERSAL)
        #, 
        #        size=(480,250))

        self.grid = wx.grid.Grid(self)
        self.grid.CreateGrid(0, 3)
        self.grid.SetRowLabelSize(0)
        self.selected = None
        self.grid.SetColLabelValue(0,"Valid")
        self.grid.SetColLabelValue(1,"Name")
        self.grid.SetColLabelValue(2,"Description")
        self.grid.SetColSize(0,20)
        self.grid.SetColSize(1,200)
        self.grid.SetColSize(2,400)
        self.grid.SetColFormatBool(0)
        btnOk = wx.Button(self, wx.ID_OK, "OK")
        btnCancel= wx.Button(self, wx.ID_CANCEL, "Cancel")
        btnOk.SetDefault()

        btnsizer = wx.BoxSizer(wx.HORIZONTAL)
        btnsizer.Add(btnOk, 1, wx.EXPAND)
        btnsizer.Add(btnCancel, 1, wx.EXPAND)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.grid, 1, wx.EXPAND|wx.ALL)
        sizer.Add(btnsizer, 0,wx.ALIGN_CENTER, 5)

        self.SetSizer(sizer)
        self.Layout()
        #sizer.Fit(self)
        #self.Fit()

        self.Bind(wx.grid.EVT_GRID_CELL_LEFT_CLICK, self.Clicked, self.grid)
        #self.Bind(wx.EVT_BUTTON, self.SetSelected, btnOk)

if __name__ == '__main__':
    from MyViewGraphInfo import GraphInfo
    sn = 106174
    graphinfo = GraphInfo()
    graphinfo.load('plot_default.txt')
    datainfo = DataInfo(graphinfo, 'DataModules/datamodules.txt', 'DataModules/enabled.txt', 'LabelModules/labelmodules.txt')
    datamodules = datainfo.datamodules

    app = wx.PySimpleApp()

    dialog = MyViewSetModuleDlg(None)
    modules = datamodules.GetDataModules()
    enabled = datamodules.GetEnabledModules()
    synopses = [datainfo.getDataModuleSynopsis(e) for e in modules]
    urls = [datainfo.getDataModuleReference(e) for e in modules]
    #print 'urls=',urls
    
    dialog.SetModules(modules, enabled, synopses, urls)

    result = dialog.ShowModal()
    if ( result == wx.ID_OK ):
        l = dialog.GetEnabled()
        print(('list=',l))

    dialog.Destroy()
