#!/usr/bin/env python
# coding: utf-8
# $Id$

import wx
import wx.lib.newevent
import os
import matplotlib


class delayInput(wx.SpinCtrl):
    """
    size入力に特化したスピンボックスコントロール
    """
    def __init__(self, parent=None):
        wx.SpinCtrl.__init__(self, parent, -1, "")

        self.delay = 0
        self.delayMin = 0
        self.delayMax = 10

        self.SetRange(self.delayMin, self.delayMax)
        self.SetValue(self.delay)

        self.Bind(wx.EVT_SPINCTRL, self.OnSpin, self)

    def OnSpin(self, event):
        self.delay = self.GetValue()

    def get(self):
        return self.delay

    def set(self, delay):
        self.SetValue(delay)

    def range(self, min, max):
        self.delayMin = min
        self.delayMax = max
        self.SetRange(min, max)

class durationInput(wx.SpinCtrl):
    """
    size入力に特化したスピンボックスコントロール
    """
    def __init__(self, parent=None):
        wx.SpinCtrl.__init__(self, parent, -1, "")

        self.duration = 150
        self.durationMin = 0
        self.durationMax = 300

        self.SetRange(self.durationMin, self.durationMax)
        self.SetValue(self.duration)

        self.Bind(wx.EVT_SPINCTRL, self.OnSpin, self)

    def OnSpin(self, event):
        self.duration = self.GetValue()

    def get(self):
        return self.duration

    def set(self, duration):
        self.SetValue(duration)

    def range(self, min, max):
        self.durationMin = min
        self.durationMax = max
        self.SetRange(min, max)

class SetRealtimeDlg(wx.Dialog):
    def __init__(self, parent, vprint, diag, delay, duration):
        wx.Dialog.__init__(self, parent, -1, 'myView Realtime', 
                style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER)

        self.print_check = wx.CheckBox(self, -1, "Send to Virtual Printer")
        self.print_check.SetValue(False)
#        self.diag_tbox = textCtrl(self)
        self.diag_tbox = wx.TextCtrl(self, -1, '', style=wx.TE_PROCESS_ENTER)
        self.delayInput = delayInput(self)
        self.durationInput = durationInput(self)

        self.okButton = wx.Button(self, wx.ID_OK, "OK")
        self.okButton.SetDefault()
        self.okButton.SetLabel("START")
        self.cancelButton = wx.Button(self, wx.ID_CANCEL, "Cancel")

        sizer = wx.BoxSizer(wx.VERTICAL)

        st_print = wx.StaticText(self, -1, " ")
        st_diag = wx.StaticText(self, -1, "Diagnostics")
        st_delay = wx.StaticText(self, -1, "Delay")
        st_duration = wx.StaticText(self, -1, "Duration (sec)")

        fgsizer = wx.FlexGridSizer(rows=4, cols=2, hgap=5, vgap=5)

        fgsizer.Add(st_print, 0, wx.ALIGN_CENTER_VERTICAL)
        fgsizer.Add(self.print_check, 0, wx.EXPAND)

        fgsizer.Add(st_diag, 0, wx.ALIGN_CENTER_VERTICAL)
        fgsizer.Add(self.diag_tbox, 0, wx.EXPAND)

        fgsizer.Add(st_delay, 0, wx.ALIGN_CENTER_VERTICAL)
        fgsizer.Add(self.delayInput, 0, wx.EXPAND)

        fgsizer.Add(st_duration, 0, wx.ALIGN_CENTER_VERTICAL)
        fgsizer.Add(self.durationInput, 0, wx.EXPAND)

        sizer.Add(fgsizer, 0, wx.EXPAND|wx.ALL)
        
        btnsizer = wx.StdDialogButtonSizer()
        btnsizer.Add(self.okButton)
        btnsizer.Add(self.cancelButton)
        btnsizer.Realize()
        sizer.Add(btnsizer, 0, wx.EXPAND|wx.ALL, 5)

        self.SetSizer(sizer)
        self.Fit()

        self.Bind(wx.EVT_CHECKBOX, self.OnPrint, self.print_check)
       
        self.set_initial_state(vprint,diag,delay, duration)

    def set_initial_state(self, vprint, diag, delay, duration):
        self.print_check.SetValue(vprint)
        self.diag_tbox.SetValue(diag)
        if vprint:
            self.diag_tbox.Enable(True)
        else:
            self.diag_tbox.Enable(False)
        self.delayInput.set(delay)
        self.durationInput.set(duration)

    def OnPrint(self, event):
        prt = self.print_check.GetValue()
        if prt:
            self.diag_tbox.Enable(True)
        else:
            self.diag_tbox.Enable(False)
    
    def get_result(self):
        flg_prt = self.print_check.GetValue()
        diag = self.diag_tbox.GetValue().strip()
        delay = self.delayInput.get()
        duration = self.durationInput.get()
        return [flg_prt, diag, delay, duration]

if __name__ == '__main__':
    app = wx.PySimpleApp()

    dialog = SetRealtimeDlg(None, True, 'lastshot',1, 150)
    result = dialog.ShowModal()
    addlist = []
    if result == wx.ID_OK:
        print((dialog.get_result()))

    dialog.Destroy()

