#!/usr/bin/env python
# coding: utf-8

import wx
import wx.lib.mixins.listctrl
import wx.lib.newevent
import os
import sys

def InsertStringItemToList(lc, i, s):
    info = wx.ListItem()
    info.SetId(i)
    info.SetText(s)
    return lc.InsertItem(info)

def SetStringItemToList(lc, index, c, s):
    info = wx.ListItem()
    info.SetId(index)
    info.SetColumn(c)
    info.SetText(s)
    lc.SetItem(info)

class SearchListCtrl(wx.ListCtrl,
        wx.lib.mixins.listctrl.ListCtrlAutoWidthMixin):

    def __init__(self, parent, size, records, titles, widths):
        """
        records = [rec1, rec2, rec3, ..., recN]
        rec1 = [v1, v2, ..., vM]
        titles = [t1, t2, ..., tM]
        widths = [w1, w2, ..., wM]
        """
        wx.ListCtrl.__init__(self, parent, style=wx.LC_REPORT, size=size) 
        wx.lib.mixins.listctrl.ListCtrlAutoWidthMixin.__init__(self)
        self.records = records
        self.titles = titles
        self.widths = widths
        self.setList()
        self.Bind(wx.EVT_SIZE, self.OnResize)
        #self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnListItemActivated)

    def OnResize(self, event):
        self.setColumnWidthByRatio(self.widths)
        self.EnsureVisible(0)
        pass

    def OnListItemActivated(self, event):
        wx.PostEvent(parent, event)

    def getSelectedItems(self):
        """
        Return the List of Indexs of Selected Items.
        """
        l = []
        item = self.GetFirstSelected()
        while(item != -1):
            l.append(item)
            item = self.GetNextSelected(item)
        return l

    def resetRecords(self, records):
        self.records = records
        self.setList()

    def setList(self):
        self.DeleteAllItems()
        #self.DeleteAllColumns()

        for col, text in enumerate(self.titles):
            self.InsertColumn(col, text)
        
        for i, rec in enumerate(self.records): 
            idx = InsertStringItemToList(self, i, rec[0])
            #idx = self.InsertStringItem(i, rec[0])
            for col, item in enumerate(rec[1:]):
                SetStringItemToList(self, idx, col+1, item)
                #self.SetStringItem(idx, col+1, item)
            if i%2 == 1:
                self.SetItemTextColour(idx, '#000000')
                self.SetItemBackgroundColour(idx, '#EEEEEE')
            else:
                self.SetItemTextColour(idx, '#000000')
                self.SetItemBackgroundColour(idx, '#FFFFFF')

        #for col, text in enumerate(self.titles):
        #    self.SetColumnWidth(col, wx.LIST_AUTOSIZE)
        self.setColumnWidthByRatio(self.widths)

    def setColumnWidthByRatio(self, widths):
        """
        widths = [w1, w2, ..., wM]
        """
        mywidth, myheight = self.GetSize()
        total = sum(widths)
        for i, width in enumerate(widths):
            w = width/total
            self.SetColumnWidth(i, int(w*mywidth))

SAMPLE_RECORDS = [
        ("module1", "This is sample record of module1"),
        ("module2", "This is sample record of module2"),
        ("module3", "This is sample record of module3"),
        ("module4", "This is sample record of module4"),
        ]
SAMPLE_TITLES = ["modulename", "abstruct"]
SAMPLE_WIDTHS = [0.2, 0.8]

       
class SearchListPanel(wx.Panel):
    def __init__(self, parent, 
            record_sources=["value"], titles=["title"], widths=[1.0],
            default_key="", default_keys=[], 
            size=(500, 400)):
        """
        """
        wx.Panel.__init__(self, parent, -1, size=size)
        self.parent = parent
        self.record_sources = record_sources
        self.titles = titles
        self.widths = widths
        self.default_key = default_key
        self.default_keys = default_keys

        records, self.filtered_indices = self.FilteredRecords(self.default_key)

        stFind = wx.StaticText(self, -1, "Find Word:")
        self.cbFind = wx.ComboBox(self, -1, choices=default_keys, style=wx.TE_PROCESS_ENTER)
        if self.default_key in self.default_keys:
            pos = self.default_keys.index(self.default_key)
            self.cbFind.SetSelection(pos)
        else:
            self.cbFind.SetValue(self.default_key)
        self.lcList = SearchListCtrl(self, (500, 400), 
                records, self.titles, self.widths)

        self.SetupWiget()
        
        sizer_h = wx.BoxSizer(wx.HORIZONTAL)
        sizer_h.Add(stFind, 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_h.Add((20, 20), 0, wx.ALIGN_CENTER_VERTICAL, 0)
        sizer_h.Add(self.cbFind, 1, wx.EXPAND, 0)
        
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(sizer_h, 0, wx.EXPAND, 0)
        sizer.Add(self.lcList, 1, wx.EXPAND, 0)
        
        self.SetSizer(sizer)
        self.Fit()

        self.ListItemActivatedEvent, self.LIST_ITEM_ACTIVATED = wx.lib.newevent.NewEvent()

        self.cbFind.Bind(wx.EVT_COMBOBOX, self.OnFind, self.cbFind)
        self.cbFind.Bind(wx.EVT_TEXT_ENTER, self.OnFind, self.cbFind)
        #self.Bind(EVT_LIST_ITEM_ACTIVATED, self.OnListItemActivated, self.lcList)
        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnListItemActivated, self.lcList)
 

    def OnFind(self, event=None):
        key = self.cbFind.GetValue()
        records, self.filtered_indices = self.FilteredRecords(key)
        self.lcList.resetRecords(records)
    
    def OnListItemActivated(self, event):
        evt = self.ListItemActivatedEvent()
        wx.PostEvent(self, evt)

    def SetupWiget(self):
        pass

    def FilteredRecords(self, key):
        records = self.record_sources
        if key == "":
            return records, [i for i in range(len(records))]
        texts = [" ".join(r) for r in records]
        indices = [i for i, t in enumerate(texts) if -1 != t.find(key)]
        return ([records[i] for i in indices], indices)

    def OnSelected(self, evt=None):
        pass

    def get_values(self):
        selected = self.lcList.getSelectedItems()
        selected_items = [self.filtered_indices[i] for i in selected]
        return [self.record_sources[i][0] for i in selected_items]

class myFrame(wx.Frame):
    def __init__(self):
        wx.Frame.__init__(self, None, -1, u'FindAndSelectItem', 
                style=wx.DEFAULT_FRAME_STYLE)
        #self.SetBackgroundColour(wx.Colour(200, 200, 200))
        self.font = wx.Font(11, wx.MODERN, wx.NORMAL, wx.NORMAL, False)

        self.cwd = os.getcwd()
        
        self.panel = SearchListPanel(self, 
                record_sources=SAMPLE_RECORDS,
                titles=SAMPLE_TITLES,
                widths=SAMPLE_WIDTHS,
                default_key="1",
                default_keys=['Te', 'Ti', 'mmwr'])
        
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.panel, 1, wx.EXPAND|wx.ALL, 0)
        self.SetSizer(sizer)
        self.Fit()

        self.Bind(wx.EVT_CLOSE, self.OnQuit)

    def OnQuit(self, evt=None):
        self.Destroy()

class myApp(wx.App):
    def __init__(self):
        wx.App.__init__(self, False)

    def OnInit(self):
        frame = myFrame()
        frame.Show(True)
        return True

if __name__ == '__main__':
    app = myApp()
    app.MainLoop()
    #app = wx.App()
    #choice = Choices()





