#!/usr/bin/env python
# coding: utf-8
# $Id$

# メイン
import matplotlib
matplotlib.use('WXAgg')
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
from matplotlib.backends.backend_wxagg import NavigationToolbar2WxAgg as NavigationToolbar
from matplotlib.figure import Figure
import matplotlib.cm
import matplotlib.colorbar
import matplotlib.mlab
from matplotlib.widgets import MultiCursor
import scipy.interpolate
import numpy
import math

import wx
import wx.lib.newevent
import wx3_patch
import wx4_patch

if matplotlib.__version__ >= "3.3":
    pass
else:
    NavigationToolbar._init_toolbar = wx4_patch.NavigationToolbar2Wx_init_toolbar 

import os
import getopt
import sys
import platform
import time
import shutil
import subprocess
import threading
threading.Thread.isAlive = threading.Thread.is_alive
import socket
import six
if six.PY3:
    import socketserver
else:
    import SocketServer as socketserver



import select
import vprint_client
import logging
import re

from MyViewGraphInfo import *
from MyViewDataInfo import *
from MyViewSetGroupDlg import *
from MyViewSetPanelDlg import *
from MyViewSetActiveLabelDlg import *
from MyViewSetStaticLabelDlg import *
from MyViewSetTraceDlg import *
from MyViewSetTraceSymbolDlg import *
from MyViewGetBatchFromToDlg import *
from MyViewCache import *
from MyViewSetFolderDlg import *
from MyViewSetIndexStartStepDlg import *
from MyViewSetRealtimeDlg import  SetRealtimeDlg
from MyViewAboutDlg import AboutDlg
from MyViewGnuplot import *
from MyViewColorPicker import *
from MyViewEnv import Runtime
#from pyPdf import PdfFileWriter, PdfFileReader
from PyPDF2 import PdfFileWriter, PdfFileReader
from MyViewGlobal import logging, realtime_debug, debug, fileHandler, variables, globalError
from MyViewConfig import *
from MyViewRecorderFrame import MyViewRecorderFrame
from DataModules.egdbcommon import checkDBConnection
from MyViewSetModuleDlg import *
from MyViewPresets import Presets
from MyViewSetPresetDlg import *
from MyViewSelectGroupAndPanelDlg import *
import util
import shot_notifier
import MyViewIDControl
import MyViewConvLayoutDlg
import EGList
import MyViewProfile
import MyViewSetProfileDlg


if matplotlib.__version__ >= '3.9.0':
    def get_cmap(name):
        return matplotlib.colormaps[name]
    matplotlib.cm.get_cmap = get_cmap

# ダミーイベント定義
import wx.lib.newevent
DummyEvent, EVT_DUMMY = wx.lib.newevent.NewEvent()


def CallNow(callableObj, *args, **kw):
    callableObj(*args, **kw)

if platform.system() == 'Linux':
    CallAfter = wx.CallAfter
else:
    CallAfter = CallNow

try:
    _dummy = wx.Colour
except NameError:
    wx.Colour = wx.Color

INIT_FILE = 'plot_default.txt'
CONFIG_FILE = 'myView2.cnf'

LockReload = threading.Lock()

def my_linspace(_min, _max, _n):
    delta = (_max - _min) / (_n - 1)
    #xi = numpy.arange(_min, _max + 1e-14, delta/(_n-1))
    #xi = numpy.arange(_n,dtype='float128') * delta + _min
    xi = numpy.arange(_n,dtype='float64') * delta + _min
    return xi
    
class UpdateContext():
    """
    Realtime モードで必要な情報を管理するクラス
    """
    def __init__(self, graphinfo, datainfo):
        self.flg_print = False
        self.print_dialog = graphinfo.realtime_printer
        self.auto_delay = graphinfo.realtime_delay
        self.graphinfo = graphinfo
        self.datainfo = datainfo
        self.trace_remain = {}
        self.alabel_remain = {}
        self.last_updated = 0
        self.duration = graphinfo.realtime_duration

    def getdata(self, targetidx):
        """
        plotdata の中から targetindex に一致するデータを返す
        """
        plotdata = self.datainfo.plotdata
        for idx, data in plotdata:
            if targetidx == idx:
                return data
        return None
    
    def getlabel(self, targetidx):
        """
        labeldata の中からtargetidx に一致するラベルを取得
        """
        for idx, text in self.datainfo.labeldata:
            if targetidx == idx:
                return text
        return ""
    
    def reset(self):
#        print "UpdateContext.reset()"
        self.trace_remain = {}
        self.last_updated = 0
        for no, t in enumerate(self.graphinfo.traces):
            self.trace_remain[no] = t
        self.alabel_remain = {}
        for no, l in enumerate(self.graphinfo.activelabels):
            self.alabel_remain[no] = l

    def updateRemain(self):
        logger.debug("UpdateContext.updateRemain()")
        for no in list(self.trace_remain.keys()):
            logger.debug("UpdateContext.updateRemain() no=%d" % (no))
            data = self.getdata(no)
            logger.debug("data = " + str(data))
            if ( data != None ):
                title, x, xe, y, ye, z, ze = data
                if ( len(x) > 0 ):
                    del self.trace_remain[no]
                    if ( self.last_updated == 0 ):
                        self.last_updated = time.time()
                
        #for no in self.alabel_remain.keys():
        #    data = self.getlabel(no)
        #    if ( data != "" ):
        #        del self.alabel_remain[no]
        
def isMeshedData(x, y, z):
    y_size = len(set(y))
    x_size = len(set(x))
    if x_size*y_size != len(z):
        return False

    xs = [x[i] for i in range(0, x_size*y_size, y_size)]
    if x_size != len(set(xs)):
        return False

    for i in range(0, x_size*y_size, y_size):
        ys = y[i:i+y_size]
        if y_size != len(set(ys)):
            return False

    return True

def rotate2Dlist(lst):
    z = []
    for v in zip(*lst):
        z.append(v)
    return z

def ExchangeVerticalAndHorizontal_old(lst):
    z = []
    for v in zip(*lst):
        z.append(v)
    return z

def ExchangeVerticalAndHorizontal(lst):
    return list(zip(*lst))

def makeMapData(xorg, yorg, zorg):
    x_size = len(set(xorg))
    y_size = len(set(yorg))
    #print len(xorg), len(yorg), len(zorg), x_size, y_size
    x = numpy.array([xorg[i] for i in range(0, len(xorg), y_size)])
    y = numpy.array(yorg[:y_size])
    z = [zorg[i:i+y_size] for i in range(0, len(zorg), y_size)]
    z = numpy.array(ExchangeVerticalAndHorizontal(z))
    #print z.shape
    return x, y, z

def FormErrors(errs):
    """
    make errs to fit the matplotlib low_err, high_err.
    if 
       errs = [v_err, v_err, ....] =>
       return errs

    if 
       err = [(v_err_low, v_err_high), (v_err_low, v_err_high), ....] =>
       err_low = [v_err_low, v_err_low, ...]
       err_high = [v_err_high, v_err_high, ...]
       return [err_low, err_high]
    """
    errs_ary = numpy.array(errs)
    if len(errs_ary.shape) == 1:
        return errs
    return errs_ary.swapaxes(0, 1)         

class PlotTraceBase():
    """
    描画用スレッドベースクラス
    """
    def __init__(self, figure, graphinfo, datainfo):
        self.figure = figure
        self.graphinfo = graphinfo
        self.datainfo = datainfo
        self.traces = self.graphinfo.traces
        self.colorConverter = matplotlib.colors.ColorConverter()
        self.figure.dict_trace_color = {}

    def getdata(self, targetidx, plotdata):
        """
        plotdata の中から targetindex に一致するデータを返す
        """
        for e in plotdata:
            if e :
                idx, data = e
                if targetidx == idx:
                    return data
        return None

    def linear(self, ary, factor, offset, ymin, ymax):
        """
        ary に factor 倍して offset を加えたものを返す
        """
        if ( ary.__class__ != numpy.ndarray ):
            ary2 = numpy.array(ary)
        else:
            ary2 = ary
        if ( factor != 1.0 ):
            ary2 *= factor
        if ( offset != 0.0 and (ymax > ymin)):
            ary2 += offset * (ymax - ymin)
        return ary2

    def linear_normal(self, ary, factor, offset):
        """
        ary に factor 倍して offset を加えたものを返す
        """
        if ( ary.__class__ != numpy.ndarray ):
            ary2 = numpy.array(ary)
        else:
            ary2 = ary.copy()
        if ( factor != 1.0):
            ary2 *= factor
        if ( offset != 0.0):
            ary2 += offset
        return ary2


    def get_interval(self, zmin, zmax):
        """
        適当な目盛りの間隔を返す
        """
        eff = zmax - zmin
        sft = 1.0
        while ( eff >= 10 ):
            eff = eff / 10
            sft = sft * 10
        while ( eff < 1 ):
            eff = eff * 10
            sft = sft / 10

        if ( eff >= 5 ):
            return sft
        elif ( eff >= 2 ):
            return sft * 0.5
        else:
            return sft * 0.2

    def get_levels(self, zmin, zmax, interval):
        """
        zmin から zmax まで interval 間隔の目盛り配列を返す。
        interval = 0.0 の時は適当な値を自動選択
        """
        if ( zmin >= zmax ):
            return []
        if ( interval <= 0.0 ):
            interval = self.get_interval(zmin, zmax)
        
        #print 'zmin=',zmin,'zmap=',zmax,'interval=',interval
        _zmin = int(zmin / interval)*interval
        if ( _zmin < zmin ):
            _zmin = _zmin + interval
        ary = numpy.arange(_zmin, zmax, interval)
        if ( len(ary) < 3 ): # レベル数が 2以下だとうまく動かない
            ary = [_zmin, (_zmin+interval), (_zmin+2*interval)]
        return ary

    def get_levels2(self, levels, interval):
        """
        levels の配列の要素の中で interval で割り切れるものを配列として返す。
        """
        ary = []
        epsilon = 1.0e-3
        if ( interval == 0 ):
            return ary
        for level in levels:
            n = math.ceil(level / interval)
            if (math.fabs(level - interval * n)/interval < epsilon ):
                ary.append(level)
        return ary

    def plot(self, i, skip=True):
        """
        i 番目のトレースを描画
        skip==True の時、データがない場合はプロットしない。
        返値：
        真：描画したとき
        偽：描画しなかった
        """
        #print "***** PlotTraceBase.plot(",i,") *****"
        trace = self.traces[i]
        plotdata = self.datainfo.plotdata[:]
        if not trace.flgshow: return False
        # i 番目のデータを取得
        data = self.getdata(i, plotdata)
        if data is None: return False
        title, x, xe, y, ye, z, ze = data
        if ( skip and len(x) == 0 ):
            return False
        removetext = "@%s" % self.graphinfo.dataid
        title = title.replace(removetext, "")

        # 表示情報の取得
        panelid = trace.panel.panelid
        groupid = trace.panel.group.groupid
        symbol = trace.symbol
        symbolsize = trace.symbolsize
        try:
            ss = int(symbolsize)
        except:
            ss = 4 
        linestyle = trace.linestyle
        linewidth = trace.linewidth
        try:
            lw = int(linewidth)
        except:
            lw = 2

        if lw == 0: ls = ''
        else: ls = linestyle

        if  ss == 0: s = ''
        else: s = symbol
        if len(s) >= 2 and s[-1] == '_':
            fcol = self.figure.colorConverter.to_rgb('#FFFFFF')
            s = s[:-1]
        else:
            fcol = self.figure.colorConverter.to_rgb(trace.color)
        col = self.figure.colorConverter.to_rgb(trace.color)

        mark = s+ls
        if mark == '': mark = '.'

        offset = trace.offset
        factor = trace.factor
        xoffset = trace.xoffset
        xfactor = trace.xfactor
        zoffset = trace.zoffset
        zfactor = trace.zfactor
        panel = trace.panel
        group = panel.group
        if group.flgcommony:
            ymin = group.ymin
            ymax = group.ymax
        else:
            ymin = panel.ymin
            ymax = panel.ymax
            
        if ( factor != 1.0 or offset != 0.0 ):
            #y = self.linear(y, factor, offset, ymin, ymax)
            #ye = self.linear(ye, factor, 0, ymin, ymax )
            y = self.linear_normal(y, factor, offset)
            ye = self.linear_normal(ye, factor, 0)

        if ( xfactor != 1.0 or xoffset != 0.0 ):
            x = self.linear_normal(x, xfactor, xoffset)
            xe = self.linear_normal(xe, xfactor, 0)
        
        if ( zfactor != 1.0 or zoffset != 0.0 ):
            if ( trace.plottype == 'xyz' ):
                z = self.linear_normal(z, zfactor, zoffset)
                ze = self.linear_normal(ze, zfactor, 0)

        ax, axsub = self.figure.dict_axes.get((groupid, panelid))
        traces_in_axes = self.figure.dict_traces_in_axes.get(ax,[])
        traces_in_subaxes = self.figure.dict_traces_in_axes.get(axsub,[])

        # グラフの種類毎に分岐
        # Y1 軸を使って描画
        if trace.plottype == 'xy' and ax:
            flgp = False
            if len(xe)==0 and len(ye)==0:
                flgp = True
                ax.plot(x, y, mark, label=title, color=fcol, mec=col, 
                    markersize=ss, linewidth=lw)
            elif len(x) == len(xe) and len(y) == len(ye):
                ax.errorbar(x, y, xerr=FormErrors(xe), yerr=FormErrors(ye), fmt=mark, 
                        label=title, color=fcol, ecolor=col, mec=col, 
                    markersize=ss, linewidth=lw, elinewidth=1)
            elif len(x) == len(xe):
                ax.errorbar(x, y, xerr=FormErrors(xe), fmt=mark, 
                        label=title, color=fcol, ecolor=col, mec=col, 
                    markersize=ss, linewidth=lw, elinewidth=1)
            elif len(y) == len(ye):
                ax.errorbar(x, y, yerr=FormErrors(ye), fmt=mark, 
                        label=title, color=fcol, ecolor=col, mec=col, 
                    markersize=ss, linewidth=lw, elinewidth=1)
            collist = self.figure.dict_trace_color.get(ax, [[],[]])
            if flgp:
                collist[0].append(col)
            else:
                collist[1].append(col)
            self.figure.dict_trace_color[ax] = collist
            traces_in_axes.append(trace)
            self.figure.dict_traces_in_axes[ax]=traces_in_axes
        # Y2 軸を使って描画
        elif trace.plottype == 'xy2' and axsub:
            flgp = False
            if len(xe)==0 and len(ye)==0:
                axsub.plot(x, y, mark, label=title, color=col, mec=col, 
                    markersize=ss, linewidth=lw)
                flgp = True
            elif len(x) == len(xe) and len(y) == len(ye):
                axsub.errorbar(x, y, xerr=FormErrors(xe), yerr=FormErrors(ye), fmt=mark, 
                        label=title, color=col, ecolor=col, mec=col, 
                    markersize=ss, linewidth=lw, elinewidth=1)
            elif len(x) == len(xe):
                axsub.errorbar(x, y, xerr=FormErrors(xe), fmt=mark, 
                        label=title, color=col, ecolor=col, mec=col, 
                    markersize=ss, linewidth=lw, elinewidth=1)
            elif len(y) == len(ye):
                axsub.errorbar(x, y, yerr=FormErrors(ye), fmt=mark, 
                        label=title, color=col, ecolor=col, mec=col, 
                    markersize=ss, linewidth=lw, elinewidth=1)
            collist = self.figure.dict_trace_color.get(axsub, [[],[]])
            if flgp:
                collist[0].append(col)
            else:
                collist[1].append(col)
            self.figure.dict_trace_color[axsub] = collist
            traces_in_subaxes.append(trace)
            self.figure.dict_traces_in_axes[axsub]=traces_in_subaxes
        # 3 次元データ
#        elif trace.plottype == 'xyz' and ax and axsub and trace.panel.ptype==2:
        elif trace.plottype == 'xyz' and ax and axsub and trace.panel.ptype==2 and len(x) > 0 :
            """
            for contour plot
            """

            if isMeshedData(x, y, z):
                xi, yi, zi = makeMapData(x, y, z)
            else:
                x = numpy.array(x)
                y = numpy.array(y)
                z = numpy.array(z)
#                xi = numpy.linspace(min(x), max(x), len(set(x)))
#                yi = numpy.linspace(min(y), max(y), len(set(y)))

                # 精度の問題で xi, yi が numy.linspace では、等間隔と判定されないため、
                # 4倍精度の浮動少数を使用している。
                # matplotlib のバグかもしれない。
                #
                #   ValueError: output grid must have constant spacing when using interp='linear'
                #
                #xi = my_linspace(min(x), max(x), len(set(x)))
                #yi = my_linspace(min(y), max(y), len(set(y)))
                xi, yi = np.mgrid[min(x): max(x): complex(len(set(x))), 
                        min(y):max(y):complex(len(set(y)))] 
                #print 'xi.dtype=',xi.dtype
                #print 'yi.dtype=',yi.dtype
                #print 'xi=',xi
                #print 'yi=',yi
                #dx = xi[1:]-xi[0:-1]
                #dy = yi[1:]-yi[0:-1]
                #epsx = numpy.finfo(xi.dtype).resolution
                #epsy = numpy.finfo(yi.dtype).resolution
                #print 'dx.max()-dx.min()=',dx.max()-dx.min()
                #print 'espx=',epsx
                #print 'dy.max()-dy.min()=',dy.max()-dy.min()
                #print 'espy=',epsy
                #zi = matplotlib.mlab.griddata(x, y, z, xi, yi, interp='linear')
                points = np.insert(np.vstack(x), 1, y, axis=1) # add y in column 1 
                zi = scipy.interpolate.griddata(points, z, (xi, yi), method='linear')
                zi = zi.T
                xi = xi[:,0]
                yi = yi[0,:]
            #contour = ax.contour(x, y, z, 10, linewidths=lw, colors=col)
            vmin = trace.panel.smin
            vmax = trace.panel.smax
            if (lw != 0.0) and ((len(xi) > 1) and (len(yi) > 1)):
                if '1.4.0' > matplotlib.__version__ >= '1.3.0':
                    pass
                else:
                    col = trace.color

               
                if ( vmin < vmax ):
                    levels = self.get_levels(vmin, vmax, trace.panel.interval)
                else:
                    z_without_nan = [z_i for z_i in z if not math.isnan(z_i)]
                    minz, maxz = numpy.min(z_without_nan), numpy.max(z_without_nan)
                    #print minz, maxz
                    levels = self.get_levels(minz, maxz, trace.panel.interval)

                if ( len(levels) > 0 ):
                    #print( 'levels=',levels)
                    contour = ax.contour(xi, yi, zi, levels=levels, linewidths=lw, colors=col)
                    levels2 = self.get_levels2(levels, trace.panel.interval2)
                    if ( len(levels2) > 0 ):
                        contour2 = ax.contour(xi, yi, zi, levels=levels2, linewidths=lw*2, colors=col)
                else:
                    contour = ax.contour(xi, yi, zi, 10, linewidths=lw, colors=col)
            
            if trace.cmap == 'none':
                cbar = None
            else:
                # shift x y point for correct positioning on pcolormesh
                #dx = [0] + [b-a for a, b in zip(xi, xi[1:])]
                #dy = [0] + [b-a for a, b in zip(yi, yi[1:])]
                #shifted_x = numpy.array([a-b*0.5 for a, b in zip(xi, dx)])
                #shifted_y = numpy.array([a-b*0.5 for a, b in zip(yi, dy)])
                shifted_x = numpy.r_[xi[0], xi[1:] - numpy.diff(xi)*0.5, xi[-1]]
                shifted_y = numpy.r_[yi[0], yi[1:] - numpy.diff(yi)*0.5, yi[-1]]
                #cmap = matplotlib.cm.get_cmap('rainbow')
                if ( trace.cmap != '' ):
                    cmap = matplotlib.cm.get_cmap(trace.cmap)
                else:
                    cmap = matplotlib.cm.jet
                if vmin == vmax:
                    #mesh = ax.pcolormesh(xi, yi, zi, cmap = cmap)
                    mesh = ax.pcolormesh(shifted_x, shifted_y, zi, cmap = cmap)
                    pass
                else:
                    axsub.set_yscale('linear')
                    if trace.panel.flglogs:
                        norm = matplotlib.colors.LogNorm(vmin=vmin, vmax=vmax)
                    else:
                        norm = matplotlib.colors.Normalize(vmin=vmin, vmax=vmax)
                    #mesh = ax.pcolormesh(shifted_x, shifted_y, zi,
                    #        cmap = cmap, norm=norm, vmin=vmin, vmax=vmax)
                    mesh = ax.pcolormesh(shifted_x, shifted_y, zi,
                            cmap = cmap, norm=norm)
                cbar = self.figure.colorbar(mesh, cax=axsub)
                
                cbar.update_ticks()
                

            # plot cbar label
            if cbar is not None:
                label = trace.panel.labels
                text = label.text
                if text != "":
                    family = label.fontfamily
                    style = label.fontstyle
                    weight = label.fontweight
                    col = self.colorConverter.to_rgb(label.fontcolor)
                    valign = label.valignment
                    halign = label.halignment
                    try:
                        rotation = float(label.rotation)
                    except:
                        rotation = 0.0
                    try:
                        size = int(label.fontsize)
                    except:
                        size = label.fontsize
                    label_font = matplotlib.font_manager.FontProperties(family=family, 
                        style=style, size=size, weight=weight, stretch='normal')
                    fontdict = {'fontsize': size, 'family':family, 'style':style,
                                'weight':weight,
                                'verticalalignment': valign,
                                'horizontalalignment': halign}
                    cbar.set_label(text, rotation=rotation, color=col,
                            fontdict = fontdict)
            # plot tile
            if trace.panel.flgshowlegs:
                lstyle_s = trace.panel.legendstyles
                left, bottom, width, height = trace.panel.group.getPanelsRect(trace.panel)
                l = self.figure.text(left + width/2.0, bottom + height, title)
                col = self.figure.colorConverter.to_rgb(trace.color)
                label_font = matplotlib.font_manager.FontProperties(
                    family=lstyle_s.fontfamily, 
                    style=lstyle_s.fontstyle, size=lstyle_s.fontsize, 
                    weight=lstyle_s.fontweight, stretch='normal')
                l.set_horizontalalignment('center')
                l.set_verticalalignment('top')
                l.set_fontproperties(label_font)
                l.set_color(col)
        return True
#            if ( self.figure.dict_traces_in_axes.has_key(ax) ):
#                self.figure.dict_traces_in_axes[ax].append(trace)
#            else:
#                self.figure.dict_traces_in_axes[ax] = [trace]
class PlotTraceThread(threading.Thread, PlotTraceBase):
    """
    描画用スレッドクラス
    """
    def __init__(self, figure, graphinfo, datainfo):
        threading.Thread.__init__(self)
        PlotTraceBase.__init__(self, figure, graphinfo, datainfo)

    def run(self):
        """
        メイン処理
        """

        # データスレッドが終了するまで待ち合わせる
        flgwaiting = True
        while flgwaiting:
            count = 0
            for t in self.datainfo.datathreads:
                if t.isAlive():
                    count = count + 1
            if count == 0:
                flgwaiting = False
            #print "Waiting Fetch Data (%d)..." % count
            #time.sleep(0.5)
        #print "Finished !"

        plotdata = self.datainfo.plotdata[:]
#        for i, data in plotdata:
#            print i, data[0]

        # 全トレースの表示ループ
        for i in range(len(self.traces)):
            self.plot(i)
#            self.figure.drawLegendOf(i)
# デバッグ 2014-04-08
        #--- End Of This Thread ---
        if 0 != len(plotdata):
            self.figure.drawLegend()
        try:
            #self.figure.parent.canvas.draw()
            CallAfter(self.figure.parent.canvas.draw)
        except(AttributeError):
            pass

class PlotTraceRealThread(threading.Thread, PlotTraceBase):
    """
    描画用スレッドクラス(非同期)
    """
    def __init__(self, figure, graphinfo, datainfo):
        logger.debug("*** PlotTraceRealThread.__init__")
        threading.Thread.__init__(self)
        PlotTraceBase.__init__(self, figure, graphinfo, datainfo)

    def run(self):
        """
        メイン処理
        """
        logger.debug("PlotTraceRealThread.run")
        #logger.debug("len(self.datainfo.datathreads)=%d" % len(self.datainfo.datathreads))
        alive = [(no, t) for no, t in enumerate(self.datainfo.datathreads)]
        while len(alive) > 0:
            #終了したスレッドを探す
            _alive = []
            terminated = []
            for no, t  in alive:
                if t.isAlive():
                    _alive.append((no,t))
                else:
                    terminated.append(no)
            alive = _alive

            # 終了したスレッドのデータを表示
            plotdata = self.datainfo.plotdata[:]
            for no in terminated:
                print(('PLOTING : ',no))
                if ( self.plot(no, True) ):
                    self.figure.drawLegendOf(no)
            time.sleep(0.5)

        #print 'ALL DONE:'
        #--- End Of This Thread ---
#        plotdata = self.datainfo.plotdata[:]
#        if 0 != len(plotdata):
#            self.figure.drawLegend()
        try:
            #self.figure.parent.canvas.draw()
            CallAfter(self.figure.parent.canvas.draw)
        except(AttributeError):
            pass


def SetLabelPropaties(target, label):
    """
    ラベル属性の設定
    """
    colorConverter = matplotlib.colors.ColorConverter()
    family = label.fontfamily
    style = label.fontstyle
    weight = label.fontweight
    col = colorConverter.to_rgb(label.fontcolor)
    valign = label.valignment
    halign = label.halignment
    try:
        rotation = float(label.rotation)
    except:
        rotation = 0.0
    try:
        size = int(label.fontsize)
    except:
        size = label.fontsize
    try:
        target.set_color(col)
    except (AttributeError):
        # label of color bar doesn't have the method.
        pass

    target.set_rotation(rotation)
    target.set_horizontalalignment(halign)
    target.set_verticalalignment(valign)
    label_font = matplotlib.font_manager.FontProperties(family=family, 
        style=style, size=size, weight=weight, stretch='normal')
    target.set_fontproperties(label_font)


class PlotActiveLabelBase():
    """
    アクティブラベルを表示するスレッドの基底クラス
    """
    def __init__(self, figure, graphinfo, datainfo):
        self.figure = figure
        self.graphinfo = graphinfo
        self.datainfo = datainfo
        self.activelabels = self.graphinfo.activelabels

    def getlabel(self, targetidx, labeldata):
        """
        labeldata の中からtargetidx に一致するラベルを取得
        """
        for idx, text in labeldata:
            if targetidx == idx:
                return text
        return ""

    def draw(self, i):
        """
        i 番目のラベルを表示
        """
        logger.debug("***** PlotActiveLabelBase.draw(%d) *****" % (i))
        labeldata = self.datainfo.labeldata[:]
        alabel = self.activelabels[i]
        logger.debug(labeldata)
        text = self.getlabel(i, labeldata)
        if text == "": return
        alabel.label.text = text
        posx = alabel.posx
        posy = alabel.posy
        
        family = alabel.label.fontfamily
        style = alabel.label.fontstyle
        weight = alabel.label.fontweight
        valign = alabel.label.valignment
        halign = alabel.label.halignment
        col = self.figure.colorConverter.to_rgb(alabel.label.fontcolor)
        try:
            rotation = float(alabel.label.rotation)
        except:
            rotation = 0.0
        try:
            size = int(alabel.label.fontsize)
        except:
            size = alabel.label.fontsize
        l = self.figure.text(posx, posy, text)
        l.set_color(col)
        l.set_rotation(rotation)
        l.set_horizontalalignment(halign)
        l.set_verticalalignment(valign)
        label_font = matplotlib.font_manager.FontProperties(family=family, 
            style=style, size=size, weight=weight, stretch='normal')
        l.set_fontproperties(label_font)
        logger.debug("***** PlotActiveLabelBase.draw exit")

class PlotActiveLabelThread(threading.Thread, PlotActiveLabelBase):
    """
    アクティブラベルを表示するスレッド
    """
    def __init__(self, figure, graphinfo, datainfo):
        threading.Thread.__init__(self)
        PlotActiveLabelBase.__init__(self, figure, graphinfo, datainfo)

    def getlabel(self, targetidx, labeldata):
        """
        labeldata の中からtargetidx に一致するラベルを取得
        """
        for idx, text in labeldata:
            if targetidx == idx:
                return text
        return ""

    def run(self):
        """
        メイン処理
        """
        flgwaiting = True
        while flgwaiting:
            count = 0
            for t in self.datainfo.labelthreads:
                if t.isAlive():
                    count = count + 1
            if count == 0:
                flgwaiting = False
            #print "Waiting Fetch Labels (%d)..." % count
            time.sleep(0.5)
        #print "Finished !"

        #print labeldata
        for i in range(len(self.activelabels)):
            self.draw(i)

        try:
            #self.figure.parent.canvas.draw()
            CallAfter(self.figure.parent.canvas.draw)
        except(AttributeError):
            pass

class PlotActiveLabelRealThread(threading.Thread, PlotActiveLabelBase):
    """
    アクティブラベルを表示するスレッド(非同期)
    """
    def __init__(self, figure, graphinfo, datainfo):
        threading.Thread.__init__(self)
        PlotActiveLabelBase.__init__(self, figure, graphinfo, datainfo)

    def getlabel(self, targetidx, labeldata):
        """
        labeldata の中からtargetidx に一致するラベルを取得
        """
        for idx, text in labeldata:
            if targetidx == idx:
                return text
        return ""

    def run(self):
        """
        メイン処理
        """
        logger.debug("***** PlotActiveLabelRealThread.run()")
        alive = [(no, t) for no, t in enumerate(self.datainfo.labelthreads)]
        while len(alive) > 0 :
            _alive = []
            terminated = []
            for no, t in alive:
                if t.isAlive():
                    _alive.append((no,t))
                else:
                    terminated.append(no)
            alive = _alive

            for no in terminated:
                self.draw(no)
            time.sleep(0.5)
        try:
            #self.figure.parent.canvas.draw()
            CallAfter(self.figure.parent.canvas.draw)
        except(AttributeError):
            pass

class UpdateGraphThread(threading.Thread):
    """
    グラフの自動更新を行う。
    """
    def __init__(self, parent):
        threading.Thread.__init__(self)
        self.parent = parent
        self.enabled = True

    def stop(self):
        self.parent.StopFetching()
        self.enabled = False
    
    def run(self):
        """
        メイン処理
        """
        while ( self.enabled ):
            #CallAfter(self.parent.UpdateMain)
            self.parent.UpdateMain()
            wx.Sleep(5)
            logger.debug("running...")
        self.parent.updatemode = False
        menu=self.parent.menuDict['Realtime*']
        menu.Check(self.parent.updatemode)
            
        
class MyFigure(Figure):
    """
    グラフの表示領域クラス
    """
    def __init__(self, parent, graphinfo, datainfo):
        Figure.__init__(self)
        self.parent = parent
        self.graphinfo = graphinfo
        self.datainfo = datainfo
        self.dict_axes = {} 
        self.colorConverter = matplotlib.colors.ColorConverter()
        self.dict_trace_color = {}
        self.dict_traces_in_axes = {}
        #self.redraw()

    def make_X_Y_Y2_position(self, x, y, cwidth, cheight):
        #make new xdata, ydata
        cursor_ax = None
        cursor_axsub = None
        for ax, axsub in list(self.dict_axes.values()):
            #find ax which has cursor
            bbox = ax.get_position()
            [[left,bottom], [right, top]] = bbox.get_points()
            xc = (x+1)/float(cwidth)
            yc = y/float(cheight) 
            if ((left < xc) and (xc < right) and 
                (bottom < yc) and (yc < top)):
                cursor_ax = ax
                if axsub is not None:
                    bboxsub = axsub.get_position()
                    [[lefts, bottoms], [rights, tops]] = bboxsub.get_points()
                    if (left, bottom, right, top) == (lefts, bottoms, rights, tops):
                        cursor_axsub = axsub
                break
        if cursor_ax is None: return ()

        width = right-left
        height =top-bottom
        xmin, xmax = cursor_ax.get_xlim()
        ymin, ymax = cursor_ax.get_ylim()
        xdata = xmin + (xc - left) * (xmax - xmin)/width
        ydata = ymin + (yc - bottom) * (ymax - ymin)/height
        if cursor_axsub is not None:
            ymins, ymaxs = cursor_axsub.get_ylim()
            y2data = ymins + (yc - bottom) * (ymaxs - ymins)/height
            #print xdata, ydata, y2data
            return (xdata, ydata, y2data)
        else:
            #print xdata, ydata
            return (xdata, ydata)

    def getAxes(self):
        return self.dict_axes

    def drawLegend(self):
        """
        凡例を表示する
        """
        for key in list(self.dict_axes.keys()):
            groupid, panelid = key
            group = self.graphinfo.groups[groupid]
            panel = group.panels[panelid]
            ax, axsub = self.dict_axes[key]
            collist = self.dict_trace_color.get(ax, None)
            collistsub = self.dict_trace_color.get(axsub, None)

            lstyle_y = panel.legendstyley
            lstyle_s = panel.legendstyles

            if group.flgcommony:
                flgshowlegy = group.flgshowlegy
            else:
                flgshowlegy = panel.flgshowlegy

            if flgshowlegy and collist and len(self.dict_traces_in_axes.get(ax, []))>0:
               leg = ax.legend(loc=2, borderpad=0.02, borderaxespad=0.01, 
                            handletextpad=0.02, labelspacing=0, numpoints=1,
                            handlelength=1)
               if matplotlib.__version__ >= '3.1.0':
                   leg.set_draggable(True)
               elif matplotlib.__version__ >= '1.3.1':
                   leg.draggable(True)
               if leg:
                   frame = leg.get_frame()
                   frame.set_lw(0)
                   frame.set_fill(False)
                   texts = leg.get_texts()
                   for i, t in enumerate(texts):
                       t.set_fontname(lstyle_y.fontfamily)
                       t.set_fontsize(lstyle_y.fontsize)
                       t.set_fontweight(lstyle_y.fontweight)
                       t.set_fontstyle(lstyle_y.fontstyle)
                       if len(collist[0]) != 0 :
                           col = collist[0].pop(0)
                           t.set_color(col)
                       elif len(collist[1]) != 0 :
                           col = collist[1].pop(0)
                           t.set_color(col)

            if (panel.flgshowlegs and panel.ptype==1 and 
                    axsub and len(self.dict_traces_in_axes.get(axsub, [])))>0:
                leg = axsub.legend(loc=1, borderpad=0.02, borderaxespad=0.01, 
                                   handletextpad=0.02, labelspacing=0, numpoints=1,
                                   handlelength=1)
                if leg:
                    frame = leg.get_frame()
                    frame.set_lw(0)
                    frame.set_fill(False)
                    texts = leg.get_texts()
                    for t in texts:
                        t.set_fontname(lstyle_s.fontfamily)
                        t.set_fontsize(lstyle_s.fontsize)
                        t.set_fontweight(lstyle_s.fontweight)
                        t.set_fontstyle(lstyle_s.fontstyle)
                        if ( collistsub ):
                            if len(collistsub[0]) != 0 :
                                col = collistsub[0].pop(0)
                                t.set_color(col)
                            elif len(collistsub[1]) != 0 :
                                col = collistsub[1].pop(0)
                                t.set_color(col)

    def drawLegendOf(self, no):
        """
        no 番目のトレースの凡例を表示する
        """
        #print "drawLegend(",no,")"
        trace = self.graphinfo.traces[no]
        panel = trace.panel
        group = panel.group
        key = (group.groupid, panel.panelid)
        ax, axsub = self.dict_axes[key]
        traces_in_axes = self.dict_traces_in_axes.get(ax,[])
        traces_in_subaxes = self.dict_traces_in_axes.get(axsub,[])
            
        try:
            idx = traces_in_axes.index(trace)
        except:
            try:
                idx = traces_in_subaxes.index(trace)
            except:
                idx = -1
        col = trace.color
        if group.flgcommony:
            flgshowlegy = group.flgshowlegy
        else:
            flgshowlegy = panel.flgshowlegy
        if flgshowlegy:
            leg = ax.legend(loc=2, borderpad=0.02, borderaxespad=0.01, 
                            handletextpad=0.02, labelspacing=0, numpoints=1,
                            handlelength=1)
            if matplotlib.__version__ >= '1.3.1':
                if ( leg ):
                    leg.draggable(True)
            if ( leg and idx >= 0 ):
                frame = leg.get_frame()
                frame.set_lw(0)
                frame.set_fill(False)
                texts = leg.get_texts()
                t = texts[idx]
                t.set_fontsize('small')
                t.set_color(col)
                    
        if panel.flgshowlegs and panel.ptype==1 and axsub:
            leg = axsub.legend(loc=1, borderpad=0.02, borderaxespad=0.01, 
                               handletextpad=0.02, labelspacing=0, numpoints=1,
                               handlelength=1)
            if ( leg and idx >= 0 ):
                frame = leg.get_frame()
                frame.set_lw(0)
                frame.set_fill(False)
                texts = leg.get_texts()
                t = texts[idx]
                t.set_fontsize('small')
                t.set_color(col)
               
    def drawPanels(self, panel, l, b, w, h):
        """
        MyFigure
        """
        rect = [l, b, w, h]
        ax = self.add_axes(rect)
        ax.grid(panel.flggrid)

        if panel.ptype == 0:
            axsub = None
        elif panel.ptype == 1:
            axsub = ax.twinx()
        else:
            #subrect = [l+w+0.001, b, 0.01, h]
            subrect = [l+w-0.02*w, b, 0.02*w, h]
            axsub = self.add_axes(subrect)
            self.tickLabelOff('x', axsub)
            self.tickLabelOff('y', axsub)
        return [ax, axsub]

    def drawGroupTitle(self, ax, label):
        """
        グループタイトル表示
        """
        text = label.text
        if text == "": return
        l = ax.set_title(text)
        SetLabelPropaties(l, label)
        
    def drawGroupLabelX(self, ax, label):
        """
        グループ X 軸タイトル表示
        """
        text = label.text
        if text == "": return
        l = ax.set_xlabel(text)
        SetLabelPropaties(l, label)

    def drawGroupLabelY(self, ax, label):
        """
        グループ Y 軸タイトル表示
        """
        text = label.text
        if text == "": return
        l = ax.set_ylabel(text)
        SetLabelPropaties(l, label)

    def labelControl(self):
        """
        メモリ等の設定
        """
        for group in self.graphinfo.groups:
            for r in range(group.numrows):
                for c in range(group.numcols):
                    groupid = group.groupid
                    panelid = c + r * group.numcols
                    panel = group.panels[panelid]
                    axes = self.dict_axes.get((groupid, panelid), None)
                    if axes:
                        ax, axsub = axes
                        # ticklabel control
                        if group.flgcommony:
                            ticklabely = group.ticklabely
                        else:
                            ticklabely = panel.ticklabely
                        ticklabelx = group.ticklabelx
                        ticklabels = panel.ticklabels
                        if c != 0:
                            self.tickLabelOff('y', ax)
                        else:
                            self.tickLabelOn('y', ax, ticklabely)

                        if c != group.numcols -1 and axsub:
                            self.tickLabelOff('y', axsub)
                        elif axsub:
                            self.tickLabelOn('y', axsub, ticklabels)

                        if r != group.numrows -1:
                            self.tickLabelOff('x', ax)
                        else:
                            self.tickLabelOn('x', ax, ticklabelx)

    def drawGroups(self):
        """
        グループの表示
        """
        for group in self.graphinfo.groups:
            for r in range(group.numrows):
                for c in range(group.numcols):
                    panelid = c + r * group.numcols
                    panel = group.panels[panelid]
                    rect = group.panelsrect[panelid]
                    if rect:
                        ax, axsub = self.drawPanels(panel, *rect)
                        self.set_ticklabel_format(group, panel, ax, axsub)

                    if axsub:
                        self.setScaleY(axsub, panel.smin, panel.smax, panel.flglogs)
                    self.dict_axes[(group.groupid, panelid)] = (ax, axsub)
                   
                    # Range Control
                    self.setScaleX(ax, group.xmin, group.xmax, group.flglogx)
                    if group.flgcommony:
                        self.setScaleY(ax, group.ymin, group.ymax, group.flglogy)
                        #ticklabely = group.ticklabely
                    else:
                        self.setScaleY(ax, panel.ymin, panel.ymax, panel.flglogy)
                        #ticklabely = panel.ticklabely

                    #ticklabelx = group.ticklabelx
                    #ticklabels = panel.ticklabels
                    
                    # ticklabel control
                    """
                    if c != 0:
                        self.tickLabelOff('y', ax)
                    else:
                        self.tickLabelOn('y', ax, ticklabely)

                    if c != group.numcols -1 and axsub:
                        self.tickLabelOff('y', axsub)
                    elif axsub:
                        self.tickLabelOn('y', axsub, ticklabels)

                    if r != group.numrows -1:
                        self.tickLabelOff('x', ax)
                    else:
                        self.tickLabelOn('x', ax, ticklabelx)
                    """

                    # draw title on each axis
                    if r == 0 and c == (group.numcols)/2:
                        self.drawGroupTitle(ax, group.labelt)
                    #if r == group.numrows-1 and c == (group.numcols)/2:
                    if r == group.numrows-1:
                        self.drawGroupLabelX(ax, group.labelx)
                    #if group.flgcommony and r == (group.numrows)/2 and c == 0:
                    if group.flgcommony and c == 0:
                        self.drawGroupLabelY(ax, group.labely)
                    else:
                        self.drawGroupLabelY(ax, panel.labely)
                    if c == group.numcols-1 and panel.ptype==1:
                        self.drawGroupLabelY(axsub, panel.labels)

    def setScaleX(self, ax, rangemin, rangemax, flglog):
        """
        X 軸のオートスケール、ログスケールの設定
        """
        if (rangemin == rangemax):
            ax.set_autoscalex_on(True)
        else:
            ax.set_autoscalex_on(False)
            ax.set_xlim(rangemin, rangemax)
        if flglog:
            if ax.get_autoscalex_on():
                ax.set_xscale('log')
            else:
                rmin, rmax = ax.get_xlim()
                if rmin and rmax > 0:
                    ax.set_xscale('log')

    def setScaleY(self, ax, rangemin, rangemax, flglog):
        """
        Y 軸のオートスケール、ログスケールの設定
        """
        if (rangemin == rangemax):
            ax.set_autoscaley_on(True)
        else:
            ax.set_autoscaley_on(False)
            ax.set_ylim(rangemin, rangemax)
        if flglog:
            if ax.get_autoscaley_on():
                ax.set_yscale('log')
            else:
                rmin, rmax = ax.get_ylim()
                if rmin and rmax > 0:
                    ax.set_yscale('log')

    def get_FormatterFunc(self, fmt):
        def formatterfunc(value, tick_number):
            try:
                s = fmt % value 
                parts = s.split('e')
                if len(parts)==1:
                    return s
                coef, exponent = parts[:2]
                exponent = "%d" % int(exponent)
                tickl = '$%s$' % (r'%s\!\times\!10^{%s}' % (coef, exponent))
            except:
                tickl = "%g" % value
            return tickl

        return formatterfunc 

    def change_ticklabel_format(self, at, ax, fmt, flglog):
        if fmt == "":
            if flglog:
                formatter = matplotlib.ticker.LogFormatter()
            else:
                formatter = matplotlib.ticker.ScalarFormatter()
        else:
            func = self.get_FormatterFunc(fmt)
            formatter = matplotlib.ticker.FuncFormatter(func)

        if at == 'x':
            ax.xaxis.set_major_formatter(formatter)
        elif at == 'y':
            ax.yaxis.set_major_formatter(formatter)

    def set_ticklabel_format(self, group, panel, ax, axsub):
        """
        MyFigure
        """
        #import matplotlib.ticker.FuncFormatter
        flglogs = panel.flglogs
        ticklabels = panel.ticklabels
        flglogx = group.flglogx
        ticklabelx = group.ticklabelx
        if group.flgcommony:
            flglogy = group.flglogy
            ticklabely = group.ticklabely
        else:
            flglogy = panel.flglogy
            ticklabely = panel.ticklabely
        fmty = ticklabely.fmt
        fmts = ticklabels.fmt
        fmtx = ticklabelx.fmt
        self.change_ticklabel_format('x', ax, fmtx, flglogx)
        self.change_ticklabel_format('y', ax, fmty, flglogy)
        if axsub:
            self.change_ticklabel_format('y', axsub, fmts, flglogs)

    def tickLabelOff(self, at, ax):
        """
        Tick ラベル OFF
        """
        if at == 'x':
            labels = ax.get_xticklabels()
        elif at == 'y':
            labels = ax.get_yticklabels()
        for label in labels:
            label.set_visible(False)
    
    def tickLabelOn(self, at, ax, ticklabel):
        """
        Tick ラベル ON
        """
        if at == 'x':
            labels = ax.get_xticklabels()
        elif at == 'y':
            labels = ax.get_yticklabels()
        """
        for i, l in enumerate(labels):
            print "%d, %s, %s" % (i, at, l.get_text())
        print
        """
        for l in labels:
            SetLabelPropaties(l, ticklabel)
            l.set_visible(True)
        if len(labels) > 0:
            if ticklabel.flgtrimfirst:
                labels[0].set_visible(False)
            if ticklabel.flgtrimlast:
                labels[-1].set_visible(False)

    def drawStaticLabels(self):
        """
        固定ラベル表示
        """
        for slabel in self.graphinfo.staticlabels:
            posx = slabel.posx
            posy = slabel.posy
            text = slabel.label.text
            if text == "": continue
            
            family = slabel.label.fontfamily
            style = slabel.label.fontstyle
            weight = slabel.label.fontweight
            valign = slabel.label.valignment
            halign = slabel.label.halignment
            col = self.colorConverter.to_rgb(slabel.label.fontcolor)
            try:
                rotation = float(slabel.label.rotation)
            except:
                rotation = 0.0
            try:
                size = int(slabel.label.fontsize)
            except:
                size = slabel.label.fontsize
            l = self.text(posx, posy, text)
            l.set_color(col)
            l.set_rotation(rotation)
            l.set_horizontalalignment(halign)
            l.set_verticalalignment(valign)
            label_font = matplotlib.font_manager.FontProperties(family=family, 
                style=style, size=size, weight=weight, stretch='normal')
            l.set_fontproperties(label_font)

    def redraw(self):
        """
        再描画
        """
        self.init_frame()


        self.threadTrace = PlotTraceThread(self, self.graphinfo, self.datainfo)
        self.threadTrace.start()
        self.threadActiveLabel = PlotActiveLabelThread(self, self.graphinfo, self.datainfo)
        self.threadActiveLabel.start()

    def init_frame(self):
        """
        静的な部分のみの描画
        """
#        print "MyFigure.init_frame()"
        self.dict_axes = {}
#        print "init_frame : clear()"
        self.clf()
        #CallAfter(self.clear)
        #time.sleep(0.05)
#        print "init_frame : drawGroups()"
        self.drawGroups()
        #CallAfter(self.drawGroups)
#        print "init_frame : drawStaticlabels()"
        self.drawStaticLabels()
        #CallAfter(self.drawStaticLabels)

    def update_figure(self):
        """
        必要な部分のみ描画する
        """
        #self.threadTrace = PlotTraceRealThread(self, self.graphinfo, self.datainfo)
        self.threadTrace = PlotTraceThread(self, self.graphinfo, self.datainfo)
        self.threadTrace.start()
        #self.threadActiveLabel = PlotActiveLabelRealThread(self, self.graphinfo, self.datainfo)
        self.threadActiveLabel = PlotActiveLabelThread(self, self.graphinfo, self.datainfo)
        self.threadActiveLabel.start()


class MyFigureCanvas(FigureCanvas):
    """
    カスタマイズされたFigureCanvas
    """
    def __init__(self, parent, wid, figure):
        FigureCanvas.__init__(self, parent, wid, figure)
        self.parent = parent
        self.Bind(wx.EVT_LEFT_DCLICK, self.OnDoubleClicked, self)
        self.Bind(wx.EVT_LEFT_DOWN, self.OnLeftClicked, self)
        self.Bind(wx.EVT_RIGHT_DOWN, self.OnRightClicked, self)
        self.Bind(wx.EVT_RIGHT_DCLICK, self.OnRightDoubleClicked, self)
        
        self.mdc = wx.MemoryDC()
        self.cdc = wx.ClientDC(self)
        self.positions = []
        self.brect = None
        self.cid = self.mpl_connect('draw_event', self.ondraw)
        #fig.canvas.mpl_disconnect(cid)

    def ondraw(self, event):
        """
        draw イベントのコールバック
        """
        self.positions = []
        self.brect = None
        self.mdc.SelectObject(wx.NullBitmap)

    def PointDraw(self):
        """
        クリックされた場所に点または線を引く
        """
        w, h = self.GetSize()
        self.parent.graphinfo.positions = []
        for position in self.positions:
            px, py = position
            x = float(px) / float(w)
            y = float((h - py)) / float(h)
            self.parent.graphinfo.positions.append((x, y))
        if self.brect:
            brect = self.brect
            self.cdc.Blit(brect[0], brect[1], brect[2], brect[3], self.mdc, 0, 0)
            self.mdc.SelectObject(wx.NullBitmap)
        if 1 == len(self.positions):
            px, py = self.positions[0]
            
            brect = (px-3, py-3, 3*2, 3*2)
            bitmap = wx.EmptyBitmap(brect[2], brect[3], -1)
            self.mdc.SelectObject(bitmap)
            self.mdc.Blit(0, 0, brect[2], brect[3], self.cdc, brect[0], brect[1])
            self.brect = brect

            self.cdc.SetPen(wx.Pen("#C87137", 1))
            self.cdc.SetBrush(wx.Brush("#C87137", style=wx.SOLID))
            self.cdc.DrawCircle(px, py, 3) 
            
        elif 2 == len(self.positions):
            px1, py1 = self.positions[0]
            px2, py2 = self.positions[1]
            x1 = min([px1, px2])
            x2 = max([px1, px2])
            y1 = min([py1, py2])
            y2 = max([py1, py2])
            brect = (x1, y1, x2-x1+1, y2-y1+1)
            bitmap = wx.EmptyBitmap(brect[2], brect[3], -1)
            self.mdc.SelectObject(bitmap)
            self.mdc.Blit(0, 0, brect[2], brect[3], self.cdc, brect[0], brect[1])
            self.brect = brect
            
            self.cdc.SetPen(wx.Pen("#C87137", 1))
            self.cdc.SetBrush(wx.Brush("#C87137", style=wx.SOLID))
            self.cdc.DrawLine(px1, py1, px2, py2)

    def OnDoubleClicked(self, event):
        """
        ダブルクリック時のコールバック
        """
        #event.Skip()
        w, h = self.GetSize()
        mx, my = event.GetPosition()
        x = float(mx)/float(w)
        y = float(h - my)/float(h)
        panels = []
        for group in self.parent.graphinfo.groups:
            panels = panels + group.getPanelsAt(x, y)
        for p in panels:
            if self.parent.parent.groupframe.panelpanel.OpenAndSetWithDialog(p):
                if p.group.flgcommony:
                    self.parent.parent.graphinfo.setGroupLink(p.group)
                else:
                    self.parent.parent.graphinfo.setPanelLink(p)
                self.parent.parent.groupframe.panelpanel.updateList()
                self.parent.parent.groupframe.grouppanel.updateList()
                self.parent.parent.groupframe.tracepanel.updateList()

    def OnRightDoubleClicked(self, event):
        """
        右ダブルクリック
        """
        #event.Skip() 
        w, h = self.GetSize()
        mx, my = event.GetPosition()
        x = float(mx)/float(w)
        y = float(h - my)/float(h)
        panels = []
        for group in self.parent.graphinfo.groups:
            panels = panels + group.getPanelsAt(x, y)
        if 0 != len(panels):
            panel = panels[0]
            panelid = panel.panelid
            groupid = panel.group.groupid
            self.parent.parent.groupframe.SetSelection(groupid, panelid)
            #self.parent.parent.OnGroupAndTrace()
            if self.parent.parent.groupframe.IsShown():
                # Move to on the top of Screen
                self.parent.parent.groupframe.Raise()
            else:
                # Show the frame and move to on the top of Screen
                self.parent.parent.showGroupFrame()


    def OnLeftClicked(self, event=None):
        """
        左ボタンクリックの時のコールバック
        """
        event.Skip() # <= on for navi toolbar
        px, py = event.GetPosition()
        self.positions.append((px, py)) 
        if len(self.positions) > 2:
            self.positions = self.positions[1:]
        self.PointDraw()
        if self.parent.parent.IsRecorderValid():
            cwidth, cheight = self.GetSize()
            xydata = self.figure.make_X_Y_Y2_position(px, cheight-py, cwidth, cheight)
            if len(xydata) >= 2:
                self.parent.parent.Record(xydata)

    def OnRightClicked(self, event=None):
        """
        右ボタンクリックの時のコールバック
        """
        event.Skip()
        if 0 == len(self.positions): return 
        self.positions.pop() 
        self.PointDraw()

class CanvasPanel(wx.Panel):
    """
    描画用パネル
    """
    def __init__(self, parent, graphinfo, datainfo, size):
        wx.Panel.__init__(self, parent, -1, size=size)
        self.parent = parent

        self.graphinfo = graphinfo
        self.datainfo = datainfo

        self.figure = MyFigure(self, self.graphinfo, self.datainfo)
        self.canvas = MyFigureCanvas(self, -1, self.figure)
        self.canvas.SetSize(size) 
        #self.canvas.SetSize(self.graphinfo.canvassize) # ? コメントアウトの理由は？
        self.navitoolbar = NavigationToolbar(self.canvas)
        self.navitoolbar.Hide()

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.canvas, 1, wx.EXPAND|wx.ALL, 0)
        self.SetSizer(sizer)
        self.Fit()


    def update(self):
        """
        パネルの更新
        """
        col = self.figure.colorConverter.to_rgb(self.graphinfo.facecolor)
        self.figure.set_facecolor(col)
        self.figure.redraw()
        self.canvas.draw()
        self.figure.labelControl()

        if self.figure.threadTrace.isAlive():
            self.figure.threadTrace.join()
        if self.figure.threadActiveLabel.isAlive():
            self.figure.threadActiveLabel.join()

        self.parent.flgBusy = False

    def init_frame(self):
        """
        外枠の表示
        """
#        print "CanvasPanel.init_frame()"
        col = self.figure.colorConverter.to_rgb(self.graphinfo.facecolor)
        self.figure.set_facecolor(col)
        self.figure.init_frame()
        
    def update_real(self):
        """
        パネルの更新
        """
        logger.debug("CanvasPanel.update_real()")

        col = self.figure.colorConverter.to_rgb(self.graphinfo.facecolor)
        self.figure.set_facecolor(col)
        self.figure.update_figure()
        self.canvas.draw()
        self.figure.labelControl()

        if self.figure.threadTrace.isAlive():
            self.figure.threadTrace.join()
        if self.figure.threadActiveLabel.isAlive():
            self.figure.threadActiveLabel.join()
        
        self.parent.flgBusy = False

class LegendPanel(wx.Panel):
    """
    凡例パネル
    """
    def __init__(self, parent, wid):
        wx.Panel.__init__(self, parent, wid)
        self.parent = parent
        self.graphinfo = self.parent.parent.graphinfo
        self.datainfo = self.parent.parent.datainfo
        self.targets = []

        self.lcLegend = wx.ListCtrl(self, -1, style=wx.LC_REPORT)
        self.btnAddSLabel = wx.Button(self, -1, 'Add S Label')
        self.btnAddALabel = wx.Button(self, -1, 'Add A Label')
        self.btnEdit = wx.Button(self, -1, 'Edit')
        self.btnRemove = wx.Button(self, -1, 'Remove')
        self.clmname = ['idx', 'type', 'data', 'arg', 'idc', 'pos1', 'pos2']
        for col, name in enumerate(self.clmname):
            self.lcLegend.InsertColumn(col, name) # append column
        self.listdata = self.graphinfo.getLegendList()
        self.setList(self.listdata)
        btnsizer = wx.BoxSizer(wx.HORIZONTAL)
        btnsizer.Add (self.btnAddSLabel,  1, wx.EXPAND)
        btnsizer.Add (self.btnAddALabel,  1, wx.EXPAND)
        btnsizer.Add (self.btnEdit,  1, wx.EXPAND)
        btnsizer.Add (self.btnRemove,  1, wx.EXPAND)
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add (self.lcLegend,  1, wx.EXPAND)
        sizer.Add (btnsizer,  0, wx.EXPAND)
        self.SetSizer(sizer)
        self.Fit()

        self.Bind(wx.EVT_BUTTON, self.OnButtonAddSLabel, self.btnAddSLabel)
        self.Bind(wx.EVT_BUTTON, self.OnButtonAddALabel, self.btnAddALabel)
        self.Bind(wx.EVT_BUTTON, self.OnButtonEdit, self.btnEdit)
        self.Bind(wx.EVT_BUTTON, self.OnButtonRemove, self.btnRemove)
        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnButtonEdit, self.lcLegend)
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.OnItemSelected, self.lcLegend)
        self.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.OnItemSelected, self.lcLegend)

    def getSelectedItems(self):
        """
        選択された項目を返す
        """
        lst = []
        num = self.lcLegend.GetSelectedItemCount()
        if 0 < num:
            item = self.lcLegend.GetFirstSelected()
            lst.append(item)
            for i in range(num-1):
                item = self.lcLegend.GetNextSelected(item)
                lst.append(item)
        return lst

    def updateList(self):
        """
        ラベル一覧の更新
        """
        listdata = self.graphinfo.getLegendList()
        self.setList(listdata)
        self.targets = []
        self.Refresh()

    def setList(self, listdata):
        """
        凡例をセットする
        """
        self.listdata = listdata
        self.lcLegend.DeleteAllItems()
        for i, rowdata in enumerate(listdata):
            index = self.lcLegend.InsertStringItem(i, rowdata[0])
            for col, item in enumerate(rowdata[1:-1]):
                #self.lcLegend.SetStringItem(index, col+1, item)
                self.lcLegend.SetItem(index, col+1, item)
        
        for i in range(len(self.clmname)): 
            self.lcLegend.SetColumnWidth(i, wx.LIST_AUTOSIZE)
        self.lcLegend.SetColumnWidth(0, 40)

        # set color
        for i, rowdata in enumerate(listdata):
            self.lcLegend.SetItemTextColour(i, rowdata[-1])

    def OnItemSelected(self, event):
        """
        マウスクリックで選択
        """
        items = self.getSelectedItems()
        self.targets = self.GetTargets()

    def GetTargets(self):
        """
        選択されたラベルリストを返す
        """
        items = self.getSelectedItems()
        targets = []
        if len(items) != 0:
            for idx in items:
                targetid = int(self.listdata[idx][0])-1
                ltype = self.listdata[idx][1]
                target = None
                if ltype == 'Static Label':
                    target = (ltype, self.graphinfo.staticlabels[targetid])
                elif ltype == 'Active Label':
                    target = (ltype, self.graphinfo.activelabels[targetid])
                if target:
                    targets.append(target)
        return targets

    def SetWithDialogStaticLabel(self, target):
        """
        In Legend Panel
        """
        dlg = SetStaticLabelDlg(self, target, self.graphinfo)
        result = dlg.ShowModal()
        flg = False
        if result == wx.ID_OK:
            dlgdata = dlg.GetResult()
            [posx, posy, text, 
             family, style, weight, size, color, rotation, halign, valign] = dlgdata

            target.setData(
                posx = posx,
                posy = posy,
                )
            target.label.setData(
                text = text, 
                fontfamily = family,
                fontstyle = style,
                fontweight = weight,
                fontsize = size,
                fontcolor = color,
                rotation = rotation,
                halignment = halign,
                valignment = valign)
            flg = True
        dlg.Destroy()
        return flg

    def SetWithDialogActiveLabel(self, target):
        """
        In Legend Panel
        """
        dlg = SetActiveLabelDlg(self, target, self.graphinfo, self.datainfo)
        result = dlg.ShowModal()
        flg = False
        if result == wx.ID_OK:
            dlgdata = dlg.GetResult()
            [posx, posy, modulename, dataidc, arg, 
             family, style, weight, size, color, rotation, halign, valign] = dlgdata

            target.setData(
                posx = posx,
                posy = posy,
                modulename = modulename,
                dataidc = dataidc,
                arg = arg)
            target.label.setData(
                fontfamily = family,
                fontstyle = style,
                fontweight = weight,
                fontsize = size,
                fontcolor = color,
                rotation = rotation,
                halignment = halign,
                valignment = valign)
            flg = True
        dlg.Destroy()
        return flg

    def OnButtonAddSLabel(self, event=None):
        """
        Add S Label のコールバック
        """
        target = StaticLabel()
        if 0 != len(self.parent.parent.graphinfo.positions):
            x, y = self.parent.parent.graphinfo.positions[0]
            target.posx = x
            target.posy = y
        if self.SetWithDialogStaticLabel(target):
            self.graphinfo.addStaticLabel(target)
            self.updateList()

    def OnButtonAddALabel(self, event=None):
        """
        Add A Label のコールバック
        """
        target = ActiveLabel()
        if 0 != len(self.parent.parent.graphinfo.positions):
            x, y = self.parent.parent.graphinfo.positions[0]
            target.posx = x
            target.posy = y
        if self.SetWithDialogActiveLabel(target):
            if target.modulename:
                self.graphinfo.addActiveLabel(target)
                self.updateList()

    def OnButtonEdit(self, event=None):
        """
        Editボタンのコールバック
        """
        targets = self.GetTargets()
        if 0 != len(targets):
            ltype, target = targets[0]
            if ltype == 'Static Label':
                if self.SetWithDialogStaticLabel(target):
                    self.updateList()
            elif ltype == 'Active Label':
                if self.SetWithDialogActiveLabel(target):
                    self.updateList()

    def OnButtonRemove(self, event=None):
        """
        Remove ボタンのコールバック
        """
        targets = self.GetTargets()
        slabeltargets = []
        alabeltargets = []
        for ltype, target in targets:
            if ltype == 'Static Label':
                slabeltargets.append(target)
            elif ltype == 'Active Label':
                alabeltargets.append(target)
        self.graphinfo.removeStaticLabels(slabeltargets)
        self.graphinfo.removeActiveLabels(alabeltargets)
        self.updateList()

class MyViewLegendFrame(wx.Frame):
    """
    凡例設定用ウィンドウ
    """
    def __init__(self, parent, wid):
        wx.Frame.__init__(self, parent, wid, 'myView Legends', 
                style=wx.DEFAULT_FRAME_STYLE|wx.STAY_ON_TOP)
        self.parent = parent
        self.legendpanel = LegendPanel(self, -1)
       
        btnRedraw = wx.Button(self, -1, "Redraw")
        btnsizer = wx.BoxSizer(wx.HORIZONTAL)
        btnsizer.Add(btnRedraw, 1, wx.EXPAND)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(btnsizer, 0, wx.EXPAND|wx.ALL, 1)
        sizer.Add(self.legendpanel, 1, wx.EXPAND|wx.ALL, 1)
        self.SetSizer(sizer)
        self.Fit()
        self.Bind(wx.EVT_CLOSE, self.OnClose)
        self.Bind(wx.EVT_BUTTON, self.OnRedraw)

    def OnRedraw(self, event=None):
        self.parent.OnReload(event)

    def Update(self):
        self.legendpanel.updateList()

    def OnClose(self, event):
        self.Hide()
        self.parent.menuDict['Legend*'].Check(False)

class GroupPanel(wx.Panel):
    """
    グループ設定用パネル
    """
    def __init__(self, parent, wid):
        wx.Panel.__init__(self, parent, wid)
        self.parent = parent
        self.graphinfo = self.parent.parent.graphinfo
        self.targets = []

        self.lcGroup = wx.ListCtrl(self, -1, style=wx.LC_REPORT)
        self.btnAdd = wx.Button(self, -1, 'Add')
        self.btnEdit = wx.Button(self, -1, 'Edit')
        self.btnRemove = wx.Button(self, -1, 'Remove')
        self.btnMoveUp = wx.Button(self, -1, 'Move Up')
        self.btnAll = wx.Button(self, -1, 'All')
        self.clmname = ['groupid', 'left', 'bottom', 'width', 'height']
        for col, name in enumerate(self.clmname):
            self.lcGroup.InsertColumn(col, name) # append column
        self.listdata = self.graphinfo.getGroupList()
        self.setList(self.listdata)
        btnsizer = wx.BoxSizer(wx.HORIZONTAL)
        btnsizer.Add (self.btnAdd,  1, wx.EXPAND)
        btnsizer.Add (self.btnEdit,  1, wx.EXPAND)
        btnsizer.Add (self.btnRemove,  1, wx.EXPAND)
        btnsizer.Add (self.btnMoveUp,  1, wx.EXPAND)
        btnsizer.Add (self.btnAll,  1, wx.EXPAND)
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add (self.lcGroup,  1, wx.EXPAND)
        sizer.Add (btnsizer,  0, wx.EXPAND)
        self.SetSizer(sizer)
        self.Fit()

        self.flgpanelupdate = True

        self.Bind(wx.EVT_BUTTON, self.OnButtonAdd, self.btnAdd)
        self.Bind(wx.EVT_BUTTON, self.OnButtonEdit, self.btnEdit)
        self.Bind(wx.EVT_BUTTON, self.OnButtonRemove, self.btnRemove)
        self.Bind(wx.EVT_BUTTON, self.OnButtonMoveUp, self.btnMoveUp)
        self.Bind(wx.EVT_BUTTON, self.OnButtonAll, self.btnAll)
        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnButtonEdit, self.lcGroup)
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.OnItemSelected, self.lcGroup)
        self.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.OnItemSelected, self.lcGroup)
    
    def selectGroup(self, groupid):
        """
        指定したグループを選択状態にする
        """
        for i in range(self.lcGroup.GetItemCount()):
            self.lcGroup.SetItemState(i, 0, wx.LIST_STATE_SELECTED)
        self.lcGroup.SetItemState(groupid, wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)

    def selectAll(self):
        """
        全グループを選択状態にする
        """
        self.flgpanelupdate = False
        itemcount = self.lcGroup.GetItemCount()
        for i in range(itemcount -1):
            self.lcGroup.SetItemState(i, wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)

        self.flgpanelupdate = True
        self.lcGroup.SetItemState(itemcount-1, wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)

    def getSelectedItems(self):
        """
        選択されたグループのリストを返す
        """
        lst = []
        num = self.lcGroup.GetSelectedItemCount()
        if 0 < num:
            item = self.lcGroup.GetFirstSelected()
            lst.append(item)
            for i in range(num-1):
                item = self.lcGroup.GetNextSelected(item)
                lst.append(item)
        return lst

    def updateList(self):
        """
        グループ一覧の更新
        """
        listdata = self.graphinfo.getGroupList()
        self.setList(listdata)
        self.targets = []
        self.Refresh()

    def setList(self, listdata):
        """
        In Group Panel
        """
        self.listdata = listdata
        self.lcGroup.DeleteAllItems()
        for i, rowdata in enumerate(listdata):
            index = self.lcGroup.InsertStringItem(i, rowdata[0])
            for col, item in enumerate(rowdata[1:]):
                #self.lcGroup.SetStringItem(index, col+1, item)
                self.lcGroup.SetItem(index, col+1, item)
        
        for i in range(len(self.clmname)): 
            self.lcGroup.SetColumnWidth(i, wx.LIST_AUTOSIZE)
        self.lcGroup.SetColumnWidth(0, 40)

    def OnItemSelected(self, event):
        """
        グループ選択時のコールバック
        """
        items = self.getSelectedItems()
        self.targets = self.GetTargets()
        if self.flgpanelupdate:
            self.parent.panelpanel.updateList()
            self.parent.tracepanel.updateList()

    def GetTargets(self):
        """
        In Group Panel
        """
        items = self.getSelectedItems()
        targets = []
        if len(items) != 0:
            for idx in items:
                groupid = int(self.listdata[idx][0])-1
                target = self.graphinfo.groups[groupid]
                targets.append(target)
        return targets

    def OpenAndSetWithDialog(self, target):
        """
        In Group Panel
        """
        dlg = SetGroupDlg(self, target, self.graphinfo)
        result = dlg.ShowModal()
        flg = False
        if result == wx.ID_OK:
            dlgdata = dlg.GetResult()
            [left, bottom, width, height, gap, 
             numrows, numcols, textt, xmin, xmax, flglogx, textx, 
             flgcommony, ymin, ymax, flglogy, flgshowlegy, texty,
             linkidx, linkidy] = dlgdata
            if linkidx != -1:
                linkx = self.graphinfo.groups[linkidx] 
            else:
                linkx = None
            if linkidy != -1:
                linky = self.graphinfo.groups[linkidy] 
            else:
                linky = None

            target.setData(
                left = left,
                bottom = bottom,
                width = width,
                height = height,
                gap = gap,
                numrows = numrows,
                numcols = numcols,
                xmin = xmin,
                xmax = xmax,
                flglogx = flglogx,
                flgcommony = flgcommony,
                ymin = ymin,
                ymax = ymax,
                flglogy = flglogy,
                flgshowlegy = flgshowlegy,
                linkx = linkx,
                linky = linky)
            target.labelt.setData(text = textt)
            target.labelx.setData(text = textx)
            target.labely.setData(text = texty)
            flg = True
        dlg.Destroy()
        return flg

    def OnButtonAdd(self, event=None):
        """
        Add ボタンのコールバック
        """
        targets = self.GetTargets()
        if 0 == len(targets):
            target = Group()
        else:
            target = targets[0].copy()

        if self.OpenAndSetWithDialog(target):
            target.groupid = self.graphinfo.addGroup(target)
            self.graphinfo.setGroupLink(target)
            self.updateList()
            self.parent.panelpanel.updateList()
            self.parent.tracepanel.updateList()

    def OnButtonEdit(self, event=None):
        """
        EDIT ボタンのコールバック
        """
        targets = self.GetTargets()
        if 0 != len(targets):
            target = targets[0]
            if self.OpenAndSetWithDialog(target):
                self.graphinfo.setGroupLink(target)
                self.graphinfo.correctPanelInTraces()
                self.updateList()
                self.parent.panelpanel.updateList()
                self.parent.tracepanel.updateList()

    def OnButtonRemove(self, event=None):
        """
        Remove ボタンのコールバック
        """
        targets = self.GetTargets()
        self.graphinfo.removeGroups(targets)
        self.graphinfo.correctLink()
        self.updateList()
    
    def OnButtonMoveUp(self, event=None):
        """
        上へ
        """
        targets = self.GetTargets()
        if 0 == len(targets): return
        target = targets[0]
        self.graphinfo.moveUpGroup(target)
        self.updateList()

    def OnButtonAll(self, event=None):
        """
        In Group Panel
        """
        self.selectAll()

class PanelPanel(wx.Panel):
    """
    パネル一覧用
    """
    def __init__(self, parent, wid):
        wx.Panel.__init__(self, parent, wid)
        self.parent = parent
        self.graphinfo = self.parent.parent.graphinfo
        self.targets = []

        self.lcPanel = wx.ListCtrl(self, -1, style=wx.LC_REPORT)
        self.btnEdit = wx.Button(self, -1, 'Edit')
        self.btnMoveUp = wx.Button(self, -1, 'MoveUp')
        self.btnSwap = wx.Button(self, -1, 'Swap')
        self.btnAll = wx.Button(self, -1, 'All')
        self.clmname = ['groupid', 'panelid', 'ymin', 'ymax', 'smin', 'smax']
        for col, name in enumerate(self.clmname):
            self.lcPanel.InsertColumn(col, name) # append column
        groups = self.parent.grouppanel.targets
        self.listdata = self.graphinfo.getPanelList(groups)
        self.setList(self.listdata)
        btnsizer = wx.BoxSizer(wx.HORIZONTAL)
        btnsizer.Add (self.btnEdit,  1, wx.EXPAND)
        btnsizer.Add (self.btnMoveUp,  1, wx.EXPAND)
        btnsizer.Add (self.btnSwap,  1, wx.EXPAND)
        btnsizer.Add (self.btnAll,  1, wx.EXPAND)
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add (self.lcPanel,  1, wx.EXPAND)
        sizer.Add (btnsizer,  0, wx.EXPAND)
        self.SetSizer(sizer)
        self.Fit()
        self.flgtraceupdate = True

        self.Bind(wx.EVT_BUTTON, self.OnButtonEdit, self.btnEdit)
        self.Bind(wx.EVT_BUTTON, self.OnButtonMoveUp, self.btnMoveUp)
        self.Bind(wx.EVT_BUTTON, self.OnButtonSwap, self.btnSwap)
        self.Bind(wx.EVT_BUTTON, self.OnButtonAll, self.btnAll)
        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnButtonEdit, self.lcPanel)
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.OnItemSelected, self.lcPanel)
        self.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.OnItemSelected, self.lcPanel)

    def selectPanel(self, panelid):
        """
        In Panel Panel
        """
        for i in range(self.lcPanel.GetItemCount()):
            self.lcPanel.SetItemState(i, 0, wx.LIST_STATE_SELECTED)
        self.lcPanel.SetItemState(panelid, wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)

    def selectAll(self):
        """
        In Panel Panel
        """
        self.flgtraceupdate = False
        itemcount = self.lcPanel.GetItemCount()
        for i in range(itemcount-1):
            self.lcPanel.SetItemState(i, wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)
        self.flgtraceupdate = True
        self.lcPanel.SetItemState(itemcount-1, wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)

    def getSelectedItems(self):
        """
        In Panel Panel
        """
        lst = []
        num = self.lcPanel.GetSelectedItemCount()
        if 0 < num:
            item = self.lcPanel.GetFirstSelected()
            lst.append(item)
            for i in range(num-1):
                item = self.lcPanel.GetNextSelected(item)
                lst.append(item)
        return lst

    def updateList(self):
        """
        In Panel Panel
        """
        groups = self.parent.grouppanel.targets
        listdata = self.graphinfo.getPanelList(groups)
        self.setList(listdata)
        self.targets = []
        self.Refresh()

    def setList(self, listdata):
        """
        In Panel Panel
        """
        self.listdata = listdata
        self.lcPanel.DeleteAllItems()
        for i, rowdata in enumerate(listdata):
            index = self.lcPanel.InsertStringItem(i, rowdata[0])
            for col, item in enumerate(rowdata[1:]):
                #self.lcPanel.SetStringItem(index, col+1, item)
                self.lcPanel.SetItem(index, col+1, item)
        
        for i in range(len(self.clmname)): 
            self.lcPanel.SetColumnWidth(i, wx.LIST_AUTOSIZE)
        self.lcPanel.SetColumnWidth(0, 40)

    def OnItemSelected(self, event):
        """
        In Panel Panel
        """
        items = self.getSelectedItems()
        self.targets = self.GetTargets()
        if self.flgtraceupdate:
            self.parent.tracepanel.updateList()

    def GetTargets(self):
        """
        In Panel Panel
        """
        items = self.getSelectedItems()
        targets = []
        if len(items) != 0:
            for idx in items:
                groupid = int(self.listdata[idx][0])-1
                panelid = int(self.listdata[idx][1])-1
                group = self.graphinfo.groups[groupid]
                panel = group.panels[panelid]
                targets.append(panel)
        return targets

    def OpenAndSetWithDialog(self, target):
        """
        In Panel Panel
        """
        dlg = SetPanelDlg(self, target, self.graphinfo)
        result = dlg.ShowModal()
        flg = False
        if result == wx.ID_OK:
            dlgdata = dlg.GetResult()
            [xmin, xmax, flglogx, textx, 
             flgcommony, ymin, ymax, flglogy, flggrid, flgshowlegy, texty,
             ptype, smin, smax, flglogs, flgshowlegs, texts, linkidg, linkidp, interval, interval2] = dlgdata
#            print 'interval2=',interval2
            if linkidg != -1:
                grp = self.graphinfo.groups[linkidg]
                pnl = grp.panels[linkidp]
                linky = (grp, pnl)
            else:
                linky = None
            target.setData(
                flggrid = flggrid,
                ptype = ptype,
                smin = smin,
                smax = smax,
                flglogs = flglogs,
                flgshowlegs = flgshowlegs,
                linky = linky,
                interval = interval,
                interval2 = interval2)
            target.labels.setData(text = texts)
            if flgcommony:
                target.group.setData(
                        xmin = xmin,
                        xmax = xmax,
                        flglogx = flglogx,
                        ymin = ymin,
                        ymax = ymax,
                        flglogy = flglogy,
                        flgshowlegy = flgshowlegy,
                        flgcommony = flgcommony)
                target.group.labelx.setData(text = textx)
                target.group.labely.setData(text = texty)
            else:
                target.group.setData(
                        xmin = xmin,
                        xmax = xmax,
                        flglogx = flglogx,
                        flgcommony = flgcommony)
                target.group.labelx.setData(text = textx)
                target.setData(
                        ymin = ymin,
                        ymax = ymax,
                        flglogy = flglogy,
                        flgshowlegy = flgshowlegy)
                target.labely.setData(text = texty)
            flg = True
        dlg.Destroy()
        return flg

    def OnButtonEdit(self, event=None):
        """
        In Panel Panel
        """
        targets = self.GetTargets()
        if 0 != len(targets):
            target = targets[0]
            if self.OpenAndSetWithDialog(target):
                if target.group.flgcommony:
                    self.graphinfo.setGroupLink(target.group)
                else:
                    self.graphinfo.setPanelLink(target)
                self.updateList()
                self.parent.grouppanel.updateList()
                self.parent.tracepanel.updateList()

    def OnButtonMoveUp(self, event=None):
        """
        In Panel Panel
        """
        targets = self.GetTargets()
        if 0 != len(targets):
            target = targets[0]
            if self.graphinfo.moveUpPanel(target):
                self.updateList()
                self.parent.grouppanel.updateList()
                self.parent.tracepanel.updateList()
                self.parent.grouppanel.selectGroup(target.group.groupid)
                self.selectPanel(target.panelid)
    
    def OnButtonSwap(self, event=None):
        """
        In Panel Panel
        """
        targets = self.GetTargets()
        if 2 > len(targets):
            return

        target1 = targets[0]
        target2 = targets[-1]
        if self.graphinfo.swapPanel(target1, target2):
            self.updateList()
            self.parent.grouppanel.updateList()
            self.parent.tracepanel.updateList()
            self.parent.grouppanel.selectGroup(target1.group.groupid)
            self.selectPanel(target1.panelid)

    def OnButtonAll(self, event=None):
        """
        In Panel Panel
        """
        self.selectAll()

class TracePanel(wx.Panel):
    def __init__(self, parent, wid):
        """
        トレース設定パネル
        """
        wx.Panel.__init__(self, parent, wid)
        self.parent = parent
        self.graphinfo = self.parent.parent.graphinfo
        self.datainfo = self.parent.parent.datainfo
        self.targets = []

        self.lcTrace = wx.ListCtrl(self, -1, style=wx.LC_REPORT)
        self.btnAdd = wx.Button(self, -1, 'Add')
        self.btnEdit = wx.Button(self, -1, 'Edit')
        self.btnCopy = wx.Button(self, -1, 'Copy')
        self.btnRemove = wx.Button(self, -1, 'Remove')
        self.btnMoveUp = wx.Button(self, -1, 'Move Up')
        self.btnShowHide = wx.Button(self, -1, 'Show/Hide')
        self.btnSetIndex = wx.Button(self, -1, 'Set Index')
        self.btnSetSymbol = wx.Button(self, -1, 'Set Symbol')
        self.btnAutoColor = wx.Button(self, -1, 'Auto Color')
        self.btnAll = wx.Button(self, -1, 'All')

        self.clmname = ['traceid', 'groupid', 'panelid', 'modulename', 'dataidc', 
                'varx', 'vary', 'varz', 'index', 'option', 
                'xfactor', 'xoffset', 
                'yfactor', 'yoffset', 
                'zfactor', 'zoffset', 
                'type']
        for col, name in enumerate(self.clmname):
            self.lcTrace.InsertColumn(col, name) # append column
        panels = self.parent.panelpanel.targets
        self.listdata = self.graphinfo.getTraceList(panels)
        self.setList(self.listdata)

        btnsizer = wx.BoxSizer(wx.HORIZONTAL)
        btnsizer.Add(self.btnAdd,  1, wx.EXPAND)
        btnsizer.Add(self.btnEdit,  1, wx.EXPAND)
        btnsizer.Add(self.btnCopy,  1, wx.EXPAND)
        btnsizer.Add(self.btnRemove,  1, wx.EXPAND)
        btnsizer.Add((20, 20),  1, wx.EXPAND)
        btnsizer.Add(self.btnMoveUp,  1, wx.EXPAND)
        btnsizer.Add(self.btnShowHide,  1, wx.EXPAND)
        btnsizer.Add(self.btnSetIndex,  1, wx.EXPAND)
        btnsizer.Add(self.btnSetSymbol,  1, wx.EXPAND)
        btnsizer.Add(self.btnAutoColor,  1, wx.EXPAND)
        btnsizer.Add(self.btnAll,  1, wx.EXPAND)
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add (self.lcTrace,  1, wx.EXPAND)
        sizer.Add (btnsizer,  0, wx.EXPAND)
        self.SetSizer(sizer)
        self.Fit()

        self.Bind(wx.EVT_BUTTON, self.OnButtonAdd, self.btnAdd)
        self.Bind(wx.EVT_BUTTON, self.OnButtonEdit, self.btnEdit)
        self.Bind(wx.EVT_BUTTON, self.OnButtonCopy, self.btnCopy)
        self.Bind(wx.EVT_BUTTON, self.OnButtonRemove, self.btnRemove)
        self.Bind(wx.EVT_BUTTON, self.OnButtonMoveUp, self.btnMoveUp)
        self.Bind(wx.EVT_BUTTON, self.OnButtonShowHide, self.btnShowHide)
        self.Bind(wx.EVT_BUTTON, self.OnButtonSetIndex, self.btnSetIndex)
        self.Bind(wx.EVT_BUTTON, self.OnButtonSetSymbol, self.btnSetSymbol)
        self.Bind(wx.EVT_BUTTON, self.OnButtonAutoColor, self.btnAutoColor)
        self.btnAutoColor.Bind(wx.EVT_RIGHT_UP, self.OnButtonAutoColorRight)
        self.Bind(wx.EVT_BUTTON, self.OnButtonAll, self.btnAll)
        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnButtonEdit, self.lcTrace)
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.OnItemSelected, self.lcTrace)
        self.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.OnItemSelected, self.lcTrace)

    def selectTargets(self, target_numbers):
        """
        In Trace Panel
        """
        for i in target_numbers:
            self.lcTrace.SetItemState(i, wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)

    def OnButtonAutoColorRight(self, event=None):
        """
        In Trace Panel
        """
        self.OnButtonAutoColor(event='right')

    def OnButtonAutoColor(self, event=None):
        """
        In Trace Panel
        """
        def code2values(color_code):
            return [int(color_code[i:i+2], 16) for i in range(1, len(color_code), 2)]

        def values2code(color_rgb):
            return '#%02X%02X%02X' % tuple(color_rgb)
        
        def makecolors(n):
            ln = int(math.ceil(n/3.0))
            dc = 200.0/float(ln)
            cols = []
            for r,g,b in [(255, 0, 0), (0, 255, 0), (0, 0, 255)]:
                for i in range(ln):
                    new_r = int(r - dc * i) if r > 0 else 0
                    new_g = int(g - dc * i) if g > 0 else 0
                    new_b = int(b - dc * i) if b > 0 else 0
                    cols.append((new_r, new_g, new_b))
            return cols

        def makecolors_for_contrast(n):
            ln = int(math.ceil(n/3.0))
            dc = 200.0/float(ln)
            cols = []
            for i in range(ln):
                for r,g,b in [(255, 0, 0), (0, 255, 0), (0, 0, 255)]:
                    new_r = int(r - dc * i) if r > 0 else 0
                    new_g = int(g - dc * i) if g > 0 else 0
                    new_b = int(b - dc * i) if b > 0 else 0
                    cols.append((new_r, new_g, new_b))
            return cols

        targets = self.GetTargets()
        if 1 >= len(targets): 
            return

        first_target = targets[0]
        if event == 'right':
            cols =  [values2code(c) for c in makecolors_for_contrast(len(targets))]
        else:
            cols =  [values2code(c) for c in makecolors(len(targets))]
        lst_kwargs = [{'color':v} for v in cols]
        
        for trace, kwargs in zip(targets, lst_kwargs):
            trace.setData(**kwargs)

        self.updateList()

    def OnButtonShowHide(self, event=None):
        """
        In Trace Panel
        """
        targets = self.GetTargets()
        if len(targets) < 1: 
            return
        for trace in targets:
            trace.setData(flgshow=(not trace.flgshow))

        self.updateList()
    
    def selectAll(self):
        """
        全トレースを選択状態にする
        """
        for i in range(self.lcTrace.GetItemCount()):
            self.lcTrace.SetItemState(i, wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)


    def getSelectedItems(self):
        """
        選択されたトレースのリストを買えす
        """
        lst = []
        num = self.lcTrace.GetSelectedItemCount()
        if 0 < num:
            item = self.lcTrace.GetFirstSelected()
            lst.append(item)
            for i in range(num-1):
                item = self.lcTrace.GetNextSelected(item)
                lst.append(item)
        return lst

    def updateList(self):
        """
        In Trace Panel
        """
        panels = self.parent.panelpanel.targets
        listdata = self.graphinfo.getTraceList(panels)
        self.setList(listdata)
        self.targets = []
        self.Refresh()

    def setList(self, listdata):
        """
        In Trace Panel
        """
        self.listdata = listdata
        self.lcTrace.DeleteAllItems()
        for i, rowdata in enumerate(listdata):
            #index = self.lcTrace.InsertStringItem(i, rowdata[0])
            index = self.lcTrace.InsertItem(i, rowdata[0])
            for col, item in enumerate(rowdata[1:-1]):
                self.lcTrace.SetStringItem(index, col+1, item)
                pass
        
        for i in range(len(self.clmname)): 
            self.lcTrace.SetColumnWidth(i, wx.LIST_AUTOSIZE)
        self.lcTrace.SetColumnWidth(0, 40)

        # set color
        for i, rowdata in enumerate(listdata):
            self.lcTrace.SetItemTextColour(i, rowdata[-1])

    def OnItemSelected(self, event):
        """
        In Trace Panel
        """
        items = self.getSelectedItems()
        self.targets = []
        if len(items) != 0:
            for idx in items:
                traceid = int(self.listdata[idx][0])-1
                groupid = int(self.listdata[idx][1])-1
                panelid = int(self.listdata[idx][2])-1
                trace = self.graphinfo.traces[traceid]
                group = self.graphinfo.groups[groupid]
                panel = group.panels[panelid]
                self.targets.append(trace)

    def GetTargetsWithItemNumber(self):
        """
        In Trace Panel
        """
        items = self.getSelectedItems()
        targets = []
        if len(items) != 0:
            for idx in items:
                traceid = int(self.listdata[idx][0])-1
                target = self.graphinfo.traces[traceid]
                targets.append((idx, target))
        return targets

    def GetTargets(self):
        """
        In Trace Panel
        """
        index_and_targets = self.GetTargetsWithItemNumber()
        return [t for i, t in index_and_targets]

    def OpenAndSetWithDialog(self, targets):
        """
        In Trace Panel
        """
        if len(targets) == 1:
            partial = False
        else:
            partial = True

        dlg = SetTraceDlg(self, targets[0], self.graphinfo, self.datainfo,
                partial=partial)
        result = dlg.ShowModal()
        flg = False
        if result == wx.ID_OK:
            dlgdata, checks = dlg.GetResult()
            dlg.Destroy()
        else:
            dlg.Destroy()
            return False

        (groupid, panelid, modulename, dataidc,
         varx, vary, varz, index, option, plottype,
         symbol, symbolsize, linestyle, linewidth, color,
         factor, offset, flgshow, cmap, xfactor, xoffset, zfactor, zoffset) = dlgdata

        group = self.graphinfo.groups[groupid]
        panel = group.panels[panelid]

        for trace in targets:
            modulename_checks = checks['modulename']
            varx_checks = checks['varx']
            vary_checks = checks['vary'] 
            varz_checks = checks['varz']
            index_checks = checks['index']
            option_checks = checks['option'] 
            plottype_checks = checks['plottype'] 

            checks_mod_dependent_prms = (modulename_checks, 
                    varx_checks, vary_checks, varz_checks,
                    index_checks, option_checks, plottype_checks)

            if trace.modulename != modulename and any(checks_mod_dependent_prms):
                modulename_checks = True
                varx_checks= True
                vary_checks = True
                varz_checks = True
                index_checks = True
                optoin_checks = True
                plottype_checks = True
                if plottype != 'xyz':
                    varz = ""

            #('parameter', value, flg)
            parameters = (
                ('panel', panel, checks['groupid'] or checks['panelid']), 
                ('modulename', modulename, modulename_checks),
                ('dataidc', dataidc, checks['dataidc']),
                ('varx', varx, varx_checks),
                ('vary', vary, vary_checks),
                ('varz', varz, varz_checks),
                ('index', index, index_checks),
                ('option', option, option_checks),
                ('symbol', symbol, checks['symbol']),
                ('symbolsize', symbolsize, checks['symbolsize']),
                ('linestyle', linestyle, checks['linestyle']),
                ('linewidth', linewidth, checks['linewidth']),
                ('color', color, checks['color']),
                ('cmap', cmap, checks['cmapname']),
                ('offset', offset, checks['offset']),
                ('factor', factor, checks['factor']),
                ('xoffset', xoffset, checks['offset']),
                ('xfactor', xfactor, checks['factor']),
                ('zoffset', zoffset, checks['offset']),
                ('zfactor', zfactor, checks['factor']),
                ('plottype', plottype, plottype_checks),
                ('flgshow', flgshow, checks['flgshow']))
            
            kwargs = {}
            for k, v, f in parameters:
                if f:
                    kwargs[k] = v
                    logger.debug("%s => %s" % (k, v))
            trace.setData(**kwargs)
        return True

    def OnButtonAdd(self, event=None):
        """
        In Trace Panel
        """
        targets = self.GetTargets()
        if 0 == len(targets):
            panels = self.parent.panelpanel.targets
            if 0 == len(panels): return
            panel = panels[0]
            target = Trace(panel)
        else:
            target = targets[0].copy()
        if self.OpenAndSetWithDialog((target,)):
            self.graphinfo.addTrace(target)
            self.updateList()

    def OnButtonEdit(self, event=None):
        """
        In Trace Panel
        """
        targets = self.GetTargets()
        if 0 != len(targets):
            #target = targets[0]
            if self.OpenAndSetWithDialog(targets):
                self.updateList()

    def OnButtonCopy(self, event=None):
        """
        In Trace Panel
        """
        targets = self.GetTargets()
        if len(targets) == 0:
            return
        
        dlg = SelectGroupAndPanelDlg(self, targets[0], self.graphinfo)
        result = dlg.ShowModal()
        if result == wx.ID_OK:
            groupid, panelid = dlg.GetResult()
            dlg.Destroy()
        else:
            dlg.Destroy()
            return
        
        group = self.graphinfo.groups[groupid]
        panel = group.panels[panelid]

        for target in targets:
            new_trace = target.copy()
            new_trace.setData(panel=panel)
            self.graphinfo.addTrace(new_trace)

        self.updateList()

    def OnButtonRemove(self, event=None):
        """
        In Trace Panel
        """
        targets = self.GetTargets()
        self.graphinfo.removeTraces(targets)
        self.updateList()

    def OnButtonMoveUp(self, event=None):
        """
        In Trace Panel
        """
        items = self.getSelectedItems()
        if 0 == len(items): return
        idx = items[0]
        if idx == 0: return
        #num = self.lcTrace.GetItemCount()
        traceid0 = int(self.listdata[idx][0])-1
        traceid1 = int(self.listdata[idx-1][0])-1
        self.graphinfo.swapTrace(traceid0, traceid1)
        self.updateList()

    def OnButtonSetIndex(self, event=None):
        """
        In Trace Panel
        """
        targets = self.GetTargets()
        if 0 != len(targets):
            start = self.graphinfo.getIndexValue(targets[0])
            if not start:
                start = 0.0
            dlg = SetIndexStartStepDlg(self, start, 0.0)
            result = dlg.ShowModal()
            if result == wx.ID_OK:
                dlgdata = dlg.GetResult()
            dlg.Destroy()
            start, step = dlgdata
            i = 0
            for target in targets:
                value = start + step * i
                idxs, idxvalues = self.datainfo.getIndexs(target)
                if idxvalues:
                    list_dv = [abs(v-value) for v in idxvalues]
                    minv = min(list_dv)
                    idx = list_dv.index(minv)
                    target.setData(index = idxs[idx])
                    i = i+1
        self.updateList()

    def OnButtonSetSymbol(self, event=None):
        """
        In Trace Panel
        """
        targets = self.GetTargets()
        if 0 == len(targets): 
            return

        first_target = targets[0]
        dlg = SetTraceSymbolDlg(self, first_target, 
                self.graphinfo)
        result = dlg.ShowModal()
        flg = False
        if result == wx.ID_OK:
            dlgdata, checks = dlg.GetResult()
            flg = True
        dlg.Destroy()

        if not flg: return
        
        name = ('symbol', 'symbolsize', 'linestyle', 'linewidth', 'color')

        kwargs = {}
        for k, v, f in zip(name, dlgdata, checks):
            if f:
                kwargs[k] = v
        #kwargs = {k:v for k, v, f in zip(name, dlgdata, checks) if f} 

        for trace in targets:
            trace.setData(**kwargs)

        self.updateList()

    
    def OnButtonAll(self, event=None):
        """
        In Trace Panel
        """
        index_and_targets = self.GetTargetsWithItemNumber()
        if 0 == len(index_and_targets): 
            self.selectAll()
            return
      
        modulename = index_and_targets[0][1].modulename
        target_numbers = []
        for idx, ld in enumerate(self.listdata):
            traceid = int(ld[0])-1
            trace = self.graphinfo.traces[traceid]
            if trace.modulename == modulename:
                target_numbers.append(idx)
        self.selectTargets(target_numbers)

    def SetRefID(self, dataid):
        self.parent.parent.tcDataIdRef.SetValue(dataid)
        self.graphinfo.dataid_ref = dataid

class MyViewGroupAndTraceFrame(wx.Frame):
    """
    グループ＆トレース設定ウィンドウ
    """
    def __init__(self, parent, wid):
        wx.Frame.__init__(self, parent, wid, 'myView Groups',
                style=wx.DEFAULT_FRAME_STYLE)
#                style=wx.DEFAULT_FRAME_STYLE|wx.STAY_ON_TOP)
        self.parent = parent
        self.grouppanel = GroupPanel(self, -1)
        self.panelpanel = PanelPanel(self, -1)
        self.tracepanel = TracePanel(self, -1)
       
        stGroup = wx.StaticText(self, -1, "Group:")
        stPanel = wx.StaticText(self, -1, "Panel:")
        stTrace = wx.StaticText(self, -1, "Trace:")
        self.btnRedraw = wx.Button(self, -1, "Redraw")

        vsizerl = wx.BoxSizer(wx.VERTICAL)
        vsizerl.Add(stGroup, 0, wx.ALIGN_LEFT|wx.ALL, 2)
        vsizerl.Add(self.grouppanel, 1, wx.EXPAND|wx.ALL, 2)
        vsizerl.Add(stPanel, 0, wx.ALIGN_LEFT|wx.ALL, 2)
        vsizerl.Add(self.panelpanel, 2, wx.EXPAND|wx.ALL, 2)
        
        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        hsizer.Add(stTrace, 0, wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL, 5)
        hsizer.Add((10, 10), 1, wx.EXPAND|wx.ALL, 2)
        hsizer.Add(self.btnRedraw, 1, wx.EXPAND|wx.ALL, 2)

        vsizerr = wx.BoxSizer(wx.VERTICAL)
        vsizerr.Add(hsizer, 0, wx.EXPAND|wx.ALL, 2)
        vsizerr.Add(self.tracepanel, 1, wx.EXPAND|wx.ALL, 2)
        
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(vsizerl, 1, wx.EXPAND|wx.ALL, 2)
        sizer.Add(vsizerr, 2, wx.EXPAND|wx.ALL, 2)
        self.SetSizer(sizer)
        self.Fit()
        self.Bind(wx.EVT_CLOSE, self.OnClose)
        self.Bind(wx.EVT_BUTTON, self.OnRedraw, self.btnRedraw)

    def Update(self):
        self.grouppanel.updateList()
        self.panelpanel.updateList()
        self.tracepanel.updateList()

    def SetSelection(self, groupid, panelid):
        self.grouppanel.selectGroup(groupid)
        self.panelpanel.selectPanel(panelid)
    
    def OnRedraw(self, event=None):
        self.parent.OnReload(event)

    def OnClose(self, event):
        """
        MyViewGroupFrame
        """
        self.Hide()
        self.parent.menuDict['&Group and Trace*'].Check(False)


class MyMultiCursor(MultiCursor):
    """
    Multi Cursor クラス
    """
    def __init__(self, target, canvas, axes, color='r', lw=1,  
            useblit = True): 
        self.target = target

        if matplotlib.__version__ >= '1.3.0':
            self.x = True
        else:
            self.x = False

        if self.x:
            MultiCursor.__init__(self, canvas, axes, color=color, 
                    lw=lw, useblit=True, horizOn=False, vertOn=False)
        else:
            MultiCursor.__init__(self, canvas, axes, color=color, lw=lw, useblit=True)
        # Produce New Event for Cursor Move Notification to parent window.
        self.CursorMoveEvent, self.EVT_CURSOR_MOVE = wx.lib.newevent.NewEvent()

        # Patch to produce the horizontal lines with right values of the ymid.
        # Please use the __init__() of the 'MultiCursor (super class)' 
        # with the option horizOn=False vertOn=False 
        # to avoid the producing of the lines in the __init__() of the MultiCursor.
        vertOn, horizOn = True, True
        xmids = [0.5 * (xmin + xmax) for xmin, xmax in [ax.get_xlim() for ax in axes]] 
        ymids = [0.5 * (ymin + ymax) for ymin, ymax in [ax.get_ylim() for ax in axes]] 

        lineprops = {'color': color, 'lw':lw}
        if self.useblit:
            lineprops['animated'] = True
        if vertOn:
            self.vlines = [ax.axvline(xmids[i], visible=False, **lineprops)
                           for i, ax in enumerate(axes)]
        else:
            self.vlines = []

        if horizOn:
            self.hlines = [ax.axhline(ymids[i], visible=False, **lineprops)
                           for i, ax in enumerate(axes)]
        else:
            self.hlines = []

        self.vertOn, self.horizOn = vertOn, horizOn
        # Patch end

        self.visible = False
        
    def onmove(self, event):
        #print event.x, event.y, self.canvas.GetSize()
        MultiCursor.onmove(self, event)
        cwidth, cheight = self.canvas.GetSize()
        if event.inaxes is None:
            return
        if not self.canvas.widgetlock.available(self):
            return
        self.needclear = True
        if not self.visible:
            self.visible = True
            return
        #make new xdata, ydata
        x, y = event.x, event.y
        xydata = self.canvas.parent.figure.make_X_Y_Y2_position(x, y, cwidth, cheight)
        
        if len(xydata) == 0: 
            return
        xdata = xydata[0]
        ydata = xydata[1]

        if self.x:
            if self.vertOn:
                for line in self.vlines:
                    line.set_xdata((xdata, xdata))
                    line.set_visible(self.visible)
            if self.horizOn:
                for line in self.hlines:
                    line.set_ydata((ydata, ydata))
                    line.set_visible(self.visible)
            self._update()
            if len(xydata) == 2:
                text = "(%10.5g, %10.5g)" % (xdata, ydata)
            elif len(xydata) == 3:
                text = "(%10.5g, %10.5g, %10.5g)" % (xdata, ydata, xydata[2])
            self.target.SetLabel(text)

        else:
            for line in self.lines:
                line.set_xdata((event.xdata, event.xdata))
                line.set_visible(self.visible)
            text = "(%10.5g)" % (xdata)
            self.target.SetLabel(text)
        self.visible = True

FRAME_GLOBAL = None 
class NotifyThread(threading.Thread):
    """
    通知用のプロセスを起動するスレッド
    """
    def __init__(self, parent, filepath):
        threading.Thread.__init__(self)
        self.parent = parent
        self.stop_request = False
        self.notifypath = filepath

    def stop(self):
        self.stop_request = True

    def run(self):
#        print "Start Notify mode ..."
        while not self.stop_request:
            if os.path.exists(self.notifypath):
                self.proc_notify()
            else:
                time.sleep(5)
#        print "Stop Notify mode"

    def proc_notify(self):
        """
        ファイルに書かれたコマンドを処理する
        """
        path, filename = os.path.split(self.notifypath)
        if path == "":
            path = "."
        fname, ext = os.path.splitext(filename)
        lockfilename = fname + ".loc"
        lockfilepath = os.path.join(path, lockfilename)
        while not self.stop_request and os.path.exists(lockfilepath):
            time.sleep(1)
        fpi = open(self.notifypath, "r")
        lines = fpi.readlines()
        fpi.close()
        os.unlink(self.notifypath)
        for l in lines:
            l = l.strip('\n')
            l = l.strip('\r')
            if 0 == len(l): continue
            if l[0] == '#': continue
            frame = FRAME_GLOBAL
            clm = l.split(' ')
            cmd = clm[0].upper()
            if cmd == 'RELOAD':
                while (frame.flgBusy):
                    logger.debug("Waiting ...")
                    time.sleep(1)
                if len(clm) == 1:
#                    frame.OnReload(flgprogress=False)
                    CallAfter(frame.OnReload, flgprogress=False)
                elif len(clm) >= 2:
#                    frame.tcDataId.SetValue(clm[1])
                    CallAfter(frame.tcDataId.SetValue,clm[1])
#                    frame.OnReload(flgprogress=False)
                    CallAfter(frame.OnReload, flgprogress=False)
            elif cmd == 'DATAID':
                if len(clm) == 1:
#                    frame.tcDataId.SetValue("")
                    CallAfter(frame.tcDataId.SetValue,clm[1])
                if len(clm) >= 2:
#                    frame.tcDataId.SetValue(clm[1])
                    CallAfter(frame.tcDataId.SetValue,clm[1])
            elif cmd == 'REFID':
                if len(clm) == 1:
#                    frame.tcDataIdRef.SetValue("")
                    CallAfter(frame.tcDataIdRef.SetValue,"")
                if len(clm) >= 2:
#                    frame.tcDataIdRef.SetValue(clm[1])
                    CallAfter(frame.tcDataIdRef.SetValue,clm[1])

    
class MyTCPHandler(socketserver.BaseRequestHandler):
    """
    TCP コネクション処理
    """
    def handle(self):
        # self.request is the TCP socket connected to the client
        self.data = self.request.recv(1024).strip()
        logger.info("%s wrote:" % self.client_address[0])
        logger.info(self.data)
        self.parse_command(self.data)
        # just send back the same data, but upper-cased
        try:
            self.request.send(self.data.upper())
            self.request.send('finished')
        except:
            logger.error("Client was closed.")
            pass

    def parse_command(self, data):
        """
        クライアントからのコマンドを処理する
        """
        frame = FRAME_GLOBAL
        clm = data.split(' ')
        cmd = clm[0].upper()
        if cmd == 'RELOAD':
            while (frame.flgBusy):
                try:
                    self.request.send("Waiting ...")
                except:
                    logger.info("Client was closed.")
                    return
                time.sleep(1)
            if len(clm) == 1:
#                frame.OnReload(flgprogress=False)
                CallAfter(frame.OnReload, flgprogress=False)
            elif len(clm) >= 2:
#                frame.tcDataId.SetValue(clm[1])
                CallAfter(frame.tcDataId.SetValue,clm[1])
#                frame.OnReload(flgprogress=False)
                CallAfter(frame.OnReload, flgprogress=False)
        elif cmd == 'DATAID':
            if len(clm) == 1:
#                frame.tcDataId.SetValue("")
                CallAfter(frame.tcDataId.SetValue,"")
            if len(clm) >= 2:
#                frame.tcDataId.SetValue(clm[1])
                CallAfter(frame.tcDataId.SetValue,clm[1])
        elif cmd == 'REFID':
            if len(clm) == 1:
#                frame.tcDataIdRef.SetValue("")
                CallAfter(frame.tcDataIdRef.SetValue,"")
            if len(clm) >= 2:
#                frame.tcDataIdRef.SetValue(clm[1])
                CallAfter(frame.tcDataIdRef.SetValue,clm[1])

class ServerThread(threading.Thread):
    """
    サーバスレッド
    """
    def __init__(self, parent):
        threading.Thread.__init__(self)
        self.parent = parent
        self.stop_request = False

    def stop(self):
        self.stop_request = True

    def run(self):
        HOST, PORT = "localhost", 9999
        server = socketserver.TCPServer((HOST, PORT), MyTCPHandler)
        srvfd = server.fileno()
        logger.info("Start Server ...")
        while not self.stop_request:
            ready = select.select([srvfd],[],[],1) #Give one second for incoming connection so we can stop the server in seconds
            if srvfd in ready[0]:
                server.handle_request()
            else:
                pass # No incomming connection, retry

class BatchThread(threading.Thread):
    def __init__(self, parent, lst_dataid, flg_pdf, flg_print, flg_export):
        threading.Thread.__init__(self)
        #self.setDaemon(True)
        self.parent = parent
        self.lst_dataid = lst_dataid
        self.flg_pdf = flg_pdf
        self.flg_print = flg_print
        self.flg_export = flg_export
        self.flg_stop = False
        self.flg_append = False
        self.pdffile = "%s@%s-%s.pdf" % ("graph", lst_dataid[0], lst_dataid[-1])

    def stop(self):
        self.flg_stop = True

    def run(self):
        print("Start Batch")
        while not self.flg_stop:
            if len(self.lst_dataid) == 0:
                break
            
            while not self.parent.batchrequest_finish:
                time.sleep(0.5)
            sn = self.lst_dataid.pop(0)
            rsn = None
            evt = self.parent.ReloadRequest(
                    sn = sn,
                    rsn = rsn,
                    lst_dataid = self.lst_dataid,
                    flg_pdf = self.flg_pdf,
                    flg_print = self.flg_print,
                    flg_export = self.flg_export,
                    flg_append = self.flg_append,
                    pdffile = self.pdffile,
                    request_from = 'batch')
            self.parent.batchrequest_finish = False
            wx.PostEvent(self.parent, evt)
            time.sleep(1.0)
            self.flg_append = True

        print("Finish Batch")
        evt = self.parent.BatchFinish()
        wx.PostEvent(self.parent, evt)
        
def CreateNondupedFilename(filepath):
    folder, filename = os.path.split(filepath) 
    base, ext = os.path.splitext(filename)
    files = os.listdir(folder)
    count = 1
    fname = filename[:]
    while fname in files:
        fname = base+ "(%d)" % count + ext
        count = count+1
    return os.path.join(folder, fname)
        
def Copyfile(source_path, dist_path):
    if source_path == dist_path:
        return
    non_duped_dist_path = CreateNondupedFilename(dist_path)
    with open(source_path, "rb") as fp:
        buff = fp.read()
    with open(non_duped_dist_path, "wb") as fp:
        fp.write(buff)
    return non_duped_dist_path

class MyViewFrame(wx.Frame):
    """
    アプリケーションメインウィンドウ
    """
    def __init__(self):
        wx.Frame.__init__(self, None, -1, 'myView2')
        self.config = MyViewConfig(CONFIG_FILE)
        self.openfilepath = INIT_FILE
        self.openfolderpath, self.openfilename = os.path.split(self.openfilepath)
        variables['%LAYOUT_FILE%']=self.openfilename
        variables['%CONFIG%']=self.config
        if self.openfolderpath == "":
            self.openfolderpath = "user/layout"
            if not os.path.exists(self.openfolderpath):
                print("! Created layout folder")
                os.mkdir(self.openfolderpath, mode=666)
#        self.exportfolderpath = "user/export"
#        self.printfolderpath = "user/print"

        self.graphinfo = GraphInfo()
        self.graphinfo.load(self.openfilepath)

        self.notifypath = os.path.join(self.config['notifyfoldername'], 'notify.txt')
        cachefilepath = os.path.join(self.config['cachefoldername'],self.config['cachefilename'])
        datamodulefilepath = os.path.join(self.config['datamodulefoldername'],self.config['datamodulefilename'])
        enabledmodulefilepath = os.path.join(self.config['datamodulefoldername'],self.config['enabledmodulefilename'])
        labelmodulefilepath = os.path.join(self.config['labelmodulefoldername'],self.config['labelmodulefilename'])
        #self.tempfolderpath = self.graphinfo.tempfolderpath
        #self.commandfolderpath = self.graphinfo.commandfolderpath
        self.datainfo = DataInfo(self.graphinfo, datamodulefilepath, enabledmodulefilepath, labelmodulefilepath)
        self.cache = MyViewCache(cachefilepath, self.config['tempfoldername'])

        self.graphinfo.args['cache'] = self.cache
        self.graphinfo.args['temppath'] = self.config['tempfoldername']
        self.graphinfo.args['cmdpath'] = self.config['commandfoldername']
        Runtime.addPath(self.config['commandfoldername'])
        self.ReloadModuleList()
        self.canvaspanel = CanvasPanel(self, self.graphinfo, self.datainfo, size=self.graphinfo.canvassize)

        if ( self.config['webapi'] ):
            Runtime.set("WIGETFILE",self.config['webapi'])

        self.createMenuBar()
        self.createToolBar()

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.canvaspanel, 1, wx.EXPAND|wx.ALL, 0)
        self.SetSizer(sizer)
        self.Fit()

        self.groupframe = MyViewGroupAndTraceFrame(self, -1)
        self.legendframe = MyViewLegendFrame(self, -1)
        self.recorderframe = MyViewRecorderFrame(self, -1)

        self.ReloadRequest, self.EVT_RELOAD_REQUEST = wx.lib.newevent.NewEvent() 
        self.BatchFinish, self.EVT_BATCH_FINISH = wx.lib.newevent.NewEvent() 
        self.Bind(self.EVT_RELOAD_REQUEST, self.OnReloadRequest)
        self.Bind(self.EVT_BATCH_FINISH, self.OnBatchFinish)

        self.Bind(wx.EVT_CLOSE, self.OnQuit)
        self.Bind(wx.EVT_CHECKBOX, self.OnCursor, self.cbCursor)
        self.Bind(EVT_DUMMY, self.OnDummy)

        self.tcDataId.Bind(wx.EVT_TEXT_ENTER, self.OnDataID, self.tcDataId)
        self.tcDataId.Bind(wx.EVT_KILL_FOCUS, self.OnDataID, self.tcDataId)
        self.tcDataIdRef.Bind(wx.EVT_TEXT_ENTER, self.OnDataID, self.tcDataIdRef)
        self.tcDataIdRef.Bind(wx.EVT_KILL_FOCUS, self.OnDataID, self.tcDataIdRef)

        self.OnCursor()
        
        self.batchmode = False
        self.batchThread = None
        self.notifymode = False
        self.notifyThread = None
        self.servermode = False
        self.serverThread = None
        self.updatemode = False
        self.updateThread = None
        self.shot_notifier = shot_notifier.ShotNotifier()
        if ( realtime_debug ):
            self.shot_notifier = shot_notifier.ShotNotifier_d()
        self.updateContext = UpdateContext(self.graphinfo, self.datainfo)
        self.SetBackgroundColour(self.graphinfo.facecolor)
        col = self.canvaspanel.figure.colorConverter.to_rgb(self.graphinfo.facecolor)

        # setting with defalt value
        self.tcDataId.SetValue(self.graphinfo.dataid)
        self.tcDataIdRef.SetValue(self.graphinfo.dataid_ref)
        #self.canvaspanel.canvas.SetSize(self.graphinfo.canvassize)
        #self.canvaspanel.SetSize(self.graphinfo.canvassize)
        self.canvaspanel.update()
        self.SetMyViewTitle()
        self.batchrequest_finish = True
        self.flgBusy = False

    def UpdateProfile(self):
        self.config = MyViewConfig(CONFIG_FILE)
        self.openfilepath = INIT_FILE
        self.openfolderpath, self.openfilename = os.path.split(self.openfilepath)
        variables['%LAYOUT_FILE%']=self.openfilename
        variables['%CONFIG%']=self.config
        if self.openfolderpath == "":
            self.openfolderpath = "user/layout"
            if not os.path.exists(self.openfolderpath):
                print("! Created layout folder")
                os.mkdir(self.openfolderpath, mode=666)
#        self.exportfolderpath = "user/export"
#        self.printfolderpath = "user/print"

        self.graphinfo = GraphInfo()
        self.graphinfo.load(self.openfilepath)

        self.notifypath = os.path.join(self.config['notifyfoldername'], 'notify.txt')
        cachefilepath = os.path.join(self.config['cachefoldername'],self.config['cachefilename'])
        datamodulefilepath = os.path.join(self.config['datamodulefoldername'],self.config['datamodulefilename'])
        enabledmodulefilepath = os.path.join(self.config['datamodulefoldername'],self.config['enabledmodulefilename'])
        labelmodulefilepath = os.path.join(self.config['labelmodulefoldername'],self.config['labelmodulefilename'])
        #self.tempfolderpath = self.graphinfo.tempfolderpath
        #self.commandfolderpath = self.graphinfo.commandfolderpath
        self.datainfo = DataInfo(self.graphinfo, datamodulefilepath, enabledmodulefilepath, labelmodulefilepath)
        self.cache = MyViewCache(cachefilepath, self.config['tempfoldername'])

        self.graphinfo.args['cache'] = self.cache
        self.graphinfo.args['temppath'] = self.config['tempfoldername']
        self.graphinfo.args['cmdpath'] = self.config['commandfoldername']
        Runtime.addPath(self.config['commandfoldername'])
        if ( self.config['webapi'] ):
            Runtime.set("WIGETFILE",self.config['webapi'])
        
    def Update(self):
        """
        更新
        """
        self.canvaspanel.Refresh()
        self.groupframe.Update()
        self.legendframe.Update()
        self.SetMyViewTitle()
        self.OnCursor()

    def menuData(self):
        """
        & : with Short Cut
        * : Toggle menu
        """
        return (("File", ("&Open", "", self.OnOpen),
                         ("Open Preset", "", self.OnPreset),
                         ("&Save", "", self.OnSave),
                         ("&Export", "", self.OnExport),
                         ("&Print", "", self.OnPrint),
                         ("&Quit", "", self.OnQuit)),
                ("Edit", ("&Copy", "", self.OnCopy)),
                ("Setting", ("&Group and Trace*", "", self.OnGroupAndTrace),
                         ("Legend*", "", self.OnLegend),
                         ("", "", None),
                         ("Font Size>", "",
                          (
                           ("Increase", "",  self.OnBiggerFont),
                           ("Decrease", "", self.OnSmallerFont),
                          )
                         ),
                         ("", "", None),
                         ("Folder Path", "", self.OnFolderPath),
                         ("", "", None)),
                ("Tool", ("Batch*", "", self.OnBatch),
                         ("Notify*", "", self.OnNotify),
                         ("Server*", "", self.OnServer),
                         ("Realtime*", "", self.OnRealtime),
                         ("Recorder*", "", self.OnRecorder),
                         ("Clear Display Cache", "", self.OnClearDisplayCache),
                         ("Delete Cache Data", "", self.OnCache),
                         ("Rebuild cache.txt", "", self.OnRebuildCache),
                         ("Module Setting", "", self.OnModule),
                         ("Convert Layoutfiles", "", self.OnConvLayout),
                         ("Rebuild EGList", "", self.OnRebuildEGList),
                         ("Reset EGList", "", self.OnResetEGList),
                         ("Profile", "", self.OnChangeProfile)),
                ("Help", ("About", "", self.OnAbout)))
                

    def createMenuBar(self):
        """
        Create Menu Bar
        """
        self.menuDict = {}
        menuBar = wx.MenuBar()
        for eachMenuData in self.menuData():
            menuLabel = eachMenuData[0]
            menuItems = eachMenuData[1:]
            menuBar.Append(self.createMenu(menuItems), menuLabel)
        self.SetMenuBar(menuBar)

    def createMenu(self, Items):
        """
        Create Menu
        """
        menu = wx.Menu()
        for item in Items:
            label = item[0]
            status = item[1]
            handler = item[2]
            if not label:
                menu.AppendSeparator()
                continue
            if label[-1] == '*':
                menuItem = menu.AppendCheckItem(-1, label[:-1], status)
            elif label[-1] == '>':
                menuItem = wx.Menu()
                submenus = handler
                handler = None
                for subitem in submenus:
                    sublabel = subitem[0]
                    substatus = subitem[1]
                    subhandler = subitem[2]
                    subMenuItem = menuItem.Append(-1,sublabel, substatus)
                    self.menuDict[sublabel] = subMenuItem
                    self.Bind(wx.EVT_MENU, subhandler, subMenuItem)
                menu.AppendSubMenu(menuItem, label[0:-1])
            else:
                menuItem = menu.Append(-1, label, status)
            self.menuDict[label] = menuItem
            if handler:
                self.Bind(wx.EVT_MENU, handler, menuItem)
        return menu

    def createToolBar(self):
        """
        Create Tool Bar
        """
        self.toolbar = self.CreateToolBar()
        tooliconfolder = 'ToolIcons'
        tbIconLDpath = os.path.join(tooliconfolder, "LDirection24.bmp")
        tbIconRDpath = os.path.join(tooliconfolder, "RDirection24.bmp")
        tbIconRLDpath = os.path.join(tooliconfolder, "Reload24.bmp")
        tbIconCPNGpath = os.path.join(tooliconfolder, "CapturePNG24.bmp")
        tbIconCEPSpath = os.path.join(tooliconfolder, "CaptureEPS24.bmp")
        tbIconZOOMpath = os.path.join(tooliconfolder, "zoom.jpg")
        tbIconPANpath = os.path.join(tooliconfolder, "pan.jpg")
        tbIconHOMEpath = os.path.join(tooliconfolder, "home.jpg")

        tbIconLD = wx.Image(tbIconLDpath, wx.BITMAP_TYPE_BMP).ConvertToBitmap()
        tbIconRD = wx.Image(tbIconRDpath, wx.BITMAP_TYPE_BMP).ConvertToBitmap()
        tbIconRLD = wx.Image(tbIconRLDpath, wx.BITMAP_TYPE_BMP).ConvertToBitmap()
        tbIconCPNG = wx.Image(tbIconCPNGpath, wx.BITMAP_TYPE_BMP).ConvertToBitmap()
        tbIconCEPS = wx.Image(tbIconCEPSpath, wx.BITMAP_TYPE_BMP).ConvertToBitmap()
        tbIconZOOM = wx.Image(tbIconZOOMpath, wx.BITMAP_TYPE_JPEG).ConvertToBitmap()
        tbIconPAN = wx.Image(tbIconPANpath, wx.BITMAP_TYPE_JPEG).ConvertToBitmap()
        tbIconHOME = wx.Image(tbIconHOMEpath, wx.BITMAP_TYPE_JPEG).ConvertToBitmap()
        
        id_tool_dec = wx.NewId()
        tool_dec = self.toolbar.AddSimpleTool(id_tool_dec, tbIconLD, "Dec", "Previous Shot #.")
        id_tool_inc = wx.NewId()
        tool_inc = self.toolbar.AddSimpleTool(id_tool_inc, tbIconRD, "Inc", "Next Shot #.")
        id_tool_rel = wx.NewId()
        tool_rel = self.toolbar.AddSimpleTool(id_tool_rel, tbIconRLD, "ReLoad", "Reload")
        id_tool_cpng = wx.NewId()
        tool_cpng = self.toolbar.AddSimpleTool(id_tool_cpng, tbIconCPNG, "PNG", "Capture")
        id_tool_ceps = wx.NewId()
        tool_ceps = self.toolbar.AddSimpleTool(id_tool_ceps, tbIconCEPS, "EPS", "Capture")

        self.toolbar.AddSeparator()
        id_tool_zoom = wx.NewId()
        tool_zoom = self.toolbar.AddSimpleTool(id_tool_zoom, tbIconZOOM, "ZOOM", "ZOOM")
        id_tool_pan = wx.NewId()
        tool_pan = self.toolbar.AddSimpleTool(id_tool_pan, tbIconPAN, "PAN", "PAN")
        id_tool_home = wx.NewId()
        tool_home = self.toolbar.AddSimpleTool(id_tool_home, tbIconHOME, "HOME", "HOME")

        self.toolbar.AddSeparator()

        #self.btnZoom = wx.Button(self.toolbar, -1, "Z", size=(40,24))
        #self.toolbar.AddControl(self.btnZoom)
        #self.btnPan = wx.Button(self.toolbar, -1, "P", size=(40, 24))
        #self.toolbar.AddControl(self.btnPan)
        #self.btnHome = wx.Button(self.toolbar, -1, "H", size=(40, 24))
        #self.toolbar.AddControl(self.btnHome)
        #self.toolbar.AddSeparator()

        st_shot = wx.StaticText(self.toolbar, -1, "SHOT ", 
                                  size=(60, 18), style=wx.ALIGN_RIGHT)
        self.toolbar.AddControl(st_shot)
        st_dataid = wx.StaticText(self.toolbar, -1, "Current:", 
                                  size=(60, 18), style=wx.ALIGN_RIGHT)
        self.toolbar.AddControl(st_dataid)

        self.tcDataId = wx.TextCtrl(self.toolbar, -1, "", size=(100, 18),
                style=wx.TE_PROCESS_ENTER)
        self.toolbar.AddControl(self.tcDataId) 

        st_dataid_ref = wx.StaticText(self.toolbar, -1, "Ref:", 
                                      size=(60, 18), style=wx.ALIGN_RIGHT)
        self.toolbar.AddControl(st_dataid_ref)

        self.tcDataIdRef = wx.TextCtrl(self.toolbar, -1, "", size=(100, 18),
                style=wx.TE_PROCESS_ENTER)
        self.toolbar.AddControl(self.tcDataIdRef) 
        
#        self.btnCache = wx.Button(self.toolbar, -1, "Cache")
        self.stOnline = wx.StaticText(self.toolbar, -1, 'Mode:ON-line',
                                      size = (100,18), style = wx.ALIGN_RIGHT)
        self.toolbar.AddControl(self.stOnline)

        self.btnOnline = wx.Button(self.toolbar, -1, "On/Off")
        self.toolbar.AddControl(self.btnOnline)
        self.online_mode = 'ONLINE'
        self.graphinfo.args['cachebutton'] = self.online_mode

        st_cursor = wx.StaticText(self.toolbar, -1, "Cursor:", 
                size=(60, 18), style=wx.ALIGN_RIGHT)
        self.toolbar.AddControl(st_cursor)
        self.cbCursor = wx.CheckBox(self.toolbar, -1, "")
        self.cbCursor.SetValue(False)
        self.toolbar.AddControl(self.cbCursor)

        self.st_position = wx.StaticText(self.toolbar, -1, "-"*33, size=(300, 18))
        self.toolbar.AddControl(self.st_position)
        
        
        self.toolbar.Realize()

        self.Bind(wx.EVT_MENU, self.OnDecrement, tool_dec)
        self.Bind(wx.EVT_MENU, self.OnIncrement, tool_inc)
        self.Bind(wx.EVT_MENU, self.OnReload, tool_rel)
        self.Bind(wx.EVT_MENU, self.OnCapturePNG, tool_cpng)
        self.Bind(wx.EVT_MENU, self.OnCaptureEPS, tool_ceps)

        self.Bind(wx.EVT_BUTTON, self.OnButtonCache, self.btnOnline)

        self.Bind(wx.EVT_MENU, self.OnButtonZoom, tool_zoom)
        self.Bind(wx.EVT_MENU, self.OnButtonPan, tool_pan)
        self.Bind(wx.EVT_MENU, self.OnButtonHome, tool_home)
        self.toolbaritem = {'Dec':id_tool_dec, 'Inc':id_tool_inc,
                'ReLoad':id_tool_rel, 'PNG':id_tool_cpng, 'EPS':id_tool_ceps}

        self.ChangeOnlineMode('ONLINE')
    def OnButtonZoom(self, event=None):
        self.canvaspanel.navitoolbar.zoom()
    def OnButtonPan(self, event=None):
        self.canvaspanel.navitoolbar.pan()
    def OnButtonHome(self, event=None):
        self.canvaspanel.navitoolbar.home()

    def OnCursor(self, event=None):
        """
        カーソルのチェックのコールバック
        """
        if self.cbCursor.GetValue():
            dictaxes = self.canvaspanel.figure.getAxes()
            lstaxes = []
            for k in list(dictaxes.keys()):
                ax, axsub = dictaxes[k]
                #print k, ax, axsub
                lstaxes.append(ax)
            self.multi = MyMultiCursor(self.st_position, 
                    self.canvaspanel.canvas, 
                    lstaxes, color='r', lw=1)
        else:
            try:
                del self.multi
            except(AttributeError):
                pass
            self.st_position.SetLabel("-" * 33)
        self.canvaspanel.canvas.draw()

    def OnDataID(self, event=None):
        event.Skip()
        self.graphinfo.dataid = self.tcDataId.GetValue()
        self.graphinfo.dataid_ref = self.tcDataIdRef.GetValue()

    def ChangeOnlineMode(self, mode ):
        self.online_mode = mode
        if mode == 'ONLINE':
            self.stOnline.SetLabel('Mode:ON-line')
        else:
            self.stOnline.SetLabel('Mode:OFF-line')

    def OnButtonCache(self, event=None):
        """
        CACHEボタンのコールバック
        """
        if self.flgBusy: return

        self.datainfo.clearInstances()
        ClearDisplayCache()
        if self.online_mode == 'ONLINE':
            self.ChangeOnlineMode('OFFLINE')
        else:
            self.ChangeOnlineMode('ONLINE')

        self.graphinfo.args['cachebutton'] = self.online_mode

    def OnDecrement(self, event=None):
        """
        ショット番号を-1する
        """
        if self.flgBusy: return
        dataid = self.tcDataId.GetValue()
        self.tcDataId.SetValue(MyViewIDControl.decrementID(dataid))

    def OnIncrement(self, event=None):
        """
        ショット番号を+1する
        """
        if self.flgBusy: return
        dataid = self.tcDataId.GetValue()
        self.tcDataId.SetValue(MyViewIDControl.incrementID(dataid))

    def OnReloadRequest(self, event=None):
        sn, rsn, request_from = event.sn, event.rsn, event.request_from
        if sn is not None:
            self.tcDataId.SetValue(sn)
        if rsn is not None:
            self.tcDataIdRef.SetValue(rsn)
        wx.Sleep(0.1)
        ClearDisplayCache()
        self.OnReload(flgprogress=True)
        if request_from == 'notify':
            pass
        elif request_from == 'server':
            pass
        elif request_from == 'batch':
            base = variables['%LAYOUT_FILE%']
            flg_print = event.flg_print
            flg_export = event.flg_export

            if (event.flg_pdf):
                path = self.config['printfoldername']
                name = base + '@%s.png' % sn 
                pngpath = os.path.join(path, name)
                self.canvaspanel.canvas.print_png(pngpath)

            if (event.flg_export):
                path = self.config['exportfoldername']
                name = base + '@%s.txt' % sn 
                exportpath = os.path.join(path, name)
                self.datainfo.save_compact(exportpath, False)

            self.batchrequest_finish = True
        else:
            pass

    def OnReload(self, event=None, flgprogress=True):
        """
        RELOADボタンのコールバック
        """
        # for active labels
        logger.debug("OnReload")
        alabels = []
        if self.flgBusy: return

        if self.online_mode == 'ONLINE' :
            if not ('webapi' in self.config)  and not checkDBConnection():
                diag = wx.MessageDialog(None,
                                        "Access to EG-Serve failed.\nDo you want retrieve data with OFFLINE mode?",
                                        "Error",
                                        wx.YES_NO )
                
                ret = diag.ShowModal()
                diag.Destroy()
                if ( ret == wx.ID_YES ):
                    self.ChangeOnlineMode('OFFLINE')
                else:
                    return

        # アクティブラベル読み込み用スレッドの開始
        self.flgBusy = True
        self.graphinfo.dataid = self.tcDataId.GetValue()
        self.graphinfo.dataid_ref = self.tcDataIdRef.GetValue()
        globalError.clear()
        for i, alabel in enumerate(self.graphinfo.activelabels):
            modulename = alabel.modulename
            dataidc = alabel.dataidc
            arg = alabel.arg
            if dataidc.upper() == 'C':
                dataid = self.tcDataId.GetValue()
            elif dataidc.upper() == 'R':
                dataid = self.tcDataIdRef.GetValue()
            else:
                shot, sub, rshot, rsub  = util.ShotStringToInteger(
                        self.tcDataId.GetValue(),
                        self.tcDataIdRef.GetValue())
                dataid = dataidc
                dataid = util.ReplaceShotnumberInFilename(
                        dataid, shot, sub, rshot, rsub)
                
            self.graphinfo.args = {}
            self.graphinfo.args['cache'] = self.cache
#            self.graphinfo.args['cache'] = 'ONLINE'
            self.graphinfo.args['cachebutton'] = self.online_mode
#            self.graphinfo.args['cachebutton'] = 'ONLINE'
            self.graphinfo.args['temppath'] = self.config['tempfoldername']
            self.graphinfo.args['cmdpath'] = self.config['commandfoldername']
            Runtime.addPath(self.config['commandfoldername'])
            if dataid != "":
                alabels.append((i, modulename, dataid, arg, self.graphinfo.args))
        labelcount = len(alabels)
        self.datainfo.FetchLabels(alabels)

        # データ読み込み用スレッドの開始
        # for plot data
        traces = []
        args = []
        for i, trace in enumerate(self.graphinfo.traces):
            if not trace.flgshow: continue
            modulename = trace.modulename
            dataidc = trace.dataidc
            option = trace.option
            if dataidc.upper() == 'C':
                dataid = self.tcDataId.GetValue()
            elif dataidc.upper() == 'R':
                dataid = self.tcDataIdRef.GetValue()
            else:
                shot, sub, rshot, rsub = util.ShotStringToInteger(
                        self.tcDataId.GetValue(),
                        self.tcDataIdRef.GetValue())
                dataid = dataidc
                dataid = util.ReplaceShotnumberInFilename(
                        dataid, shot, sub, rshot, rsub)
            self.graphinfo.args = {}
            self.graphinfo.args['varx'] = trace.varx
            self.graphinfo.args['vary'] = trace.vary
            self.graphinfo.args['varz'] = trace.varz
            self.graphinfo.args['index'] = trace.index
            self.graphinfo.args['trace'] = trace
            self.graphinfo.args['type'] = trace.plottype
            self.graphinfo.args['cache'] = self.cache
            self.graphinfo.args['cachebutton'] = self.online_mode
            self.graphinfo.args['temppath'] = self.config['tempfoldername']
            self.graphinfo.args['cmdpath'] = self.config['commandfoldername']
            args.append(self.graphinfo.args)
            if dataid != "":
                traces.append((i, modulename, dataid, option, self.graphinfo.args))
        datacount = len(traces)
        self.datainfo.FetchData(traces)

        # スレッドが終了するまで待ち合わせる
        if flgprogress:
            #---------- Progress Dialog -------------------------
            maxcount = datacount + labelcount
            dialog = wx.ProgressDialog("Reloading",
                    "Thread Remaining", maxcount, 
                    style = wx.PD_CAN_ABORT|wx.PD_ELAPSED_TIME|wx.PD_REMAINING_TIME|wx.PD_AUTO_HIDE)
            flgwaiting = True
            flgabort = False
            while flgwaiting:
                count = self.datainfo.GetCount()
                threads = self.datainfo.datathreads + self.datainfo.labelthreads
                if any([t.isAlive() for t in threads]):
                    flgwaiting = True
                else:
                    flgwaiting = False

                if flgwaiting and globalError.get():
                    errDiag = wx.MessageDialog(None,
                                               "Network Error occurred.\nDo you cancel retrieving?",
                                               "Error",
                                               wx.YES_NO )

                    ret = errDiag.ShowModal()
                    if ( ret == wx.ID_YES ):
                        flgabort = True
                    errDiag.Destroy()

                if flgabort or (not dialog.Update(count)[0]):
                    for t in self.datainfo.datathreads:
                        if t.isAlive():
                            t.abort()
                    for t in self.datainfo.labelthreads:
                        if t.isAlive():
                            t.abort()
                    flgwaiting = False
                wx.Sleep(1)
            dialog.Destroy()
            #-----------------------------------------------------
        self.canvaspanel.update()
        self.groupframe.Update()
        # 描画終了後にダミーのイベントを発生させる MacOS 対策
        self.Refresh()
        wx.PostEvent(self, DummyEvent())

    def StopFetching(self):
        """
        データ取得用スレッドの終了を待ち合わせる
        """
        logger.debug("StopFetching")
        #---------- Progress Dialog -------------------------
        maxcount = len(self.datainfo.datathreads)
        maxcount = maxcount + len(self.datainfo.labelthreads)
        flgwaiting = True
        while flgwaiting:
            count = 0
            for t in self.datainfo.datathreads:
                if t.isAlive():
                    count = count + 1
            for t in self.datainfo.labelthreads:
                if t.isAlive():
                    count = count + 1
            if count == 0:
                flgwaiting = False
            else:
                wx.Sleep(0.01)
    
    def StartFetching(self):
        """
        """

        # for active labels
        logger.debug("StartFetching")
#        print "self.flgBusy=",self.flgBusy
        alabels = []
#        if self.flgBusy: return
        
        # アクティブラベル読み込み用スレッドの開始
#        self.flgBusy = True
        self.graphinfo.dataid = self.tcDataId.GetValue()
        self.graphinfo.dataid_ref = self.tcDataIdRef.GetValue()

#        for i, alabel in enumerate(self.updateContext.alabel_remain):
        logger.debug("*** START LABEL FETCH ***")
        logger.debug("self.updateContext.alabel_remain.items()=%d" % (len(list(self.updateContext.alabel_remain.items()))))
        for i, alabel in list(self.updateContext.alabel_remain.items()):
            modulename = alabel.modulename
            dataidc = alabel.dataidc
            arg = alabel.arg
            if dataidc.upper() == 'C':
                dataid = self.tcDataId.GetValue()
            elif dataidc.upper() == 'R':
                dataid = self.tcDataIdRef.GetValue()
            else:
                dataid = dataidc
            self.graphinfo.args = {}
            self.graphinfo.args['cache'] = self.cache
            self.graphinfo.args['cachebutton'] = self.online_mode
            self.graphinfo.args['temppath'] = self.config['tempfoldername']
            self.graphinfo.args['cmdpath'] = self.config['commandfoldername']
            Runtime.addPath(self.config['commandfoldername'])
            if dataid != "":
                alabels.append((i, modulename, dataid, arg, self.graphinfo.args))
        self.datainfo.FetchLabels(alabels)

        # データ読み込み用スレッドの開始
        # for plot data
        traces = []
#        for i, trace in enumerate(self.updateContext.trace_remain):
        logger.debug("*** START DATA FETCH ***")
        logger.debug("self.updateContext.trace_remain.items()=%d" % (len(list(self.updateContext.trace_remain.items()))))
        for i, trace in list(self.updateContext.trace_remain.items()):
            if not trace.flgshow: continue
            modulename = trace.modulename
            dataidc = trace.dataidc
            option = trace.option
            if dataidc.upper() == 'C':
                dataid = self.tcDataId.GetValue()
            elif dataidc.upper() == 'R':
                dataid = self.tcDataIdRef.GetValue()
            else:
                dataid = dataidc
            self.graphinfo.args = {}
            self.graphinfo.args['varx'] = trace.varx
            self.graphinfo.args['vary'] = trace.vary
            self.graphinfo.args['varz'] = trace.varz
            self.graphinfo.args['index'] = trace.index
            self.graphinfo.args['trace'] = trace
            self.graphinfo.args['type'] = trace.plottype
            self.graphinfo.args['cache'] = self.cache
            self.graphinfo.args['cachebutton'] = self.online_mode
            self.graphinfo.args['temppath'] = self.config['tempfoldername']
            self.graphinfo.args['cmdpath'] = self.config['commandfoldername']
            if dataid != "":
                traces.append((i, modulename, dataid, option, self.graphinfo.args))
                ClearDisplayDataCache(i, modulename, dataid, option, self.graphinfo.args)
        #print "traces=",traces
        logger.debug("&&&&& traces="+str(traces))
        self.datainfo.FetchData(traces)
        #print "len(self.datainfo.datathreads)=",len(self.datainfo.datathreads)
        

    def CancelUpdate(self):
        """
        自動更新を中断する
        """
        if ( self.updateThread != None ):
            self.updateThread.stop()
            self.updateThread.join()
            self.updateThread = None
        #print "CancelUpdate()"

    def StartUpdate(self):
        """
        自動更新を始める
        """
        logger.debug("***** StartUpdate *****")
        self.updateThread = UpdateGraphThread(self)
        self.updateThread.start()
    
    def UpdateGraph(self):
        """
        更新されたデータを描画する
        """
        logger.debug("***** UpdateGraph() *****")
        #---------- Progress Dialog -------------------------
        if self.flgBusy: return
        self.flgBusy = True
        maxcount = len(self.datainfo.datathreads)
        maxcount = maxcount + len(self.datainfo.labelthreads)
        flgwaiting = True
        # データ取得スレッドが終了するまで待ち合わせる
        while flgwaiting:
            count = 0
            for t in self.datainfo.datathreads:
                if t.isAlive():
                    count = count + 1
            for t in self.datainfo.labelthreads:
                if t.isAlive():
                    count = count + 1
            if count == 0:
                flgwaiting = False
            time.sleep(0.5)
            #-----------------------------------------------------
        #self.canvaspanel.update_real()
        logger.debug("*** Complete Fetching ****")
        CallAfter(self.canvaspanel.update_real)

#        self.canvaspanel.update_real()

    def UpdateMain(self):
        """
        自動更新メイン部分
        """
        logger.debug("UpdateMain()")
        
        con = self.updateContext
        try:
            shotNo = "%d" % (self.shot_notifier.getCurrent() - con.auto_delay)
            self.graphinfo.dataid = self.tcDataId.GetValue()
            self.graphinfo.dataid_ref = self.tcDataIdRef.GetValue()
            if ( self.graphinfo.dataid != shotNo ):
                now = time.time()
                last = self.updateContext.last_updated
                duration = self.updateContext.duration 
                if ( ( now - last ) > duration ):
                    self.StopFetching()
                    self.updateContext.reset()
                    if con.flg_print:
                    	#印刷を行う
                        #self.SendToVirtualPrinter(con.print_dialog, int(shotNo))
                        CallAfter(self.SendToVirtualPrinter,con.print_dialog, int(shotNo))
                    #self.tcDataId.SetValue(shotNo)
                    CallAfter(self.tcDataId.SetValue,shotNo)
                    self.graphinfo.dataid = shotNo
                    #time.sleep(5.0)
                    CallAfter(self.canvaspanel.init_frame)
            self.StartFetching()
            self.UpdateGraph()
            wx.Sleep(0.01)
            CallAfter(con.updateRemain)
            wx.Sleep(0.01)
        except:
            logger.error("*****************")
            import sys,traceback
            logger.error(traceback.print_exc())
        
    def OnCapturePNG(self, event=None):
        """
        PNGボタンのコールバック
        """
        if self.flgBusy: return
        dataid = self.tcDataId.GetValue() 
        base, ext = os.path.splitext(self.openfilename)
        default_filename = "%s@%s.png" % (base, dataid)
        wildcard = "png file (*.png)|*.png" 
        dialog = wx.FileDialog(None, "Input Save filename", self.config['printfoldername'],
                default_filename, wildcard, wx.SAVE)
        result = dialog.ShowModal()
        if result == wx.ID_OK:
            filepath = dialog.GetPath()
        dialog.Destroy()
        if result == wx.ID_OK:
            """
            delete multi cursor once, then save and recover
            """
            try:
                del self.multi
            except(AttributeError):
                pass
            self.canvaspanel.canvas.draw()
            #self.canvaspanel.canvas.print_png(filepath)
            self.canvaspanel.figure.savefig(filepath)
            self.OnCursor()

    def OnCaptureEPS(self, event=None):
        """
        EPSボタンのコールバック
        """
        if self.flgBusy: return
        dataid = self.tcDataId.GetValue() 
        base, ext = os.path.splitext(self.openfilename)
        default_filename = "%s@%s.eps" % (base, dataid)
        wildcard = "eps file (*.eps)|*.eps" 
        dialog = wx.FileDialog(None, "Input Save filename", self.config['printfoldername'],
                default_filename, wildcard, wx.SAVE)
        result = dialog.ShowModal()
        if result == wx.ID_OK:
            filepath = dialog.GetPath()
        dialog.Destroy()
        if result == wx.ID_OK:
            """
            delete multi cursor once, then save and recover
            """
            try:
                del self.multi
            except(AttributeError):
                pass
            self.canvaspanel.canvas.draw()
            #self.canvaspanel.canvas.print_eps(filepath)
            self.canvaspanel.figure.savefig(filepath)
            self.OnCursor()

    def OnOpen(self, event=None):
        """
        Open メニューのコールバック
        """
        if self.flgBusy: return
        wildcard = "MyView def (*.mvd)|*.mvd|text file (*.txt)|*.txt|" \
                "All files (*.*)|*.*"
        dialog = wx.FileDialog(None, "Input Load filename", self.openfolderpath,
                "", wildcard, wx.OPEN)
        result = dialog.ShowModal()
        if result == wx.ID_OK:
            filepath = dialog.GetPath()
            self.openfilepath = filepath
            print('filepath=',filepath)
            self.openfolderpath, self.openfilename = os.path.split(self.openfilepath)
        dialog.Destroy()
        if result == wx.ID_OK:
            self.graphinfo.load(filepath)
            self.tcDataId.SetValue(self.graphinfo.dataid)
            self.tcDataIdRef.SetValue(self.graphinfo.dataid_ref)
            self.ReloadModuleList()

            if wx.__version__ >= "2.9":
                #print self.canvaspanel.canvas.GetSize()
                #print self.canvaspanel.GetSize()
                #print self.GetClientSize()
                #print "-------------"
                #self.canvaspanel.canvas.SetSize(self.graphinfo.canvassize)
                self.SetClientSize(self.graphinfo.canvassize)
                #print self.canvaspanel.canvas.GetSize()
                #print self.canvaspanel.GetSize()
                #print self.GetClientSize()
            else:
                self.canvaspanel.canvas.SetSize(self.graphinfo.canvassize)
            
            # cache change here
            # temppath change here
            self.Update()

    def OnSave(self, event=None):
        """
        Save メニューのコールバック
        """
        #dataid = self.tcDataId.GetValue() 
        #base, ext = os.path.splitext(self.openfilename)
        #default_filename = "%s@%s%s" % (base, dataid, ext)
        default_filename = self.openfilename

        wildcard = "MyView def (*.mvd)|*.mvd|text file (*.txt)|*.txt" 
        dialog = wx.FileDialog(None, "Input Save filename", self.openfolderpath,
                default_filename, wildcard, wx.SAVE)
        result = dialog.ShowModal()
        if result == wx.ID_OK:
            filepath = dialog.GetPath()
            self.openfilepath = filepath
            self.openfolderpath, self.openfilename = os.path.split(self.openfilepath)
            self.SetMyViewTitle()
        dialog.Destroy()
        if result == wx.ID_OK:
            self.graphinfo.save(filepath)

    def OnExport(self, event=None):
        """
        Export メニューのコールバック
        """
        if self.flgBusy: return
        dataid = self.tcDataId.GetValue() 
        base, ext = os.path.splitext(self.openfilename)
        default_filename = "%s@%s.txt" % (base, dataid)

        wildcard = "text file (*.txt)|*.txt" 
        dialog = wx.FileDialog(None, "Input Export filename", self.config['exportfoldername'],
                default_filename, wildcard, wx.SAVE)
        result = dialog.ShowModal()
        if result == wx.ID_OK:
            filepath = dialog.GetPath()
            self.config['exportfoldername'], filename = os.path.split(filepath)
        dialog.Destroy()
        if result == wx.ID_OK:
            # 2014-03-14 DEBUG
            #retv = self.datainfo.save(filepath, False)
            retv = self.datainfo.save_compact(filepath, False)

    def OnExportGP(self, event=None):
        """
        Gnuplot のコマンドとして出力
        """
        if self.flgBusy: return
        dataid = self.tcDataId.GetValue() 
        base, ext = os.path.splitext(self.openfilename)
        default_filename = "%s_traces@%s%s" % (base, dataid, ext)

        wildcard = "text file (*.txt)|*.txt" 
        dialog = wx.FileDialog(None, "Input Export filename", self.config['exportfoldername'],
                default_filename, wildcard, wx.SAVE)
        result = dialog.ShowModal()
        if result == wx.ID_OK:
            filepath = dialog.GetPath()
            self.config['exportfoldername'], filename = os.path.split(filepath)
        dialog.Destroy()
        if result == wx.ID_OK:
            retv = self.datainfo.save(filepath, True)
            #retv = self.datainfo.save(filepath, False)
            """
            for idx, clms, flgs in retv:
                print "%d:\t%d\t%d\t%d\t%d\t%d\t%d" % (idx, clms[0], clms[1],
                        clms[2], clms[3],
                        clms[4], clms[5])
            """
            base, ext = os.path.splitext(filepath)
            plotfilepath = base + '.plt'
            plotdata = self.datainfo.plotdata[:]
            gnuplot = SaveGnuplot(filepath, self.graphinfo, retv, plotdata)
            gnuplot.output(plotfilepath)

    def PrintPS(self, filepath):
        """
        PSファイルの出力
        """
        self.canvaspanel.figure.savefig("myviewtemp.ps", orientation='landscape')
        cwidth, cheight = self.canvaspanel.canvas.GetSize()
        fpi = open('myviewtemp.ps', 'r')
        lines = fpi.readlines()
        fpi.close()
        # read bounding box from ps file
        for line in lines:
            line = line.strip('\n')
            line = line.strip('\r')
            if -1 != line.find('%%BoundingBox:'):
                clm = line.split(' ')
                bbleft = float(clm[1])
                bbbtm = float(clm[2])
                bbright = float(clm[3])
                bbtop = float(clm[4])
                break
        bbwidth = bbright - bbleft + 1
        bbheight = bbtop - bbbtm + 1
        lmargin = 40
        bmargin = 40
        a4width = 590 # full width = 610 there is no print region
        a4height = 770 # full height = 790 there is no print region
        rwidth = a4width - lmargin
        rheight = a4height - bmargin
        scalewidth = float(rwidth)/float(bbwidth)
        scaleheight = float(rheight)/float(bbheight)
        if scalewidth < scaleheight:
            scale = scalewidth
        else:
            scale = scaleheight
        offsetx = int(bbleft * -1.0 + lmargin/scale)
        offsety = int(bbbtm * -1.0 + bmargin/scale)

        fpo = open(filepath, 'w')
        for line in lines:
            line = line.strip('\n')
            line = line.strip('\r')
            if -1 != line.find('%%DocumentPaperSizes:'):
                l = '%%%%DocumentPaperSize: a4\n'
                fpo.write(l)
            elif -1 != line.find('%%BeginProlog'):
                l = '%%%%BeginProlog\n'
                s = '%4.2f %4.2f scale\n' % (scale, scale)
                l = l + s
                s = '%d %d translate\n' % (offsetx, offsety)
                l = l + s
                fpo.write(l)
            elif -1 != line.find('%%BoundingBox:'):
                l = "%%%%BoundingBox: %d %d %d %d\n" % (bbleft, bbbtm, bbright, bbtop)
                fpo.write(l)
            else:
                fpo.write(line)
                fpo.write('\n')
        fpo.close()

    def OnPrint(self, event=None):
        """
        Printメニューのコールバック
        """
        dataid = self.tcDataId.GetValue() 
        base, ext = os.path.splitext(self.openfilename)
        default_filename = "%s@%s.ps" % (base, dataid)

        wildcard = "ps file (*.ps)|*.ps" 
        dialog = wx.FileDialog(None, "Input Print filename",  self.config['printfoldername'], 
                default_filename, wildcard, wx.SAVE)
        result = dialog.ShowModal()
        if result == wx.ID_OK:
            filepath = dialog.GetPath()
        dialog.Destroy()
        if result == wx.ID_OK:
            self.PrintPS(filepath)

    def OnQuit(self, event=None):
        """
        Qutiメニューのコールバック
        """
        for t in self.datainfo.datathreads:
            t.join()
        for t in self.datainfo.labelthreads:
            t.join()
        if self.batchThread:
            self.batchThread.join()
        if self.serverThread:
            self.serverThread.stop()
            self.serverThread.join()
        if self.notifyThread:
            self.notifyThread.stop()
            self.notifyThread.join()
        if self.updateThread:
            self.updateThread.stop()
            self.updateThread.join()
        self.Destroy()

    def OnCopy(self, event=None):
        """
        delete multi cursor once, then copy to clipboard
        """
        try:
            del self.multi
        except(AttributeError):
            pass
        self.canvaspanel.canvas.draw()
        self.canvaspanel.canvas.Copy_to_Clipboard()
        self.OnCursor()

    def OnGroupAndTrace(self, event=None):
        """
        Group & Trace メニューのコールバック
        """
#        if self.groupframe.IsShown():
#            self.groupframe.Hide()
#            self.menuDict['&Group and Trace*'].Check(False)
#        else:
#            self.groupframe.Show()
#            self.menuDict['&Group and Trace*'].Check(True)
#            self.groupframe.Centre()
        if self.groupframe.IsShown():
            self.hideGroupFrame()
            self.menuDict['&Group and Trace*'].Check(False)
        else:
            self.showGroupFrame()
            self.menuDict['&Group and Trace*'].Check(True)

    
    def OnLegend(self, event=None):
        """
        Legend メニューのコールバック
        """
        if self.legendframe.IsShown():
            self.legendframe.Hide()
            self.menuDict['Legend*'].Check(False)
        else:
            self.legendframe.Show()
            self.menuDict['Legend*'].Check(True)
            self.legendframe.Centre()

    def OnBiggerFont(self, event=None):
        """
        Bigger Font メニューのコールバック
        """
        self.graphinfo.changeFontSize(+1)
        
    def OnSmallerFont(self, event=None):
        """
        Smaller Font メニューのコールバック
        """
        self.graphinfo.changeFontSize(-1)

    def OnFolderPath(self, event=None):
        """
        Folder Path のコールバック
        """
        if self.flgBusy: return
        dialog = SetFolderDlg(self, self.config)
        result = dialog.ShowModal()
        if result == wx.ID_OK:
            dialogdata = dialog.GetResult()
            [commandfolderpath,
             datamodulefolderpath,
             labelmodulefolderpath,
             cachefolderpath,
             tempfolderpath,
             exportfolderpath,
             printfolderpath,
             notifyfolderpath] = dialogdata
            self.config['commandfoldername'] = commandfolderpath
            self.config['datamodulefoldername'] = datamodulefolderpath
            self.config['labelmodulefoldername'] = labelmodulefolderpath
            self.config['cachefoldername'] = cachefolderpath
            self.config['tempfoldername'] = tempfolderpath
            self.config['exportfoldername'] = exportfolderpath
            self.config['printfoldername'] = printfolderpath
            self.config['notifyfoldername'] = notifyfolderpath
            self.notifypath = os.path.join(notifyfolderpath, 'notify.txt')
            self.cache.changePath(os.path.join(self.config['cachefoldername'], self.config['cachefilename']))
            self.datainfo.datamodules.changePath(os.path.join(self.config['datamodulefoldername'], self.config['datamodulefilename']),
                                                 os.path.join(self.config['datamodulefoldername'], self.config['enabledmodulefilename']))
            self.datainfo.labelmodules.changePath(os.path.join(self.config['labelmodulefoldername'], self.config['labelmodulefilename']))
            self.graphinfo.args['temppath'] = self.config['tempfoldername']
            self.graphinfo.args['cmdpath'] = self.config['commandfoldername']
        dialog.Destroy()
        
    def StopBatchThread(self):
        self.batchThread.stop()
        self.batchThread.join()
        self.batchThread = None
        self.batchmode = False

    def StartBatchThread(self, lst_dataid, flg_pdf, flg_print, flg_export):
        self.batchThread = BatchThread(self, lst_dataid, flg_pdf, flg_print, flg_export)
        self.batchThread.start()
        self.batchmode = True

    def OnBatchFinish(self, evt=None):
        self.batchThread = None
        self.batchmode = False
        self.menuDict['Batch*'].Check(self.batchmode)

    def OnBatch(self, event=None):
        """
        Batch メニューのコールバック
        """
        batchmenu=self.menuDict['Batch*']
        if self.batchmode:
            # stop thread here 
            self.StopBatchThread()
            #self.batchThread.stop()
            #self.batchThread.join()
            #self.batchThread = None
            #self.batchmode = False
            batchmenu.Check(self.batchmode)
        else:
            dialog = GetBatchFromToDlg(self, self.tcDataId.GetValue())
            result = dialog.ShowModal()
            if result == wx.ID_OK:
                str_shots, create_pdf, print_out, export_flg = dialog.GetResult()
                shots = []
                lst_shots = str_shots.split(',')
                for shot in lst_shots:
                    m = re.search(r"(\d+)\s*-\s*(\d+)",shot)
                    if ( m ):
                        s = int(m.group(1))
                        e = int(m.group(2))
                        shots.extend(list(range(s,e+1)))
                    else:
                        shots.append(int(shot))
                lst_dataid = [str(s) for s in shots]
                #start thread here
                self.batchThread = BatchThread(
                        self, lst_dataid, create_pdf, print_out, export_flg)
                self.batchThread.start()
                self.batchmode = True
                batchmenu.Check(self.batchmode)
            else:
                self.batchmode = False
                batchmenu.Check(self.batchmode)
            dialog.Destroy()
   
    def OnNotify(self, event=None):
        """
        Notifyメニューのコールバック
        """
        notifymenu=self.menuDict['Notify*']
        if self.notifymode:
            # stop thread here 
            self.notifyThread.stop()
            print("Starting Notify mode")
            self.notifyThread.join()
            self.notifyThread = None
            self.notifymode = False
            notifymenu.Check(self.notifymode)
        else:
            print("Starting Notify mode")
            self.notifyThread = NotifyThread(self, self.notifypath)
            self.notifyThread.start()
            self.notifymode = True
            notifymenu.Check(self.notifymode)

    def OnServer(self, event=None):
        """
        サーバーメニューのコールバック
        """
        servermenu=self.menuDict['Server*']
        if self.servermode:
            # stop thread here 
            self.serverThread.stop()
            print("Stopping Server")
            self.serverThread.join()
            self.serverThread = None
            self.servermode = False
            servermenu.Check(self.servermode)
        else:
            print("Starting Server")
            self.serverThread = ServerThread(self)
            self.serverThread.start()
            self.servermode = True
            servermenu.Check(self.servermode)

    def OnRealtime(self, event=None):
        """
        Realtimeメニューのコールバック
        """
        realtimemenu=self.menuDict['Realtime*']

        if self.updatemode:
            print("Stopping Updater")
            self.CancelUpdate()
            self.updatemode = False
            realtimemenu.Check(self.updatemode)
        else:
            c = self.updateContext
            dialog = SetRealtimeDlg(None, c.flg_print, c.print_dialog, c.auto_delay, c.duration)
            result = dialog.ShowModal()
            if ( result == wx.ID_OK ):
                c.reset()
                l = dialog.get_result()
#                print "l=",l
#                c.flg_print, c.print_dialog, c.auto_delay = dialog.get_result()
                c.flg_print, c.print_dialog, c.auto_delay, c.duration = l
                self.graphinfo.realtime_printer = c.print_dialog
                self.graphinfo.realtime_delay = c.auto_delay
                self.graphinfo.realtime_duration = c.duration
                self.StartUpdate()
                self.updatemode = True
                realtimemenu.Check(self.updatemode)
            dialog.Destroy()

    def OnRecorder(self, event):
        if self.recorderframe.IsShown():
            self.recorderframe.Hide()
            self.menuDict["Recorder*"].Check(False)
        else:
            self.recorderframe.Show()
            self.menuDict["Recorder*"].Check(True)
            self.recorderframe.Raise()

    def IsRecorderValid(self):
        return self.menuDict["Recorder*"].IsChecked()

    def Record(self, xydata):
        self.recorderframe.Add(xydata)

    def OnAbout(self, event):
        """
        About メニューのコールバック
        """
        dlg = AboutDlg(self)
        dlg.ShowModal()
        dlg.Destroy()
    
    def OnClearDisplayCache(self, event=None):
        if self.flgBusy: return
        ClearDisplayCache()
        print("*********** Cleared display cache *************")

    def OnCache(self, event=None):
        if self.flgBusy: return
        diag = wx.MessageDialog(None,
                                "Are you sure to delete all the cache data?",
                                "WARNING",
                                wx.YES_NO|wx.NO_DEFAULT)
        
        ret = diag.ShowModal()
        diag.Destroy()
        if ( ret == wx.ID_YES ):
            self.cache.removeall()

    def OnRebuildCache(self, event=None):
        if self.flgBusy: return

        cachefilepath = os.path.join(
                self.config['cachefoldername'],self.config['cachefilename'])
        RebuildCache(cachefilepath)
        self.cache.changePath(cachefilepath)

    def ReloadModuleList(self):
        datamodulefilepath = os.path.join(self.config['datamodulefoldername'],self.config['datamodulefilename'])
        enabledmodulefilepath = os.path.join(self.config['datamodulefoldername'],self.config['enabledmodulefilename'])
        self.datainfo.datamodules.ModuleLoad(datamodulefilepath)
        if ( os.path.exists(enabledmodulefilepath )):
            self.datainfo.datamodules.EnabledModuleLoad(enabledmodulefilepath)
        else:
            self.datainfo.datamodules.SetAllDisabled()
            self.datainfo.datamodules.EnabledModuleSave(enabledmodulefilepath)

        self.datainfo.datamodules.AddEnabledModules(self.graphinfo.getModules())

    def OnModule(self, event=None):
        if self.flgBusy: return
        self.ReloadModuleList()
        
        dialog = MyViewSetModuleDlg(self)
        modules = self.datainfo.datamodules.GetDataModules()
        enabled = self.datainfo.datamodules.GetEnabledModules()
        synopses = [self.datainfo.getDataModuleSynopsis(e) for e in modules]
        urls = [self.datainfo.getDataModuleReference(e) for e in modules]
        dialog.SetModules(modules, enabled, synopses, urls)

        result = dialog.ShowModal()
        if ( result == wx.ID_OK ):
            modules = dialog.GetEnabled()
            self.datainfo.datamodules.SetEnabledModules(modules)
            enabledmodulefilepath = os.path.join(self.config['datamodulefoldername'],self.config['enabledmodulefilename'])
            self.datainfo.datamodules.EnabledModuleSave(enabledmodulefilepath)
            self.datainfo.datamodules.AddEnabledModules(self.graphinfo.getModules())
        labelmodulefilepath = os.path.join(self.config['labelmodulefoldername'],self.config['labelmodulefilename'])
        self.datainfo.labelmodules.ModuleLoad(labelmodulefilepath)

    def OnPreset(self, event=None):
        if self.flgBusy: return
        dialog = MyViewSetPresetDlg(self)
        presets = Presets()
        presetfilepath = os.path.join(self.config['presetfoldername'],self.config['presetfilename'])
        presets.PresetLoad(presetfilepath)
        dialog.SetPresets(presets)
        result = dialog.ShowModal()

        if ( result == wx.ID_OK ):
            selected = dialog.GetSelected()
            sourcepath = os.path.join(self.config['presetfoldername'],selected)

            layoutpath = 'user/layout'
            distpath = os.path.join(layoutpath, selected)
            #self.openfilepath = filepath
            self.openfolderpath = layoutpath
            self.openfilepath = Copyfile(sourcepath, distpath) 
            base, self.openfilename = os.path.split(self.openfilepath)

        dialog.Destroy()
        if result == wx.ID_OK:
            print('filepath=',self.openfilepath)
            self.graphinfo.load(self.openfilepath)
            self.tcDataId.SetValue(self.graphinfo.dataid)
            self.tcDataIdRef.SetValue(self.graphinfo.dataid_ref)
            self.ReloadModuleList()

            if wx.__version__ >= "2.9":
                #print self.canvaspanel.canvas.GetSize()
                #print self.canvaspanel.GetSize()
                #print self.GetClientSize()
                #print "-------------"
                #self.canvaspanel.canvas.SetSize(self.graphinfo.canvassize)
                self.SetClientSize(self.graphinfo.canvassize)
                #print self.canvaspanel.canvas.GetSize()
                #print self.canvaspanel.GetSize()
                #print self.GetClientSize()
            else:
                self.canvaspanel.canvas.SetSize(self.graphinfo.canvassize)
            
            # cache change here
            # temppath change here
            self.Update()
        
        
    def OnConvLayout(self, event=None):
        if self.flgBusy: return
        MyViewConvLayoutDlg.ConvertLayout(self, self.openfolderpath)

    def OnRebuildEGList(self, event=None):
        EGList.Rebuild_EGList('eglist.txt')

    def OnResetEGList(self, event=None):
        EGList.Rebuild_EGList('eglist.txt', flg_count_reset=True)

    def OnChangeProfile(self, event=None):
        profile = MyViewProfile.Profile()
        current = self.config['profile']
        dlg = MyViewSetProfileDlg.SetProfileDlg(self, profile, current)
        result = dlg.ShowModal()
        if result == wx.ID_OK:
            choice = dlg.GetResult()
            profile.update(choice)
            wx.MessageBox("You need to restart myView2 to take this change into effect")
#            sys.exit(0)
        
        
    def OnDummy(self, event=None):
        """
        """
        pass


    def catPDF(self, iflns, ofln):
        """
        PDFを連結する
        """
        output = PdfFileWriter()
        fps = []
        for fln in iflns:
            fp = open(fln, "rb")
            fps.append(fp)
            input = PdfFileReader(fp)
            for i in range(input.getNumPages()):
                output.addPage(input.getPage(i))
        outputStream = file(ofln, "wb")
        output.write(outputStream)
        outputStream.close()
        for fp in fps:
            fp.close()

    def PrintPDF(self, fname, Append=False):
        """
        PDF出力
        """
        d=os.path.dirname(fname)
        temppdf=os.path.join(d,'temp.pdf')
        if ( not Append ):
            self.canvaspanel.figure.savefig(fname)
            return
        temppdf1=os.path.join(d,'temp1'+util.random_str()+'.pdf')
        temppdf2=os.path.join(d,'temp2'+util.random_str()+'.pdf')
        try:
            self.canvaspanel.figure.savefig(temppdf1)
            self.catPDF([fname, temppdf1], temppdf2)
            os.unlink(fname)
            shutil.move(temppdf2, fname)
            os.unlink(temppdf1)
        except:
            import sys,traceback
            logger.error(traceback.print_exc())

    def SendToVirtualPrinter(self, diag, shotno):
        """
        バーチャルプリンタに出力する
        """
        fname = "temp.ps"
#        diag=u'lastshot'
        self.PrintPS(fname)
        printer = vprint_client.VirtualPrinter()
        printer.printout(diag, shotno, fname)
    
    def showGroupFrame(self):
        self.groupframe.Show()
        self.menuDict['&Group and Trace*'].Check(True)
        self.groupframe.Centre()

    def hideGroupFrame(self):
        self.groupframe.Hide()
        self.menuDict['&Group and Trace*'].Check(False)

    def SetMyViewTitle(self):
        try:
            version = int(open('version.txt').read().strip())
        except:
            version = 0
        profile = self.config['profile']
        filename = os.path.basename(self.openfilepath)
        title = "MyView2 [ver.%d] (%s) [%s]" % (version, filename, profile)
        self.SetTitle(title)
        variables['%LAYOUT_FILE%']=filename
        variables['%VERSION%']="%d" % (version)
        variables['%TITLE%']=title
        
class MyViewApp(wx.App):
    DEBUG=False
    """
    アプリケーション起動用
    """
    def __init__(self):
        wx.App.__init__(self, False, 'log.txt')

    def OnInit(self):
        global FRAME_GLOBAL
        frame = MyViewFrame()
        frame.Show(True)
        FRAME_GLOBAL = frame
        return True


if __name__ == '__main__':
    os.chdir(os.path.dirname(sys.argv[0]) or '.')
    try:
        opts, args = getopt.getopt(sys.argv[1:], "adrlm:h",["call-after", "debug","realtime-debug","log","message-level=","help"])
    except getopt.GetoptError:
        pass
    opt_debug = False
    opt_rdebug = False
    opt_logging = False
    opt_call_after = False
    opt_level = 3
    LEVELS = [logging.DEBUG, logging.INFO, logging.WARNING, logging.ERROR, logging.CRITICAL ]
    
    for o, a in opts:
        if ( o in ('-a','--call-after')):
            opt_call_after=True
        if ( o in ('-d','--debug')):
            opt_debug=True
        if ( o in ('-r','--realtime-debug')):
            opt_rdebug=True
        if ( o in ('-l','--log')):
            opt_logging=True
        if ( o in ('-m','--message-level')):
            opt_level=int(a)
        if ( o in ('-h','--help')):
            print("usage: python myView2.py [options..] [layout file]")
            print("option: ")
            print("-a / --call-after                    : use Call After ")
            print("-d / --debug                         : debug mode on ")
            print("-r / --realtime-debug                : debug mode for realtime on ")
            print("-l / --log                           : logging on ")
            print("-m [12345] / --message-level=[12345] : set message level (1 -  5)")
            sys.exit(0)
    if opt_call_after:
        CallAfter = wx.CallAfter
    if opt_debug:
        debug = True
    if opt_rdebug:
        realtime_debug = True
    if opt_logging:
        logger.addHandler(fileHandler)
    if ( 0 <= opt_level < len(LEVELS)):
        logger.setLevel(LEVELS[opt_level-1])
    if ( len(args) == 1 ):
        INIT_FILE = args[0]
    
    app = MyViewApp()
    logger.info('My View2 starts')
    if wx.__version__ >= '4.1.0':
        app.ResetLocale()
    app.MainLoop()

        
