#!/usr/bin/env python
# coding: utf-8
# $Id$

import itertools
from email.generator import _make_boundary
import mimetypes
import urllib.request, urllib.parse, urllib.error
import urllib.request, urllib.error, urllib.parse
import os

URL='http://egftp1.lhd.nifs.ac.jp/cgi-bin/virtual_printer.cgi'

class MultiPartForm(object):
    """Accumulate the data to be used when posting a form."""

    def __init__(self):
        self.form_fields = []
        self.files = []
        self.boundary = _make_boundary()
        return

    def get_content_type(self):
        return 'multipart/form-data; boundary=%s' % self.boundary

    def add_field(self, name, value):
        """Add a simple field to the form data."""
        self.form_fields.append((name, value))
        return

    def add_file(self, fieldname, filename, fileHandle, mimetype=None):
        """Add a file to be uploaded."""
        body = fileHandle.read()
        if mimetype is None:
            mimetype = mimetypes.guess_type(filename)[0] or 'application/octet-stream'
        self.files.append((fieldname, filename, mimetype, body))
        return
    
    def __str__(self):
        """Return a string representing the form data, including attached files."""
        # Build a list of lists, each containing "lines" of the
        # request.  Each part is separated by a boundary string.
        # Once the list is built, return a string where each
        # line is separated by '\r\n'.
        parts = []
        part_boundary = '--' + self.boundary
        
        # Add the form fields
        parts.extend(
            [ part_boundary,
              'Content-Disposition: form-data; name="%s"' % name,
              '',
              value,
            ]
            for name, value in self.form_fields
            )
        
# Add the files to upload
        parts.extend(
            [ part_boundary,
              'Content-Disposition: file; name="%s"; filename="%s"' % \
                  (field_name, filename),
              'Content-Type: %s' % content_type,
              '',
              body,
            ]
            for field_name, filename, content_type, body in self.files
            )
        
        # Flatten the list and add closing boundary marker,
        # then return CR+LF separated data
        flattened = list(itertools.chain(*parts))
        flattened.append('--' + self.boundary + '--')
        flattened.append('')
        return '\r\n'.join(flattened)

class VirtualPrinter():
    def __init__(self):
        pass

    def printout(self, diagnostics, shotno, filename):
#        print "diagnostics=",diagnostics
#        print "shotno=",shotno
#        print "filename=",filename
        if ( type(diagnostics) == str ):
            diag = diagnostics.encode('ascii','ignore')
        else:
            diag = diagnostics
        fp = open(filename,'r')
        form = MultiPartForm()
        form.add_field('diagnostics', diag)
        form.add_field('shotno', str(shotno))
        
        # Add a fake file
        form.add_file('file', os.path.basename(filename),
                      fileHandle=fp)
        
        # Build the request
        request = urllib.request.Request(URL)
        request.add_header('User-agent', 'vprint')
        body = str(form)
        request.add_header('Content-type', form.get_content_type())
        request.add_header('Content-length', len(body))
        request.add_data(body)
        urllib.request.urlopen(request)
        fp.close()
        
if __name__ == '__main__':
    # Create the form with simple fields

    diagnostics = 'tsmap'
    shotno = 117135
#    filename = 'tsmap_117135.ps'
    filename = 'temp.ps'

    printer = VirtualPrinter()
    printer.printout(diagnostics, shotno, filename)



